Package: BulkSignalR
Type: Package
Title: Infer Ligand-Receptor Interactions from bulk expression
        (transcriptomics/proteomics) data, or spatial transcriptomics
Version: 1.2.0
Authors@R: c(
    person("Jacques","Colinge",email="jacques.colinge@inserm.fr",role="aut",comment = c(ORCID = "0000-0003-2466-4824")),
    person("Jean-Philippe","Villemin",email="jpvillemin@gmail.com",role="cre",comment = c(ORCID = "0000-0002-1838-5880")))
Description: Inference of ligand-receptor (LR) interactions from bulk
    expression (transcriptomics/proteomics) data, or spatial transcriptomics.
    BulkSignalR bases its inferences
    on the LRdb database included in our other package, SingleCellSignalR
    available from Bioconductor. It relies on a statistical model that
    is specific to bulk data sets. Different visualization and data
    summary functions are proposed to help navigating prediction results.
URL: https://github.com/ZheFrench/BulkSignalR
BugReports: https://github.com/ZheFrench/BulkSignalR/issues
License: CeCILL | file LICENSE
Encoding: UTF-8
LazyData: FALSE
Depends: R (>= 4.5)
biocViews: Network, RNASeq, Software, Proteomics, Transcriptomics,
        NetworkInference, Spatial
Imports: BiocFileCache, httr2, RCurl, cli, curl, rlang, jsonlite,
        matrixStats, methods, doParallel, glmnet, ggalluvial, ggplot2,
        gridExtra, grid, Rtsne, ggrepel, foreach, multtest, igraph,
        orthogene, stabledist, circlize (>= 0.4.14), ComplexHeatmap (>=
        2.0.0), stats, scales, RANN, SpatialExperiment,
        SummarizedExperiment, tools
Suggests: knitr, markdown, rmarkdown, STexampleData, testthat (>=
        3.0.0), codetools, Matrix, lattice, cluster, survival, MASS,
        nlme
Config/testthat/edition: 3
RoxygenNote: 7.3.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/BulkSignalR
git_branch: RELEASE_3_22
git_last_commit: 4e95251
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 02:57:05 UTC; biocbuild
Author: Jacques Colinge [aut] (ORCID: <https://orcid.org/0000-0003-2466-4824>),
  Jean-Philippe Villemin [cre] (ORCID:
    <https://orcid.org/0000-0002-1838-5880>)
Maintainer: Jean-Philippe Villemin <jpvillemin@gmail.com>

# Generated by roxygen2: do not edit by hand

export(BSRClusterComp)
export(BSRDataModel)
export(BSRDataModelComp)
export(BSRInference)
export(BSRInferenceComp)
export(BSRSignature)
export(BSRSignatureComp)
export(alluvialPlot)
export(assignCellTypesToInteractions)
export(bubblePlotPathwaysLR)
export(cacheClear)
export(cacheInfo)
export(cacheVersion)
export(cellTypeFrequency)
export(cellularNetwork)
export(cellularNetworkTable)
export(chordDiagramLR)
export(convertToHuman)
export(createResources)
export(findOrthoGenes)
export(generateSpatialPlots)
export(getLRIntracellNetwork)
export(getLRNetwork)
export(getResource)
export(maxLigandSpatialCounts)
export(relateToGeneSet)
export(resetLRdb)
export(resetNetwork)
export(resetPathways)
export(scoreSignatures)
export(separatedLRPlot)
export(signatureHeatmaps)
export(simpleHeatmap)
export(smoothSpatialCounts)
export(spatialAssociation)
export(spatialAssociationPlot)
export(spatialDiversityPlot)
export(spatialIndexPlot)
export(spatialPlot)
export(summarizedCellularNetwork)
exportClasses(BSRClusterComp)
exportClasses(BSRDataModel)
exportClasses(BSRDataModelComp)
exportClasses(BSRInference)
exportClasses(BSRInferenceComp)
exportClasses(BSRSignature)
exportClasses(BSRSignatureComp)
exportMethods(LRinter)
exportMethods(LRinterScore)
exportMethods(LRinterShort)
exportMethods(addClusterComp)
exportMethods(coerce)
exportMethods(colClusterA)
exportMethods(colClusterB)
exportMethods(comparison)
exportMethods(comparisonName)
exportMethods(differentialStats)
exportMethods(getPathwayStats)
exportMethods(inferenceParameters)
exportMethods(initialOrganism)
exportMethods(initialOrthologs)
exportMethods(learnParameters)
exportMethods(ligands)
exportMethods(logTransformed)
exportMethods(mu)
exportMethods(ncounts)
exportMethods(normalization)
exportMethods(parameters)
exportMethods(pathways)
exportMethods(receptors)
exportMethods(reduceToBestPathway)
exportMethods(reduceToLigand)
exportMethods(reduceToPathway)
exportMethods(reduceToReceptor)
exportMethods(removeClusterComp)
exportMethods(rescoreInference)
exportMethods(resetToInitialOrganism)
exportMethods(scoreLRGeneSignatures)
exportMethods(sourceComparisonName)
exportMethods(tgCorr)
exportMethods(tgExpr)
exportMethods(tgGenes)
exportMethods(tgLogFC)
exportMethods(tgPval)
exportMethods(updateInference)
import(BiocFileCache)
import(ComplexHeatmap)
import(SpatialExperiment)
import(SummarizedExperiment)
import(doParallel)
import(ggalluvial)
import(ggplot2)
import(glmnet)
import(grid)
import(httr2)
import(igraph)
import(jsonlite)
import(multtest)
importFrom(BiocFileCache,BiocFileCache)
importFrom(BiocFileCache,bfcinfo)
importFrom(BiocFileCache,bfcquery)
importFrom(BiocFileCache,bfcremove)
importFrom(BiocFileCache,removebfc)
importFrom(ComplexHeatmap,"%v%")
importFrom(RANN,nn2)
importFrom(RCurl,url.exists)
importFrom(Rtsne,Rtsne)
importFrom(circlize,chordDiagramFromDataFrame)
importFrom(circlize,circos.axis)
importFrom(circlize,circos.clear)
importFrom(circlize,circos.par)
importFrom(circlize,circos.text)
importFrom(circlize,circos.trackPlotRegion)
importFrom(circlize,colorRamp2)
importFrom(circlize,get.cell.meta.data)
importFrom(cli,cli_alert)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_inform)
importFrom(curl,has_internet)
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(ggrepel,geom_text_repel)
importFrom(gridExtra,grid.arrange)
importFrom(matrixStats,colQuantiles)
importFrom(matrixStats,colSums2)
importFrom(matrixStats,rowMeans2)
importFrom(matrixStats,rowSums2)
importFrom(methods,is)
importFrom(methods,new)
importFrom(orthogene,convert_orthologs)
importFrom(rlang,.data)
importFrom(scales,rescale)
importFrom(stabledist,dstable)
importFrom(stabledist,pstable)
importFrom(stats,quantile)

# BulkSignalR <img  width="120" height="139" src="man/figures/logo.png" align="right" />
  

<!-- badges: start -->
![Bioconductor Time](https://bioconductor.org/shields/years-in-bioc/BulkSignalR.svg)
![Bioconductor Downloads](https://bioconductor.org/shields/downloads/release/BulkSignalR.svg)
<!-- badges: end -->


## Overview

**BulkSignalR** is used to infer ligand-receptor (L-R) interactions from bulk
expression (transcriptomics/proteomics) data, or medium resolution spatial
transcriptomics such as the 10x Genomics Visium platform. Particular types of
bulk data are also supported such as purified cell populations transcriptomics
and proteomics, or PDX mRNA-sequencing.

Known L-R interactions are taken from the
LR*db* database, which was previously included in our other package 
**SingleCellSignalR**, also available from Bioconductor 
[here](https://www.bioconductor.org/packages/release/bioc/html/SingleCellSignalR.html).
Now, LR*db* is distributed and updated separately. 

Inferences rely on a statistical model linking potential L-R interactions
with biological pathways downstream the receptor as defined in Reactome or
GO Biological Processes.

A number of visualization and data summary functions are proposed to
help navigating the predicted interactions.

Note that due to BulkSignalR introduction in Bioconductor, we modified certain
class definitions and functions to adhere to Bioconductor standards. An overview
of changes in BulkSignalR usage compared to early versions is provided in the
companion project: BulkSignalR_companion.


<img   src="man/figures/workflow.png" align="center" width="85%" height="85%" />
  

## Installation

``` R

# BulkSignalR directly from Bioconductor.
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("BulkSignalR")

# or Installation goes via GitHub:
# install.packages("devtools")
devtools::install_github("jcolinge/BulkSignalR",build_vignettes = TRUE)

# To read the vignette
# browseVignettes("BulkSignalR")

```

## Notes

For a version history/change logs, see the [NEWS file](https://github.com/jcolinge/BulksignalR/blob/master/NEWS).

**Previous versions of BulkSignalR** (before introduction in Bioconductor) are availabe in
the branch named before_BC at GitHub.com/jcolinge/BulkSignalR.

**BulkSignalR** has been successfully installed on Mac OS X, Linux, and Windows using R version 4.5.


The code in this repository is published with the [CeCILL](https://github.com/jcolinge/BulksignalR/blob/master/LICENSE.md) License.


<!-- badges: start -->
[![Generic badge](https://img.shields.io/badge/License-CeCILL-green.svg)](https://shields.io/)
<!-- badges: end -->

#' Internal function to generate expression matrix permutation indices
#'
#' @param ncounts         A matrix of normalized read counts.
#' @param n.bins          Number of bins.
#'
#' @return A list containing a vectors of row indices. The vector at index
#'   \code{i} in the list contains the row indices of rows with mean normalized
#'   read count in bin \code{i}.
#'
#' @importFrom foreach %do% %dopar%
#' @importFrom stats quantile
#' @importFrom matrixStats rowMeans2 colQuantiles
#' @keywords internal
.buildPermutationIndices <- function(ncounts, n.bins = 20) {
    rm <- matrixStats::rowMeans2(ncounts, na.rm = TRUE)
    breaks <- matrixStats::colQuantiles(data.matrix(rm),
        probs = (seq(0, n.bins)) / n.bins)

    breaks[1] <- 0

    lapply(
        seq(2, length(breaks)),
        function(i) which(rm > breaks[i - 1] & rm <= breaks[i])
    )
} # .buildPermutationIndices


#' Internal function to shuffle permutation indices
#'
#' @param pind      Permutation indices such as returned by
#'   \code{\link{.buildPermutationIndices}}.
#'
#' @return A list with same structure as \code{pind} with shuffled indices
#'   within each bin.
#'
#' @importFrom foreach %do% %dopar%
#' @keywords internal
.shufflePermutationIndices <- function(pind) {
    lapply(
        pind,
        function(x) sample(x, length(x))
    )
} # .shufflePermutationIndices


#' Internal function to generate a randomized expression matrix
#'
#' @param ncounts    A matrix of normalized read counts.
#' @param pind      Permutation indices such as returned by
#'   \code{\link{.buildPermutationIndices}}.
#'
#' @return \code{ncount} with shuffled row names (gene symbols). Shuffling is
#'   performed within rows of comparable average expression.
#'
#' @keywords internal
.buildPermutatedCountMatrix <- function(ncounts, pind) {
    symbols <- rownames(ncounts)
    rind <- .shufflePermutationIndices(pind)

    for (i in seq_len(length(pind))) {
        symbols[pind[[i]]] <- symbols[rind[[i]]]
    }
    rownames(ncounts) <- symbols

    ncounts
} # .buildPermutatedCountMatrix


#' Internal function to fit a censored Gaussian distribution
#'
#' @description Maximum-likelihood estimators are used.
#'
#' @param d   A vector of values to fit.
#' @param title     A plot title.
#' @param verbose   Provide details on computations.
#' @param file.name   The file name of a PDF file.
#'
#' @return A list with the mean (\code{mu}) and standard deviation
#'   (\code{sigma}) estimates.
#'
#'   If \code{file.name} is provided, a control plot is generated in a PDF with
#'   a data histogram and the fitted Gaussian. \code{title} is used to give this
#'   plot a main title.
#' @keywords internal
.getGaussianParam <- function(d, title, verbose = FALSE, file.name = NULL) {
    if (!is.null(file.name)) {
        grDevices::pdf(
            file = file.name, width = 4, height = 4,
            pointsize = 10, useDingbats = FALSE
        )
        graphics::hist(d,
            freq = FALSE, main = paste0(title, " / censored normal"),
            xlab = "Spearman correlation", breaks = 30
        )
    }

    # initial fit with a Gaussian over ]-infty;+infty[]
    mu <- mean(d)
    sigma <- stats::sd(d)
    if (verbose) {
        message("Initial estimate of the mean: ", mu)
        message("Initial estimate of the standard deviation: ", sigma)
    }

    # ML fit of a censored Gaussian on [-1;1]
    GaussianLL <- function(par) {
        q <- stats::pnorm(1, par[1], par[2]) - stats::pnorm(-1, par[1], par[2])
        -sum(stats::dnorm(d, par[1], par[2], log = TRUE)) + length(d) * log(q)
    }
    par.0 <- c(mu, sigma)
    lo.bound <- c(mu - 0.5, 0.5 * sigma)
    hi.bound <- c(mu + 0.5, 1.5 * sigma)
    res <- stats::optim(par.0, GaussianLL,
        method = "L-BFGS-B",
        lower = lo.bound, upper = hi.bound,
        control = list(maxit = 2000)
    )
    if (res$convergence != 0) {
        stop("optim() could not fit the normal distribution parameters")
    }
    mu <- res$par[1]
    sigma <- res$par[2]
    if (verbose) {
        message("Censored normal mean:" , mu)
        message("Censored normal standard deviation: ", sigma)
    }
    q <- stats::pnorm(1, mu, sigma) - stats::pnorm(-1, mu, sigma)
    start <- stats::pnorm(-1, mu, sigma)
    params <- list(
        mu = mu, sigma = sigma, factor = q,
        start = start, distrib = "censored_normal"
    )

    # KS test D statistics
    x <- seq(-1, 1, by = 0.005)
    y <- stats::dnorm(x, mu, sigma) / q
    params$D <- stats::ks.test(d, y, simulate.p.value=TRUE)$statistic
    if (inherits(params$D, "try-error")) {
        params$D <- NULL
    }
    if (is.null(params$D)){
        stop("Call to `stats::ks.test` failed")
    }
    params$D <- as.numeric(params$D)

    # Chi2
    x <- seq(-1, 1, by = 0.05)
    h <- graphics::hist(d, breaks = x, plot = FALSE)
    hist.rf <- h$counts / length(d)
    gauss.rf <- .cdfGaussian(h$breaks[-1], params) -
        .cdfGaussian(h$breaks[-length(h$breaks)], params)
    params$Chi2 <- sum((hist.rf - gauss.rf)**2)

    # control plot
    if (!is.null(file.name)) {
        x <- seq(-1, 1, by = 0.002)
        graphics::lines(
            x = x, y = stats::dnorm(x, mu, sigma) / q,
            col = "blue", type = "l"
        )
        graphics::legend(
            x = "topright", lty = 1, legend = "Model",
            col = "blue", bty = "n", pt.cex = 0.5
        )
        grDevices::dev.off()
        # fn <- gsub("pdf$", "txt", file.name)
        # write.table(d, file=fn, row.names = FALSE)
    }

    params
    
} # .getGaussianParam


#' Internal function to compute a censored Gaussian CDF
#'
#' @param x   A vector of observed values.
#' @param par A list containing the censored Gaussian model parameters.
#'
#' @return A vector of probabilities P(X<x|par).
#' @keywords internal
.cdfGaussian <- function(x, par) {
    (stats::pnorm(x, par$mu, par$sigma) - par$start) / par$factor
} # .cdfGaussian


#' Internal function to fit a censored mixed-Gaussian distribution
#'
#' @description Maximum-likelihood estimators are used.
#'
#' @param d   A vector of values to fit.
#' @param title     A plot title.
#' @param verbose   Provide details on computations.
#' @param file.name   The file name of a PDF file.
#'
#' @return A list with the mean (\code{mu}) and standard deviation
#'   (\code{sigma}) estimates of each distribution along with the
#'   weight alpha applied to the first distribution.
#'
#'   If \code{file.name} is provided, a control plot is generated in a PDF with
#'   a data histogram and the fitted Gaussian. \code{title} is used to give this
#'   plot a main title.
#' @keywords internal
.getMixedGaussianParam <- function(d, title, verbose = FALSE,
    file.name = NULL) {
    if (!is.null(file.name)) {
        grDevices::pdf(
            file = file.name, width = 4, height = 4,
            pointsize = 10, useDingbats = FALSE
        )
        graphics::hist(d,
            freq = FALSE,
            main = paste0(title, " / censored mixed normal"),
            xlab = "Spearman correlation", breaks = 30
        )
    }

    # ML fit of a censored mixed-Gaussian on [-1;1]
    mixedGaussianLL <- function(par) {
        alpha <- par[1]
        mu1 <- par[2]
        sigma1 <- par[3]
        mu2 <- par[4]
        sigma2 <- par[5]
        q <- alpha * stats::pnorm(1, mu1, sigma1) +
            (1 - alpha) * stats::pnorm(1, mu2, sigma2) -
            (alpha * stats::pnorm(-1, mu1, sigma1) +
                (1 - alpha) * stats::pnorm(-1, mu2, sigma2)
            )
        -sum(log(alpha * stats::dnorm(d, mu1, sigma1) +
            (1 - alpha) * stats::dnorm(d, mu2, sigma2))) + length(d) * log(q)
    }
    mu <- mean(d)
    sigma <- stats::sd(d)
    par.0 <- c(0.7, mu - 0.1, 0.75 * sigma, mu + 0.1, 3 * sigma)
    lo.bound <- c(0.5, mu - 0.5, 0.5 * sigma, mu - 0.2, 2 * sigma)
    hi.bound <- c(1.0, mu + 0.2, 1.5 * sigma, mu + 0.4, 10 * sigma)
    res <- stats::optim(par.0, mixedGaussianLL,
        method = "L-BFGS-B",
        lower = lo.bound, upper = hi.bound,
        control = list(maxit = 2000)
    )
    if (res$convergence != 0) {
        stop("optim() could not fit the mixed normal distribution parameters")
    }
    if (verbose) {
        message(
            "Censored mixed normal parameters ",
            "(alpha, mean1, sd1, mean2, sd2): ",
            paste(res$par, collapse = ", ")
        )
    }
    alpha <- res$par[1]
    mu1 <- res$par[2]
    sigma1 <- res$par[3]
    mu2 <- res$par[4]
    sigma2 <- res$par[5]
    q <- alpha * stats::pnorm(1, mu1, sigma1) +
        (1 - alpha) * stats::pnorm(1, mu2, sigma2) -
        (alpha * stats::pnorm(-1, mu1, sigma1) +
            (1 - alpha) * stats::pnorm(-1, mu2, sigma2)
        )
    start <- alpha * stats::pnorm(-1, mu1, sigma1) +
        (1 - alpha) * stats::pnorm(-1, mu2, sigma2)
    params <- list(
        alpha = alpha, mu1 = mu1, sigma1 = sigma1, mu2 = mu2, sigma2 = sigma2,
        factor = q, start = start, distrib = "censored_mixed_normal"
    )

    # KS test D statistics
    x <- seq(-1, 1, by = 0.005)
    y <- alpha * stats::dnorm(x, mu1, sigma1) +
        (1 - alpha) * stats::dnorm(x, mu2, sigma2) / q

    params$D <- stats::ks.test(d, y, simulate.p.value=TRUE)$statistic
    if (inherits(params$D, "try-error")) {
        params$D <- NULL
    }
    if (is.null(params$D)){
        stop("Call to `stats::ks.test` failed")
    }
    params$D <- as.numeric(params$D)

    # Chi2
    x <- seq(-1, 1, by = 0.05)
    h <- graphics::hist(d, breaks = x, plot = FALSE)
    hist.rf <- h$counts / length(d)
    mixed.rf <- .cdfMixedGaussian(h$breaks[-1], params) -
        .cdfMixedGaussian(h$breaks[-length(h$breaks)], params)
    params$Chi2 <- sum((hist.rf - mixed.rf)**2)

    # control plot
    if (!is.null(file.name)) {
        x <- seq(-1, 1, by = 0.002)
        graphics::lines(
            x = x, y = alpha * stats::dnorm(x, mu1, sigma1) +
                (1 - alpha) * stats::dnorm(x, mu2, sigma2) / q,
            col = "blue", type = "l"
        )
        graphics::legend(
            x = "topright", lty = 1, legend = "Model",
            col = "blue", bty = "n", pt.cex = 0.5
        )
        grDevices::dev.off()
    }

    params
    
} # .getMixedGaussianParam


#' Internal function to compute a censored mixed-Gaussian CDF
#'
#' @param x   A vector of observed values.
#' @param par A list containing the censored mixed-Gaussian model parameters.
#'
#' @return A vector of probabilities P(X<x|par).
#' @keywords internal
.cdfMixedGaussian <- function(x, par) {
    (par$alpha * stats::pnorm(x, par$mu1, par$sigma1) +
        (1 - par$alpha) * stats::pnorm(x, par$mu2, par$sigma2) -
        par$start
    ) / par$factor
} # .cdfMixedGaussian


#' Internal function to fit an empirical distribution
#'
#' @description Based on stats::ecdf.
#'
#' @param d   A vector of values to fit.
#' @param title     A plot title.
#' @param verbose   Provide details on computations.
#' @param file.name   The file name of a PDF file.

#' @return A list with the step function implementing the CDF of
#'   the empirical distribution (\code{empirCDF}).
#'
#'   If \code{file.name} is provided, a control plot is generated in a PDF with
#'   a data histogram and the fitted Gaussian. \code{title} is used to give this
#'   plot a main title.
#' @keywords internal
.getEmpiricalParam <- function(d, title, verbose = FALSE, file.name = NULL) {
    if (!is.null(file.name)) {
        grDevices::pdf(
            file = file.name, width = 4, height = 4,
            pointsize = 10, useDingbats = FALSE
        )
        graphics::hist(d,
            freq = FALSE, main = paste0(title, " / empirical"),
            xlab = "Spearman correlation", breaks = 30
        )
    }

    empir <- stats::ecdf(d)

    # control plot
    if (!is.null(file.name)) {
        step <- 0.005
        x <- seq(-1, 1, by = step)
        cd <- empir(x)
        n <- length(x)
        left <- cd[-c(n - 1, n)]
        right <- cd[-c(1, 2)]
        dens <- c(0, (right - left) / 2 / step, 0)
        graphics::lines(x = x, y = dens, col = "blue", type = "l")
        graphics::legend(
            x = "topright", lty = 1, legend = "Model",
            col = "blue", bty = "n", pt.cex = 0.5
        )
        grDevices::dev.off()
    }

    list(empirCDF = empir, distrib = "empirical")
    
} # .getEmpiricalParam


#' Internal function to compute an empirical CDF
#'
#' @param x   A vector of observed values.
#' @param par A list containing the step function implementing the CDF.
#'
#' @return A vector of probabilities P(X<x|par).
#' @keywords internal
.cdfEmpirical <- function(x, par) {
    par$empirCDF(x)
} # .cdfEmpirical


#' Internal function to fit a Gaussian kernel-based empirical distribution
#'
#' @description Based on stats::density.
#'
#' @param d   A vector of values to fit.
#' @param title     A plot title.
#' @param verbose   Provide details on computations.
#' @param file.name   The file name of a PDF file.
#' @param n  The number of grid points for density FFT
#'
#' @return A list with the step function implementing the CDF of
#'   the empirical distribution (\code{kernelCDF}).
#'
#'   If \code{file.name} is provided, a control plot is generated in a PDF with
#'   a data histogram and the fitted Gaussian. \code{title} is used to give this
#'   plot a main title.
#' @keywords internal
.getKernelEmpiricalParam <- function(d, title, verbose = FALSE,
    file.name = NULL, n = 512) {
    if (!is.null(file.name)) {
        grDevices::pdf(
            file = file.name, width = 4, height = 4,
            pointsize = 10, useDingbats = FALSE
        )
        graphics::hist(d,
            freq = FALSE, main = paste0(title, " / kernel empirical"),
            xlab = "Spearman correlation", breaks = 30
        )
    }

    df <- stats::density(d, from = -1, to = 1, n = n)
    cd <- cumsum(df$y)
    cd <- cd / cd[n]
    params <- list(
        kernelCDF = stats::stepfun(df$x, c(0, cd)),
        distrib = "kernel_empirical"
    )

    # KS test D statistics
    params$D <- stats::ks.test(d,df$y, simulate.p.value=TRUE)$statistic
    if (inherits(params$D, "try-error")) {
        params$D <- NULL
    }
    if (is.null(params$D)){
        stop("Call to `stats::ks.test` failed")
    }
    params$D <- as.numeric(params$D)
    
    # Chi2
    x <- seq(-1, 1, by = 0.05)
    h <- graphics::hist(d, breaks = x, plot = FALSE)
    hist.rf <- h$counts / length(d)
    kernel.rf <- .cdfKernelEmpirical(h$breaks[-1], params) -
        .cdfKernelEmpirical(h$breaks[-length(h$breaks)], params)
    params$Chi2 <- sum((hist.rf - kernel.rf)**2)

    # control plot
    if (!is.null(file.name)) {
        # left <- cd[-c(n-1,n)]
        # right <- cd[-c(1,2)]
        # step <- 1/(n/2)
        # dens <- c(0, (right-left)/2/step, 0)
        # graphics::lines(x = df$x, y = dens, col = "blue", type = "l")
        graphics::lines(df, col = "blue", type = "l")
        graphics::legend(
            x = "topright", lty = 1, legend = "Model",
            col = "blue", bty = "n", pt.cex = 0.5
        )
        grDevices::dev.off()
    }

    params
    
} # .getKernelEmpiricalParam


#' Internal function to compute a Gaussian kernel-based empirical CDF
#'
#' @param x   A vector of observed values.
#' @param par A list containing the step function implementing the CDF.
#'
#' @return A vector of probabilities P(X<x|par).
#' @keywords internal
.cdfKernelEmpirical <- function(x, par) {
    par$kernelCDF(x)
} # .cdfKernelEmpirical


#' Internal function to fit a censored (alpha-) stable distribution
#'
#' @description Maximum-likelihood estimators are used.
#'
#' @param d   A vector of values to fit.
#' @param title     A plot title.
#' @param verbose   Provide details on computations.
#' @param file.name   The file name of a PDF file.
#'
#' @return A list with the four stable distribution parameters (S0
#'   representation).
#'
#'   If \code{file.name} is provided, a control plot is generated in a PDF with
#'   a data histogram and the fitted Gaussian. \code{title} is used to give this
#'   plot a main title.
#' @importFrom stabledist pstable dstable
#' @keywords internal
.getAlphaStableParam <- function(d, title, verbose = FALSE, file.name = NULL) {
    if (!is.null(file.name)) {
        grDevices::pdf(
            file = file.name, width = 4, height = 4,
            pointsize = 10, useDingbats = FALSE
        )
        graphics::hist(d,
            freq = FALSE,
            main = paste0(title, " / censored stable"),
            breaks = 30, xlab = "Spearman correlation"
        )
    }

    # ML fit of a censored stable on [-1;1]
    stableLL <- function(par) {
        q <- stabledist::pstable(1,
            alpha = par[1], beta = par[2],
            gamma = (par[3])**2, delta = par[4]
        ) -
            stabledist::pstable(-1,
                alpha = par[1], beta = par[2],
                gamma = (par[3])**2, delta = par[4]
            )
        -sum(stabledist::dstable(d,
            alpha = par[1], beta = par[2],
            gamma = (par[3])**2, delta = par[4], log = TRUE
        )) + length(d) * log(q)
    }
    par.0 <- c(1.5, 0.5, sqrt(stats::sd(d)), mean(d))
    if (verbose) {
        message(
            "Starting stable distribution parameter estimation. ",
            "This can take a few dozens of minutes..."
        )
    }
    res <- stats::optim(par.0, stableLL,
        control = list(
            reltol = 1e-5, maxit = 800,
            trace = ifelse(verbose, 1, 0)
        )
    )
    if (res$convergence != 0) {
        stop("optim() could not fit the censored stable distribution")
    }
    if (verbose) {
        message(
            "Censored stable parameters (alpha, beta, gamma, delta): ",
            paste(res$par, collapse = ", ")
        )
    }
    alpha <- res$par[1]
    beta <- res$par[2]
    gamma <- (res$par[3])**2
    delta <- res$par[4]
    q <- stabledist::pstable(1,
        alpha = alpha, beta = beta,
        gamma = gamma, delta = delta
    ) -
        stabledist::pstable(-1,
            alpha = alpha, beta = beta,
            gamma = gamma, delta = delta
        )
    start <- stabledist::pstable(-1,
        alpha = alpha, beta = beta,
        gamma = gamma, delta = delta
    )

    # control plot
    if (!is.null(file.name)) {
        x <- seq(-1, 1, by = 0.002)
        graphics::lines(
            x = x, y = stabledist::dstable(x,
                alpha = alpha,
                beta = beta, gamma = gamma, delta = delta
            ) / q,
            col = "blue", type = "l"
        )
        graphics::legend(
            x = "topright", lty = 1, legend = "Model",
            col = "blue", bty = "n", pt.cex = 0.5
        )
        grDevices::dev.off()
    }
    list(
        alpha = alpha, beta = beta, gamma = gamma, delta = delta, factor = q,
        start = start, distrib = "censored_stable"
    )
    
} # .getAlphaStableParam


#' Internal function to compute a censored alpha-) stable CDF
#'
#' @param x   A vector of observed values.
#' @param par A list containing the censored stable model parameters.
#'
#' @return A vector of probabilities P(X<x|par).
#' @importFrom stabledist pstable dstable
#' @keywords internal
.cdfAlphaStable <- function(x, par) {
    (stabledist::pstable(x,
        alpha = par$alpha, beta = par$beta, gamma = par$gamma,
        delta = par$delta
    ) - par$start) / par$factor
} # .cdfAlphaStable


#' Sampling of correlations downstream the receptors null distribution
#'
#' Perform receptor downstream analysis with
#' \code{.checkReceptorSignaling} based on randomized expression data and
#' ligand-receptor pairs selected from the same randomized data.
#'
#' @param obj   A BSRDatamodel without learned paramaters.
#'
#' @return A list of \code{n.rand} tables such as output by
#'   \code{.checkReceptorSignaling}. Each table is computed from a randomized
#'   expression matrix (randomized \code{ncounts}).
#'
#' @details A large number of correlations 
#'   (ligand-receptor and receptor-downstream target genes) 
#'   is reported in each randomized matrix. Therefore,
#'   \code{n.rand} should be
#'   given a modest value to avoid unnecessarily long computations.
#'
#'   See \code{\link{.checkReceptorSignaling}} for more details about the
#'   parameters.
#'
#' @importFrom foreach %do% %dopar%
#' @keywords internal
.getEmpiricalNull <- function(obj){

    pindices <- .buildPermutationIndices(obj@ncounts)
    r.ds <- BSRDataModel(obj@ncounts,
        normalize = FALSE, method = "ALREADY",
        min.LR.found = 0
    )
    if (foreach::getDoParWorkers() > 1) {
        foreach::foreach(k = seq_len(obj@param$n.rand.RT), 
        .combine = c) %dopar% {
            ncounts(r.ds) <- .buildPermutatedCountMatrix(obj@ncounts, pindices)
            r.LR <- .getCorrelatedLR(r.ds, min.cor = obj@param$min.corr.LR)
            list(.checkReceptorSignaling(r.ds, r.LR,
                with.complex = obj@param$with.complex,
                max.pw.size = obj@param$max.pw.size,
                min.pw.size = obj@param$min.pw.size,
                min.positive = obj@param$min.positive,
                use.full.network = FALSE
            ))
        }
    } else {
        foreach::foreach(k = seq_len(obj@param$n.rand.RT), 
        .combine = c) %do% {
            ncounts(r.ds) <- .buildPermutatedCountMatrix(obj@ncounts, pindices)
            r.LR <- .getCorrelatedLR(r.ds, min.cor = obj@param$min.corr.LR)
            list(.checkReceptorSignaling(r.ds, r.LR,
                with.complex = obj@param$with.complex, 
                max.pw.size = obj@param$max.pw.size,
                min.pw.size = obj@param$min.pw.size, 
                min.positive = obj@param$min.positive,
                use.full.network = FALSE
            ))
        }
    }
    
} # .getEmpiricalNull


#' Sampling of ligand-receptor correlation null distribution
#'
#' Perform a ligand-receptor Spearman correlation analysis based
#' on randomized expression data.
#'
#' @param obj   A BSRDatamodel without learned paramaters.
#'
#' @return A list of \code{n.rand.LR} tables such as output by
#'   \code{\link{.getCorrelatedLR}}. Each table is computed from a randomized
#'   expression matrix (randomized \code{ncounts}).
#'
#' @details A large number of correlations is reported in each randomized
#'   matrix. Therefore, \code{n.rand.LR} 
#'   should be given a modest value to avoid unnecessarily long
#'   computations.
#'
#'   See \code{\link{.getCorrelatedLR}} for more details about the parameters.
#'
#' @importFrom foreach %do% %dopar%
#' @keywords internal
.getEmpiricalNullCorrLR <- function(obj) {

    pindices <- .buildPermutationIndices(obj@ncounts)
    r.ds <- BSRDataModel(obj@ncounts,
        normalize = FALSE, method = "ALREADY",
        min.LR.found = 0
    )

    if (foreach::getDoParWorkers() > 1) {
        foreach::foreach(
            k = seq_len(obj@param$n.rand.LR), .combine = "c",
            .packages = "BulkSignalR"
        ) %dopar% {
            ncounts(r.ds) <- .buildPermutatedCountMatrix(obj@ncounts, pindices)
            list(.getCorrelatedLR(r.ds, min.cor = obj@param$min.corr.LR))
        }
    } else {
        foreach::foreach(k = seq_len(obj@param$n.rand.LR), .combine = c) %do% {
            ncounts(r.ds) <- .buildPermutatedCountMatrix(obj@ncounts, pindices)
            list(.getCorrelatedLR(r.ds, min.cor = obj@param$min.corr.LR))
        }
    }
    
} # .getEmpiricalNullCorrLR

#' BulkSignalR: A package to infer ligand-receptor
#' interactions from bulk transcriptomics or proteomics
#'
#' Inference of ligand-receptor interactions from bulk
#' (transcriptomic or proteomic) data. BulkSignalR bases its inferences
#' on the LRdb database. It relies on a statistical model that
#' is specific to bulk data sets. Different visualization and data
#' summary functions are proposed to help navigating results.
#'
#' @keywords internal 
"_PACKAGE"

utils::globalVariables(c(
    "BulkSignalR_LRdb",
    "%v%", "%>%", "%do%", "%dopar%", "ComplexHeatmap"
))

#' Get correlated ligand-receptor pairs
#'
#' Internal function to compute the Spearman correlations
#' of all the ligand-receptor
#' pairs in LRdb and return those above a minimum value.
#'
#' @param ds              A BSRDataModel object.
#' @param min.cor         The minimum correlation required.
#' @param restrict.genes  A list of gene symbols that restricts ligands and
#'   receptors.
#'
#' @return A data frame containing putative ligand-receptor pairs along
#'   with their correlations above \code{min.cor}. This table is the first step
#'   of a ligand-receptor analysis.
#'
#'
#' @details The \code{restrict.genes} parameter is used for special cases where
#'   LRdb must be further restricted to a subset.
#'   The putative ligand-receptor pairs has 3 columns : R, L and corr.
#'
#' @importFrom methods is
#' @importFrom foreach %do% %dopar%
#' @import doParallel
#' @keywords internal
.getCorrelatedLR <- function(ds, min.cor = 0.25, restrict.genes = NULL) {
    # local binding
    i <- NULL

    if ((min.cor < -1) || (min.cor > 1)) {
        stop("min.cor must lie in [-1;+1]")
    }
    if (!is(ds, "BSRDataModel")) {
        stop("ds must be an object of class BSRDataModel")
    }
    
    lrgenes <- intersect(c(
        .SignalR$BulkSignalR_LRdb$ligand,
        .SignalR$BulkSignalR_LRdb$receptor
    ), rownames(ncounts(ds)))
    if (!is.null(restrict.genes)) {
        lrgenes <- intersect(lrgenes, restrict.genes)
    }

    # compute all the correlations at once
    corlr <- stats::cor(t(ncounts(ds)[lrgenes, ]), method = "spearman")

    # get the pairs
    pairs <- foreach::foreach(
        i = seq_len(nrow(.SignalR$BulkSignalR_LRdb)),
        .combine = rbind
    ) %do% {
        if (.SignalR$BulkSignalR_LRdb$ligand[i] %in% rownames(corlr) &&
            .SignalR$BulkSignalR_LRdb$receptor[i] %in% rownames(corlr)) {
            data.frame(
                L = .SignalR$BulkSignalR_LRdb$ligand[i],
                R = .SignalR$BulkSignalR_LRdb$receptor[i],
                corr = corlr[
                    .SignalR$BulkSignalR_LRdb$ligand[i],
                    .SignalR$BulkSignalR_LRdb$receptor[i]
                ],
                stringsAsFactors = FALSE
            )
        } else {
            NULL
        }
    }

    good <- pairs$corr >= min.cor
    pairs[good, ]
    
} # .getCorrelatedLR


#' Internal function to check receptor signaling downstream
#'
#' @param lr              A data frame as returned by
#'   \code{.getCorrelatedLR()}.
#' @param pw              A table defining the reference pathways.
#' @param pw.size         A named vector with pathway sizes (names are pathway
#'   IDs).
#' @param rncounts        A matrix of normalized read counts with at
#'   least all the ligands, receptors, and genes in the reference pathways.
#' @param id.col          Column index or name in \code{pw} for the pathway IDs.
#' @param gene.col        Column index or name in \code{pw} for the gene
#'   symbols.
#' @param pw.col          Column index or name in \code{pw} for the pathway
#'   names.
#' @param min.positive    Minimum number of target genes to be found in a given
#'   pathway.
#' @param with.complex    A logical indicating whether receptor co-complex
#'   members should be included in the target genes.
#'
#' @return A table reporting all the ligand-receptor pairs provided in \code{lr}
#'   along with the pathways found and data about target gene correlations with
#'   the receptor.
#'
#' @importFrom foreach %do% %dopar%
#' @keywords internal
.downstreamSignaling <- function(lr, pw, pw.size, rncounts, id.col, gene.col,
    pw.col, min.positive, with.complex = TRUE) {
    if (!is.matrix(rncounts)) {
        stop("rncounts must be a matrix")
    }

    # local binding
    r <- p <- pl <- id <- NULL

    # define interaction types
    control.int <- c(
        "positively-controls-expression-of",
        "negatively-controls-expression-of",
        "regulates-transcription-of"
    )

    incomplex.int <- c("in-complex-with", "interacts-with")

    directed.int <- c(
        "controls-transport-of",
        "catalysis-precedes",
        "positively-controls-expression-of",
        "negatively-controls-expression-of",
        "regulates-transcription-of",
        "controls-dephospho-of",
        "controls-phospho-of",
        "regulates-dephospho-of",
        "regulates-phospho-of",
        "controls-state-change-of",
        "controls-state-of-by-metab"
    )


    if (with.complex) {
        correlated.int <- union(control.int, incomplex.int)
    } else {
        correlated.int <- control.int
    }

    # compute downstream correlations
    corrg <- stats::cor(t(rncounts), method = "spearman")

    # the global computation above is faster than restricted to the receptors
    corrg <- corrg[unique(lr$R), ]

    # check each putative LR pair, loop over the receptors
    reg.proc <- foreach::foreach(r = unique(lr$R), .combine = rbind) %do% {
        # reg.proc <- NULL
        # for (r in unique(lr$putative.pairs$R)){

        # loop over the pathways containing the receptor r
        pa <- intersect(pw[pw[[gene.col]] == r, id.col], names(pw.size))
        if (length(pa) > 0) {
            receptor.ligands <- unique(lr$L[lr$R == r])
            best.2nd <- foreach::foreach(p = pa, .combine = rbind) %do% {
                # best.2nd <- NULL
                # for (p in pa){
                int <- .SignalR$BulkSignalR_Network[
                    .SignalR$BulkSignalR_Network$a.gn %in% 
                    pw[pw[[id.col]] == p, gene.col] &
                        .SignalR$BulkSignalR_Network$b.gn %in% 
                        pw[pw[[id.col]] == p, gene.col],
                ]

                # extract the target genes of receptor r
                if (r %in% int$a.gn || r %in% int$b.gn) {
                    # double the undirected interactions 
                    # and generate a directed graph
                    directed <- int$type %in% directed.int
                    ret <- int[!directed, c("a.gn", "b.gn")]
                    from <- ret$a.gn
                    ret$a.gn <- ret$b.gn
                    ret$b.gn <- from
                    d.int <- unique(rbind(int[, c("a.gn", "b.gn")], ret))
                    g <- igraph::graph_from_data_frame(d.int, directed = TRUE)

                    # putative targets in the pathway
                    target.genes <- setdiff(
                        c(
                            int[int$type %in% correlated.int & 
                            int$a.gn == r, "b.gn"],
                            int[int$type %in% correlated.int & 
                            int$b.gn == r, "a.gn"],
                            int[int$type %in% directed.int, "b.gn"]
                        ),
                        r
                    )

                    # reduce putative to reachable from the receptor
                    sp <- igraph::distances(g, r, target.genes)
                    target.genes <- colnames(sp)[!is.infinite(sp[r, ])]

                    # eliminate ligands of the receptor if present
                    target.genes <- intersect(
                        rownames(rncounts),
                        setdiff(target.genes, receptor.ligands)
                    )

                    if (length(target.genes) >= min.positive) {
                        # if all conditions are met, list all target genes with
                        # their correlations to the receptor in a data frame
                        # row. Target genes are sorted wrt correlations.
                        c <- corrg[r, target.genes]
                        o <- order(c)
                        c <- c[o]
                        target.genes <- target.genes[o]
                        data.frame(
                            pathway = p, target.corr = paste(c, collapse = ";"),
                            target.genes = paste(target.genes, collapse = ";"),
                            len = length(c),
                            stringsAsFactors = FALSE
                        )
                    } else {
                        NULL
                    }
                } else {
                    NULL
                }
            }
            if (!is.null(best.2nd)) {
                # one or several pathways containing the receptor r were found,
                # combine them in |-separated strings
                data.frame(
                    R = r, pathways = paste(best.2nd$pathway, collapse = "|"),
                    target.corr = paste(best.2nd$target.corr, collapse = "|"),
                    target.genes = paste(best.2nd$target.genes, collapse = "|"),
                    len = paste(best.2nd$len, collapse = "|"),
                    stringsAsFactors = FALSE
                )
            } else {
                NULL
            }
        } else {
            NULL
        }
    }

    # combine LR pair correlations with R-target gene correlations
    rownames(reg.proc) <- reg.proc$R
    conf.pairs <- lr[lr$R %in% reg.proc$R, ]
    conf.pairs$pwid <- reg.proc[conf.pairs$R, "pathways"]
    conf.pairs$target.corr <- reg.proc[conf.pairs$R, "target.corr"]
    conf.pairs$len <- reg.proc[conf.pairs$R, "len"]
    conf.pairs$target.genes <- reg.proc[conf.pairs$R, "target.genes"]
    pw.name <- unique(pw[, c(id.col, pw.col)])
    pw2name <- stats::setNames(pw.name[[2]], pw.name[[1]])
    conf.pairs$pwname <- foreach::foreach(pl = conf.pairs$pwid,
        .combine = c) %do% {
        paste(
            foreach::foreach(
                id = unlist(strsplit(pl, "\\|")),
                .combine = c
            ) %do% {
                pw2name[id]
            },
            collapse = "|"
        )
    }

    conf.pairs[, c(
        "L", "R", "corr", "pwid", "pwname", "len", "target.genes",
        "target.corr"
    )]
    
} # .downstreamSignaling


#' Internal function to check receptor signaling downstream
#'
#' Assess the existence of correlations between a receptor,
#' part of a ligand-receptor pair, and
#' genes coding for proteins forming a complex with the receptor or genes
#' regulated by the receptor downstream signaling.
#'
#' @param ds              A BSRDataModel object.
#' @param lr              A table as returned by \code{.getCorrelatedLR()}.
#' @param reference       Which pathway reference should be used ("REACTOME"
#'   for Reactome, "GOBP" for GO Biological Process,
#'   or "REACTOME-GOBP" for both).
#' @param max.pw.size     Maximum pathway size to consider from the pathway
#'   reference.
#' @param min.pw.size     Minimum pathway size to consider from the pathway
#'   reference.
#' @param min.positive    Minimum number of target genes to be found in a given
#'   pathway.
#' @param restrict.pw     A list of pathway IDs to restrict the application of
#'   the function.
#' @param with.complex    A logical indicating whether receptor co-complex
#'   members should be included in the target genes.
#' @param use.full.network  A logical to avoid limiting the reference network
#'   to the detected genes and use the whole reference network.
#' @return A data frame extending \code{lr} content with the pathways found to
#' contain the receptors and data about target gene correlations with those
#' receptors. Strings in semi-colon-separated format are used to report
#' target genes and their Spearman correlations with the receptor in the
#' data frame. The target genes are sorted according to the correlation
#' coefficient.
#'
#' In a pathway of the reference, i.e., a Reactome pathway or the genes of a
#' GOBP term, the target genes are the
#' genes coding for proteins forming a complex with the receptor and the
#' genes in the pathway downstream the receptor,
#' which are given as regulated by the pathway. If \code{with.complex} is
#' set to \code{FALSE}, then only the
#' regulated genes are considered. Participation to a complex and being
#' regulated as well as the pathway directed topologies
#' are defined by Reactome and KEGG pathways as provided by PathwayCommons.
#'
#' The maximum pathway size is used to limit the redundancy inherent to GOBP
#' and Reactome. The minimum pathway size is
#' used to avoid overspecific, noninformative results.
#'
#' @importFrom methods is
#' @keywords internal
.checkReceptorSignaling <- function(ds, lr, reference = c(
                                        "REACTOME-GOBP",
                                        "REACTOME", "GOBP"
                                    ),
                                    max.pw.size = 400, min.pw.size = 5,
                                    min.positive = 4, use.full.network = FALSE,
                                    restrict.pw = NULL, with.complex = TRUE) {
    if (!is(ds, "BSRDataModel")) {
        stop("ds must be a BSRDataModel object")
    }

    reference <- match.arg(reference)
    results <- list()

    # Reactome pathways
    if (reference %in% c("REACTOME-GOBP", "REACTOME")) {
        pw.size <- table(.SignalR$BulkSignalR_Reactome$`Reactome ID`)
        pw.size <- pw.size[pw.size >= min.pw.size & pw.size <= max.pw.size]
        if (use.full.network) {
            react <- .SignalR$BulkSignalR_Reactome
        } else {
            react <- .SignalR$BulkSignalR_Reactome[
            .SignalR$BulkSignalR_Reactome$`Gene name` 
            %in% rownames(ncounts(ds)), ]
        }
        if (!is.null(restrict.pw)) {
            react <- react[react$`Reactome ID` %in% restrict.pw, ]
        }
        contains.receptors <- react[react$`Gene name` %in% lr$R, "Reactome ID"]
        pw.size <- pw.size[names(pw.size) %in% contains.receptors]
        corgenes <- intersect(
            rownames(ncounts(ds)),
            c(lr$R, react[react$`Reactome ID` %in% names(pw.size), "Gene name"])
        )
        results$reactome.pairs <- .downstreamSignaling(lr, react, pw.size,
            ncounts(ds)[corgenes, ],
            id.col = "Reactome ID", gene.col = "Gene name",
            pw.col = "Reactome name", min.positive, with.complex = with.complex
        )
    }

    # GOBP
    if (reference %in% c("REACTOME-GOBP", "GOBP")) {
        pw.size <- table(.SignalR$BulkSignalR_Gobp$`GO ID`)
        pw.size <- pw.size[pw.size >= min.pw.size & pw.size <= max.pw.size]
        if (use.full.network) {
            go <- .SignalR$BulkSignalR_Gobp
        } else {
            go <- .SignalR$BulkSignalR_Gobp[
            .SignalR$BulkSignalR_Gobp$`Gene name` %in% rownames(ncounts(ds)), ]
        }
        if (!is.null(restrict.pw)) {
            go <- go[go$`GO ID` %in% restrict.pw, ]
        }
        contains.receptors <- go[go$`Gene name` %in% lr$R, "GO ID"]
        pw.size <- pw.size[names(pw.size) %in% contains.receptors]
        corgenes <- intersect(
            rownames(ncounts(ds)),
            c(lr$R, go[go$`GO ID` %in% names(pw.size), "Gene name"])
        )
        results$gobp.pairs <- .downstreamSignaling(lr, go, pw.size,
            ncounts(ds)[corgenes, ],
            id.col = "GO ID", gene.col = "Gene name", pw.col = "GO name",
            min.positive, with.complex = with.complex
        )
    }

    # merge
    if (reference == "REACTOME-GOBP") {
        pairs <- unique(rbind(
            results$reactome.pairs[, c(1, 2)],
            results$gobp.pairs[, c(1, 2)]
        ))
        react.keys <- paste(results$reactome.pairs[[1]],
            results$reactome.pairs[[2]],
            sep = "|"
        )
        gobp.keys <- paste(results$gobp.pairs[[1]],
            results$gobp.pairs[[2]],
            sep = "|"
        )
        results$merged.pairs <- rbind(
            results$reactome.pairs,
            results$gobp.pairs[!(gobp.keys %in% react.keys), ]
        )
    } else if (reference == "REACTOME") {
        results$merged.pairs <- results$reactome.pairs
    } else {
        results$merged.pairs <- results$gobp.pairs
    }

    results$merged.pairs
    
} # .checkReceptorSignaling


#' Internal function to assign P-values to LR interactions
#'
#' Estimate the P-value of each ligand-receptor pair based
#' on the data frame output by \code{\link{.checkReceptorSignaling}}.
#'
#' @param pairs         A data frame output by \code{checkReceptorSignaling}.
#' @param param         A list containing the statistical model parameters.
#' @param rank.p        A number between 0 and 1 defining the rank of the last
#'   considered target genes.
#' @param fdr.proc      The procedure for adjusting P-values according to
#'   \code{\link[multtest]{mt.rawp2adjp}}.
#'
#' @return A data.frame with the data in \code{pairs} complemented with
#' P-values and adjusted P-values.
#' @keywords internal
.pValuesLR <- function(pairs, param, rank.p = 0.75,
    fdr.proc = c(
        "BH", "Bonferroni", "Holm", "Hochberg",
        "SidakSS", "SidakSD", "BY", "ABH", "TSBH")) {
    
    if (rank.p < 0 || rank.p > 1) {
        stop("rank.p must lie in [0;1]")
    }
    fdr.proc <- match.arg(fdr.proc)

    # prepare the chosen model CDF
    LR.par <- param$LR.0$model
    RT.par <- param$RT.0$model
    if (LR.par$distrib != RT.par$distrib) {
        stop("Distinct statistical models for LR and RT nulls are not allowed")
    }
    if (LR.par$distrib == "censored_normal") {
        cdf <- .cdfGaussian
    } else if (LR.par$distrib == "censored_mixed_normal") {
        cdf <- .cdfMixedGaussian
    } else if (LR.par$distrib == "censored_stable") {
        cdf <- .cdfAlphaStable
    } else if (LR.par$distrib == "empirical") {
        cdf <- .cdfEmpirical
    } else if (LR.par$distrib == "kernel_empirical") {
        cdf <- .cdfKernelEmpirical
    } else {
        stop("Unknown statistical model: ", LR.par$LR.0$model$distrib)
    }

    # estimate P-values
    res <- NULL
    for (i in seq_len(nrow(pairs))) {
        # all the data related to each pathway containing a given
        # receptor were collapsed separated by |
        # we need to split those pathways
        pwid <- unlist(strsplit(pairs$pwid[i], split = "\\|"))
        pwname <- unlist(strsplit(pairs$pwname[i], split = "\\|"))
        tg <- unlist(strsplit(pairs$target.genes[i], split = "\\|"))
        spear <- unlist(strsplit(pairs$target.corr[i], split = "\\|"))
        len <- as.numeric(unlist(strsplit(pairs$len[i], split = "\\|")))

        # estimate the LR correlation P-value
        if (pairs$corr[i] >= 0) {
            # normal case
            p.lr <- 1 - cdf(pairs$corr[i], LR.par)
        } else {
            # to enable searching for inhibitory L-R interactions
            p.lr <- cdf(pairs$corr[i], LR.par)
        }

        # estimate the target gene correlation P-value based on rank statistics
        # for the individual correlation Gaussian model
        for (k in seq_len(length(len))) {
            spears <- as.numeric(strsplit(spear[k], split = ";")[[1]])
            r <- min(max(1, trunc(rank.p * len[k])), len[k])
            rank.corr <- spears[r]
            # r-1 correlations are < rank.corr, prob to have r-1 or less
            # corr < rank.corr is given by a binomial with success rate
            # equal to the probability to get a corr < rank.corr, i.e.,
            # cdf(rank.corr, RT.par). If rank.corr is high, it becomes
            # difficult to get as little as r-1 corr < rank.corr by chance!
            p.rt <- stats::pbinom(r - 1, len[k], cdf(rank.corr, RT.par))
            res <- rbind(res, data.frame(pairs[i, c("L", "R")],
                LR.corr = pairs[i, "corr"], pw.id = pwid[k],
                pw.name = pwname[k], rank = r, len = len[k],
                rank.corr = rank.corr, target.genes = tg[k],
                target.corr = spear[k], pval = p.lr * p.rt,
                stringsAsFactors = FALSE
            ))
        }
    }

    # avoid the impossible
    key <- paste(res$L, res$R, res$pw.id, sep = "||")
    bad <- duplicated(key)
    res <- res[!bad, ]

    # multiple hypothesis correction
    rawp <- res$pval
    if (length(rawp) > 1){
        adj <- multtest::mt.rawp2adjp(rawp, fdr.proc)
        res$qval <- adj$adjp[order(adj$index), fdr.proc]
    }
    else {
        res$qval <- rawp
    }

    res
    
} # .pValuesLR

#' BulkSignalR cluster comparison-based inference object
#'
#' An S4 class to represent ligand-receptor interactions inferred from
#' a comparison between two clusters of samples. This class inherits from
#' BSRInference.
#'
#' @slot cmp.name  The name of the BSRClusterComp object in a BSRDataModelComp
#' object comp list.
#' @slot src.cmp.name  The name of an optional BSRClusterComp object in a
#' BSRDataModelComp object comp list in case paracrine inferences were
#' performed.
#' @slot tg.pval  A list of target gene P-values, one
#' entry per interaction
#' @slot tg.logFC  A list of target gene logFC, one
#' entry per interaction
#' @slot tg.expr  A list of target gene expression, one
#' entry per interaction
#'
#' @details This class is contains inferred LR interactions along with
#' their statistical significance. Data representation supports subsequent
#' reductions to pathways, etc. See reduction functions
#' \code{"\link[=BSRInferenceComp-class]{reduceToBestPathway}"},
#' \code{"\link[=BSRInferenceComp-class]{reduceToLigand}"},
#' \code{"\link[=BSRInferenceComp-class]{reduceToReceptor}"} and
#' \code{"\link[=BSRInferenceComp-class]{reduceToPathway}"}.
#' @export
#' @examples
#' new("BSRInferenceComp")
setClass("BSRInferenceComp",
    contains = "BSRInference",
    slots = c(
        cmp.name = "character",
        src.cmp.name = "character",
        tg.pval = "list",
        tg.logFC = "list",
        tg.expr = "list"
    ),
    prototype = list(
        cmp.name = "myComparison",
        src.cmp.name = "",
        tg.pval = list(c(0.05, 0.1, 0.008)),
        tg.logFC = list(c(-1, 0, 2)),
        tg.expr = list(c(1, 2, 3))
    )
)


setValidity(
    "BSRInferenceComp",
    function(object) {
        if (!is.character(object@cmp.name)) {
            return("cmp.name is not of character type")
        }
        if (!is.character(object@src.cmp.name)) {
            return("src.cmp.name is not of character type")
        }
        if (length(object@cmp.name) == 0) {
            return("cmp.name must have a length > 0")
        }
        if (!is.list(object@tg.pval)) {
            return("tg.pval is not a list")
        }
        if (!is.list(object@tg.logFC)) {
            return("tg.logFC is not a list")
        }
        if (!is.list(object@tg.expr)) {
            return("tg.expr is not a list")
        }

        TRUE
    }
)


setMethod(
    "show", "BSRInferenceComp",
    function(object) {
        callNextMethod()
        cat("Cluster comparison name:", object@cmp.name, "\n")
        cat("Source cluster comparison name:", object@src.cmp.name, "\n")
    }
)


# Constructor ========================================================

#' Inference of ligand-receptor interactions based on regulation
#'
#' This method supports two configurations that we refer to
#' as paracrine and autocrine.
#'
#' In the autocrine case, a single cluster comparison name is provided.
#' In the corresponding cluster comparison, a group of samples A was
#' compared to a group of samples B to determine fold-changes and associated
#' P-values. The inferred ligand-receptor interactions take place in the
#' samples of group A. A typical single-cell example would be a population of
#' macrophages (group A) compared to all the other populations (group B) to
#' represent specific increased or decreased expression in macrophages. The
#' resulting ligand-receptor interactions will be autocrine interactions
#' that are exacerbated (or reduced depending on the chosen parameters) in
#' macrophages.
#'
#' In the paracrine case, two cluster comparison names must be provided.
#' For instance, a first comparison could involve macrophages versus all
#' the other cell populations as above. The second comparison could be
#' B-cells against all the other populations. Now, calling
#' \code{BSRInferenceComp()}
#' with comparison macrophages *versus* the rest and, as source comparison,
#' B-cells *versus* the rest, will result in inferring interactions between
#' B-cells (ligands) and macrophages (receptors and downstream pathways). To
#' obtain macrophages to B-cells paracrine interactions, it is necessary to call
#' the method a second time with permuted cluster comparison names. Another
#' example in spatial transcriptomics could be two thin bands at the boundary of
#' two tissue regions, one emitting the ligand and the other one expressing the
#' receptor.
#'
#' In this initial inference, all the receptor-containing pathways are reported,
#' see reduction functions to reduce this list.
#'
#' @name BSRInferenceComp
#'
#' @param obj       A BSRDataModelComp object.
#' @param cmp.name        The name of the cluster comparison that should be used
#' for the inference. Autocrine interactions if only this comparison name is
#' provided, paracrine if a source comparison name is provided as well.
#' @param src.cmp.name    The name of the source cluster comparison that should
#' be used for paracrine interaction inferences.
#' @param rank.p        A number between 0 and 1 defining the rank of the last
#'   considered target genes.
#' @param max.pval        The maximum P-value imposed to both the ligand
#'   and the receptor.
#' @param min.logFC       The minimum log2 fold-change allowed for
#'   both the receptor and the ligand.
#' @param neg.receptors     A logical indicating whether receptors are only
#'   allowed to be upregulated (FALSE), or up- and downregulated (TRUE).
#' @param fdr.proc      The procedure for adjusting P-values according to
#' \code{\link[multtest]{mt.rawp2adjp}}.
#' @param reference       Which pathway reference should be used ("REACTOME"
#'   for Reactome, "GOBP" for GO Biological Process,
#'   or "REACTOME-GOBP" for both).
#' @param max.pw.size     Maximum pathway size to consider from the pathway
#'   reference.
#' @param min.pw.size     Minimum pathway size to consider from the pathway
#'   reference.
#' @param min.positive    Minimum number of target genes to be found in a given
#'   pathway.
#' @param with.complex    A logical indicating whether receptor co-complex
#'   members should be included in the target genes.
#' @param min.t.logFC     The minimum log2 fold-change allowed for
#'   targets in case pos.targets or neg.targets are used.
#' @param pos.targets   A logical imposing that all the network targets must
#'   display positive logFC, i.e. logFC >= min.t.logFC.
#' @param neg.targets   A logical imposing that all the network targets must
#'   display negative logFC, i.e. logFC <= - min.t.logFC.
#' @param restrict.pw     A list of pathway IDs to restrict the application of
#'   the function.
#' @param restrict.genes  A list of gene symbols that restricts ligands and
#'   receptors.
#' @param use.full.network  A logical to avoid limiting the reference network
#' to the detected genes and use the whole reference network.
#'
#' @details Perform the initial ligand-receptor inference. Initial means that
#' no reduction is applied. All the (ligand, receptor, downstream pathway)
#' triples are reported, i.e., a given LR pair may appear multiple times
#' with different pathways downstream the receptor. Specific reduction
#' functions are available from the package to operate subsequent
#' simplifications based on the BSRInferenceComp object created by this method.
#'
#' Here, ligand-receptor interactions are inferred based on gene or protein
#' regulation-associated P-values when comparing two clusters of samples. Since
#' a BSRDataModelComp object can contain several such comparisons, the name
#' of the comparison to use must be specified (parameter \code{cmp.name}).
#'
#' Note that since the introduction of the use.full.network parameter
#' (April 29, 2024), the pathway sizes are always computed before potential
#' intersection with the observed data (use.full.network set to FALSE) for
#' consistency. Accordingly, the minimum and maximum pathway default values
#' have been raised from 5 & 200 to 5 & 400 respectively. By default,
#' use.full.network is set to FALSE.
#'
#' In addition to statistical significance estimated according to BulkSignalR
#' statistical model, we compute SingleCellSignalR original LR-score,
#' based on L and R cluster average expression. 
#' In the paracrine case, L average expression
#' is taken from the source cluster.
#'
#' @return A BSRInferenceComp object with initial inferences set.
#'
#' @export
#'
#' @examples
#' data(bsrdm.comp, package = "BulkSignalR")
#' data(immune.signatures, package = "BulkSignalR")
#' 
#' # infer ligand-receptor interactions from the comparison
#' bsrinf.comp <- BSRInferenceComp(bsrdm.comp, max.pval = 1, 
#' reference="REACTOME",
#' "random.example")
#' 
#' @importFrom methods new
BSRInferenceComp <- function(obj, cmp.name, 
    src.cmp.name = NULL, rank.p = 0.55,
    max.pval = 0.01, min.logFC = 1, neg.receptors = FALSE,
    pos.targets = FALSE, neg.targets = FALSE,
    min.t.logFC = 0.5, restrict.genes = NULL,
    use.full.network = FALSE,
    reference = c("REACTOME-GOBP", "REACTOME", "GOBP"),
    max.pw.size = 400, min.pw.size = 5, min.positive = 2,
    restrict.pw = NULL, with.complex = TRUE,
    fdr.proc = c(
        "BH", "Bonferroni", "Holm", "Hochberg",
        "SidakSS", "SidakSD", "BY", "ABH", "TSBH"
        )) {


    if (!(cmp.name %in% names(comparison(obj)))) {
        stop("cmp.name must exist in the names ",
            "of comparisons contained in obj")
    }
    if (!is.null(src.cmp.name) && !(src.cmp.name %in% names(comparison(obj)))) {
        stop("src.cmp.name must exist in the names",
        " of comparisons contained in obj")
    }
    reference <- match.arg(reference)
    fdr.proc <- match.arg(fdr.proc)
    if (min.logFC <= 0) {
        stop("min.logFC must be >0")
    }
    if (min.t.logFC <= 0) {
        stop("min.t.logFC must be >0")
    }
    if (rank.p < 0 || rank.p > 1) {
        stop("rank.p must lie in [0;1]")
    }
    if (neg.targets && pos.targets) {
        stop("neg.targets and pos.targets cannot be TRUE simultaneously")
    }

    # retrieve the BSRClusterComp object(s)
    cc <- comparison(obj)[[cmp.name]]
    if (!is.null(src.cmp.name)) {
        scc <- comparison(obj)[[src.cmp.name]]
    } else {
        scc <- NULL
    }

    # store inference parameters and retrieve relevant L & R
    inf.param <- list()
    inf.param$log.transformed.data <- logTransformed(obj)
    inf.param$mu <- mu(obj)
    inf.param$col.clusterA <- colClusterA(cc)
    inf.param$col.clusterB <- colClusterB(cc)
    if (is.null(src.cmp.name)) {
        inf.param$inference.type <- "autocrine"
    } else {
        inf.param$inference.type <- "paracrine"
        inf.param$src.colA <- colClusterA(scc)
        inf.param$src.colB <- colClusterB(scc)
    }
    inf.param$max.pval <- max.pval
    inf.param$min.logFC <- min.logFC
    inf.param$neg.receptors <- neg.receptors
    inf.param$pos.targets <- pos.targets
    inf.param$neg.targets <- neg.targets
    inf.param$min.t.logFC <- min.t.logFC
    inf.param$restrict.genes <- restrict.genes
    inf.param$use.full.network <- use.full.network
    lr <- .getRegulatedLR(obj, cc, scc,
        max.pval = max.pval, min.logFC = min.logFC,
        neg.receptors = neg.receptors, restrict.genes = restrict.genes
    )

    # apply BSR model on the targets
    inf.param$reference <- reference
    inf.param$min.pw.size <- min.pw.size
    inf.param$max.pw.size <- max.pw.size
    inf.param$with.complex <- with.complex
    inf.param$min.positive <- min.positive
    inf.param$restrict.pw <- restrict.pw
    pairs <- .checkRegulatedReceptorSignaling(obj, cc, lr,
        reference = reference,
        pos.targets = pos.targets, neg.targets = neg.targets,
        min.t.logFC = min.logFC, use.full.network = use.full.network,
        min.pw.size = min.pw.size, max.pw.size = max.pw.size,
        min.positive = min.positive, with.complex = with.complex,
        restrict.pw = restrict.pw
    )
    inf.param$fdr.proc <- fdr.proc
    inf.param$rank.p <- rank.p

    # compute P-values
    inter <- .pValuesRegulatedLR(pairs, 
        parameters(obj), rank.p = rank.p, fdr.proc = fdr.proc)

    # compute LR-score for compatibility with SingleCellSignalR version 1
    if (is.null(scc)) {
        inter$L.expr <- differentialStats(cc)[inter$L, "expr"]
    } else {
        inter$L.expr <- differentialStats(scc)[inter$L, "expr"]
    }
    inter$R.expr <- differentialStats(cc)[inter$R, "expr"]
	if (sum(inter$L.expr<0) + sum(inter$R.expr<0) > 0){
	    # impossible to compute LR-score with negative values,
		# this may happen for scProt-MS data
		inter$LR.score <- 0
	}
	else{
	    # normal case
        if (inf.param$log.transformed.data) {
            sq <- sqrt(inter$L.expr * inter$R.expr)
        } else {
            sq <- sqrt(log1p(inter$L.expr) / log(2) * log1p(inter$R.expr) / log(2))
        }
        inter$LR.score <- sq / (inf.param$mu + sq)
	}

    # prepare the accompanying lists
    ligands <- strsplit(inter$L, ";")
    receptors <- strsplit(inter$R, ";")
    tg <- strsplit(inter$target.genes, ";")
    tgpval <- lapply(
        strsplit(inter$target.pval, ";"),
        function(x) as.numeric(x)
    )
    tglogFC <- lapply(
        strsplit(inter$target.logFC, ";"),
        function(x) as.numeric(x)
    )
    tgcorr <- lapply(
        strsplit(inter$target.corr, ";"),
        function(x) as.numeric(x)
    )
    tgexpr <- lapply(
        strsplit(inter$target.expr, ";"),
        function(x) as.numeric(x)
    )
    inf.param$ligand.reduced <- FALSE
    inf.param$receptor.reduced <- FALSE
    inf.param$pathway.reduced <- FALSE

    # instantiate the object
    if (is.null(src.cmp.name)) {
        src.cmp.name.char <- ""
    } else {
        src.cmp.name.char <- src.cmp.name
    }

    new("BSRInferenceComp",
        LRinter = inter[, c(
            "L", "R", "pw.id", "pw.name", "pval", "qval",
            "L.logFC", "R.logFC", "LR.pval", "LR.corr",
            "rank", "len", "rank.pval", "rank.corr",
            "LR.score", "L.expr", "R.expr"
        )],
        ligands = ligands, 
        receptors = receptors, 
        tg.genes = tg, 
        tg.corr = tgcorr, inf.param = inf.param,
        tg.pval = tgpval, tg.logFC = tglogFC, tg.expr = tgexpr, 
        cmp.name = cmp.name, src.cmp.name = src.cmp.name.char
    )
    
} # BSRInferenceComp


# Accessors & setters ========================================================

setGeneric("comparisonName", signature="x",
    function(x) standardGeneric("comparisonName")
)
#' Comparison name accessor
#'
#' @name comparisonName
#' @aliases comparisonName,BSRInferenceComp-method
#' @param x BSRInferenceComp object
#' @return cmp.name
#' @examples
#' bsrinf <- new("BSRInferenceComp")
#' comparisonName(bsrinf)
#' @export
setMethod("comparisonName", "BSRInferenceComp", function(x) x@cmp.name)


setGeneric("comparisonName<-", signature=c("x", "value"),
    function(x, value) standardGeneric("comparisonName<-")
)
#' Comparison name setter (internal use only)
#' 
#' @param x BSRInferenceComp object
#' @param value value to be set for bsrinf
#' @return returns \code{NULL}
#' @keywords internal
setMethod("comparisonName<-", "BSRInferenceComp", function(x, value) {
    x@cmp.name <- value
    methods::validObject(x)
    x
})


setGeneric("sourceComparisonName", signature="x",
    function(x) standardGeneric("sourceComparisonName")
)
#' Source comparison name accessor
#'
#' @name sourceComparisonName
#' @aliases sourceComparisonName,BSRInferenceComp-method
#' @param x BSRInferenceComp object
#' @return src.comp.name
#' @examples
#' bsrinf <- new("BSRInferenceComp")
#' sourceComparisonName(bsrinf)
#' @export
setMethod("sourceComparisonName", "BSRInferenceComp", 
    function(x) x@src.cmp.name)


setGeneric("sourceComparisonName<-", signature=c("x", "value"),
    function(x, value) standardGeneric("sourceComparisonName<-")
)
#' Source comparison name setter (internal use only)
#' @param x BSRInferenceComp object
#' @param value value to be set for bsrinf
#' @return returns \code{NULL}
#' @keywords internal
setMethod("sourceComparisonName<-", "BSRInferenceComp", function(x, value) {
    x@src.cmp.name <- value
    methods::validObject(x)
    x
})


setGeneric("tgPval", signature="x",
    function(x) standardGeneric("tgPval")
)

#' Target gene P-values accessor
#'
#' @name tgPval
#' @aliases tgPval,BSRInferenceComp-method
#' @param x BSRInferenceComp object
#' @return tgPval
#' @examples
#' bsrinf <- new("BSRInferenceComp")
#' tgPval(bsrinf)
#' @export
setMethod("tgPval", "BSRInferenceComp", function(x) x@tg.pval)


setGeneric("tgPval<-", signature=c("x", "value"),
    function(x, value) standardGeneric("tgPval<-")
)
#' Target gene P-values setter (internal use only)
#'
#' @param x BSRInferenceComp object
#' @param value value to be set for bsrinf
#' @return returns \code{NULL}
#' @keywords internal
setMethod("tgPval<-", "BSRInferenceComp", function(x, value) {
    x@tg.pval <- value
    methods::validObject(x)
    x
})


setGeneric("tgLogFC", signature="x",
    function(x) standardGeneric("tgLogFC")
)
#' Target gene logFC accessor
#'
#' @name tgLogFC
#' @aliases tgLogFC,BSRInferenceComp-method
#' @param x BSRInferenceComp object
#' @return tgLogFC
#' @examples
#' bsrinf <- new("BSRInferenceComp")
#' tgLogFC(bsrinf)
#' @export
setMethod("tgLogFC", "BSRInferenceComp", function(x) x@tg.logFC)


setGeneric("tgLogFC<-", signature=c("x", "value"),
    function(x, value) standardGeneric("tgLogFC<-")
)
#' Target gene logFC setter (internal use only)
#' @param x BSRInferenceComp object
#' @param value value to be set for bsrinf
#' @return returns \code{NULL}
#' @keywords internal
setMethod("tgLogFC<-", "BSRInferenceComp", function(x, value) {
    x@tg.logFC <- value
    methods::validObject(x)
    x
})


setGeneric("tgExpr", signature="x",
    function(x) standardGeneric("tgExpr")
)
#' Target gene expression accessor
#'
#' @name tgExpr
#' @aliases tgExpr,BSRInferenceComp-method
#' @param x BSRInferenceComp object
#' @return tgExpr
#' @examples
#' bsrinf <- new("BSRInferenceComp")
#' tgExpr(bsrinf)
#' @export
setMethod("tgExpr", "BSRInferenceComp", function(x) x@tg.expr)


setGeneric("tgExpr<-", signature=c("x", "value"),
    function(x, value) standardGeneric("tgExpr<-")
)
#' Target gene expression setter (internal use only)
#'
#' @param x BSRInferenceComp object
#' @param value value to be set for bsrinf
#' @return returns \code{NULL}
#' @keywords internal
setMethod("tgExpr<-", "BSRInferenceComp", function(x, value) {
    x@tg.expr <- value
    methods::validObject(x)
    x
})


# simplified table views =======================================================

#' Simplified LRinter accessor reporting the essential columns
#'
#' @name LRinterShort
#' @aliases LRinterShort,BSRInferenceComp-method
#' @param x BSRInferenceComp object
#' @return LRinterShort
#' @examples
#' data(bsrinf.comp, package = "BulkSignalR")
#' LRinterShort(bsrinf.comp)[5,]
#' @export
setMethod(
    "LRinterShort", "BSRInferenceComp",
    function(x) {
        x@LRinter[, c(
            "L", "R", "pw.id", "pw.name",
            "qval", "L.logFC", "R.logFC", "len"
        )]
    }
)


setGeneric("LRinterScore", signature="x",
    function(x) standardGeneric("LRinterScore")
)
#' Simplified LRinter accessor with focus on the LR-score
#'
#' @name LRinterScore
#' @aliases LRinterScore,BSRInferenceComp-method
#' @param x BSRInferenceComp object
#' @return LRinterScore
#' @examples
#' data(bsrinf.comp, package = "BulkSignalR")
#' LRinterScore(bsrinf.comp)[5,]
#' @export
setMethod(
    "LRinterScore", "BSRInferenceComp",
    function(x) {
        x@LRinter[, c(
            "L", "R", "pw.id", "pw.name",
            "LR.score", "L.expr", "R.expr", "len"
        )]
    }
)


# Rescoring & updating =========================================================

#' Inference re-scoring
#'
#' A method to re-score an existing BSRInferenceComp object
#' (P- and Q-value estimations).
#'
#' @name rescoreInference
#' @aliases rescoreInference,BSRInferenceComp-method
#'
#' @param obj BSRInferecenceComp object.
#' @param param NULL by default
#' @param rank.p        A number between 0 and 1 defining the rank of the last
#'   considered target genes.
#' @param fdr.proc      The procedure for adjusting P-values according to
#'   \code{\link[multtest]{mt.rawp2adjp}}.
#'
#' @details A BSRInferenceComp object should be created by calling
#' \code{"\link[=BSRInferenceComp-class]{BSRInferenceComp}"}
#'
#' @return A BSRInferenceComp object.
#'
#' @export
#' @examples
#' data(bsrinf.comp, package = "BulkSignalR")
#'
#' bsrinf.less <- rescoreInference(bsrinf.comp, 
#' rank.p = 0.75)
setMethod("rescoreInference", "BSRInferenceComp", function(obj,
    param = NULL, 
    rank.p = 0.55,
    fdr.proc = c("BH", "Bonferroni", "Holm",
    "Hochberg", "SidakSS", "SidakSD", "BY", "ABH", "TSBH"
    )) {

    if (rank.p < 0 || rank.p > 1) {
        stop("rank.p must lie in [0;1]")
    }
    fdr.proc <- match.arg(fdr.proc)

    # extract the necessary data from the BSRInferenceComp object
    pairs <- LRinter(obj)
    tg.genes <- tgGenes(obj)
    tg.pval <- tgPval(obj)
    tg.corr <- tgCorr(obj)

    # recompute P-values, LR- and LRT-scores
    for (i in seq_len(nrow(pairs))) {
        tg <- tg.genes[[i]]
        spvals <- tg.pval[[i]]

        # get the LR correlation P-value
        p.lr <- pairs$LR.pval[i]

        # estimate the target gene correlation P-value based on rank statistics
        # for the individual correlation Gaussian model
        len <- pairs$len[i]
        r <- min(max(1, trunc(rank.p * len)), len)
        pairs$rank[i] <- r
        rank.pval <- spvals[r]
        # r-1 P-values are > rank.pval, prob to have r-1 or less
        # P-values > rank.pval is given by a binomial with success rate
        # equal to the probability to get a P-value > rank.pval, i.e.,
        # 1-rank.pval. If rank.pval is low (i.e., highly significant),
        # it becomes difficult to get as little 
        # as r-1 P-values > rank.pval by chance!
        p.rt <- stats::pbinom(r - 1, len, 1 - rank.pval) # cdf is punif here!
        pairs$pval[i] <- p.lr * p.rt
        pairs$rank.pval[i] <- rank.pval
        pairs$rank.corr[i] <- tg.corr[[i]][r]
    }

    # recompute the Q-values
    rawp <- pairs$pval
    if (length(rawp) > 1){
        adj <- multtest::mt.rawp2adjp(rawp, fdr.proc)
        pairs$qval <- adj$adjp[order(adj$index), fdr.proc]
    }
    else {
        pairs$qval <- rawp
    }

    # update the BSRInference object
    inf.param <- inferenceParameters(obj)
    inf.param$fdr.proc <- fdr.proc
    inf.param$rank.p <- rank.p
    inferenceParameters(obj) <- inf.param
    LRinter(obj) <- pairs

    obj
    
}) # rescoreInference


setGeneric("updateInference", signature="obj",
    function(obj,...) standardGeneric("updateInference")
)
#' Inference updating
#'
#' A method to update the data underlying statistical significance estimations
#' prior to rescoring for an existing BSRInferenceComp object
#' (P- and Q-value estimations as well as LR-score).
#'
#' @name updateInference
#' @aliases updateInference,BSRInferenceComp-method
#'
#' @param obj BSRInferenceComp object.
#' @param bsrcc BSRClusterComp object relative to target cells.
#' @param ncounts Matrix counts normalized.
#' @param src.bsrcc BSRClusterComp object relative to source cells.
#' @param rank.p        A number between 0 and 1 defining the rank of the last
#'   considered target genes.
#' @param max.pval        The maximum P-value imposed to both the ligand
#'   and the receptor.
#' @param min.logFC       The minimum log2 fold-change allowed for
#'   both the receptor and the ligand.
#' @param min.LR.score       The minimum LR-score allowed for the interaction.
#' @param neg.receptors     A logical indicating whether receptors are only
#'   allowed to be upregulated (FALSE), or up- and downregulated (TRUE).
#' @param fdr.proc      The procedure for adjusting P-values according to
#' \code{\link[multtest]{mt.rawp2adjp}}.
#' @param min.positive    Minimum number of target genes to be found in a given
#'   pathway.
#' @param min.t.logFC     The minimum log2 fold-change allowed for
#'   targets in case pos.targets or neg.targets are used.
#' @param pos.targets   A logical imposing that all the network targets must
#'   display positive logFC, i.e. logFC >= min.t.logFC.
#' @param neg.targets   A logical imposing that all the network targets must
#'   display negative logFC, i.e. logFC <= - min.t.logFC.
#'
#' @details A BSRInferenceComp object should be created by calling
#' \code{"\link[=BSRInferenceComp-class]{BSRInferenceComp}"}
#'
#' @return A BSRInferenceComp object. The main application of this
#' method is to take a "universal" inference obtained by assigning
#' each gene to good logFC, P-values and expression levels whose role
#' is to find all the reachable targets per receptor/pathway, and
#' to update it by using actual logFC, P-values, and expression data.
#' The benefit is to save time when multiple sample comparisons are
#' performed, only one network exploration is necessary. Note that
#' if a restrictive logic such as \code{positive.targets=TRUE} is used,
#' the result will be correct provided all the targets were in the
#' initial BSRInferenceComp object. If a restriction on the targets
#' was applied, then the update is likely to miss some targets, i.e.,
#' the statistical analysis will be wrong.
#'
#' In case no L-R interaction is above the required thresholds, the
#' value `NULL` is returned.
#'
#' Note that correlations are set to 1 to avoid
#' lengthy computations with scRNA-seq data and multiple cell
#' populations.
#'
#' The main role of this method is to support our SingleCellSignalR Version 2
#' package.
#'
#' @export
#' @examples
#' data(bsrdm.comp, package = "BulkSignalR")
#' data(bsrinf.comp, package = "BulkSignalR")
#' colA <- as.integer(1:2)
#' colB <- as.integer(3:4)
#' 
#' #bsrdm.comp <- as(bsrdm, "BSRDataModelComp")
#'
#' n <- nrow(ncounts(bsrdm.comp))
#' stats <- data.frame(pval = runif(n),
#' logFC = rnorm(n, 0, 2),
#' expr = runif(n, 0, 10))
#' rownames(stats) <- rownames(ncounts(bsrdm.comp))
#' 
#' 
#' # update
#' stats$pval <- stats$pval / 100
#' stats$logFC <- stats$logFC + 0.5
#' 
#' bsrcc.2 <- BSRClusterComp(bsrdm.comp, colA, colB, stats)
#' bsrinf.updated <- updateInference(bsrinf.comp, bsrcc.2,
#' max.pval = 1, min.logFC = 0.1)
setMethod("updateInference", "BSRInferenceComp", function(obj, bsrcc,
    ncounts, src.bsrcc = NULL, rank.p = 0.55,
    max.pval = 0.01, min.logFC = 1, min.LR.score = 0,
    neg.receptors = FALSE,
    pos.targets = FALSE, neg.targets = FALSE,
    min.t.logFC = 0.5, min.positive = 2,
    fdr.proc = c(
        "BH", "Bonferroni", "Holm", "Hochberg",
        "SidakSS", "SidakSD", "BY", "ABH", "TSBH"
        )) {
    if (!is(obj, "BSRInferenceComp")) {
        stop("obj must be a BSRInferenceComp object")
    }
    if (!is(bsrcc, "BSRClusterComp")) {
        stop("bsrcc must be a BSRClusterComp object")
    }
    if (!is.null(src.bsrcc) && !is(src.bsrcc, "BSRClusterComp")) {
        stop("src.bsrcc must be a BSRClusterComp object")
    }

    # get BSRInferenceComp object details
    inter <- LRinter(obj)
    L <- ligands(obj)
    R <- receptors(obj)
    t <- tgGenes(obj)
    c <- tgCorr(obj)
    lfc <- tgLogFC(obj)
    p <- tgPval(obj)
    e <- tgExpr(obj)
    par <- inferenceParameters(obj)
    mu <- par$mu
    logTransf <- par$log.transformed.data
    R.stats <- differentialStats(bsrcc)
    if (!is.null(src.bsrcc)) {
        L.stats <- differentialStats(src.bsrcc)
    } # paracrine
    else {
        L.stats <- R.stats
    } # autocrine

    # update parameters
    par$col.clusterA <- colClusterA(bsrcc)
    par$col.clusterB <- colClusterB(bsrcc)
    if (is.null(src.bsrcc)) {
        par$inference.type <- "autocrine"
    } else {
        par$inference.type <- "paracrine"
        par$src.colA <- colClusterA(src.bsrcc)
        par$src.colB <- colClusterB(src.bsrcc)
    }
    par$max.pval <- max.pval
    par$min.logFC <- min.logFC
    par$min.LR.score <- min.LR.score
    par$neg.receptors <- neg.receptors
    par$pos.targets <- pos.targets
    par$neg.targets <- neg.targets
    par$min.t.logFC <- min.t.logFC
    par$min.positive <- min.positive
    par$fdr.proc <- fdr.proc
    par$rank.p <- rank.p

    # assign correct logFC, P-values, correlations, and expression to L and R
    inter$L.logFC <- L.stats[inter$L, "logFC"]
    inter$R.logFC <- R.stats[inter$R, "logFC"]
    inter$LR.pval <- L.stats[inter$L, "pval"] * R.stats[inter$R, "pval"]
    inter$L.expr <- L.stats[inter$L, "expr"]
    inter$R.expr <- R.stats[inter$R, "expr"]

    # if (is.null(src.bsrcc)){
    #   corlr <- stats::cor(t(ncounts[, c(colA(bsrcc),colB(bsrcc))]),
    # method = "spearman")
    # }
    # else {
    #   corlr <- stats::cor(t(ncounts[, c(colA(bsrcc),colA(src.bsrcc))]),
    # method = "spearman")
    #}
    # for (i in seq_len(nrow(inter)))
    #   inter$LR.corr[i] <- corlr[inter$L[i], inter$R[i]]
    inter$LR.corr <- 1

    # LR-score
	if (sum(inter$L.expr<0) + sum(inter$R.expr<0) > 0){
	    # impossible to compute LR-score with negative values,
		# this may happen for scProt-MS data
		inter$LR.score <- 0
	}
	else{
	    # normal case
        if (logTransf) {
            sq <- sqrt(inter$L.expr * inter$R.expr)
        } else {
            sq <- sqrt(log1p(inter$L.expr) / log(2) *
			           log1p(inter$R.expr) / log(2))
        }
        inter$LR.score <- sq / (mu + sq)
	}

    # select on L & R as well as LR-scores
    good <- L.stats[inter$L, "pval"] <= max.pval & inter$L.logFC >= min.logFC &
        R.stats[inter$R, "pval"] <= max.pval
    if (neg.receptors) {
        good <- good & abs(inter$R.logFC) >= min.logFC
    } else {
        good <- good & inter$R.logFC >= min.logFC
    }
    good <- good & inter$LR.score >= min.LR.score
    if (sum(good) == 0) {
        return(NULL)
    }
    inter <- inter[good, ]
    L <- L[good]
    R <- R[good]
    t <- t[good]
    c <- c[good]
    lfc <- lfc[good]
    p <- p[good]
    e <- e[good]

    # assign correct logFC, P-values, 
    # correlations, and expression to the targets
    # and select the targets
    keep <- NULL
    for (i in seq_len(nrow(inter))) {
        genes <- t[[i]]
        logfc <- R.stats[genes, "logFC"]
        if (pos.targets || neg.targets) {
            if (pos.targets) {
                genes <- genes[logfc >= min.t.logFC]
            } else {
                genes <- genes[logfc <= -min.t.logFC]
            }
        } else {
            genes <- genes[abs(logfc) >= min.t.logFC]
        }
        if (length(genes) < min.positive) {
            keep <- c(keep, FALSE)
        } else {
            # if all conditions are met, list all target genes with
            # their regulation P-values in a data frame
            # row. Target genes are sorted wrt P-values in decreasing
            # order to keep the compatibility with correlation analysis,
            # where the most significant values are at the end.
            pv <- R.stats[genes, "pval"]
            o <- order(pv, decreasing = TRUE)
            pv <- pv[o]
            logfc <- R.stats[genes, "logFC"]
            logfc <- logfc[o]
            expr <- R.stats[genes, "expr"]
            expr <- expr[o]
            genes <- genes[o]
            co <- rep(1, length(genes)) # corlr[inter[i, "R"], genes]
            t[[i]] <- genes
            c[[i]] <- co
            lfc[[i]] <- logfc
            p[[i]] <- pv
            e[[i]] <- expr
            len <- length(genes)
            inter[i, "len"] <- len
            rank <- min(max(1, trunc(rank.p * len)), len)
            inter[i, "rank"] <- rank
            # rank.corr and rank.pval are updated by
            #  calling rescoreInference() below
            keep <- c(keep, TRUE)
        }
    }
    if (sum(keep) == 0) {
        return(NULL)
    }
    inter <- inter[keep, ]
    L <- L[keep]
    R <- R[keep]
    t <- t[keep]
    c <- c[keep]
    lfc <- lfc[keep]
    p <- p[keep]
    e <- e[keep]

    # update object
    LRinter(obj) <- inter
    ligands(obj) <- L
    receptors(obj) <- R
    tgGenes(obj) <- t
    tgCorr(obj) <- c
    tgPval(obj) <- p
    tgLogFC(obj) <- lfc
    tgExpr(obj) <- e
    inferenceParameters(obj) <- par

    # rescore and return
    rescoreInference(obj, par, rank.p = rank.p, fdr.proc = fdr.proc)
    
}) # updateInferenceComp


# Reduction and pathway stat methods ===========================================

#' Keep one pathway per ligand-receptor pair
#'
#' @name reduceToBestPathway
#' @aliases reduceToBestPathway,BSRInferenceComp-method
#'
#' @param obj BSRInferenceComp object
#'
#' @return A BSRInferenceComp object reduced to only report one pathway per
#' ligand-receptor pair. The pathway with the
#' smallest P-value is selected.
#'
#' @details Ligand-receptor pairs
#' are evaluated in relation with pathways that allow checking receptor
#' downstream correlations. It is thus possible
#' that several pathways are reported for a same LR pair.
#'
#' @export
#' @examples
#' data(bsrinf.comp, package = "BulkSignalR")
#' 
#' 
#' reduceToBestPathway(bsrinf.comp)
#'
#' @importFrom rlang .data
setMethod("reduceToBestPathway", "BSRInferenceComp", function(obj) {
    # Here we access the object slots directly as this procedure
    # is dependent of actual data representation

    # get best p-value pathway per LR pair
    ligands <- list()
    receptors <- list()
    tg.genes <- list()
    tg.corr <- list()
    tg.pval <- list()
    tg.logFC <- list()
    tg.expr <- list()

    LRinter <- NULL
    pairs <- obj@LRinter
    LR <- unique(pairs[, c("L", "R")])
    for (i in seq_len(nrow(LR))) {
        L <- LR$L[i]
        R <- LR$R[i]

        pwr <- pairs[pairs$L == L & pairs$R == R, ]
        k <- which.min(pwr$pval)
        j <- which(pairs$L == L & pairs$R == R & pairs$pw.id == pwr$pw.id[k])
        ligands <- c(ligands, obj@ligands[j])
        receptors <- c(receptors, obj@receptors[j])
        tg.genes <- c(tg.genes, obj@tg.genes[j])
        tg.corr <- c(tg.corr, obj@tg.corr[j])
        tg.pval <- c(tg.pval, obj@tg.pval[j])
        tg.logFC <- c(tg.logFC, obj@tg.logFC[j])
        tg.expr <- c(tg.expr, obj@tg.expr[j])
        LRinter <- rbind(LRinter, pairs[j, ])
    }

    # update the object
    obj@LRinter <- LRinter
    obj@ligands <- ligands
    obj@receptors <- receptors
    obj@tg.genes <- tg.genes
    obj@tg.corr <- tg.corr
    obj@tg.pval <- tg.pval
    obj@tg.logFC <- tg.logFC
    obj@tg.expr <- tg.expr
    obj@inf.param$pathway.reduced <- TRUE

    obj
    
}) # reduceToBestPathway


#' Aggregate the ligands of a same receptor
#'
#' Simplifies a ligand-receptor table to focus on the receptors.
#'
#' @name reduceToReceptor
#' @aliases reduceToReceptor,BSRInferenceComp-method
#'
#' @return BSRInferenceComp object reduced to one row per receptor.
#' All the ligands are combined in a
#' semi-colon-separated list surrounded by curly brackets in the tabular
#' slot \code{LRinter}, and in vectors in the \code{ligands} (list) slot.
#'
#' The reported P-value and target genes are those from the line with the
#' pathway featuring the smallest P-value.
#' The same logic applies to the LR-score, and the ligand
#' expression.
#' @param obj BRSInferenceComp object
#' @export
#' @examples
#' data(bsrinf.comp, package = "BulkSignalR")
#' # reduction
#' bsrinf.redR <- reduceToReceptor(bsrinf.comp)
#' @importFrom rlang .data
setMethod("reduceToReceptor", "BSRInferenceComp", function(obj) {
    # Here we access the object slots directly as this procedure
    # is dependent of actual data representation

    if (inferenceParameters(obj)$ligand.reduced) {
        stop("Already reduced to receptor")
    } # because ligands were reduced

    # pool the ligands
    ligands <- list()
    receptors <- list()
    tg.genes <- list()
    tg.corr <- list()
    tg.pval <- list()
    tg.logFC <- list()
    tg.expr <- list()
    LRinter <- NULL
    pairs <- obj@LRinter
    for (R in unique(pairs$R)) {
        lig <- pairs[pairs$R == R, ]
        k <- which.min(lig$pval)
        j <- which(pairs$R == lig$R[k] & pairs$pw.id == lig$pw.id[k])[1]
        ligands <- c(ligands, list(unique(lig$L)))
        receptors <- c(receptors, list(R))
        tg.genes <- c(tg.genes, obj@tg.genes[j])
        tg.corr <- c(tg.corr, obj@tg.corr[j])
        tg.pval <- c(tg.pval, obj@tg.pval[j])
        tg.logFC <- c(tg.logFC, obj@tg.logFC[j])
        tg.expr <- c(tg.expr, obj@tg.expr[j])
        to.add <- pairs[j, ]
        to.add[1, "L"] <- paste0("{", paste(unique(lig$L), collapse = ";"), "}")
        to.add[1, "LR.score"] <- max(lig$LR.score)
        to.add[1, "L.expr"] <- max(lig$L.expr)
        LRinter <- rbind(LRinter, to.add)
    }

    # update the object
    obj@LRinter <- LRinter
    obj@ligands <- ligands
    obj@receptors <- receptors
    obj@tg.genes <- tg.genes
    obj@tg.corr <- tg.corr
    obj@tg.pval <- tg.pval
    obj@tg.logFC <- tg.logFC
    obj@tg.expr <- tg.expr
    obj@inf.param$pathway.reduced <- TRUE
    obj@inf.param$ligand.reduced <- TRUE

    obj
    
}) # reduceToReceptor


#' Aggregate the receptors of a same ligand
#'
#' Simplifies a ligand-receptor table to focus on the ligands.
#'
#' @name reduceToLigand
#' @aliases reduceToLigand,BSRInferenceComp-method
#'
#' @return A BSRInferenceComp object but reduced to one row per ligand.
#' All the receptors are combined in a
#' semi-colon-separated list surrounded by curly brackets in the tabular
#' slot \code{LRinter}, and in vectors in the \code{ligands} (list) slot.
#'
#' The reported P-value and target genes are those from the pathway with
#' the smallest P-value.
#' The same logic applies to the LR-score, and the receptor
#' expression.
#' @param obj BSRInferenceComp object
#' @export
#' @examples
#' data(bsrinf.comp, package = "BulkSignalR")
#' 
#' bsrinf.redL <- reduceToLigand(bsrinf.comp)
#' @importFrom rlang .data
setMethod("reduceToLigand", "BSRInferenceComp", function(obj) {
    # Here we access the object slots directly as this procedure
    # is dependent of actual representation

    if (inferenceParameters(obj)$receptor.reduced) {
        stop("Already reduced to ligand")
    } # because receptors were reduced

    # pool the receptors
    ligands <- list()
    receptors <- list()
    tg.genes <- list()
    tg.corr <- list()
    tg.pval <- list()
    tg.logFC <- list()
    tg.expr <- list()
    LRinter <- NULL
    pairs <- obj@LRinter
    for (L in unique(pairs$L)) {
        rec <- pairs[pairs$L == L, ]
        k <- which.min(rec$pval)
        j <- which(pairs$L == L & 
            pairs$R == rec$R[k] & 
            pairs$pw.id == rec$pw.id[k])
        ligands <- c(ligands, list(L))
        receptors <- c(receptors, list(unique(rec$R)))
        tg.genes <- c(tg.genes, obj@tg.genes[j])
        tg.corr <- c(tg.corr, obj@tg.corr[j])
        tg.pval <- c(tg.pval, obj@tg.pval[j])
        tg.logFC <- c(tg.logFC, obj@tg.logFC[j])
        tg.expr <- c(tg.expr, obj@tg.expr[j])
        to.add <- pairs[j, ]
        to.add[1, "R"] <- paste0("{", paste(unique(rec$R), collapse = ";"), "}")
        to.add[1, "LR.score"] <- max(rec$LR.score)
        to.add[1, "R.expr"] <- max(rec$R.expr)
        LRinter <- rbind(LRinter, to.add)
    }

    # update the object
    obj@LRinter <- LRinter
    obj@ligands <- ligands
    obj@receptors <- receptors
    obj@tg.genes <- tg.genes
    obj@tg.corr <- tg.corr
    obj@tg.pval <- tg.pval
    obj@tg.logFC <- tg.logFC
    obj@tg.expr <- tg.expr
    obj@inf.param$pathway.reduced <- TRUE
    obj@inf.param$receptor.reduced <- TRUE

    obj
    
}) # reduceToLigand


#' Aggregate ligands and receptors at the pathway level
#'
#' Simplifies a ligand-receptor inference object to focus on
#' the pathways.
#'
#' @name reduceToPathway
#' @aliases reduceToPathway,BSRInferenceComp-method
#'
#' @return A BSRInferenceComp object reduced to only report one row per pathway.
#' The information of which ligand interacted with which receptor is lost as
#' all the ligands and all the receptors forming pairs related to a certain
#' pathway are combined.
#' For a given pathway, the reported P-values and target genes are those of
#' the best ligand-receptor pair that was in this pathway.
#' The same logic applies to the LR-score, and the ligand and receptor
#' expression.
#' Receptors and ligands are combined in two semi-colon-separated
#' lists surrounded by curly brackets in the tabular slot \code{LRinter},
#' while the list representation slots (\code{ligands} and
#' \code{receptors}) are update accordingly.
#'
#' @param obj BSRInferenceComp object
#' @export
#' @examples
#' data(bsrinf.comp, package = "BulkSignalR")
#' 
#' bsrinf.redP <- reduceToPathway(bsrinf.comp)
#' @importFrom rlang .data
setMethod("reduceToPathway", "BSRInferenceComp", function(obj) {
    # Here we access the object slots directly as this procedure
    # is dependent of actual representation

    if (inferenceParameters(obj)$receptor.reduced) {
        stop("Already reduced to ligand")
    } # because receptors were reduced
    if (inferenceParameters(obj)$ligand.reduced) {
        stop("Already reduced to receptor")
    } # because ligands were reduced

    # reduce to unique pathways
    ligands <- list()
    receptors <- list()
    tg.genes <- list()
    tg.corr <- list()
    tg.pval <- list()
    tg.logFC <- list()
    tg.expr <- list()
    LRinter <- NULL
    pairs <- obj@LRinter
    for (p in unique(pairs$pw.id)) {
        j <- which(pairs$pw.id == p)[1]
        ligands <- c(ligands, list(unique(pairs$L[pairs$pw.id == p])))
        receptors <- c(receptors, list(unique(pairs$R[pairs$pw.id == p])))
        tg.genes <- c(tg.genes, obj@tg.genes[j])
        tg.corr <- c(tg.corr, obj@tg.corr[j])
        tg.pval <- c(tg.pval, obj@tg.pval[j])
        tg.logFC <- c(tg.logFC, obj@tg.logFC[j])
        tg.expr <- c(tg.expr, obj@tg.expr[j])
        to.add <- pairs[j, ]
        to.add[1, "L"] <- paste0("{", paste(unique(pairs$L[pairs$pw.id == p]),
            collapse = ";"
        ), "}")
        to.add[1, "R"] <- paste0("{", paste(unique(pairs$R[pairs$pw.id == p]),
            collapse = ";"
        ), "}")
        to.add[1, "LR.score"] <- max(pairs$LR.score[pairs$pw.id == p])
        to.add[1, "R.expr"] <- max(pairs$R.expr[pairs$pw.id == p])
        to.add[1, "L.expr"] <- max(pairs$L.expr[pairs$pw.id == p])
        LRinter <- rbind(LRinter, to.add)
    }

    # update the object
    obj@LRinter <- LRinter
    obj@ligands <- ligands
    obj@receptors <- receptors
    obj@tg.genes <- tg.genes
    obj@tg.corr <- tg.corr
    obj@tg.pval <- tg.pval
    obj@tg.logFC <- tg.logFC
    obj@tg.expr <- tg.expr
    obj@inf.param$ligand.reduced <- TRUE
    obj@inf.param$receptor.reduced <- TRUE

    obj
    
}) # reduceToPathway

## ----setup, include = FALSE---------------------------------------------------
knitr::knit_hooks$set(optipng = knitr::hook_optipng)
options(rmarkdown.html_vignette.check_title = FALSE)

## ----load-libs, message = FALSE,  warning = FALSE, results = FALSE------------
library(BulkSignalR)

## ----diffmode1,eval=TRUE------------------------------------------------------
data(sdc, package = "BulkSignalR")
normal <- grep("^N", names(sdc))
bsrdm.comp <- BSRDataModelComp(sdc[, -normal])

## ----diffmode2,eval=TRUE------------------------------------------------------
colA <- as.integer(1:3)
colB <- as.integer(12:15)

n <- nrow(ncounts(bsrdm.comp))
stats <- data.frame(pval = runif(n),
logFC = rnorm(n, 0, 2),
expr = runif(n, 0, 10))
rownames(stats) <- rownames(ncounts(bsrdm.comp))

## ----diffmode3,eval=TRUE------------------------------------------------------

bsrcc <- BSRClusterComp(bsrdm.comp, colA, colB, stats)
bsrdm.comp <- addClusterComp(bsrdm.comp, bsrcc, "random.example")


## ----diffmode4,eval=TRUE------------------------------------------------------

subset <- c("REACTOME_BASIGIN_INTERACTIONS",
"REACTOME_SYNDECAN_INTERACTIONS",
"REACTOME_ECM_PROTEOGLYCANS",
"REACTOME_CELL_JUNCTION_ORGANIZATION")

reactSubset <- BulkSignalR:::.SignalR$BulkSignalR_Reactome[
BulkSignalR:::.SignalR$BulkSignalR_Reactome$`Reactome name` %in% subset,]

resetPathways(dataframe = reactSubset,
resourceName = "Reactome")

bsrinf.comp <- BSRInferenceComp(bsrdm.comp,
reference="REACTOME",
max.pval = 1, 
"random.example")

head(LRinter(bsrinf.comp))


## ----session-info-------------------------------------------------------------
sessionInfo()

#nameEnv <- ".SignalR-Env"
.SignalR<- new.env(parent = emptyenv()) 

.onLoad <- function(...) {
    
    # handle directory creation over different OS
    cacheDir <- tools::R_user_dir("BulkSignalR", which="cache")
    assign("BulkSignalR_CACHEDIR", cacheDir, envir = .SignalR)

    url <- "https://partage-dev.montp.inserm.fr:9192/CBSB/"

    assign("BulkSignalR_CORE_URL", url, envir = .SignalR)

    .testRemoteServer()
    .testCacheFiles()
    

    ################################
    ##   Resource Cache Files   ###
    ################################
    urlGo <- paste0(url,
        "SignalR/resources/gobp.rds")
    urlReactome <- paste0(url,
        "SignalR/resources/reactome.rds")
    urlNetwork <- paste0(url,
        "SignalR/resources/Network.rds")
    urlLRdb <- paste0(url,
        "SignalR/resources/LRdb.txt")

    assign("BulkSignalR_GO_URL", urlGo, 
    envir = .SignalR)
    assign("BulkSignalR_Reactome_URL", urlReactome, 
    envir = .SignalR)
    assign("BulkSignalR_Network_URL", urlNetwork, 
    envir = .SignalR)
    assign("BulkSignalR_LRdb_URL", urlLRdb, 
    envir = .SignalR)

    createResources(onRequest = FALSE)
   
    BulkSignalR_Reactome <- getResource(resourceName = "Reactome",
        cache = TRUE)
    BulkSignalR_Gobp <- getResource(resourceName = "GO-BP",
        cache = TRUE)
    BulkSignalR_Network <- getResource(resourceName = "Network",
        cache = TRUE)
    BulkSignalR_LRdb <- getResource(resourceName = "LRdb",
        cache = TRUE)  

    assign("BulkSignalR_Reactome", BulkSignalR_Reactome, 
    envir = .SignalR)
    assign("BulkSignalR_Gobp", BulkSignalR_Gobp, 
    envir = .SignalR)
    assign("BulkSignalR_Network", BulkSignalR_Network, 
    envir = .SignalR)
    assign("BulkSignalR_LRdb", BulkSignalR_LRdb, 
    envir = .SignalR)
}

#' Check whether the reference DB server is up
#'
#' @return Returns `NULL`, invisibly. 
#'
#' @importFrom RCurl url.exists
#' @importFrom curl has_internet
#' @importFrom cli cli_alert_danger cli_alert_info
.testRemoteServer <- function() {
    conf <- list("ssl.verifypeer" = 0L,
    "ssl.verifyhost" = 0L)

    hasInternet <- tryCatch(expr={curl::has_internet()}, 
        error = FALSE)

    if(hasInternet){

        if(!url.exists(.SignalR$BulkSignalR_CORE_URL,
            .opts = conf))
        {
        cli::cli_alert_danger(
            "Remote server is down. 
            {.val {(.SignalR$BulkSignalR_CORE_URL)}}"    
        )
        }
    } 

    return(invisible(NULL))
}


#' Check there is a well formatted cache
#'
#' @return Returns `NULL`, invisibly.
#'
#' @importFrom RCurl url.exists
#' @importFrom curl has_internet
#' @importFrom cli cli_alert_danger cli_alert_info
#' @importFrom BiocFileCache BiocFileCache bfcinfo
.testCacheFiles <- function() {

    hasInternet <- tryCatch(expr={curl::has_internet()}, 
        error = FALSE)

    cacheDir <- .SignalR$BulkSignalR_CACHEDIR

    cacheDir <- paste(cacheDir, "resources", sep = "/")
   
    # Create cacheDir 
    bfc <- BiocFileCache::BiocFileCache(cacheDir, ask = FALSE)
    files <- BiocFileCache::bfcinfo(bfc)$rpath

    vecSizes <- vapply(files, file.size, numeric(1))
    totSize <- sum(vecSizes)
    #if(!hasInternet){
    #    mess_info <- paste0("You need an internet connection",
    #" to download cache files.")
    #    cli::cli_alert_danger(mess_info)
    #    stop()
    #}
    # Trick
    if(totSize==0){
        mess_info <- paste0("{(.SignalR$BulkSignalR_CACHEDIR)}",
        " first set up.\n")
        cli::cli_alert_info(mess_info)
        unlink(.SignalR$BulkSignalR_CACHEDIR, recursive=TRUE)
        return(invisible(NULL))
    }

    if(totSize > 6000000 | totSize < 4000000 | length(files)!=4){
        mess_info <- paste0("{(.SignalR$BulkSignalR_CACHEDIR)}",
        " is corrupted.\n",
        " The directory will be removed and re-downloaded",
        " automatically on the next attempt",
        " when the remote server is reachable.\n")

        cli::cli_alert_info(mess_info)
        unlink(.SignalR$BulkSignalR_CACHEDIR, recursive=TRUE)
    } 

    return(invisible(NULL))
}


#' Modify LRdb database
#'
#' Users can provide a data frame with 2 columns named
#' ligand and receptor.
#' This can be used to extend or replace the existing
#' LRdb.
#'
#' @param db     A data frame with 2 columns named
#' ligand and receptor.
#' @param switch  A logical indicating whether LRdb should be extended only
#' (FALSE, default) or completely replaced (TRUE).
#'
#' @return Returns `NULL`, invisibly. 
#'
#' @importFrom cli cli_alert_info
#' @export
#' @examples
#' resetLRdb(db = data.frame(ligand = "A2M", receptor = "LRP1"), switch = FALSE)
resetLRdb <- function(db, switch = FALSE) {
    if (colnames(db)[1] == "ligand" & colnames(db)[2] == "receptor") {
        if (switch) {
            assign("BulkSignalR_LRdb", unique(db[, c("ligand", "receptor")]),
                envir = .SignalR
            )
        } else {
            db <- rbind(
                .SignalR$BulkSignalR_LRdb[, c("ligand", "receptor")],
                db[, c("ligand", "receptor")]
            )
            assign("BulkSignalR_LRdb", unique(db), 
                envir = .SignalR)
        }
    } else {
        stop(
            "db should be a data frame with",
            "2 columns named 'ligand' and 'receptor'."
        )
    }

    message("")
    cli::cli_alert_info(
        "New database defined for {.val LRdb}."
    )

    return(invisible(NULL))
    
} # resetLRdb


#' Internal function to check and extract a
#' count matrix if a more complex object than a simple matrix or data frame
#' is given as parameter. Main usage is to link with Bioconductor objects.
#'
#' @param counts A table or matrix of read counts (or protein abundance).
#' It can also be a SummarizedExperiment or SpatialExperiment
#' object from which the count matrix should be extracted.
#' See \code{\link{BSRDataModel}}.
#' @param symbol.col The index of the column containing the gene symbols in case
#' those are not the row names of \code{counts} already. In a
#' SpatialExperiment object, the index in the data frame returned by rowData().
#' @param x.col In a SpatialExperiment object, the index of the column
#' containing the x coordinates in the dafaframe returned by rowData(), usually 
#' named array_row.
#' @param y.col  In a SpatialExperiment object, the index of the column
#' containing the y coordinates in the dafaframe returned by rowData(), usually 
#' named array_col.
#' @param barcodeID.col   In a SpatialExperiment object, the index of the column
#' containing the barcodeID in the dafaframe returned by colData(), usually
#' named barcode_id.
#'
#' @return A matrix of count (or abundance) values
#'
#' @import SummarizedExperiment
#' @import SpatialExperiment
#' @keywords internal
.checkInteroperabilityForCounts <- function(counts,
    symbol.col,x.col,y.col,barcodeID.col) {
    
    countsChecked <- NULL 

        if(class(counts) %in% c("SummarizedExperiment")){

            if(length(assays(counts))>1){
                stop("Only one assay should be defined.")
            }
            countsChecked <- assays(counts)[[1]]
            if(!is.null(symbol.col)){
                rownames(countsChecked) <- rowData(counts)[[symbol.col]]
            }
        }

        else if(class(counts) %in% c("SpatialExperiment")){

            if(length(assays(counts))>1){
                stop("Only one assay should be defined.")
            }

            if (!is.numeric(x.col) || !is.numeric(y.col) ) {
                stop("x.col or y.col values are not numerics.")
            }
            if (is.null(barcodeID.col)) {
                stop("barcodeID.col should be defined.")
            }
            countsChecked <- as.matrix(assays(counts)[[1]])
            if(!is.null(symbol.col)){
                rownames(countsChecked) <- rowData(counts)[[symbol.col]]
            }

            colData(counts)$idSpatial <- paste(colData(counts)[[x.col]],
                colData(counts)[[y.col]],sep = "x")

            # Match and re-order cols 
            ord <- match(colnames(countsChecked), 
                colData(counts)[[barcodeID.col]]) 
            if(sum(is.na(ord))>0){
                stop("Some barcodeIDs are missing...")
            }
            countsChecked <- countsChecked[,ord]

            if (!all(colnames(countsChecked) == colData(counts)
            [[barcodeID.col]]))
                stop("BarcodeIDs are not well ordered.", call. = FALSE)

            colnames(countsChecked) <- colData(counts)[["idSpatial"]]
        }

        else {countsChecked <- counts}
    
    return(countsChecked)
    
} # .checkInteroperabilityForCounts


#' Constructor of the BSRDataModel class
#'
#' Take a matrix or data frame containing RNA sequencing,
#' microarray, or expression proteomics data as input parameter
#' and return a BSRDataModel
#' object ready for subsequent training.
#' 
#' Note that this constructor replaces
#' the function prepareDataset that was part of the previous version of
#' BulkSignalR library.
#'
#' @param counts     A table or matrix of read counts.
#' @param species    Data were obtained for this organism.
#' @param normalize  A logical indicating whether \code{counts} should be
#'   normalized according to \code{method} or if it was normalized beforehand.
#' @param symbol.col The index of the column containing the gene symbols in case
#'   those are not the row names of \code{counts} already.
#' @param min.count  The minimum read count of a gene to be considered expressed
#'   in a sample.
#' @param prop       The minimum proportion of samples where a gene must be
#'   expressed higher than \code{min.count} to keep that gene.
#' @param method     The normalization method ('UQ' for upper quartile or 'TC'
#'   for total count). If \code{normalize==FALSE}, then method must be
#'   used to document the name of the normalization method applied by the user.
#' @param UQ.pc      Percentile for upper-quartile normalization, number
#' between 0 and 1 (in case the default 0.75 - hence the name - is not
#' appropriate).
#' @param log.transformed  A logical indicating whether expression data were
#'   already log2-transformed, e.g., some microarray data.
#' @param min.LR.found  The minimum number of ligands or receptors found in
#'   \code{count} row names after eliminating the rows containing too many
#'   zeros according to \code{min.count} and \code{prop}.
#' @param conversion.dict  Correspondence table of HUGO gene symbols
#' human/nonhuman. Not used unless the organism is different from human.
#' @param x.col In a SpatialExperiment object, the index of the column
#' containing the x coordinates in the dafaframe returned by rowData(), usually 
#' named array_row.
#' @param y.col  In a SpatialExperiment object, the index of the column
#' containing the y coordinates in the dafaframe returned by rowData(), usually 
#' named array_col.
#' @param barcodeID.col   In a SpatialExperiment object, the index of the column
#' containing the barcodeID in the dafaframe returned by colData(), usually
#' named barcode_id.
#'
#' @return A BSRModelData object with empty model parameters.
#'
#' @details The \code{counts} matrix or table should be provided with expression
#'   levels of protein coding genes in each samples (column) and
#'   \code{rownames(counts)} set to HUGO official gene symbols.
#'   For commodity, it is also possible 
#'   to provide \code{counts} with the
#'   gene symbols stored in one of its columns. This column must be specified
#'   with \code{symbol.col}. In such a case, \code{BSRDataModel} will extract
#'   this column and use it to set the row names. Because row names must be
#'   unique, \code{BSRDataModel} will eliminate rows with duplicated gene
#'   symbols by keeping the rows with maximum average expression. Gene symbol
#'   duplication may occur in protein coding genes after genome alignment
#'   due to errors in genome feature annotation files (GTF/GFF), where a handful
#'   of deprecated gene annotations might remain, or
#'   some genes are not given their fully specific symbols. If your read count
#'   extraction pipeline does not take care of this phenomenon, the maximum mean
#'   expression selection strategy implemented here should solve this difficulty
#'   for the sake of inferring ligand-receptor interactions.
#'
#'   If \code{normalize} is \code{TRUE} then normalization is performed
#'   according to \code{method}. If those two simple methods are not satisfying,
#'   then it is possible to provide a pre-normalized matrix setting
#'   \code{normalize} to \code{FALSE}. In such a case, the parameter
#'   \code{method} must be used to document the name of the normalization
#'   algorithm used.
#'
#'   In case proteomic or microarray data are provided, \code{min.count} must be
#'   understood as its equivalent with respect to those data types.
#'
#' @importFrom matrixStats rowMeans2 rowSums2 colSums2
#' @export
#' @examples
#' data(sdc, package = "BulkSignalR")
#' idx <- sample(nrow(sdc), 4000)
#' bsrdm <- BSRDataModel(sdc[idx, c("N22","SDC17")],
#' normalize = FALSE,method="UQ")
BSRDataModel <- function(
    counts, normalize = TRUE, symbol.col = NULL, min.count = 10,
    prop = 0.1, method = c("UQ", "TC"), 
    log.transformed = FALSE, min.LR.found = 80,
    species = "hsapiens", conversion.dict = NULL,
    UQ.pc = 0.75,x.col = NULL, y.col = NULL,
    barcodeID.col = NULL) {
    if ((species != "hsapiens") && is.null(conversion.dict)) {
        stop("Non-human species ",
            "but no conversion.dict provided")
    }

    if (normalize) {
        if (prop < 0 || prop > 1) {
            stop("prop must lie in [0;1]")
        }
        if (UQ.pc <= 0 || UQ.pc > 1) {
            stop("UQ.pc must lie in ]0;1]")
        }
        if (min.count < 0) {
            stop("min.count must be positive")
        }
        method <- match.arg(method)
    } else if (nchar(method) == 0) {
        stop(
            "In case of user-normalized counts, the name of the ",
            "normalization must be documented through the parameter 'method'"
        )
    }

    if (is(counts,"SummarizedExperiment")){

    counts <- .checkInteroperabilityForCounts(counts,
        symbol.col, x.col, y.col, barcodeID.col)

    } else {

        if (!is.null(symbol.col)) {
            if (!is.numeric(symbol.col)) {
                stop("symbol.col must be the index of",
                    "the column containing the gene symbols")
            }

            # simple but desperately slow counts <-
            # aggregate(.~symbol,data=counts,FUN=max)

            # home-made but fast
            symbols <- as.character(counts[, symbol.col])
            d <- symbols[duplicated(symbols)]
            bad <- NULL
            for (s in d) {
                i <- which(symbols == s)
                t <- rowSums(counts[i, -symbol.col])
                bad <- c(bad, i[-which.max(t)])
            }

            # remove duplicates and the gene symbol column
            if (!is.null(bad)) {
                counts <- counts[-bad, -symbol.col]
                rownames(counts) <- symbols[-bad]
            } else {
                counts <- counts[, -symbol.col]
                rownames(counts) <- symbols
            }
        }
    }

    if (is.null(rownames(counts)) || typeof(rownames(counts)) != "character") {
        stop("The read count matrix ",
            "must be provided with gene symbols as row names")
    }

    # as of now we ensure that counts is a matrix
    if (!is.matrix(counts))
        counts <- data.matrix(counts)


    # avoid empty rows even if no normalization is performed here
    counts <- counts[matrixStats::rowSums2(abs(counts)) > 0, ]

    if (normalize) {
        good.c <- matrixStats::rowSums2(counts >= min.count) >= 
        prop * ncol(counts)
        counts <- counts[good.c, ]
        if (method == "UQ") {
            tot <- apply(counts, 2, function(x) {
                stats::quantile(x[x > 0],
                    prob = UQ.pc
                )
            })
            if (sum(tot == 0) > 0) {
                stop(paste0(
                    "Cannot perform UQ normalization (percentile=",
                    UQ.pc, " ), not enough signal in sample(s) ",
                    paste(colnames(counts)[tot == 0], collapse = ", ")
                ))
            }
        } else {
            tot <- matrixStats::colSums2(counts)
        }
        ncounts <- sweep(counts, 2, tot / stats::median(tot), "/")
    } else {
        ncounts <- counts
    }

    homolog.genes <- list()
    if (species != "hsapiens") {
        ncounts <- as.data.frame(ncounts)
        ncounts$human.gene.name <- rownames(ncounts)
        conversion.dict$human.gene.name <- rownames(conversion.dict)
        ncounts$id <- seq_len(nrow(ncounts))

        counts.transposed <- merge(ncounts, conversion.dict,
            by.x = "human.gene.name",
            all = FALSE, sort = FALSE
        )
        counts.transposed <- counts.transposed[order(counts.transposed$id), ]

        homolog.genes <- list(counts.transposed$Gene.name)

        counts.transposed$id <- NULL
        ncounts$id <- NULL
        ncounts$human.gene.name <- NULL
        ncounts <- data.matrix(ncounts)
        rm(counts.transposed)
    }

    nLR <- length(intersect(
        c(.SignalR$BulkSignalR_LRdb$ligand, 
        .SignalR$BulkSignalR_LRdb$receptor),
        rownames(ncounts)
    ))
    if (nLR < min.LR.found) {
        stop(
            "Not enough LR genes (", nLR, " < ", min.LR.found,
            " were found).\n"
        )
    }

    new("BSRDataModel",
        ncounts = ncounts, 
        log.transformed = log.transformed,
        normalization = toupper(method), 
        initial.organism = species,
        initial.orthologs = homolog.genes
    )
    
} # BSRDataModel


#' @title Orthologs Gene Names
#'
#' @description By default, BulkSignalR is designed to work with Homo sapiens.
#' In order to work with other organisms, gene names need to be first converted
#' to human following an orthology mapping process.
#' @param from_organism    An organism defined as in Ensembl:
#' drerio, mmusculus, celegans, dmelanogaster, etc. This is the source organism
#' from which you want to convert the gene names to Homo sapiens.
#' @param from_values   A vector of gene names from the current species studied.
#' @param method  Ortholog mapping method.
#' @return Return a data frame with 2 columns containing the gene names
#' for the two species.
#' First column is the gene name from the source organism
#' and the second column corresponds to the  homologous gene name
#' in  Homo sapiens.
#' @importFrom orthogene convert_orthologs
#'
#' @export
#' @examples
#' data(bodyMap.mouse)
#' 
#' idx <- sample(nrow(bodyMap.mouse), 20)
#' bodyMap.mouse <- bodyMap.mouse[idx,]
#' 
#' ortholog.dict <- findOrthoGenes(
#'     from_organism = "mmusculus",
#'     from_values = rownames(bodyMap.mouse)
#' )
#'
findOrthoGenes <- function(from_organism, from_values,
    method = c("homologene", "gprofiler", "babelgene")) {

    method <- match.arg(method)
    if (!method %in% c("gprofiler", "homologene", "babelgene")) {
        stop("Method selected should be gprofiler,homologene or babelgene")
    }

    orthologs_dictionary <- orthogene::convert_orthologs(
        gene_df = from_values,
        gene_input = "rownames",
        gene_output = "rownames",
        input_species = from_organism,
        output_species = "human",
        non121_strategy = "drop_both_species", # assure 1.1
        method = method,
        verbose = FALSE
    )
    orthologs_dictionary$index <- NULL
    names(orthologs_dictionary)[1] <- paste("Gene.name")
    # Keep only Gene.name
    orthologs_dictionary <- orthologs_dictionary[, "Gene.name",
    drop = FALSE]

    message(
        "Dictionary Size: ",
        dim(orthologs_dictionary)[1],
        " genes"
    )

    nL <- length(intersect(
        .SignalR$BulkSignalR_LRdb$ligand,
        rownames(orthologs_dictionary)
    ))
    message("-> ", nL, " Ligands")

    nR <- length(intersect(
        .SignalR$BulkSignalR_LRdb$receptor,
        rownames(orthologs_dictionary)
    ))
    message("-> ", nR, " Receptors")

    orthologs_dictionary
    
} # findOrthoGenes


#' @title Transpose to Human Gene Names
#'
#' @description By default, BulkSignalR is designed to work with Homo sapiens.
#' In order to work with other organisms, gene names need to be first converted
#' to human by orthology.
#' @param counts  A table or matrix of read counts.
#' @param dictionary   A data frame where the first column 
#' is made of gene symbols for the actual organism 
#' and row names are the ortholog human gene symbols.
#'
#' @return Return a counts matrix transposed for Human.
#'
#' @export
#' @examples
#' data(bodyMap.mouse)
#'
#' idx <- sample(nrow(bodyMap.mouse), 500)
#' bodyMap.mouse <- bodyMap.mouse[idx,]
#' 
#' ortholog.dict <- findOrthoGenes(
#'     from_organism = "mmusculus",
#'     from_values = rownames(bodyMap.mouse)
#' )
#'
#' matrix.expression.human <- convertToHuman(
#'     counts = bodyMap.mouse,
#'     dictionary = ortholog.dict
#' )
#'
convertToHuman <- function(counts, dictionary) {
    # we need counts to be a data.frame
    if (is.matrix(counts)) {
        was.matrix <- TRUE
        counts <- as.data.frame(counts)
    } else {
        was.matrix <- FALSE
    }
    # counts should have row names
    if (all(row.names(counts) == seq(1, nrow(counts)))) {
        stop("Rownames should be set as human ",
            "gene names for counts.", call. = FALSE)
    }
    if (all(row.names(dictionary) == seq(1, nrow(dictionary)))) {
        stop("Rownames should be set as human ",
        "gene names dictionary.", call. = FALSE)
    }

    # Check column exists
    if (!"Gene.name" %in% colnames(dictionary)) {
        stop("Gene.name column does not exist in dictionary.")
    }

    if (!all(apply(counts, 2, function(x) is.numeric(x)))) {
        stop("Some variables are not defined as numerics.", call. = FALSE)
    }

    # Transform Matrice using orthologs_dictionary
    counts$Gene.name <- rownames(counts)
    dictionary$human.gene.name <- rownames(dictionary)

    counts$id <- seq_len(nrow(counts))

    counts.transposed <- merge(counts, dictionary,
        by.x = "Gene.name", all = FALSE, sort = FALSE)
    counts.transposed <- counts.transposed[order(counts.transposed$id), ]
    counts.transposed$id <- NULL

    # aesthetics only
    counts.transposed <- counts.transposed[,
    c(which(colnames(counts.transposed) == "human.gene.name"),
        which(colnames(counts.transposed) != "human.gene.name"))]

    counts.transposed$Gene.name <- NULL

    rownames(counts.transposed) <- counts.transposed[, 1]
    counts.transposed <- counts.transposed[, -1]

    if (was.matrix) {
        # convert back to a matrix
        data.matrix(counts.transposed)
    } else {
        counts.transposed
    }
    
} # convertToHuman

#' Get regulated ligand-receptor pairs.
#'
#' Internal function to return all the pairs of ligands and receptors
#' having both a P-value below a given threshold.
#'
#' @param ds              A BSRDataModel object
#' @param cc              A BSRClusterComp object.
#' @param scc             An optional BSRClusterComp object in case ligand
#'   regulation was assessed in a separate cluster comparison.
#' @param max.pval        The maximum P-value imposed to both the ligand
#' and the receptor.
#' @param min.logFC       The maximum log2 fold-change allowed for
#'   both the receptor and the ligand.
#' @param neg.receptors     A logical indicating whether receptors are only
#'   allowed to be upregulated (FALSE), or up- and downregulated (TRUE).
#' @param restrict.genes  A list of gene symbols that restricts ligands and
#'   receptors.
#'
#' @return A data frame containing putative ligand-receptor pairs along
#'   with the product of their respective P-values. This table is the first step
#'   of a ligand-receptor analysis.
#'
#' @details The \code{restrict.genes} parameter is used for special cases where
#'   LRdb must be further restricted to a subset.
#'   The putative ligand-receptor pairs has 6 columns : R, L, LR.pval, corr,
#'   L.logFC, and R.logFC. 
#'   Note that correlations are currently set to 1 to avoid
#'   lengthy computations with scRNA-seq data and multiple cell
#'   populations.
#'
#' @importFrom methods is
#' @importFrom foreach %do% %dopar%
#' @import doParallel
#' @keywords internal
.getRegulatedLR <- function(ds, cc, scc = NULL, max.pval = 0.01, min.logFC = 1,
                            neg.receptors = FALSE, restrict.genes = NULL) {
    # local binding
    i <- NULL

    if ((max.pval <= 0) || (max.pval > 1)) {
        stop("max.pval must lie in ]0;1]")
    }
    if (min.logFC <= 0) {
        stop("min.logFC must be > 0")
    }
    if (!is(ds, "BSRDataModelComp")) {
        stop("ds must be an object of class BSRDataModelComp")
    }
    if (!is(cc, "BSRClusterComp")) {
        stop("cc must be an object of class BSRClusterComp")
    }
    if (!is.null(scc) && !is(scc, "BSRClusterComp")) {
        stop("scc must be an object of class BSRClusterComp")
    }

    lrgenes <- intersect(c(
        .SignalR$BulkSignalR_LRdb$ligand,
        .SignalR$BulkSignalR_LRdb$receptor
    ), rownames(differentialStats(cc)))
    if (!is.null(restrict.genes)) {
        lrgenes <- intersect(lrgenes, restrict.genes)
    }

    # # compute all the correlations at once
    # if (is.null(scc)){
    #   corlr <- stats::cor(t(ncounts(ds)[lrgenes,
    # c(colClusterA(cc),colClusterB(cc))]), method = "spearman")
    # }
    # else {
    #   corlr <- stats::cor(t(ncounts(ds)[lrgenes,
    # c(colClusterA(cc),colClusterA(scc))]), method = "spearman")
    # }
    # get the pairs
    R.stats <- differentialStats(cc)
    if (is.null(scc)) {
        L.stats <- differentialStats(cc)
    } else {
        L.stats <- differentialStats(scc)
    }
    pairs <- NULL
    for (i in seq_len(nrow(.SignalR$BulkSignalR_LRdb))) {
        L <- .SignalR$BulkSignalR_LRdb$ligand[i]
        R <- .SignalR$BulkSignalR_LRdb$receptor[i]
        if (L %in% lrgenes && R %in% lrgenes) {
            pL <- L.stats[L, "pval"]
            pR <- R.stats[R, "pval"]
            if (pL <= max.pval && pR <= max.pval) {
                fcL <- L.stats[L, "logFC"]
                fcR <- R.stats[R, "logFC"]
                if (fcL >= min.logFC &&
                    ((neg.receptors && abs(fcR) >= min.logFC) || 
                        fcR >= min.logFC)) {
                    pairs <- rbind(
                        pairs,
                        data.frame(
                            L = L, R = R,
                            LR.pval = pL * pR, corr = 1, # corlr[L, R],
                            L.logFC = fcL, R.logFC = fcR,
                            stringsAsFactors = FALSE
                        )
                    )
                }
            }
        }
    }

    if (is.null(pairs)) {
        stop("Dataframe `pairs` from `.getRegulatedLR` is NULL.")
    }

    pairs
} # .getRegulatedLR


#' Internal function to check receptor signaling downstream
#'
#' @param lr              A data frame as returned by
#'   \code{.getRegulatedLR()}.
#' @param pw              A table defining the reference pathways.
#' @param pw.size         A named vector with pathway sizes (names are pathway
#'   IDs).
#' @param rncounts        A matrix of normalized read counts with at
#'   least all the ligands, receptors, and genes in the reference pathways.
#' @param stats           A data.frame with a column 'pval' and
#' \code{rownames(stats)} assigned to gene symbols. Rows must at least
#'   include all the ligands, receptors, and genes in the reference pathways.
#' @param id.col          Column index or name in \code{pw} for the pathway IDs.
#' @param gene.col        Column index or name in \code{pw} for the gene
#'   symbols.
#' @param pw.col          Column index or name in \code{pw} for the pathway
#'   names.
#' @param min.positive    Minimum number of target genes to be found in a given
#'   pathway.
#' @param min.t.logFC     The minimum log2 fold-change allowed for
#'   targets in case pos.targets or neg.targets are used.
#' @param pos.targets   A logical imposing that all the network targets must
#'   display positive logFC, i.e. logFC >= min.t.logFC.
#' @param neg.targets   A logical imposing that all the network targets must
#'   display negative logFC, i.e. logFC <= - min.t.logFC.
#' @param with.complex    A logical indicating whether receptor co-complex
#'   members should be included in the target genes.
#'
#' @return A table reporting all the ligand-receptor pairs provided in \code{lr}
#'   along with the pathways found and data about target gene regulation
#'   P-values. Note that correlations are currently set to 1 to avoid
#'   lengthy computations with scRNA-seq data and multiple cell
#'   populations.
#'
#' @importFrom foreach %do% %dopar%
#' @keywords internal
.downstreamRegulatedSignaling <- function(lr, pw, pw.size, rncounts, stats,
    id.col, gene.col, pw.col,
    min.positive, pos.targets = FALSE,
    neg.targets = FALSE, min.t.logFC = 0.5,
    with.complex = TRUE) {
    if (!is.data.frame(stats)) {
        stop("stats must be a data.frame")
    }
    if (!("pval" %in% names(stats))) {
        stop("stats must contain a column named 'pval'")
    }
    if (neg.targets && pos.targets) {
        stop("neg.targets and pos.targets cannot be TRUE simultaneously")
    }
    if (min.t.logFC <= 0) {
        stop("min.t.logFC must be >0")
    }

    # local binding
    r <- p <- pl <- id <- NULL

    # define interaction types
    control.int <- c(
        "positively-controls-expression-of",
        "negatively-controls-expression-of",
        "regulates-transcription-of"
    )

    incomplex.int <- c("in-complex-with", "interacts-with")

    directed.int <- c(
        "controls-transport-of",
        "catalysis-precedes",
        "positively-controls-expression-of",
        "negatively-controls-expression-of",
        "regulates-transcription-of",
        "controls-dephospho-of",
        "controls-phospho-of",
        "regulates-dephospho-of",
        "regulates-phospho-of",
        "controls-state-change-of",
        "controls-state-of-by-metab"
    )

    if (with.complex) {
        correlated.int <- union(control.int, incomplex.int)
    } else {
        correlated.int <- control.int
    }

    # # compute downstream correlations
    # corrg <- stats::cor(t(rncounts), method = "spearman")
    #
    # # the global computation above is faster than restricted to the receptors
    # corrg <- corrg[unique(lr$R), ]

    # check each putative LR pair, loop over the receptors
    reg.proc <- foreach::foreach(r = unique(lr$R), .combine = rbind) %do% {
        # reg.proc <- NULL
        # for (r in unique(lr$putative.pairs$R)){

        # loop over the pathways containing the receptor r
        pa <- intersect(pw[pw[[gene.col]] == r, id.col], names(pw.size))
        if (length(pa) > 0) {
            receptor.ligands <- unique(lr$L[lr$R == r])
            best.2nd <- foreach::foreach(p = pa, .combine = rbind) %do% {
                # best.2nd <- NULL
                # for (p in pa){
                int <- .SignalR$BulkSignalR_Network[
                    .SignalR$BulkSignalR_Network$a.gn %in% 
                    pw[pw[[id.col]] == p, gene.col] &
                        .SignalR$BulkSignalR_Network$b.gn %in% 
                        pw[pw[[id.col]] == p, gene.col],
                ]

                # extract the target genes of receptor r
                if (r %in% int$a.gn || r %in% int$b.gn) {
                    # double the undirected interactions 
                    # and generate a directed graph
                    directed <- int$type %in% directed.int
                    ret <- int[!directed, c("a.gn", "b.gn")]
                    from <- ret$a.gn
                    ret$a.gn <- ret$b.gn
                    ret$b.gn <- from
                    d.int <- unique(rbind(int[, c("a.gn", "b.gn")], ret))
                    g <- igraph::graph_from_data_frame(d.int, directed = TRUE)

                    # putative targets in the pathway
                    target.genes <- setdiff(
                        c(
                            int[int$type %in% correlated.int & 
                            int$a.gn == r, "b.gn"],
                            int[int$type %in% correlated.int &
                            int$b.gn == r, "a.gn"],
                            int[int$type %in% directed.int, "b.gn"]
                        ),
                        r
                    )

                    # reduce putative to reachable from the receptor
                    sp <- igraph::distances(g, r, target.genes)
                    target.genes <- colnames(sp)[!is.infinite(sp[r, ])]

                    # eliminate ligands of the receptor if present
                    # and intersect with detected genes in case use.full.network
                    # parameter was set to TRUE in 
                    # .checkRegulatedReceptorSignaling
                    target.genes <- intersect(
                        rownames(stats),
                        setdiff(target.genes, receptor.ligands)
                    )

                    # check fold-change direction if required
                    if (pos.targets || neg.targets) {
                        lfc <- stats[target.genes, "logFC"]
                        if (pos.targets) {
                            target.genes <- target.genes[lfc >= min.t.logFC]
                        } else {
                            target.genes <- target.genes[lfc <= -min.t.logFC]
                        }
                    }

                    if (length(target.genes) >= min.positive) {
                        # if all conditions are met, list all target genes with
                        # their regulation P-values in a data frame
                        # row. Target genes are sorted wrt P-values
                        # in decreasing order to
                        # keep the compatibility with correlation analysis,
                        # where the most significant values are at the end.
                        pv <- stats[target.genes, "pval"]
                        o <- order(pv, decreasing = TRUE)
                        pv <- pv[o]
                        lfc <- stats[target.genes, "logFC"]
                        lfc <- lfc[o]
                        expr <- stats[target.genes, "expr"]
                        expr <- expr[o]
                        target.genes <- target.genes[o]
                        c <- rep(1, length(target.genes))
                        data.frame(
                            pathway = p, 
                            target.pval = paste(pv, collapse = ";"),
                            target.genes = paste(target.genes, collapse = ";"),
                            target.corr = paste(c, collapse = ";"),
                            target.logFC = paste(lfc, collapse = ";"),
                            target.expr = paste(expr, collapse = ";"),
                            len = length(c), stringsAsFactors = FALSE
                        )
                    } else {
                        NULL
                    }
                } else {
                    NULL
                }
            }
            if (!is.null(best.2nd)) {
                # one or several pathways containing the receptor r were found,
                # combine them in |-separated strings
                data.frame(
                    R = r, pathways = paste(best.2nd$pathway, collapse = "|"),
                    target.pval = paste(best.2nd$target.pval, collapse = "|"),
                    target.genes = paste(best.2nd$target.genes, collapse = "|"),
                    target.corr = paste(best.2nd$target.corr, collapse = "|"),
                    target.logFC = paste(best.2nd$target.logFC, collapse = "|"),
                    target.expr = paste(best.2nd$target.expr, collapse = "|"),
                    len = paste(best.2nd$len, collapse = "|"),
                    stringsAsFactors = FALSE
                )
            } else {
                NULL
            }
        } else {
            NULL
        }
    }

    # combine LR pair correlations with R-target gene correlations
    rownames(reg.proc) <- reg.proc$R
    conf.pairs <- lr[lr$R %in% reg.proc$R, ]
    conf.pairs$pwid <- reg.proc[conf.pairs$R, "pathways"]
    conf.pairs$target.pval <- reg.proc[conf.pairs$R, "target.pval"]
    conf.pairs$len <- reg.proc[conf.pairs$R, "len"]
    conf.pairs$target.genes <- reg.proc[conf.pairs$R, "target.genes"]
    conf.pairs$target.corr <- reg.proc[conf.pairs$R, "target.corr"]
    conf.pairs$target.logFC <- reg.proc[conf.pairs$R, "target.logFC"]
    conf.pairs$target.expr <- reg.proc[conf.pairs$R, "target.expr"]
    pw.name <- unique(pw[, c(id.col, pw.col)])
    pw2name <- stats::setNames(pw.name[[2]], pw.name[[1]])
    conf.pairs$pwname <- foreach::foreach(pl = conf.pairs$pwid,
        .combine = c) %do% {
        paste(
            foreach::foreach(
                id = unlist(strsplit(pl, "\\|")),
                .combine = c
            ) %do% {
                pw2name[id]
            },
            collapse = "|"
        )
    }

    conf.pairs[, c(
        "L", "R", "LR.pval", "corr", "L.logFC", "R.logFC",
        "pwid", "pwname", "len", "target.genes",
        "target.pval", "target.logFC", "target.corr",
        "target.expr"
    )]
} # .downstreamRegulatedSignaling


#' Internal function to check receptor signaling downstream
#'
#' Assess the existence of concomitant regulations between a receptor,
#' part of a ligand-receptor pair, and
#' genes coding for proteins forming a complex with the receptor or genes
#' regulated by the receptor downstream signaling.
#'
#' @param ds              A BSRDataModel object
#' @param cc              A BSRClusterComp object.
#' @param lr              A table as returned by \code{.getRegulatedLR()}.
#' @param ds              An optional BSRDataModel object.
#' @param reference       Which pathway reference should be used ("REACTOME"
#'   for Reactome, "GOBP" for GO Biological Process,
#'   or "REACTOME-GOBP" for both).
#' @param max.pw.size     Maximum pathway size to consider from the pathway
#'   reference.
#' @param min.pw.size     Minimum pathway size to consider from the pathway
#'   reference.
#' @param min.positive    Minimum number of target genes to be found in a given
#'   pathway.
#' @param restrict.pw     A list of pathway IDs to restrict the application of
#'   the function.
#' @param with.complex    A logical indicating whether receptor co-complex
#'   members should be included in the target genes.
#' @param min.t.logFC     The minimum log2 fold-change allowed for
#'   targets in case pos.targets or neg.targets are used.
#' @param pos.targets   A logical imposing that all the network targets must
#'   display positive logFC, i.e. logFC >= min.t.logFC.
#' @param neg.targets   A logical imposing that all the network targets must
#'   display negative logFC, i.e. logFC <= - min.t.logFC.
#' @param use.full.network  A logical to avoid limiting the reference network
#'   to the detected genes and use the whole reference network.
#' @return A data frame extending \code{lr} content with the pathways found to
#' contain the receptors and data about receptor target gene regulations
#' Strings in semi-colon-separated format are used to report
#' target genes and their regulation P-values in the
#' data frame. The target genes are sorted according to the P-values in
#' decreasing order.
#'
#' In case \code{ds} is set, then correlations between the receptor and
#' target genes will be computed for documentation or additional use. The
#' row names of \code{differentialStats(cc)} and \code{ncounts(ds)} must match
#' exactly (not necessarily in the same order). The same is true for
#' \code{differentialStats(scc)} in case \code{scc} is provided.
#'
#' In a pathway of the reference, i.e., a Reactome pathway or the genes of a
#' GOBP term, the target genes are the
#' genes coding for proteins forming a complex with the receptor and the
#' genes in the pathway downstream the receptor,
#' which are given as regulated by the pathway. If \code{with.complex} is
#' set to \code{FALSE}, then only the
#' regulated genes are considered. Participation to a complex and being
#' regulated as well as the pathway directed topologies
#' are defined by Reactome and KEGG pathways as provided by PathwayCommons.
#'
#' The maximum pathway size is used to limit the redundancy inherent to GOBP
#' and Reactome. The minimum pathway size is
#' used to avoid overspecific, noninformative results.
#'
#' @importFrom methods is
#' @keywords internal
.checkRegulatedReceptorSignaling <- function(ds, cc, lr,
    reference = c("REACTOME-GOBP", "REACTOME", "GOBP"),
    pos.targets = FALSE, neg.targets = FALSE, min.t.logFC = 0.5,
    use.full.network = FALSE,
    max.pw.size = 400, min.pw.size = 5,
    min.positive = 2, restrict.pw = NULL,
    with.complex = TRUE) {
    
    if (!is(cc, "BSRClusterComp")) {
        stop("cc must be a BSRClusterComp object")
    }
    if (!is(ds, "BSRDataModelComp")) {
        stop("ds must be a BSRDataModelComp object")
    }
    if (!all(rownames(differentialStats(cc)) %in% rownames(ncounts(ds)))) {
        stop("The row names of differentialStats(cc)",
        " and ncounts(ds) must match exactly")
    }
    if (neg.targets && pos.targets) {
        stop("neg.targets and pos.targets cannot be TRUE simultaneously")
    }
    if (min.t.logFC <= 0) {
        stop("min.t.logFC must be >0")
    }

    reference <- match.arg(reference)
    results <- list()

    # Reactome pathways
    if (reference %in% c("REACTOME-GOBP", "REACTOME")) {
        pw.size <- table(.SignalR$BulkSignalR_Reactome$`Reactome ID`)
        pw.size <- pw.size[pw.size >= min.pw.size & pw.size <= max.pw.size]
        if (use.full.network) {
            react <- .SignalR$BulkSignalR_Reactome
        } else {
            react <- .SignalR$BulkSignalR_Reactome[
            .SignalR$BulkSignalR_Reactome$`Gene name` %in% 
            rownames(differentialStats(cc)), ]
        }
        if (!is.null(restrict.pw)) {
            react <- react[react$`Reactome ID` %in% restrict.pw, ]
        }
        contains.receptors <- react[react$`Gene name` %in% lr$R, "Reactome ID"]
        pw.size <- pw.size[names(pw.size) %in% contains.receptors]
        corgenes <- intersect(
            rownames(differentialStats(cc)),
            c(lr$R, react[react$`Reactome ID` %in% names(pw.size), "Gene name"])
        )
        results$reactome.pairs <- .downstreamRegulatedSignaling(lr,
            react, pw.size,
            ncounts(ds)[corgenes, c(colClusterA(cc), colClusterB(cc))], 
            differentialStats(cc)[corgenes, ],
            id.col = "Reactome ID", gene.col = "Gene name",
            pw.col = "Reactome name", min.positive = min.positive,
            pos.targets = pos.targets, neg.targets = neg.targets,
            min.t.logFC = min.t.logFC,
            with.complex = with.complex
        )
    }

    # GOBP
    if (reference %in% c("REACTOME-GOBP", "GOBP")) {
        pw.size <- table(.SignalR$BulkSignalR_Gobp$`GO ID`)
        pw.size <- pw.size[pw.size >= min.pw.size & pw.size <= max.pw.size]
        if (use.full.network) {
            go <- .SignalR$BulkSignalR_Gobp
        } else {
            go <- .SignalR$BulkSignalR_Gobp[
            .SignalR$BulkSignalR_Gobp$`Gene name` %in% 
            rownames(differentialStats(cc)), ]
        }
        if (!is.null(restrict.pw)) {
            go <- go[go$`GO ID` %in% restrict.pw, ]
        }
        contains.receptors <- go[go$`Gene name` %in% lr$R, "GO ID"]
        pw.size <- pw.size[names(pw.size) %in% contains.receptors]
        corgenes <- intersect(
            rownames(differentialStats(cc)),
            c(lr$R, go[go$`GO ID` %in% names(pw.size), "Gene name"])
        )
        results$gobp.pairs <- .downstreamRegulatedSignaling(lr, go, pw.size,
            ncounts(ds)[corgenes, c(colClusterA(cc), colClusterB(cc))],
            differentialStats(cc)[corgenes, ],
            id.col = "GO ID", gene.col = "Gene name", pw.col = "GO name",
            pos.targets = pos.targets, neg.targets = neg.targets,
            min.t.logFC = min.t.logFC,
            min.positive = min.positive, with.complex = with.complex
        )
    }

    # merge
    if (reference == "REACTOME-GOBP") {
        pairs <- unique(rbind(
            results$reactome.pairs[, c(1, 2)],
            results$gobp.pairs[, c(1, 2)]
        ))
        react.keys <- paste(results$reactome.pairs[[1]],
            results$reactome.pairs[[2]],
            sep = "|"
        )
        gobp.keys <- paste(results$gobp.pairs[[1]],
            results$gobp.pairs[[2]],
            sep = "|"
        )
        results$merged.pairs <- rbind(
            results$reactome.pairs,
            results$gobp.pairs[!(gobp.keys %in% react.keys), ]
        )
    } else if (reference == "REACTOME") {
        results$merged.pairs <- results$reactome.pairs
    } else {
        results$merged.pairs <- results$gobp.pairs
    }

    results$merged.pairs
} # .checkRegulatedReceptorSignaling


#' Internal function to assign P-values to LR interactions
#'
#' Estimate the P-value of each ligand-receptor pair based
#' on the data frame output by \code{\link{.checkRegulatedReceptorSignaling}}.
#'
#' @param pairs         A data frame output by
#'   \code{checkRegulatedReceptorSignaling}.
#' @param param         A list containing the statistical model parameters.
#' @param rank.p        A number between 0 and 1 defining the rank of the last
#'   considered target genes.
#' @param fdr.proc      The procedure for adjusting P-values according to
#'   \code{\link[multtest]{mt.rawp2adjp}}.
#'
#' @return A data.frame with the data in \code{pairs} complemented with
#' P-values and adjusted P-values.
#' @keywords internal
.pValuesRegulatedLR <- function(pairs, param, rank.p = 0.75,
                                fdr.proc = c(
                                    "BH", "Bonferroni", "Holm", "Hochberg",
                                    "SidakSS", "SidakSD", "BY", "ABH", "TSBH"
                                )) {
    if (rank.p < 0 || rank.p > 1) {
        stop("rank.p must lie in [0;1]")
    }
    fdr.proc <- match.arg(fdr.proc)
    if (is.null(pairs)) {
        stop("Dataframe `pairs` from `.checkRegulatedReceptorSignaling`",
            " is NULL.")
    }

    # estimate P-values
    res <- NULL
    for (i in seq_len(nrow(pairs))) {
        # all the data related to each pathway containing a given
        # receptor were collapsed separated by |
        # we need to split those pathways
        pwid <- unlist(strsplit(pairs$pwid[i], split = "\\|"))
        pwname <- unlist(strsplit(pairs$pwname[i], split = "\\|"))
        tg <- unlist(strsplit(pairs$target.genes[i], split = "\\|"))
        spval <- unlist(strsplit(pairs$target.pval[i], split = "\\|"))
        slfc <- unlist(strsplit(pairs$target.logFC[i], split = "\\|"))
        spear <- unlist(strsplit(pairs$target.corr[i], split = "\\|"))
        sexpr <- unlist(strsplit(pairs$target.expr[i], split = "\\|"))
        len <- as.numeric(unlist(strsplit(pairs$len[i], split = "\\|")))

        # get the LR correlation P-value
        p.lr <- pairs$LR.pval[i]

        # estimate the target gene correlation P-value based on rank statistics
        # for the individual correlation Gaussian model
        for (k in seq_len(length(len))) {
            spvals <- as.numeric(strsplit(spval[k], split = ";")[[1]])
            spears <- as.numeric(strsplit(spear[k], split = ";")[[1]])
            r <- min(max(1, trunc(rank.p * len[k])), len[k])
            rank.pval <- spvals[r]
            rank.corr <- spears[r]
            # r-1 P-values are > rank.pval, prob to have r-1 or less
            # P-values > rank.pval is given by a binomial with success rate
            # equal to the probability to get a P-value > rank.pval, i.e.,
            # 1-rank.pval. If rank.pval is low (i.e., highly significant),
            # it becomes difficult to get as little 
            # as r-1 P-values > rank.pval by chance!
            p.rt <- stats::pbinom(r - 1, len[k], 1 - rank.pval) 
            res <- rbind(res, data.frame(pairs[i, c("L", "R", "LR.pval",
                "corr", "L.logFC", "R.logFC")],
                pw.id = pwid[k], pw.name = pwname[k], rank = r,
                len = len[k], rank.pval = rank.pval,
                rank.corr = rank.corr,
                target.genes = tg[k], target.pval = spval[k],
                target.logFC = slfc[k], target.corr = spear[k],
                target.expr = sexpr[k], pval = p.lr * p.rt,
                stringsAsFactors = FALSE
            ))
        }
    }
    names(res)[4] <- "LR.corr"

    # avoid the impossible
    key <- paste(res$L, res$R, res$pw.id, sep = "||")
    bad <- duplicated(key)
    res <- res[!bad, ]

    # multiple hypothesis correction
    rawp <- res$pval
    adj <- multtest::mt.rawp2adjp(rawp, fdr.proc)
    res$qval <- adj$adjp[order(adj$index), fdr.proc]

    res
} # .pValuesRegulatedLR

#' BulkSignalR ligand-receptor signature object for cluster comparisons
#'
#' S4 class to represent gene signatures associated with ligand-receptor
#' interactions that were inferred from the comparison of two clusters
#' of samples. This class inherits from BSRSignature.
#'
#' @slot cmp.name   The name of the comparison.
#' @slot tg.pval    A list of target genes P-values.
#' @slot tg.logFC   A list of target genes logFC.
#' @slot tg.expr   A list of target genes expression
#'
#' @export
#' @examples
#' new("BSRSignatureComp")
#'
setClass("BSRSignatureComp",
    contains = c("BSRSignature"),
    slots = c(
        cmp.name = "character",
        tg.expr = "list",
        tg.pval = "list",
        tg.logFC = "list"
    ),
    prototype = list(
        cmp.name = "myComparison",
        tg.expr = list(c(1, 2, 3)),
        tg.pval = list(c(0.05, 0.1, 0.008)),
        tg.logFC = list(c(-1, 0, 2))
    )
)

setValidity(
    "BSRSignatureComp",
    function(object) {
        if (!is.character(object@cmp.name)) {
            return("cmp.name is not character")
        }
        if (length(object@cmp.name) == 0) {
            return("cmp.name must have a length > 0")
        }
        if (!is.list(object@tg.expr)) {
            return("tg.expr is not a list")
        }
        if (!is.list(object@tg.pval)) {
            return("tg.pval is not a list")
        }
        if (!is.list(object@tg.logFC)) {
            return("tg.logFC is not a list")
        }

        TRUE
    }
)

setMethod("show", "BSRSignatureComp", function(object) {
    callNextMethod()
    cat("Cluster comparison name:", object@cmp.name, "\n")
})

# Constructor ========================================================

# Obtain gene signatures from a BSRInference object

#' Extract gene signatures of LR pair activity
#'
#' Obtains gene signatures reflecting ligand-receptor as well as
#' receptor downstream activity to
#' score ligand-receptor pairs across samples subsequently with
#' \code{"\link[=BSRInferenceComp-class]{scoreLRGeneSignatures}"}
#'
#' @name BSRSignatureComp
#'
#' @param obj    BSRInferenceComp object.
#' @param pval.thres    P-value threshold.
#' @param qval.thres    Q-value threshold.
#' @param with.pw.id    A logical indicating whether the ID of a pathway
#' should be concatenated to its name.
#' @return A BSRSignatureComp object containing a gene signature for each triple
#' ligand-receptor pair. A reduction to the best pathway
#' for each pair is automatically performed and the gene signature is
#' comprised of the ligand, the receptor,
#' and all the target genes with rank equal or superior to \code{pairs$rank}.
#' @export
#' @examples
#' data(bsrinf.comp, package = "BulkSignalR")
#' 
#' bsrinf.redP <- reduceToPathway(bsrinf.comp)
#' bsrsig.redP <- BSRSignatureComp(bsrinf.redP, qval.thres = 0.001)
#' @importFrom foreach %do% %dopar%
#' @importFrom methods new
BSRSignatureComp <- function(obj,
    pval.thres = NULL, qval.thres = NULL, with.pw.id = FALSE) {
    if (is.null(pval.thres) && is.null(qval.thres)) {
        stop("Either a P- or a Q-value threshold must be provided")
    }

    # reduce and select
    obj <- reduceToBestPathway(obj)
    pairs <- LRinter(obj)
    if (!is.null(pval.thres)) {
        selected <- pairs$pval <= pval.thres
    } else {
        selected <- pairs$qval <= qval.thres
    }

    # obtain the signature object
    pairs <- pairs[selected, ]
    ligands <- ligands(obj)[selected]
    receptors <- receptors(obj)[selected]
    if (with.pw.id) {
        pathways <- paste0(pairs$pw.id, ": ", pairs$pw.name)
    } else {
        pathways <- pairs$pw.name
    }
    tg.genes <- tgGenes(obj)[selected]
    t.corrs <- tgCorr(obj)[selected]
    t.pvals <- tgPval(obj)[selected]
    t.logFCs <- tgLogFC(obj)[selected]
    t.exprs <- tgExpr(obj)[selected]

    for (i in seq_len(nrow(pairs))) {
        tg <- tg.genes[[i]]
        tg.genes[[i]] <- tg[pairs$rank[i]:length(tg)]
        tc <- t.corrs[[i]]
        t.corrs[[i]] <- tc[pairs$rank[i]:length(tc)]
        tp <- t.pvals[[i]]
        t.pvals[[i]] <- tp[pairs$rank[i]:length(tp)]
        tl <- t.logFCs[[i]]
        t.logFCs[[i]] <- tl[pairs$rank[i]:length(tl)]
        te <- t.exprs[[i]]
        t.exprs[[i]] <- te[pairs$rank[i]:length(te)]

    }

    new("BSRSignatureComp",
        ligands = ligands,
        receptors = receptors, 
        tg.genes = tg.genes, 
        tg.corr = t.corrs,
        pathways = pathways, 
        cmp.name = comparisonName(obj),
        tg.pval = t.pvals, 
        tg.logFC = t.logFCs,        
        tg.expr = t.exprs
    )
} # BSRSignatureComp

# Accessors & setters ========================================================

#' Target gene expression accessor
#'
#' @name tgExpr
#' @aliases tgExpr,BSRSignatureComp-method
#' @param x BSRSignatureComp object
#' @return tg.expr
#' @export
setMethod("tgExpr", "BSRSignatureComp", function(x) x@tg.expr)

#' Target gene P-values accessor
#'
#' @name tgPval
#' @aliases tgPval,BSRSignatureComp-method
#' @param x BSRSignatureComp object
#' @return tg.pval
#' @export
setMethod("tgPval", "BSRSignatureComp", function(x) x@tg.pval)


#' Target gene logFC accessor
#'
#' @name tgLogFC
#' @aliases tgLogFC,BSRSignatureComp-method
#' @param x BSRSignatureComp object
#' @return tg.logFC
#' @export
setMethod("tgLogFC", "BSRSignatureComp", function(x) x@tg.logFC)

#' Comparison name accessor
#'
#' @name comparisonName
#' @aliases comparisonName,BSRSignatureComp-method
#' @param x BSRSignatureComp object
#' @return cmp.name
#' 
#' @export
setMethod("comparisonName", "BSRSignatureComp", function(x) x@cmp.name)

#' BulkSignalR Inference Object
#'
#' An S4 class to represent inferred ligand-receptor interactions.
#'
#' @slot LRinter  A data frame describing the (ligand,receptor,pathway) triples
#' with P- and Q-values.
#' @slot ligands   A list of ligands, one entry per LR interaction.
#' @slot receptors   A list of receptors, one entry per LR interaction.
#' @slot tg.genes  A list of target genes, one entry per LR interaction.
#' @slot tg.corr  A list of target gene correlations to the receptor, one
#' entry per interaction
#' @slot inf.param  The parameters used for the inference.
#'
#' @details This class contains inferred LR interactions along with
#' their statistical significance. Data representation supports subsequent
#' reductions to pathways, etc. See reduction functions
#' \code{"\link[=BSRInference-class]{reduceToBestPathway}"},
#' \code{"\link[=BSRInference-class]{reduceToLigand}"},
#' \code{"\link[=BSRInference-class]{reduceToReceptor}"} and
#' \code{"\link[=BSRInference-class]{reduceToPathway}"}.
#' @export
#' @examples
#' new("BSRInference")
setClass("BSRInference",
    slots = c(
        LRinter = "data.frame",
        ligands = "list",
        receptors = "list",
        tg.genes = "list",
        tg.corr = "list",
        inf.param = "list"
    ),
    prototype = list(
        LRinter = data.frame(
            L = "A", R = "B", pw.id = "123", pw.name = "one pw",
            pval = 1.0, qval = 1.0, LR.corr = 0.6, rank = 2,
            len = 50, rank.corr = 0.6,
            stringsAsFactors = FALSE
        ),
        ligands = list("A"),
        receptors = list("B"),
        tg.genes = list(c("a", "b", "c")),
        tg.corr = list(c(-0.5, 0.1, 0.8)),
        inf.param = list()
    )
)


setValidity(
    "BSRInference",
    function(object) {
        if (!is.data.frame(object@LRinter)) {
            return("LRinter is not a data frame")
        }
        if (!is.list(object@ligands)) {
            return("ligands is not a list")
        }
        if (!is.list(object@receptors)) {
            return("receptors is not a list")
        }
        if (!is.list(object@tg.genes)) {
            return("tg.genes is not a list")
        }
        if (!is.list(object@tg.corr)) {
            return("tg.corr is not a list")
        }
        if (!is.list(object@inf.param)) {
            return("inf.param is not a list")
        }

        TRUE
    }
)


setMethod(
    "show", "BSRInference",
    function(object) {
        cat("Reference database: ", object@inf.param$reference, "\n", sep = "")
        print(utils::head(object@LRinter[
            order(object@LRinter$qval),
            c("L", "R", "pval", "qval", "pw.id", "pw.name"),
        ]))[5, ]
        cat("Inference parameters:\n")
        utils::str(object@inf.param)
    }
)


# Constructor ========================================================

#' Inference of ligand-receptor interactions
#'
#' Computes putative LR interactions along with their statistical confidence.
#' In this initial inference, all the relevant pathways are reported,
#' see reduction functions to reduce this list.
#'
#' @name BSRInference
#'
#' @param obj         A BSRDataModel output by \code{\link{BSRDataModel}} with
#' statistical model parameters trained by the
#' \code{"\link[=BSRDataModel-class]{learnParameters}"} method.
#' @param rank.p        A number between 0 and 1 defining the rank of the last
#' considered target genes.
#' @param min.cor         The minimum Spearman correlation required between
#' the ligand and the receptor.
#' @param fdr.proc      The procedure for adjusting P-values according to
#' \code{\link[multtest]{mt.rawp2adjp}}.
#' @param reference       Which pathway reference should be used ("REACTOME"
#'   for Reactome, "GOBP" for GO Biological Process,
#'   or "REACTOME-GOBP" for both).
#' @param max.pw.size     Maximum pathway size to consider from the pathway
#'   reference.
#' @param min.pw.size     Minimum pathway size to consider from the pathway
#'   reference.
#' @param min.positive    Minimum number of target genes to be found in a given
#'   pathway.
#' @param with.complex    A logical indicating whether receptor co-complex
#'   members should be included in the target genes.
#' @param restrict.pw     A list of pathway IDs to restrict the application of
#'   the function.
#' @param restrict.genes  A list of gene symbols that restricts ligands and
#'   receptors.
#' @param use.full.network  A logical to avoid limiting the reference network
#'   to the detected genes and use the whole reference network.
#'
#' @details Perform the initial ligand-receptor inference. Initial means that
#' no reduction is applied. All the (ligand, receptor, downstream pathway)
#' triples are reported, i.e., a given LR pair may appear multiple times
#' with different pathways downstream the receptor. Specific reduction
#' functions are available from the package to operate subsequent
#' simplifications based on the BSRInference object created by the initial
#' inference.
#'
#' Parameters defining minimum/maximum pathway sizes, etc. are set to NULL
#' by default, meaning that their values will be taken from what was set
#' during the training of the statistical model with
#' \code{"\link[=BSRDataModel-class]{learnParameters}"}
#'
#' To use different
#' values at the time of inference sounds like a bad idea, although this
#' could be used to explore without retraining the underlying model.
#' Retraining of the model with adjusted parameters is advised following
#' such an exploration.
#'
#' @return A BSRInference object with initial inferences set.
#'
#' @export
#'
#' @examples
#' data(bsrdm, package = "BulkSignalR")
#' data(immune.signatures, package = "BulkSignalR")
#' 
#' # We use a subset of the reference to speed up
#' # inference in the context of the example.
#' 
#' reactSubset <- getResource(resourceName = "Reactome",
#' cache = FALSE)
#' 
#' subset <- c("REACTOME_BASIGIN_INTERACTIONS",
#' "REACTOME_SYNDECAN_INTERACTIONS",
#' "REACTOME_ECM_PROTEOGLYCANS",
#' "REACTOME_CELL_JUNCTION_ORGANIZATION")
#' 
#' reactSubset <- reactSubset[
#' reactSubset$`Reactome name` %in% subset,]
#' 
#' resetPathways(dataframe = reactSubset,
#' resourceName = "Reactome")
#' 
#' bsrinf <- BSRInference(bsrdm,
#'     min.cor = 0.2,restrict.genes=immune.signatures$gene,
#'     reference="REACTOME")
#' @importFrom methods new
BSRInference <- function(obj, rank.p = 0.55,
    min.cor = 0.25, restrict.genes = NULL,
    reference = c("REACTOME-GOBP", "REACTOME", "GOBP"),
    max.pw.size = NULL, min.pw.size = NULL, min.positive = NULL,
    use.full.network = FALSE, restrict.pw = NULL,
    with.complex = NULL, fdr.proc = c(
        "BH", "Bonferroni", "Holm", "Hochberg",
        "SidakSS", "SidakSD", "BY", "ABH", "TSBH")) {

    if (is.null(max.pw.size)) {
        max.pw.size <- parameters(obj)$max.pw.size
    }
    if (is.null(min.pw.size)) {
        min.pw.size <- parameters(obj)$min.pw.size
    }
    if (is.null(min.positive)) {
        min.positive <- parameters(obj)$min.positive
    }
    if (is.null(with.complex)) {
        with.complex <- parameters(obj)$with.complex
    }
    if (is.null(use.full.network)) {
        use.full.network <- parameters(obj)$use.full.network
    }
    reference <- match.arg(reference)
    fdr.proc <- match.arg(fdr.proc)
    if (rank.p < 0 || rank.p > 1) {
        stop("rank.p must lie in [0;1]")
    }

    inf.param <- list()
    inf.param$min.corr <- min.cor
    inf.param$restrict.genes <- restrict.genes
    lr <- .getCorrelatedLR(obj, min.cor = min.cor, 
        restrict.genes = restrict.genes)

    inf.param$reference <- reference
    inf.param$min.pw.size <- min.pw.size
    inf.param$max.pw.size <- max.pw.size
    inf.param$with.complex <- with.complex
    inf.param$min.positive <- min.positive
    inf.param$use.full.network <- use.full.network
    inf.param$restrict.pw <- restrict.pw
    pairs <- .checkReceptorSignaling(obj, lr,
        reference = reference,
        min.pw.size = min.pw.size, max.pw.size = max.pw.size,
        min.positive = min.positive, with.complex = with.complex,
        use.full.network = use.full.network, restrict.pw = restrict.pw
    )

    inf.param$fdr.proc <- fdr.proc
    inf.param$rank.p <- rank.p
    inter <- .pValuesLR(pairs, 
        parameters(obj), 
        rank.p = rank.p, 
        fdr.proc = fdr.proc)

    ligands <- strsplit(inter$L, ";")
    receptors <- strsplit(inter$R, ";")
    tg <- strsplit(inter$target.genes, ";")
    tgcorr <- lapply(
        strsplit(inter$target.corr, ";"),
        function(x) as.numeric(x)
    )
    inf.param$ligand.reduced <- FALSE
    inf.param$receptor.reduced <- FALSE
    inf.param$pathway.reduced <- FALSE

    new("BSRInference",
        LRinter = inter[, c(
            "L", "R", "pw.id", "pw.name", "pval", "qval",
            "LR.corr", "rank", "len", "rank.corr"
        )], ligands = ligands,
        receptors = receptors, tg.genes = tg, tg.corr = tgcorr,
        inf.param = inf.param
    )
    
} # BSRInference


# Accessors & setters ========================================================

setGeneric("LRinter", signature="x",
    function(x) standardGeneric("LRinter")
)
#' LRinter accessor
#'
#' @name LRinter
#' @aliases LRinter,BSRInference-method
#' @param x BSRInference object
#' @return LRinter
#' @examples
#' bsrinf <- new ("BSRInference")
#' LRinter(bsrinf)
#' @export
setMethod("LRinter", "BSRInference", function(x) x@LRinter)


setGeneric("LRinter<-", signature=c("x", "value"),
    function(x, value) standardGeneric("LRinter<-")
)
#' LRinter setter (internal use only)
#'
#' @param x BSRInference object
#' @param value value to be set to BSRInference
#' @return returns \code{NULL}
#' @keywords internal
setMethod("LRinter<-", "BSRInference", function(x, value) {
    x@LRinter <- value
    methods::validObject(x)
    x
})


setGeneric("ligands", signature="x",
    function(x) standardGeneric("ligands")
)
#' ligands accessor
#'
#' @name ligands
#' @aliases ligands,BSRInference-method
#' @param x BSRInference object
#' @return ligands
#' @export
setMethod("ligands", "BSRInference", function(x) x@ligands)


setGeneric("ligands<-", signature=c("x", "value"),
    function(x, value) standardGeneric("ligands<-")
)
#' ligands setter (internal use only)
#' @param x BRSInference object
#' @param value Value to be set for bsrinf
#' @return returns \code{NULL}
#' @keywords internal
setMethod("ligands<-", "BSRInference", function(x, value) {
    x@ligands <- value
    methods::validObject(x)
    x
})


setGeneric("receptors", signature="x",
    function(x) standardGeneric("receptors")
)
#' receptors accessor
#'
#' @name receptors
#' @aliases receptors,BSRInference-method
#' @param x BRSInference object
#' @return receptors
#' @export
setMethod("receptors", "BSRInference", function(x) x@receptors)


setGeneric("receptors<-", signature=c("x", "value"),
    function(x, value) standardGeneric("receptors<-")
)
#' receptors setter (internal use only)
#'
#' @param x BRSInference object
#' @param value value to be set for BRSInference
#' @return returns \code{NULL}
#' @keywords internal
setMethod("receptors<-", "BSRInference", function(x, value) {
    x@receptors <- value
    methods::validObject(x)
    x
})


setGeneric("tgGenes", signature="x",
    function(x) standardGeneric("tgGenes")
)
#' Target genes accessor
#'
#' @name tgGenes
#' @aliases tgGenes,BSRInference-method
#' @param x BSRInferance object
#' @return tgGenes
#' @export
setMethod("tgGenes", "BSRInference", function(x) x@tg.genes)


setGeneric("tgGenes<-", signature=c("x", "value"),
    function(x, value) standardGeneric("tgGenes<-")
)
#' Target genes setter (internal use only)
#' 
#' @param x BSRInference object
#' @param value value to be set BSRInference
#' @return returns \code{NULL}
#' @keywords internal
setMethod("tgGenes<-", "BSRInference", function(x, value) {
    x@tg.genes <- value
    methods::validObject(x)
    x
})


setGeneric("tgCorr", signature="x",
    function(x) standardGeneric("tgCorr")
)
#' Target gene correlations accessor
#'
#' @name tgCorr
#' @aliases tgCorr,BSRInference-method
#' @param x BSRInference object
#' @export
setMethod("tgCorr", "BSRInference", function(x) x@tg.corr)

setGeneric("tgCorr<-", signature=c("x", "value"),
    function(x, value) standardGeneric("tgCorr<-")
)
#' Target gene correlations setter (internal use only)
#' @param x BSRInference object
#' @param value value to be set for bsrinf
#' @return returns \code{NULL}
#' 
#' @keywords internal
setMethod("tgCorr<-", "BSRInference", function(x, value) {
    x@tg.corr <- value
    methods::validObject(x)
    x
})


setGeneric("inferenceParameters", signature="x",
    function(x) standardGeneric("inferenceParameters")
)
#' Inference parameters accessor
#'
#' @name inferenceParameters
#' @aliases inferenceParameters,BSRInference-method
#' @param x BRSInference object.
#' @return inf.param
#' @examples
#' bsrinf <- new ("BSRInference")
#' inferenceParameters(bsrinf)
#' @export
setMethod("inferenceParameters", "BSRInference", function(x) x@inf.param)


setGeneric("inferenceParameters<-", signature=c("x", "value"),
    function(x, value) standardGeneric("inferenceParameters<-")
)
#' Inference parameters setter (internal use only)
#' @param x BRSInference object.
#' @param value value to be set.
#' @return returns \code{NULL}
#' 
#' @keywords internal
setMethod("inferenceParameters<-", "BSRInference", function(x, value) {
    x@inf.param <- value
    methods::validObject(x)
    x
})


# simplified table view ========================================================

setGeneric("LRinterShort", signature="x",
    function(x) standardGeneric("LRinterShort")
)
#' Simplified LRinter accessor reporting the essential columns
#'
#' @name LRinterShort
#' @aliases LRinterShort,BSRInference-method
#' @param x BSRInference object
#' @return LRinterShort
#' @export
setMethod(
    "LRinterShort", "BSRInference",
    function(x) {
        x@LRinter[, c(
            "L", "R", "pw.id", "pw.name",
            "qval", "LR.corr", "len"
        )]
    }
)


# Rescoring ====================================================================

setGeneric("rescoreInference", signature="obj",
    function(obj,...) standardGeneric("rescoreInference")
)
#' Inference re-scoring
#'
#' A method to re-score an existing BSRInference object
#' (P- and Q-value estimations).
#'
#' @name rescoreInference
#' @aliases rescoreInference,BSRInference-method
#'
#' @param obj BSRInferecence object.
#' @param param The matching BSRDataModel parameters
#'              (can be obtained with the accessor \code{param})
#' @param rank.p        A number between 0 and 1 defining the rank of the last
#'   considered target genes.
#' @param fdr.proc      The procedure for adjusting P-values according to
#'   \code{\link[multtest]{mt.rawp2adjp}}.
#'
#' @details A BSRInference object should be created by calling
#' \code{"\link[=BSRDataModel-class]{BSRInference}"}
#'
#' Parameters controlling the estimation
#' of the statistical significance of the ligand/receptor/pathway triples
#' (\code{param}) are provided at the time of calling the latter method.
#'
#' Nonetheless, it
#' might be useful to change the initially-provided parameters, in
#' which case this method should not be called.
#'
#' @return A BSRInference object.
#'
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#' data(bsrdm, package = "BulkSignalR")
#' 
#' bsrinf.new <- rescoreInference(bsrinf,
#'                       param = parameters(bsrdm)
#'               )
setMethod("rescoreInference", "BSRInference", function(obj, param, 
    rank.p = 0.55, fdr.proc = c("BH", "Bonferroni", "Holm",
        "Hochberg", "SidakSS", "SidakSD", "BY", "ABH", "TSBH"
        )) {

    if (rank.p < 0 || rank.p > 1) {
        stop("rank.p must lie in [0;1]")
    }
    fdr.proc <- match.arg(fdr.proc)

    # extract the necessary data from the BSRInference object
    pairs <- LRinter(obj)
    tg.genes <- tgGenes(obj)
    tg.corr <- tgCorr(obj)

    # prepare the chosen model CDF
    LR.par <- param$LR.0$model
    RT.par <- param$RT.0$model
    if (LR.par$distrib != RT.par$distrib) {
        stop("Distinct statistical models for LR and RT nulls are not allowed")
    }
    if (LR.par$distrib == "censored_normal") {
        cdf <- .cdfGaussian
    } else if (LR.par$distrib == "censored_mixed_normal") {
        cdf <- .cdfMixedGaussian
    } else if (LR.par$distrib == "empirical") {
        cdf <- .cdfEmpirical
    } else if (LR.par$distrib == "kernel_empirical") {
        cdf <- .cdfKernelEmpirical
    } else if (LR.par$distrib == "censored_stable") {
        cdf <- .cdfAlphaStable
    } else {
        stop("Unknown statistical model: ", LR.par$LR.0$model$distrib)
    }

    # recompute P-values
    for (i in seq_len(nrow(pairs))) {
        tg <- tg.genes[[i]]
        spears <- tg.corr[[i]]

        # estimate the LR correlation P-value
        if (pairs$LR.corr[i] >= 0) {
            # normal case
            p.lr <- 1 - cdf(pairs$LR.corr[i], LR.par)
        } else {
            # to enable searching for inhibitory L-R interactions
            p.lr <- cdf(pairs$LR.corr[i], LR.par)
        }

        # estimate the target gene correlation P-value based on rank statistics
        # for the individual correlation Gaussian model
        len <- pairs$len[i]
        r <- min(max(1, trunc(rank.p * len)), len)
        pairs$rank[i] <- r
        rank.corr <- spears[r]
        p.rt <- stats::pbinom(r - 1, len, cdf(rank.corr, RT.par))
        pairs$pval[i] <- p.lr * p.rt
        pairs$rank.corr[i] <- rank.corr
    }

    # recompute the Q-values
    rawp <- pairs$pval
    adj <- multtest::mt.rawp2adjp(rawp, fdr.proc)
    pairs$qval <- adj$adjp[order(adj$index), fdr.proc]

    # update the BSRInference object
    inf.param <- inferenceParameters(obj)
    inf.param$fdr.proc <- fdr.proc
    inf.param$rank.p <- rank.p
    inferenceParameters(obj) <- inf.param
    LRinter(obj) <- pairs

    obj
    
}) # rescoreInference


# Reduction and pathway stat methods ===========================================

setGeneric("getPathwayStats", signature="obj",
    function(obj,...) standardGeneric("getPathwayStats")
)
#' Basic statistics about hit pathways
#'
#' @name getPathwayStats
#' @aliases getPathwayStats,BSRInference-method
#'
#' @param obj    BSRinf object.
#' @param pval.thres    P-value threshold.
#' @param qval.thres    Q-value threshold.
#' @return A table with the pathways selected after the chosen threshold was
#'   applied to rows in \code{LRinter(obj)}.
#' Each pathway is reported along with various statistics:
#' the number of selected receptors in this pathway, the total number of
#' receptors described this pathway,
#' the number of selected ligand-receptor pairs hitting this pathway,
#' and the total number of
#' ligand-receptor pairs described that could hit this pathway.
#'
#' Obviously, one could imagine computing enrichment in receptors or
#' ligand-receptor pairs
#' based on such statistics, but the actual meaning of such an analysis
#' would be ambiguous since
#' the pathways were already selected as significantly regulated by the
#' receptor. We thus did not implement
#' this (hypergeometric test) computation.
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#'
#' pw.stat <- getPathwayStats(bsrinf)
#' 
#'
#' @importFrom foreach %do% %dopar%
setMethod("getPathwayStats", "BSRInference", function(obj,
    pval.thres = NULL, qval.thres = NULL) {
    if (inferenceParameters(obj)$ligand.reduced || 
        inferenceParameters(obj)$receptor.reduced) {
        stop(
            "Cannot be applied to interactions involving",
            " reduced receptors or ligands"
        )
    }

    # local binding
    id <- NULL

    pairs <- LRinter(obj)
    if (!is.null(pval.thres)) {
        pairs <- pairs[pairs$pval <= pval.thres, ]
    } else {
        pairs <- pairs[pairs$qval <= qval.thres, ]
    }

    # pairs reduced to the receptor & pathway names
    pairs.R <- unique(pairs[, c("R", "pw.id", "pw.name")])
    upw <- unique(pairs[, c("pw.id", "pw.name")])
    pw.names <- stats::setNames(upw$pw.name, upw$pw.id)

    # number of "hits" on a pathway with or without the ligand combinatorics
    pw.ids <- table(pairs$pw.id)
    pw.ids.R <- table(pairs.R$pw.id)

    # number of ligands for each receptor
    R.n.comb <- table(.SignalR$BulkSignalR_LRdb$receptor)

    foreach::foreach(id = names(pw.ids), .combine = rbind) %do% {
        # number of receptors that are in the current pathway,
        # depending on whether it is a GOBP or Reactome pathway
        if (regexpr("^R-", id) != -1) {
            Rs <- intersect(
                .SignalR$BulkSignalR_LRdb$receptor,
                .SignalR$BulkSignalR_Reactome[
                .SignalR$BulkSignalR_Reactome$`Reactome ID` == id, "Gene name"]
            )
        } else {
            Rs <- intersect(
                .SignalR$BulkSignalR_LRdb$receptor,
                .SignalR$BulkSignalR_Gobp[
                .SignalR$BulkSignalR_Gobp$`GO ID` == id, "Gene name"]
            )
        }

        # non-combinatorial version (ignore ligands)
        tot.R <- length(Rs)

        # ligand combinatorics included
        tot.LR <- sum(R.n.comb[Rs])

        data.frame(
            pw.id = id, pw.name = pw.names[id], n.R = pw.ids.R[id],
            tot.R = tot.R, n.LR = pw.ids[id], tot.LR = tot.LR,
            stringsAsFactors = FALSE
        )
    }
    
}) # getPathwayStats


setGeneric("reduceToBestPathway", signature="obj",
    function(obj,...) standardGeneric("reduceToBestPathway")
)
#' Keep one pathway per ligand-receptor pair
#'
#' @name reduceToBestPathway
#' @aliases reduceToBestPathway,BSRInference-method
#'
#' @param obj BSRInference object
#'
#' @return A BSRInference object reduced to only report one pathway per
#' ligand-receptor pair. The pathway with the
#' smallest P-value is selected.
#'
#' @details Ligand-receptor pairs
#' are evaluated in relation with pathways that allow checking receptor
#' downstream correlations. It is thus possible
#' that several pathways are reported for a same LR pair.
#'
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#' bsrinf.redBP <- reduceToBestPathway(bsrinf)
#'
#' @importFrom rlang .data
setMethod("reduceToBestPathway", "BSRInference", function(obj) {
    # Here we access the object slots directly as this procedure
    # is dependent of actual data representation

    # get best p-value pathway per LR pair
    ligands <- list()
    receptors <- list()
    tg.genes <- list()
    tg.corr <- list()
    LRinter <- NULL
    pairs <- obj@LRinter
    LR <- unique(pairs[, c("L", "R")])
    for (i in seq_len(nrow(LR))) {
        L <- LR$L[i]
        R <- LR$R[i]

        pwr <- pairs[pairs$L == L & pairs$R == R, ]
        k <- which.min(pwr$pval)
        j <- which(pairs$L == L & pairs$R == R & pairs$pw.id == pwr$pw.id[k])
        ligands <- c(ligands, obj@ligands[j])
        receptors <- c(receptors, obj@receptors[j])
        tg.genes <- c(tg.genes, obj@tg.genes[j])
        tg.corr <- c(tg.corr, obj@tg.corr[j])
        LRinter <- rbind(LRinter, pairs[j, ])
    }

    # update the object
    obj@LRinter <- LRinter
    obj@ligands <- ligands
    obj@receptors <- receptors
    obj@tg.genes <- tg.genes
    obj@tg.corr <- tg.corr
    obj@inf.param$pathway.reduced <- TRUE

    obj
    
}) # reduceToBestPathway


setGeneric("reduceToReceptor", signature="obj",
    function(obj,...) standardGeneric("reduceToReceptor")
)
#' Aggregate the ligands of a same receptor
#'
#' Simplifies a ligand-receptor table to focus on the receptors.
#'
#' @name reduceToReceptor
#' @aliases reduceToReceptor,BSRInference-method
#'
#' @return BSRInference object reduced to one row per receptor.
#' All the ligands are combined in a
#' semi-colon-separated list surrounded by curly brackets in the tabular
#' slot \code{LRinter}, and in vectors in the \code{ligands} (list) slot.
#'
#' The reported P-value and target genes are those from the line with the
#' pathway featuring the smallest P-value.
#' @param obj BRSInference object
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#' 
#' bsrinf.redR <- reduceToReceptor(bsrinf)
#'
#' @importFrom rlang .data
setMethod("reduceToReceptor", "BSRInference", function(obj) {
    # Here we access the object slots directly as this procedure
    # is dependent of actual data representation

    if (inferenceParameters(obj)$ligand.reduced) {
        stop("Already reduced to receptor")
    } # because ligands were reduced

    # pool the ligands
    ligands <- list()
    receptors <- list()
    tg.genes <- list()
    tg.corr <- list()
    LRinter <- NULL
    pairs <- obj@LRinter
    for (R in unique(pairs$R)) {
        lig <- pairs[pairs$R == R, ]
        k <- which.min(lig$pval)
        j <- which(pairs$R == lig$R[k] & pairs$pw.id == lig$pw.id[k])[1]
        ligands <- c(ligands, list(unique(lig$L)))
        receptors <- c(receptors, list(R))
        tg.genes <- c(tg.genes, obj@tg.genes[j])
        tg.corr <- c(tg.corr, obj@tg.corr[j])
        to.add <- pairs[j, ]
        to.add[1, "L"] <- paste0("{", paste(unique(lig$L), collapse = ";"), "}")
        LRinter <- rbind(LRinter, to.add)
    }

    # update the object
    obj@LRinter <- LRinter
    obj@ligands <- ligands
    obj@receptors <- receptors
    obj@tg.genes <- tg.genes
    obj@tg.corr <- tg.corr
    obj@inf.param$pathway.reduced <- TRUE
    obj@inf.param$ligand.reduced <- TRUE

    obj
    
}) # reduceToReceptor


setGeneric("reduceToLigand", signature="obj",
    function(obj,...) standardGeneric("reduceToLigand")
)
#' Aggregate the receptors of a same ligand
#'
#' Simplifies a ligand-receptor table to focus on the ligands.
#'
#' @name reduceToLigand
#' @aliases reduceToLigand,BSRInference-method
#'
#' @return A BSRInference object but reduced to one row per ligand.
#' All the receptors are combined in a
#' semi-colon-separated list surrounded by curly brackets in the tabular
#' slot \code{LRinter}, and in vectors in the \code{ligands} (list) slot.
#'
#' The reported P-value and target genes are those from the pathway with
#' the smallest P-value.
#' @param obj BSRInference object
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#' 
#' bsrinf.redL <- reduceToLigand(bsrinf)
#'
#' @importFrom rlang .data
setMethod("reduceToLigand", "BSRInference", function(obj) {
    # Here we access the object slots directly as this procedure
    # is dependent of actual representation

    if (inferenceParameters(obj)$receptor.reduced) {
        stop("Already reduced to ligand")
    } # because receptors were reduced

    # pool the receptors
    ligands <- list()
    receptors <- list()
    tg.genes <- list()
    tg.corr <- list()
    LRinter <- NULL
    pairs <- obj@LRinter
    for (L in unique(pairs$L)) {
        rec <- pairs[pairs$L == L, ]
        k <- which.min(rec$pval)
        j <- which(pairs$L == L & 
            pairs$R == rec$R[k] & 
            pairs$pw.id == rec$pw.id[k])
        ligands <- c(ligands, list(L))
        receptors <- c(receptors, list(unique(rec$R)))
        tg.genes <- c(tg.genes, obj@tg.genes[j])
        tg.corr <- c(tg.corr, obj@tg.corr[j])
        to.add <- pairs[j, ]
        to.add[1, "R"] <- paste0("{", paste(unique(rec$R), collapse = ";"), "}")
        LRinter <- rbind(LRinter, to.add)
    }

    # update the object
    obj@LRinter <- LRinter
    obj@ligands <- ligands
    obj@receptors <- receptors
    obj@tg.genes <- tg.genes
    obj@tg.corr <- tg.corr
    obj@inf.param$pathway.reduced <- TRUE
    obj@inf.param$receptor.reduced <- TRUE

    obj
    
}) # reduceToLigand


setGeneric("reduceToPathway", signature="obj",
    function(obj,...) standardGeneric("reduceToPathway")
)
#' Aggregate ligands and receptors at the pathway level
#'
#' Simplifies a ligand-receptor inference object to focus on
#' the pathways.
#'
#' @name reduceToPathway
#' @aliases reduceToPathway,BSRInference-method
#'
#' @return A BSRInference object reduced to only report one row per pathway.
#' The information of which ligand interacted with which receptor is lost as
#' all the ligands and all the receptors forming pairs related to a certain
#' pathway are combined.
#' For a given pathway, the reported P-values and target genes are those of
#' the best ligand-receptor pair that
#' was in this pathway.
#' Receptors and ligands are combined in two semi-colon-separated
#' lists surrounded by curly brackets in the tabular slot \code{LRinter},
#' while the list representation slots (\code{ligands} and
#' \code{receptors}) are update accordingly.
#'
#' @param obj BSRInference object
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#' 
#' bsrinf.redP <- reduceToPathway(bsrinf)
#' @importFrom rlang .data
setMethod("reduceToPathway", "BSRInference", function(obj) {
    # Here we access the object slots directly as this procedure
    # is dependent of actual representation

    if (inferenceParameters(obj)$receptor.reduced) {
        stop("Already reduced to ligand")
    } # because receptors were reduced
    if (inferenceParameters(obj)$ligand.reduced) {
        stop("Already reduced to receptor")
    } # because ligands were reduced

    # reduce to unique pathways
    ligands <- list()
    receptors <- list()
    tg.genes <- list()
    tg.corr <- list()
    LRinter <- NULL
    pairs <- obj@LRinter
    for (p in unique(pairs$pw.id)) {
        j <- which(pairs$pw.id == p)[1]
        ligands <- c(ligands, list(unique(pairs$L[pairs$pw.id == p])))
        receptors <- c(receptors, list(unique(pairs$R[pairs$pw.id == p])))
        tg.genes <- c(tg.genes, obj@tg.genes[j])
        tg.corr <- c(tg.corr, obj@tg.corr[j])
        to.add <- pairs[j, ]
        to.add[1, "L"] <- paste0("{", paste(unique(pairs$L[pairs$pw.id == p]),
            collapse = ";"
        ), "}")
        to.add[1, "R"] <- paste0("{", paste(unique(pairs$R[pairs$pw.id == p]),
            collapse = ";"
        ), "}")
        LRinter <- rbind(LRinter, to.add)
    }

    # update the object
    obj@LRinter <- LRinter
    obj@ligands <- ligands
    obj@receptors <- receptors
    obj@tg.genes <- tg.genes
    obj@tg.corr <- tg.corr
    obj@inf.param$ligand.reduced <- TRUE
    obj@inf.param$receptor.reduced <- TRUE

    obj
    
}) # reduceToPathway


# Reset gene names to initial organism =========================================

setGeneric("resetToInitialOrganism", signature="obj",
    function(obj,...) standardGeneric("resetToInitialOrganism")
)
#'  Reset gene names to initial organism provided in the first instance
#'
#' @name resetToInitialOrganism
#' @aliases resetToInitialOrganism,BSRInference-method
#'
#' @param obj  BSRInference object
#' @param conversion.dict   A dictionnary
#'
#' @return An BSRInference object updated for gene names.
#' The gene names are replaced by the ones from
#' the organism provided in the first instance.
#'
#' @export
#' @examples
#' data(bodyMap.mouse, package = "BulkSignalR")
#' data(bsrinf.mouse, package = "BulkSignalR")
#' data(ortholog.dict, package = "BulkSignalR")
#' 
#' #idx <- sample(nrow(bodyMap.mouse), 7500)
#' 
#' #bodyMap.mouse <- bodyMap.mouse[idx,1:3]
#' 
#' #ortholog.dict <- findOrthoGenes(
#' #    from_organism = "mmusculus",
#' #    from_values = rownames(bodyMap.mouse)
#' #)
#'
#' #matrix.expression.human <- convertToHuman(
#' #    counts = bodyMap.mouse,
#' #    dictionary = ortholog.dict
#' #)
#'
#' #bsrdm <- BSRDataModel(
#' #    counts = matrix.expression.human,
#' #    species = "mmusculus",
#' #    conversion.dict = ortholog.dict
#' #)
#' 
#' #bsrdm <- learnParameters(bsrdm,
#' #    quick = TRUE  
#' #)
#' 
#' #reactSubset <- getResource(resourceName = "Reactome",
#' #cache = TRUE)
#' 
#' #subset <- c("REACTOME_BASIGIN_INTERACTIONS",
#' #"REACTOME_SYNDECAN_INTERACTIONS",
#' #"REACTOME_ECM_PROTEOGLYCANS",
#' #"REACTOME_CELL_JUNCTION_ORGANIZATION")
#' 
#' #reactSubset <- reactSubset[
#' #reactSubset$`Reactome name` %in% subset,]
#' 
#' #bsrinf.mouse <- BSRInference(bsrdm,reference="REACTOME")
#' 
#' bsrinf <- resetToInitialOrganism(bsrinf.mouse, 
#' conversion.dict = ortholog.dict)
setMethod("resetToInitialOrganism", "BSRInference", function(obj,
    conversion.dict) {
    # Need to check conversion.dict format

    conversion.dict$human.gene.name <- rownames(conversion.dict)

    LRinter(obj)$L <- .geneNameConversion(LRinter(obj)$L, conversion.dict)
    LRinter(obj)$R <- .geneNameConversion(LRinter(obj)$R, conversion.dict)

    ligands(obj) <- .geneNameConversion(ligands(obj), conversion.dict)
    receptors(obj) <- .geneNameConversion(receptors(obj), conversion.dict)
    tgGenes(obj) <- .geneNameConversion(tgGenes(obj), conversion.dict)

    obj
    
}) # resetToInitialOrganism


#' @title Convert gene symbols to another organism
#'
#' @description Convert gene symbols to another organism
#' based on a dictionary with human and orthologs in the other species.
#'
#' @param genes The genes you want to convert.
#' @param conversion.dict A data frame containing
#' gene names for the source species and Homo sapiens.
#'
#' @return Depend on the type of input genes
#' LRinter return a vector of genes
#' tgGenes receptors ligands: return list of list of genes
#' @keywords internal
.geneNameConversion <- function(genes, conversion.dict) {
    # print(".geneNameConversion")
    if (typeof(genes) == "character") {
        genes.df <- data.frame(human.gene.name = genes)

        genes.df$id <- seq_len(nrow(genes.df))

        genes.converted <- merge(genes.df, conversion.dict,
            by.x = "human.gene.name", sort = FALSE, all = FALSE
        )
        genes.converted <- genes.converted[order(genes.converted$id), ]

        genes.converted$human.gene.name <- NULL
        genes.converted$id <- NULL

        as.vector(unlist(genes.converted))
    } else if (typeof(genes) == "list") {
        list <- list()

        for (i in seq_len(length(genes))) {
            genes.df <- data.frame(human.gene.name = genes[[i]])
            genes.df$id <- seq_len(nrow(genes.df))
            genes.converted <- merge(genes.df, conversion.dict,
                by.x = "human.gene.name",
                sort = FALSE, all = FALSE
            )
            genes.converted <- genes.converted[order(genes.converted$id), ]

            genes.converted$human.gene.name <- NULL
            genes.converted$id <- NULL

            list[[i]] <- as.vector(unlist(genes.converted))
            rm(genes.df)
            rm(genes.converted)
        }

        list
    } else {
        stop("Something went wrong during gene conversion.", call. = FALSE)
    }
  
} # .geneNameConversion

#' Assign cell types to L-R interactions
#'
#' Generate a data.frame linking interactions to cell types.
#'
#' @param bsrdm   A BSRDataModel object.
#' @param bsrinf  A BSRInference object.
#' @param ct.scores   A matrix of cell type signature scores.
#' @param normalize.scores  A logical indicating whether scores should be
#' normalized before assigning cell types.
#' @param min.weight  Minimum weight to keep in the linear model (cell types
#' with lower weights will be discarded) if \code{lasso==TRUE}. Otherwise,
#' minimum correlation coefficient of each individual cell type.
#' @param min.r2  Minimum r2 between a candidate cell type and a L-R gene
#' signature score.
#' @param min.r2.after  Minimum r2 between the proposed linear model and
#' a L-R gene signature score to retain the model.
#' @param lasso  Logical indicating that the LASSO (or linear regression if
#' only one cell type satisfies the \code{min.r2} criterion) should be
#' used. Otherwise, Spearman linear correlation is used.
#' @param qval.thres  Maximum Q-value of the L-R pairs to be considered.
#'
#' @return A data.frame containing the cell type assignments for each
#' L-R interaction. Unique interactions are considered only (thanks to
#' \code{"\link[=BSRInference-class]{reduceToBestPathway}"} that is applied
#' internally). An interaction can be associated with several cell types
#' or none. In case it is associated with a single cell type, it is labelled
#' autocrine (indicative only).
#'
#' Cell type signature scores must be provided. They can be computed
#' with BulkSignalR utility function \code{"\link{scoreSignatures}"}, but
#' also any other external tool such as CIBERSORT or BisqueRNA. In case
#' such a tool would score cell types in a nonlinear fashion, we
#' recommend to transform the score matrix to restore a linear relationship
#' cell type abundance/score. By default, cell type (and L-R gene
#' signature) scores are normalized between 0 and 1 to make the weights of each
#' cel type in the linear models as comparable as possible.
#' @export
#' @examples
#' data(bsrdm, package = "BulkSignalR")
#' data(bsrinf, package = "BulkSignalR")
#' data(immune.signatures, package = "BulkSignalR")
#' data(tme.signatures, package = "BulkSignalR")
#' 
#' immune.signatures <- immune.signatures[immune.signatures$signature %in%
#'     c("T cells"), ]
#' 
#' signatures <- rbind(immune.signatures, tme.signatures[
#'     tme.signatures$signature %in% c("Fibroblasts"),
#' ])
#' 
#' tme.scores <- scoreSignatures(bsrdm, signatures)
#'
#' # assignment
#' lr2ct <- assignCellTypesToInteractions(bsrdm, bsrinf, tme.scores)
#' @import glmnet
#' @importFrom foreach %do%
assignCellTypesToInteractions <- function(bsrdm, bsrinf, ct.scores,
    normalize.scores = TRUE, min.weight = 0.1,
    min.r2 = 0.25, min.r2.after = 0.35,
    lasso = TRUE, qval.thres = 1e-3) {
    # local binding
    i <- NULL

    # L-R interaction table and scores
    bsrinf.red <- reduceToBestPathway(bsrinf)
    inter <- LRinter(bsrinf.red)
    inter <- inter[inter$qval <= qval.thres, ]
    bsrsig <- BSRSignature(bsrinf.red, qval.thres = qval.thres)
    lr.scores <- scoreLRGeneSignatures(bsrdm, bsrsig)

    # check order
    inter.keys <- paste(inter[["L"]], inter[["R"]], sep = "-")
    score.keys <- gsub("\\{", "", rownames(lr.scores))
    score.keys <- gsub("\\}", "", score.keys)
    score.keys <- gsub(" / ", "-", score.keys)
    if (!all(inter.keys == score.keys)) {
        stop("LT table and scores not in the same order")
    }

    # CT and LR score matrices preparation
    x <- t(ct.scores)
    if (normalize.scores) {
        mins <- apply(x, 2, min)
        x <- sweep(x, 2, mins, "-")
        maxs <- apply(x, 2, max)
        x <- sweep(x, 2, maxs, "/")

        mins <- apply(lr.scores, 1, min)
        lr.scores <- sweep(lr.scores, 1, mins, "-")
        maxs <- apply(lr.scores, 1, max)
        lr.scores <- sweep(lr.scores, 1, maxs, "/")
    }

    # assignment of CTs
    a <- foreach::foreach(i = seq_len(nrow(inter)), .combine = rbind) %do% {
        y <- lr.scores[i, ]
        c <- try(stats::cor(y, x, method = "spearman"),silent = TRUE)
        if (inherits(c, "try-error")) {
            c <- NULL
        }
        if (is.null(c)){
            stop("Call to `stats::cor` failed")
        }

        c2 <- c**2
        good <- c2 >= min.r2 & c > 0
        if (sum(good) > 0) {
            if (lasso) {
                # actual LASSO since >1 CTs
                if (sum(good) > 1) {
                    xL <- x[, good]
                    lb <- rep(0, length(colnames(xL)))
                    ub <- rep(Inf, length(colnames(xL)))
                    cvfit <- glmnet::cv.glmnet(xL, y,
                        grouped = FALSE,
                        lower.limits = lb, upper.limits = ub
                    )
                    lasso.fit.l <- glmnet::glmnet(
                        x = xL, y = y, lambda = cvfit$lambda.min,
                        lower.limits = lb, upper.limits = ub
                    )
                    MAE <- as.vector(glmnet::assess.glmnet(lasso.fit.l,
                        newx = xL, newy = y)$mae)
                    p <- stats::predict(lasso.fit.l, newx = xL)

                    gc2 <- try(stats::cor(y, p),silent = TRUE)
                    if (inherits(gc2, "try-error")) {
                        gc2 <- NULL
                    }
                    if (is.null(gc2)){
                        stop("Call to `stats::cor` failed")
                    }
                    gc2 <- gc2**2

                    lcoef <- stats::coef(lasso.fit.l)[-1, 1]
                    if (gc2 > min.r2.after && sum(lcoef > 0) > 0) {
                        data.frame(
                            L = inter$L[i], R = inter$R[i], 
                            cell.type = names(lcoef)[lcoef > 0],
                            weight = lcoef[lcoef > 0], MAE = MAE, 
                            r2 = gc2[1, 1],
                            alg = "LASSO", stringsAsFactors = FALSE
                        )
                    } else {
                        NULL
                    }
                } else {
                    # linear model since 1 CT
                    dat <- data.frame(y = y, x = x[, good])
                    lmfit <- stats::lm(y ~ x, data = dat)
                    p <- stats::predict(lmfit)
                    
                    gc2 <- try(stats::cor(y, p),silent = TRUE)
                    if (inherits(gc2, "try-error")) {
                        gc2 <- NULL
                    }
                    if (is.null(gc2)){
                        stop("Call to `stats::cor` failed")
                    }
                    gc2 <- gc2**2

                    lcoef <- stats::coef(lmfit)[-1]
                    if (gc2 > min.r2.after && lcoef > 0) {
                        data.frame(
                            L = inter$L[i], R = inter$R[i], 
                            cell.type = colnames(x)[good],
                            weight = lcoef, MAE = mean(abs(p - y)), 
                            r2 = gc2, alg = "lm",
                            stringsAsFactors = FALSE
                        )
                    } else {
                        NULL
                    }
                }
            } else {
                # Spearman correlation only
                data.frame(
                    L = inter$L[i], R = inter$R[i], 
                    cell.type = colnames(good)[good], r2 = c2[good],
                    alg = "Spearman.cor", stringsAsFactors = FALSE
                )
            }
        } else {
            NULL
        }
    }

    a[a$weight >= min.weight, ]
    rownames(a) <- NULL
    a
} # assignCellTypesToInteractions


#' Build a table describing a cellular network
#'
#' Generate a data.frame including all the links between cell types
#' mediated by L-R interactions with their respective weights.
#'
#' @param lr  The data.frame output by
#' \code{"\link{assignCellTypesToInteractions}"}.
#' @param autocrine  A logical indicating whether autocrine interactions should
#' be included.
#'
#' @return A data.frame containing all the links in the cellular network. A
#' link is created between two cell types as soon as there was a L-R interaction
#' that was associated with both cell types. The link is given a score
#' equal to the geometric mean of each cell type assignment r2.
#' @export
#' @examples
#' data(bsrdm, package = "BulkSignalR")
#' data(bsrinf, package = "BulkSignalR")
#' data(immune.signatures, package = "BulkSignalR")
#' data(tme.signatures, package = "BulkSignalR")
#' 
#' immune.signatures <- immune.signatures[immune.signatures$signature %in%
#'     c("T cells"), ]
#' 
#' signatures <- rbind(immune.signatures, tme.signatures[
#'     tme.signatures$signature %in% c("Fibroblasts"),
#' ])
#' 
#' tme.scores <- scoreSignatures(bsrdm, signatures)
#'
#' # assignment
#' lr2ct <- assignCellTypesToInteractions(bsrdm, bsrinf, tme.scores)
#'
#' # cellular network
#' g.table <- cellularNetworkTable(lr2ct)
#' @importFrom foreach %do%
cellularNetworkTable <- function(lr, autocrine = FALSE) {
    # global binding
    key <- j <- NULL

    # paracrine
    keys <- paste(lr$L, lr$R, sep = "-")
    t <- table(keys)
    if (sum(t > 1) > 1) {
        nt <- foreach::foreach(key = names(t)[t > 1], 
            .combine = rbind) %do% {
            rows <- which(keys == key)
            L <- lr$L[rows[1]]
            R <- lr$R[rows[1]]
            r2 <- lr$r2[rows[1]]
            foreach::foreach(i = seq_len(length(rows) - 1), 
                .combine = rbind) %do% {
                row.1 <- rows[i]
                CT1 <- lr$cell.type[row.1]
                w.1 <- lr$weight[row.1]
                foreach::foreach(j = i + seq_len(length(rows) - i), 
                    .combine = rbind) %do% {
                    row.2 <- rows[j]
                    CT2 <- lr$cell.type[row.2]
                    w.2 <- lr$weight[row.2]
                    data.frame(
                        L = L, R = R, inter = key, CT1 = CT1, 
                        CT2 = CT2, r2 = r2, weight.1 = w.1,
                        weight.2 = w.2, score = sqrt(w.1 * w.2), 
                        type = "paracrine", stringsAsFactors = FALSE
                    )
                }
            }
        }
    } else {
        nt <- NULL
    }

    # paracrine
    if (autocrine) {
        if (sum(t == 1) > 1) {
            nt <- rbind(nt, foreach::foreach(key = names(t)[t == 1], 
                .combine = rbind) %do% {
                i <- which(keys == key)
                L <- lr$L[i]
                R <- lr$R[i]
                r2 <- lr$r2[i]
                CT1 <- lr$cell.type[i]
                w.1 <- lr$weight[i]
                data.frame(
                    L = L, R = R, inter = key, CT1 = CT1, 
                    CT2 = CT1, r2 = r2, weight.1 = w.1,
                    weight.2 = w.1, score = w.1, 
                    type = "autocrine", stringsAsFactors = FALSE
                )
            })
        }
    }

    nt
} # cellularNetworkTable


#' Build a cellular network
#'
#' Generate a igraph object including all the links between cell types.
#'
#' @param tab  The data.frame output by \code{"\link{cellularNetworkTable}"}.
#'
#' @return A igraph object containing all the links in the cellular network.
#' @export
#' @examples
#' data(bsrdm, package = "BulkSignalR")
#' data(bsrinf, package = "BulkSignalR")
#' data("tme.signatures", package = "BulkSignalR")
#' data(immune.signatures, package = "BulkSignalR")
#' 
#' immune.signatures <- immune.signatures[immune.signatures$signature %in%
#'     c("T cells"), ]
#' 
#' signatures <- rbind(immune.signatures, tme.signatures[
#'     tme.signatures$signature %in% c("Fibroblasts"),
#' ])
#' 
#' tme.scores <- scoreSignatures(bsrdm, signatures)
#'
#' # assignment
#' lr2ct <- assignCellTypesToInteractions(bsrdm, bsrinf, tme.scores)
#'
#' # cellular network
#' g.table <- cellularNetworkTable(lr2ct)
#' gCN <- cellularNetwork(g.table)
#' 
#' #plot(gCN, edge.width=5*E(gCN)$score)
#' @import igraph
cellularNetwork <- function(tab) {
    df <- tab[, c("CT1", "CT2", "inter", "L", "R", "r2", "score", "type")]
    igraph::graph_from_data_frame(df, directed = FALSE)
} # cellularNetwork


#' Build a summary cellular network
#'
#' Generate a igraph object with one link between each cell type.
#'
#' @param tab  The data.frame output by \code{"\link{cellularNetworkTable}"}.
#'
#' @return A igraph object containing a summary cellular network with
#' edge weights proportional to the sum of individual link scores. Edge
#' weight are normalized to a total of one.
#' @export
#' @examples
#' data(bsrdm, package = "BulkSignalR")
#' data(bsrinf, package = "BulkSignalR")
#' data("tme.signatures", package = "BulkSignalR")
#' data(immune.signatures, package = "BulkSignalR")
#' 
#' immune.signatures <- immune.signatures[immune.signatures$signature %in%
#'     c("T cells"), ]
#' 
#' signatures <- rbind(immune.signatures, tme.signatures[
#'     tme.signatures$signature %in% c("Fibroblasts"),
#' ])
#' 
#' tme.scores <- scoreSignatures(bsrdm, signatures)
#'
#' # assignment
#' lr2ct <- assignCellTypesToInteractions(bsrdm, bsrinf, tme.scores)
#' 
#' # cellular network
#' g.table <- cellularNetworkTable(lr2ct[c(1:25),])
#' gSummary <- summarizedCellularNetwork(g.table)
#' # plot(gSummary, edge.width=1+30*E(gSummary)$score)
#' @import igraph
#' @importFrom foreach %do%
summarizedCellularNetwork <- function(tab) {
    i <- NULL
    sscore <- sum(tab$score)
    ct <- unique(tab[, c("CT1", "CT2")])
    sum.tab <- foreach::foreach(i = seq_len(nrow(ct)), .combine = rbind) %do% {
        parsum <- sum(tab[tab$CT1 == ct[i, "CT1"] & 
            tab$CT2 == ct[i, "CT2"], "score"])
        data.frame(CT1 = ct[i, "CT1"], CT2 = ct[i, "CT2"],
            score = parsum / sscore)
    }
    igraph::graph_from_data_frame(sum.tab, directed = FALSE)
} # summarizedCellularNetwork


# Relate to a gene set ========================================================


#' Relate ligands to a gene set
#'
#' Finds ligands related to a gene set by following receptor, and
#' receptor downstream pathway targets.
#'
#' @param bsrinf  BSRInference object.
#' @param gs   The gene set.
#' @param min.cor  Minimum Spearman correlation between the receptor of a triple
#' (L,R,pw) and a gene of the gene set.
#' @param qval.thres  Maximum Q-value imposed to the (L,R,pw) triples to be
#' considered.
#' @return A data.frame listing all the (L,R,pathway) triples that lead
#' to at least one gene in the gene set. The number of genes found
#' by each triple is indicated in the column \code{n.genes}.
#'
#' @export
#' @examples
#' data(bsrdm, package = "BulkSignalR")
#' data(bsrinf, package = "BulkSignalR")
#' 
#' data(p.EMT, package = "BulkSignalR")
#' p.EMT <- p.EMT$gene
#
#' triggers <- relateToGeneSet(bsrinf, p.EMT) 
#' @importFrom foreach %do%
relateToGeneSet <- function(bsrinf, gs, min.cor = 0.25, qval.thres = 0.001) {
    i <- NULL
    # get target gens and L-R interactions
    inter <- LRinter(bsrinf)
    tg <- tgGenes(bsrinf)
    tcor <- tgCorr(bsrinf)

    # impose threshold on FDR
    good <- inter$qval <= qval.thres
    inter <- inter[good, ]
    tg <- tg[good]
    tcor <- tcor[good]

    # select L-R interactions that relate to the gene set
    foreach::foreach(i = seq_len(nrow(inter)), .combine = rbind) %do% {
        k <- (tg[[i]] %in% gs) & (tcor[[i]] >= min.cor)
        if (sum(k) > 0) {
            data.frame(
                L = inter$L[i], R = inter$R[i], pw.name = inter$pw.name[i],
                qval = inter$qval[i], n.genes = sum(k),
                genes = paste(tg[[i]][k], collapse = ";"),
                cor = paste(tcor[[i]][k], collapse = ";"),
                stringsAsFactors = FALSE
            )
        } else {
            NULL
        }
    }
} # relateToGeneSet



#' Cell type frequencies in relations to gene sets
#'
#' Count how many times and with which weights cell types were involved
#' in the (L,R,pathway) triples that targeted genes in a gene set.
#'
#' @param rel  The data.frame output by
#' \code{"\link[=BSRInference-class]{relateToGeneSet}"}.
#' @param lr   The data.frame output by
#' \code{"\link{assignCellTypesToInteractions}"}.
#' @param min.n.genes  Minimum number of genes in the gene set for
#' one (L,R,pathway) triple.
#'
#' @return A list of two slots: t for counting how many times each cell type
#' is involved; s for summing the weights of each involved cell type.
#' @export
#' @examples
#' data(bsrdm, package = "BulkSignalR")
#' data(bsrinf, package = "BulkSignalR")
#' data(immune.signatures, package = "BulkSignalR")
#' data(tme.signatures, package = "BulkSignalR")
#' data(p.EMT, package = "BulkSignalR")
#' 
#' immune.signatures <- immune.signatures[immune.signatures$signature %in%
#'     c("T cells"), ]
#' 
#' signatures <- rbind(immune.signatures, tme.signatures[
#'     tme.signatures$signature %in% c("Fibroblasts"),
#' ])
#' 
#' tme.scores <- scoreSignatures(bsrdm, signatures)
#'
#' # assignment
#' lr2ct <- assignCellTypesToInteractions(bsrdm, bsrinf, tme.scores)
#'
#' # relate to p-EMT (should be done in HNSCC normally, not in SDC)
#' p.EMT <- p.EMT$gene
#' triggers <- relateToGeneSet(bsrinf, p.EMT)
#'
#' # counts
#' cf <- cellTypeFrequency(triggers, lr2ct)
cellTypeFrequency <- function(rel, lr, min.n.genes = 1) {
    rel <- rel[rel$n.genes >= min.n.genes, ]
    rel.keys <- paste(rel$L, rel$R, sep = "-")
    lr.keys <- paste(lr$L, lr$R, sep = "-")
    already <- NULL
    CTs <- NULL
    weights <- NULL
    for (key in unique(rel.keys)) {
        rows <- which(lr.keys == key)
        if (length(rows) > 0) {
            for (j in rows) {
                CTs <- c(CTs, lr$cell.type[j])
                weights <- c(weights, lr$weight[j])
            }
        }
    }

    t <- table(CTs)
    s <- NULL
    for (ct in names(t)) {
        s <- c(s, sum(weights[CTs == ct]))
    }
    names(s) <- names(t)
    list(t = t, s = s)
} # cellTypeFrequency

####################################################
###     Generic Hidden Cache Functions           ###
####################################################

#' Add cache for resources.
#'
#' Add cache for resources (pathways, lrdb, or network)
#' downloaded from the web or local database.
#' This part is handled with BiocFileCache.
#'
#' @param fpath    Path to file on the web or local system.
#' @param resourceName   Ressource name.
#' @param cacheDir  Absolute path to cache directory.
#' @param verbose   Default FALSE
#' @param download   Logical(TRUE/FALSE) Default TRUE for download.
#' @return Returns `NULL`, invisibly. 
#' 
#' @import BiocFileCache
#' @import httr2 
#' @keywords internal
.cacheAdd <- function(fpath, cacheDir, resourceName,
    verbose = FALSE, download = TRUE) {
    if (!dir.exists(cacheDir)) {
        dir.create(cacheDir, recursive = TRUE)
    }

    dir <- basename(cacheDir)

    bfc <- BiocFileCache::BiocFileCache(cacheDir, ask = FALSE)

    # safeguard
    cacheHits <- BiocFileCache::bfcquery(bfc, 
        query = resourceName, field = "rname")
    if (nrow(cacheHits) >= 1) {
        cli::cli_alert_danger("Multiple cache results found for {.var {dir}}.")
        stop("Please clear your cache with `cacheClear()`!", "\n")
    }

    req <- httr2::request(fpath)
    ssl_opts <- list(ssl_verifypeer = 0L, ssl_verifyhost = 0L) 
    config <- httr2::req_options(req,!!!ssl_opts)
    
    # if fname="exact" remove the unique identifier
    BiocFileCache::bfcadd(bfc, rname = resourceName,
        config = config$options, fpath = fpath, download = download)
    
    cli::cli_alert_info("{.val {resourceName}} added to cache with success.")

    if (verbose) {
        cli::cli_alert("{.path {BiocFileCache::bfccache(bfc)}}")
        BiocFileCache::bfcinfo(bfc)
        message("")
    }

    return(invisible(NULL))
}


#' Check existence of a record in the cache.
#'
#' Check whether the cache record exists or not by passing
#' to the function an associated keyword
#' related to the resource we are looking for.
#'
#' @param bfc Object of class BiocFileCache, created by a call to
#' BiocFileCache::BiocFileCache()
#' @param resourceName keyword associated to a specific resource name.
#'
#' @keywords internal
#' @return logical This function returns TRUE if a record with
#' the requested keyword already exists in the file cache,
#' otherwise it returns FALSE.
.cacheCheckIn <- function(bfc, resourceName) {
    cacheHits <- BiocFileCache::bfcquery(bfc, 
        query = resourceName, field = "rname")
    as.logical(nrow(cacheHits))
}

####################################################
###     Generic Public Cache Functions           ###
####################################################

#' Delete cache content.
#'
#' Delete the content of cache directory.
#'
#' @param dir Directory to remove. Can be only 'resources'.
#' @return Returns `NULL`, invisibly. 
#' 
#' @importFrom BiocFileCache removebfc
#' @importFrom cli cli_alert_danger cli_alert
#' @export
#' @examples
#' cacheClear(dir="resources")
#' # need to recreate database in order to run examples well
#' createResources(verbose=TRUE)
cacheClear <- function(dir = c("resources")) {
    dir <- match.arg(dir)

    if (!dir %in% c("resources")) {
        stop("Only `resources`  cache can be cleared.")
    }

    cacheDir <- .SignalR$BulkSignalR_CACHEDIR

    cacheDir <- paste(cacheDir, dir, sep = "/")

    # safeguard
    if (!dir.exists(cacheDir)) {
        cli::cli_alert("BulkSignalR cache for {.val {dir}} doesn't exist.")
        return(invisible(NULL))
    }

    bfc <- BiocFileCache::BiocFileCache(cacheDir, ask = FALSE)
    BiocFileCache::removebfc(bfc, ask = FALSE)

    # dir.create(resourcesCacheDir)
    cli::cli_alert("BulkSignalR cache {.val {dir}} has been deleted.\n")
    message(
        "- Location: ", cacheDir, "\n",
        "- No. of files: 0"
    )

    return(invisible(NULL))
}


#' Get cache content information.
#'
#' Get cache content information for a specific cache directory.
#'
#' @param dir Directory to remove in order to clean the cache.
#' Can be only 'resources'
#' @return Returns `NULL`, invisibly. 

#' @importFrom BiocFileCache BiocFileCache bfcinfo
#' @importFrom cli cli_alert_danger cli_alert
#' @export
#' @examples
#' cacheInfo()
cacheInfo <- function(dir = c("resources")) {
    dir <- match.arg(dir)

    if (!dir %in% c("resources")) {
        stop("Only `resources` can be cleared.")
    }

    cacheDir <- .SignalR$BulkSignalR_CACHEDIR

    cacheDir <- paste(cacheDir, dir, sep = "/")

    # safeguard
    if (!dir.exists(cacheDir)) {
        cli::cli_alert("BulkSignalR {.val {dir}} cache uninitialized.")
        message("- Location: ", cacheDir)
        return(invisible(NULL))
    }

    bfc <- BiocFileCache::BiocFileCache(cacheDir, ask = FALSE)
    files <- BiocFileCache::bfcinfo(bfc)$rpath

    if (length(files) == 0) {
        cli::cli_alert("BulkSignalR {.val {dir}} cache uninitialized.")
        message("- Location: ", cacheDir, "\n",
            "- No. of files: ", length(files)
        )
        return(invisible(NULL))
    } else {
        total_size <- sum(file.size(files))
        size_obj <- structure(total_size, class = "object_size")
        # print(total_size)
        # print(size_obj)

        cli::cli_alert("BulkSignalR {.val {dir}} cache :")
        message(
            "- Location: ", cacheDir, "\n",
            "- No. of files: ", length(files), "\n",
            "- Total size: ", format(size_obj, units = "auto")
        )
        listing <- paste0(files, "\n")
        message(listing)
    }

    return(invisible(NULL))
}


#' Check whether remote resource files have been changed.
#'
#' Check to see whether some resource
#' has been updated.
#'
#' @param dir Directory for which you want to check Version.
#' Can be only 'resources'.
#'
#' @importFrom cli cli_alert_danger cli_alert cli_alert_info
#' @importFrom cli cli_inform
#' @import BiocFileCache httr2
#' @importFrom curl has_internet
#' @return Returns `NULL`, invisibly. 
#'
#' @export
#' @examples
#' cacheVersion()
cacheVersion <- function(dir = c("resources")) {
    dir <- match.arg(dir)
    
    # bypass ssl
    config <- list(ssl_verifypeer = 0L, ssl_verifyhost = 0L)

    if (!dir %in% c("resources")) {
        stop("Only `resources` is a valid keyword.")
    }

  
    cacheDir <- .SignalR$BulkSignalR_CACHEDIR
    cacheDir <- paste(cacheDir, dir, sep = "/")

    if (!dir.exists(cacheDir)) {
        cli::cli_alert_danger("BulkSignalR {.val {dir}} cache uninitialized.")
        stop("- Location: ", cacheDir, "\n")    
    }

    word <- ifelse(dir == "resources", "have", "has")
    word2 <- ifelse(dir == "resources", "are", "is")

    hasInternet <- tryCatch(expr={curl::has_internet()}, 
        error = FALSE)
    
    if (hasInternet) {

        bfc <- BiocFileCache::BiocFileCache(cacheDir, ask = FALSE)

        if (any(BiocFileCache::bfcneedsupdate(bfc,config=config))) {
            cli::cli_alert("Remote {.val {dir}} {word} been updated.\n")
            mess_info <- paste0("To update locally,",
                " clear your cache with cacheClear({.var {dir}})\n")
            cli::cli_alert_info(mess_info)
            return(invisible(NULL))
        } else {
            cli::cli_inform("Local {.val {dir}} {word2} up to date.\n",
            class = "packageStartupMessage")
            packageStartupMessage("")
        }
    } 
    else {
        mess_info <- paste0("Your internet connection is off:",
        " remote update of {.val {dir}} won't be checked.")
        cli::cli_alert_info(mess_info)
    }

    return(invisible(NULL))

}

####################################################
###     Check / Read RDS From Cache              ###
####################################################

#' Read RDS from the cache.
#'
#' Access  resources (pathways or network
#' stored in the cache.
#'
#' @param bfc Object of class BiocFileCache, created by a call to
#' BiocFileCache::BiocFileCache()
#' @param resourceName keyword associated to a specific resourceName
#' @return Returns BiocFileCache::bfcquery object or FALSE
#' 
#' @importFrom cli cli_alert_danger cli_alert cli_alert_info
#' @import BiocFileCache
#' @keywords internal
.readRDSFromCache <- function(bfc, resourceName) {
    cacheHits <- BiocFileCache::bfcquery(bfc, query = resourceName,
        field = "rname")
    if (nrow(cacheHits) == 0) {
        cli::cli_alert_danger("No cache result found.\n")
        stop()
    } else if (nrow(cacheHits) > 1) {
        cli::cli_alert_danger("Multiple cache results found.\n")
        stop("Please, clear your cache! See cacheClear() function.")
    } else {
        test <- tryCatch(is.list(infoRDS(cacheHits$rpath[1])),
            error = function(e) {
                return(FALSE)
            }
        )
        if (test) {
            rid <- cacheHits$rid
            result <- readRDS(bfc[[rid]])
            return(result)
        }
    }
    # if not RDS
    return(FALSE)
}


#' Check for valid RDS cache file.
#'
#' This function checks whether a cache entry is a valid RDS file.
#' Returns TRUE if the cache entry is valid, FALSE otherwise.
#' In the case of an invalid file, the cache entry and file are
#' deleted.
#'
#' @param bfc Object of class BiocFileCache, created by a call to
#' BiocFileCache::BiocFileCache()
#' @param resourceName keyword associated to a specific resource name.
#' @return logical TRUE/FALSE
#' 
#' @importFrom cli cli_alert_danger cli_alert cli_alert_info
#' @importFrom BiocFileCache bfcremove bfcquery
#' @keywords internal
.checkRDSFromCache <- function(bfc, resourceName) {
    cacheHits <- BiocFileCache::bfcquery(bfc, query = resourceName,
        field = "rname")
    if (nrow(cacheHits) == 0) {
        cli::cli_alert_danger("No cache result found.\n")
        stop()
    } else if (nrow(cacheHits) > 1) {
        cli::cli_alert_danger("Multiple cache results found.\n")
        stop("Please, clear your cache! See cacheClear() function.")
    } else {
        test <- tryCatch(is.list(infoRDS(cacheHits$rpath[1])),
            error = function(e) {
                return(FALSE)
            }
        )
        if (!test) {
            BiocFileCache::bfcremove(bfc, cacheHits$rid[1])
        }
        return(test)
    }

    return(FALSE)
}

## ----setup, include = FALSE---------------------------------------------------
knitr::knit_hooks$set(optipng = knitr::hook_optipng)

## ----load-libs, message = FALSE,  warning = FALSE, results = FALSE------------
library(BulkSignalR)
library(igraph)
library(STexampleData)

## ----parallel, message=FALSE, warning=FALSE-----------------------------------
library(doParallel)
n.proc <- 1
cl <- makeCluster(n.proc)
registerDoParallel(cl)

# To add at the end of your script:
# stopCluster(cl)

## ----loading, eval=TRUE-------------------------------------------------------
data(sdc)
head(sdc)

## ----BSRDataModel, eval=TRUE,cache=FALSE--------------------------------------
bsrdm <- BSRDataModel(counts = sdc)
bsrdm

## ----learnParameters,eval=TRUE,warning=FALSE,fig.dim = c(7,3)-----------------
bsrdm <- learnParameters(bsrdm, quick=TRUE)
bsrdm

## ----BSRInference,eval=TRUE---------------------------------------------------

# We use a subset of the reference to speed up
# inference in the context of the vignette.
subset <- c("REACTOME_BASIGIN_INTERACTIONS",
"REACTOME_SYNDECAN_INTERACTIONS",
"REACTOME_ECM_PROTEOGLYCANS",
"REACTOME_CELL_JUNCTION_ORGANIZATION")

reactSubset <- BulkSignalR:::.SignalR$BulkSignalR_Reactome[
BulkSignalR:::.SignalR$BulkSignalR_Reactome$`Reactome name` %in% subset,]

resetPathways(dataframe = reactSubset,
              resourceName = "Reactome")

bsrinf <- BSRInference(bsrdm,
    min.cor = 0.3,
    reference="REACTOME")

LRinter.dataframe <- LRinter(bsrinf)

head(LRinter.dataframe[
    order(LRinter.dataframe$qval),
    c("L", "R", "LR.corr", "pw.name", "qval")])


## ----BSRInferenceTfile, eval=FALSE--------------------------------------------
# write.table(LRinter.dataframe[order(LRinter.dataframe$qval), ],
#     "./sdc_LR.tsv",
#     row.names = FALSE,
#     sep = "\t",
#     quote = FALSE
# )

## ----ReduceToPathway, eval=TRUE-----------------------------------------------
bsrinf.redP <- reduceToPathway(bsrinf)

## ----ReduceToBestPathway, eval=TRUE-------------------------------------------
bsrinf.redBP <- reduceToBestPathway(bsrinf)

## ----ReduceToLigand,eval=TRUE-------------------------------------------------
bsrinf.L <- reduceToLigand(bsrinf)
bsrinf.R <- reduceToReceptor(bsrinf)

## ----doubleReduction, eval=TRUE-----------------------------------------------
bsrinf.redP <- reduceToPathway(bsrinf)
bsrinf.redPBP <- reduceToBestPathway(bsrinf.redP)

## ----scoringLR,eval=TRUE,fig.dim = c(5,4)-------------------------------------
bsrsig.redBP <- BSRSignature(bsrinf.redBP, qval.thres=0.001)

scoresLR <- scoreLRGeneSignatures(bsrdm, bsrsig.redBP,
    name.by.pathway=FALSE
)

simpleHeatmap(scoresLR[1:20, ],
    hcl.palette="Cividis",
    pointsize=8)


## ----scoringPathway,eval=TRUE,fig.dim = c(7,3)--------------------------------

bsrsig.redPBP <- BSRSignature(bsrinf.redPBP, qval.thres=0.01)

scoresPathway <- scoreLRGeneSignatures(bsrdm, bsrsig.redPBP,
    name.by.pathway=TRUE
)

simpleHeatmap(scoresPathway,
    hcl.palette="Blue-Red 2",
    pointsize=8)

## ----heatmapMulti,eval=TRUE,fig.dim = c(7,10)---------------------------------
pathway1 <- pathways(bsrsig.redPBP)[1]
signatureHeatmaps(
    pathway = pathway1,
    bsrdm = bsrdm,
    heights = c(3,2,15),
    bsrsig = bsrsig.redPBP
)

## ----AlluvialPlot,eval=TRUE,fig.dim = c(8,3)----------------------------------
alluvialPlot(bsrinf,
    keywords = c("LAMC1"),
    type = "L",
    qval.thres = 0.01
)

## ----BubblePlot,eval=TRUE,fig.dim = c(8,4)------------------------------------

pathways <- LRinter(bsrinf)[1,c("pw.name")]
bubblePlotPathwaysLR(bsrinf,
    pathways = pathways,
    qval.thres = 0.001,
    color = "red",
    pointsize = 8
)

## ----Chordiagram,eval=TRUE,fig.dim = c(6,4.5)---------------------------------
chordDiagramLR(bsrinf,
    pw.id.filter = "R-HSA-210991",
    limit = 20,
    ligand="FYN", 
    receptor="SPN"
)

## ----network1, eval=TRUE------------------------------------------------------
# Generate a ligand-receptor network and export it in .graphML
# for Cytoscape or similar tools
gLR <- getLRNetwork(bsrinf.redBP, qval.thres = 1e-3)

# save to file
# write.graph(gLR,file="SDC-LR-network.graphml",format="graphml")

# As an alternative to Cytoscape, you can play with igraph package functions.
plot(gLR,
    edge.arrow.size = 0.1,
    vertex.label.color = "black",
    vertex.label.family = "Helvetica",
    vertex.label.cex = 0.1
)

## ----network2,  eval=TRUE-----------------------------------------------------
# You can apply other functions.


# Community detection
u.gLR <- as_undirected(gLR) # most algorithms work for undirected graphs only
comm <- cluster_edge_betweenness(u.gLR)
# plot(comm,u.gLR,
#     vertex.label.color="black",
#     vertex.label.family="Helvetica",
#     vertex.label.cex=0.1)

# Cohesive blocks
cb <- cohesive_blocks(u.gLR)
plot(cb, u.gLR,
    vertex.label.color = "black",
    vertex.label.family = "Helvetica",
    vertex.label.cex = 0.1,
    edge.color = "black"
)

## ----network3, eval=FALSE,warning=FALSE---------------------------------------
# # For the next steps, we just share the code below, but graph generation
# # functions are commented to lighten the vignette.
# 
# # Generate a ligand-receptor network complemented with intra-cellular,
# # receptor downstream pathways [computations are a bit longer here]
# #
# # You can save to a file for cystoscape or plot with igraph.
# 
# gLRintra <- getLRIntracellNetwork(bsrinf.redBP, qval.thres = 1e-3)
# 
# lay <- layout_with_kk(gLRintra)
# # plot(gLRintra,
# #     layout=lay,
# #     edge.arrow.size=0.1,
# #     vertex.label.color="black",
# #     vertex.label.family="Helvetica",
# #     vertex.label.cex=0.1)
# 
# # Reduce complexity by focusing on strongly targeted pathways
# pairs <- LRinter(bsrinf.redBP)
# top <- unique(pairs[pairs$pval <  1e-3, c("pw.id", "pw.name")])
# top
# gLRintra.res <- getLRIntracellNetwork(bsrinf.redBP,
#     qval.thres = 0.01,
#     restrict.pw = top$pw.id
# )
# lay <- layout_with_fr(gLRintra.res)
# 
# # plot(gLRintra.res,
# #     layout=lay,
# #     edge.arrow.size=0.1,
# #     vertex.label.color="black",
# #     vertex.label.family="Helvetica",
# #     vertex.label.cex=0.4)

## ----mouse,eval=TRUE,warning=FALSE--------------------------------------------
data(bodyMap.mouse)

ortholog.dict <- findOrthoGenes(
    from_organism = "mmusculus",
    from_values = rownames(bodyMap.mouse)
)

matrix.expression.human <- convertToHuman(
    counts = bodyMap.mouse,
    dictionary = ortholog.dict
)

bsrdm <- BSRDataModel(
    counts = matrix.expression.human,
    species = "mmusculus",
    conversion.dict = ortholog.dict
)

bsrdm <- learnParameters(bsrdm,quick=TRUE)

bsrinf <- BSRInference(bsrdm,reference="REACTOME")

bsrinf <- resetToInitialOrganism(bsrinf, conversion.dict = ortholog.dict)

# For example, if you want to explore L-R interactions
# you can proceed as shown above for a human dataset.

bsrinf.redBP <- reduceToBestPathway(bsrinf)
bsrsig.redBP <- BSRSignature(bsrinf.redBP, qval.thres=0.001)
scoresLR <- scoreLRGeneSignatures(bsrdm, bsrsig.redBP, name.by.pathway=FALSE)
head(LRinter(bsrinf.redBP))
#simpleHeatmap(scoresLR, column.names=TRUE,
#              pointsize=8)

## ----spatial1,eval=TRUE,message=FALSE-----------------------------------------
# load data =================================================

# We re-initialize the environment variable of Reactome
# to have all the pathways (we reduced it to a subset above to fasten
# example computations)
reactSubset <- getResource(resourceName = "Reactome",
cache = TRUE)

resetPathways(dataframe = reactSubset,
resourceName = "Reactome")

# Few steps of pre-process to subset a spatialExperiment object
# from STexampleData package ==================================

spe <- Visium_humanDLPFC()
set.seed(123)

speSubset <- spe[, colData(spe)$ground_truth%in%c("Layer1","Layer2")]

idx <- sample(ncol(speSubset), 10)
speSubset <- speSubset[, idx]

my.image.as.raster <- SpatialExperiment::imgRaster(speSubset, 
    sample_id = imgData(spe)$sample_id[1], image_id = "lowres")

colData(speSubset)$idSpatial <- paste(colData(speSubset)[[4]],
                colData(speSubset)[[5]],sep = "x")


annotation <- colData(speSubset)


## ----spatial2,eval=TRUE,warning=FALSE-----------------------------------------
# prepare data =================================================

bsrdm <- BSRDataModel(speSubset,
    min.count = 1,
    prop = 0.01,
    method = "TC",
    symbol.col = 2,
    x.col = 4,
    y.col = 5, 
    barcodeID.col = 1)

bsrdm <- learnParameters(bsrdm,
    quick = TRUE,
    min.positive = 2,
    verbose = TRUE)

bsrinf <- BSRInference(bsrdm, min.cor = -1,reference="REACTOME")

# spatial analysis ============================================

bsrinf.red <- reduceToBestPathway(bsrinf)
pairs.red <- LRinter(bsrinf.red)

thres <- 0.01
min.corr <- 0.01
pairs.red <- pairs.red[pairs.red$qval < thres & pairs.red$LR.corr > min.corr,]

head(pairs.red[
    order(pairs.red$qval),
    c("L", "R", "LR.corr", "pw.name", "qval")])

s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
scores.red <- scoreLRGeneSignatures(bsrdm,s.red)

head(scores.red)


## ----spatialPlot3,eval=TRUE,fig.dim = c(6,4.5)--------------------------------
# Visualization ============================================

# plot one specific interaction

# we have to follow the syntax with {} 
# to be compatible with the reduction functions
inter <- "{SLIT2} / {GPC1}"

# with raw tissue reference
spatialPlot(scores.red[inter, ], annotation, inter,
    ref.plot = TRUE, ref.plot.only = FALSE,
    image.raster = NULL, dot.size = 1,
    label.col = "ground_truth"
)

# or with synthetic image reference
spatialPlot(scores.red[inter, ], annotation, inter,
    ref.plot = TRUE, ref.plot.only = FALSE,
    image.raster = my.image.as.raster, dot.size = 1,
    label.col = "ground_truth"

)

## ----spatialPlot4,eval=TRUE,fig.dim = c(6,4.5)--------------------------------

separatedLRPlot(scores.red, "SLIT2", "GPC1", 
    ncounts(bsrdm), 
    annotation,
    label.col = "ground_truth")


## ----spatialPlot5,eval=TRUE---------------------------------------------------

# generate a visual index in a pdf file directly
spatialIndexPlot(scores.red, annotation,  
    label.col = "ground_truth",
    out.file="spatialIndexPlot")


## ----spatialPlot6,eval=TRUE,fig.dim = c(6,4.5)--------------------------------

# statistical association with tissue areas based on correlations
# For display purpose, we only use a subset here
assoc.bsr.corr <- spatialAssociation(scores.red[c(1:17), ],
annotation, label.col = "ground_truth",test = "Spearman")

head(assoc.bsr.corr)

spatialAssociationPlot(assoc.bsr.corr)


## ----inferCells,eval=FALSE,include=FALSE--------------------------------------
# 
# ## Additional functions
# 
# # This is not part of the main workflow for analyzing L-R interactions but
# # we offer convenience functions for inferring cell types. However
# # we recommend using other software packages specifically dedicated to this
# # purpose.
# 
# # Inferring cell types
# 
# 
# data(sdc, package = "BulkSignalR")
# bsrdm <- BSRDataModel(counts = sdc)
# bsrdm <- learnParameters(bsrdm)
# bsrinf <- BSRInference(bsrdm)
# 
# # Common TME cell type signatures
# data(immune.signatures, package = "BulkSignalR")
# unique(immune.signatures$signature)
# immune.signatures <- immune.signatures[immune.signatures$signature %in% c(
#     "B cells", "Dentritic cells", "Macrophages",
#     "NK cells", "T cells", "T regulatory cells"
# ), ]
# data("tme.signatures", package = "BulkSignalR")
# signatures <- rbind(immune.signatures,
# tme.signatures[tme.signatures$signature %in%
# c("Endothelial cells", "Fibroblasts"), ])
# tme.scores <- scoreSignatures(bsrdm, signatures)
# 
# # assign cell types to interactions
# lr2ct <- assignCellTypesToInteractions(bsrdm, bsrinf, tme.scores)
# head(lr2ct)
# 
# # cellular network computation and plot
# g.table <- cellularNetworkTable(lr2ct)
# 
# gCN <- cellularNetwork(g.table)
# 
# plot(gCN, edge.width = 5 * E(gCN)$score)
# 
# gSummary <- summarizedCellularNetwork(g.table)
# plot(gSummary, edge.width = 1 + 30 * E(gSummary)$score)
# 
# # relationship with partial EMT---
# # Should be tested HNSCC data instead of SDC!!
# 
# # find the ligands
# data(p.EMT, package = "BulkSignalR")
# gs <- p.EMT$gene
# triggers <- relateToGeneSet(bsrinf, gs)
# triggers <- triggers[triggers$n.genes > 1, ] # at least 2 target genes in the gs
# ligands.in.gs <- intersect(triggers$L, gs)
# triggers <- triggers[!(triggers$L %in% ligands.in.gs), ]
# ligands <- unique(triggers$L)
# 
# # link to cell types
# cf <- cellTypeFrequency(triggers, lr2ct, min.n.genes = 2)
# missing <- setdiff(rownames(tme.scores), names(cf$s))
# cf$s[missing] <- 0
# cf$t[missing] <- 0
# 
# op <- par(mar = c(2, 10, 2, 2))
# barplot(cf$s, col = "lightgray", horiz = T, las = 2)
# par(op)
# 
# # random selections based on random gene sets
# qval.thres <- 1e-3
# inter <- LRinter(bsrinf)
# tg <- tgGenes(bsrinf)
# tcor <- tgCorr(bsrinf)
# good <- inter$qval <= qval.thres
# inter <- inter[good, ]
# tg <- tg[good]
# tcor <- tcor[good]
# all.targets <- unique(unlist(tg))
# r.cf <- list()
# for (k in 1:100) { # should 1000 or more
#     r.gs <- sample(all.targets, length(intersect(gs, all.targets)))
#     r.triggers <- relateToGeneSet(bsrinf, r.gs, qval.thres = qval.thres)
#     r.triggers <- r.triggers[r.triggers$n.genes > 1, ]
#     r.ligands.in.gs <- intersect(r.triggers$L, r.gs)
#     r.triggers <- r.triggers[!(r.triggers$L %in% r.ligands.in.gs), ]
#     r <- cellTypeFrequency(r.triggers, lr2ct, min.n.genes = 2)
#     missing <- setdiff(rownames(tme.scores), names(r$s))
#     r$s[missing] <- 0
#     r$t[missing] <- 0
#     o <- order(names(r$t))
#     r$s <- r$s[o]
#     r$t <- r$t[o]
#     r.cf <- c(r.cf, list(r))
# }
# r.m.s <- foreach(i = seq_len(length(r.cf)), .combine = rbind) %do% {
#     r.cf[[i]]$s
# }
# 
# # plot results
# op <- par(mar = c(2, 10, 2, 2))
# boxplot(r.m.s, col = "lightgray", horizontal = T, las = 2)
# pts <- data.frame(x = as.numeric(cf$s[colnames(r.m.s)]), cty = colnames(r.m.s))
# stripchart(x ~ cty, data = pts, add = TRUE, pch = 19, col = "red")
# par(op)
# for (cty in rownames(tme.scores)) {
#     cat(cty, ": P=", sum(r.m.s[, cty] >= cf$s[cty]) / nrow(r.m.s),
#     "\n", sep = "")
# }

## ----session-info-------------------------------------------------------------
sessionInfo()

#' Generate a ligand-receptor network
#'
#' Generate a ligand-receptor network from a ligand-receptor table.
#'
#' @param bsrinf        A BSRInference object.
#' @param pval.thres    P-value threshold.
#' @param qval.thres    Q-value threshold.
#' @param node.size     Default node size in the network.
#' @param red.pairs     A data frame with columns L (ligands) and R
#' (receptors) that restrict LR pairs to those listed.
#' @return An \code{igraph} object featuring the ligand-receptor network.
#' Default colors and node sizes are assigned,
#' which can be changed afterwards if necessary.
#' @import igraph
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#' 
#' gLR <- getLRNetwork(bsrinf, qval.thres = 1e-4)
#' # plot(gLR)
#' # write.graph(gLR, file="SDC-LR-network.graphml", format="graphml")
#' @importFrom methods is
getLRNetwork <- function(bsrinf, pval.thres = NULL, qval.thres = NULL,
    node.size = 5, red.pairs = NULL) {
    if (!is(bsrinf, "BSRInference")) {
        stop("bsrinf must be a BSRInference object")
    }
    if (is.null(pval.thres) && is.null(qval.thres)) {
        stop("Either a P- or a Q-value threshold must be provided")
    }
    ipar <- inferenceParameters(bsrinf)
    if (ipar$ligand.reduced || ipar$receptor.reduced) {
        stop(
            "Does not work with inferences already reduced to the ",
            "ligands or the receptors"
        )
    }

    # get unique LR pairs with required statistical significance
    bsrinf <- reduceToBestPathway(bsrinf)
    pairs <- LRinter(bsrinf)
    if (!is.null(pval.thres)) {
        pairs <- pairs[pairs$pval <= pval.thres, ]
    } else {
        pairs <- pairs[pairs$qval <= qval.thres, ]
    }
    # different pathways for the same LR pair can have the same P-value
    ref <- paste(pairs$L, pairs$R, sep = "||")
    dup <- duplicated(ref)
    pairs <- pairs[!dup, ]

    # LR pairs restricted to a provided list
    if (!is.null(red.pairs)) {
        ref <- paste(pairs$L, pairs$R, sep = "||")
        restricted <- paste(red.pairs$L, red.pairs$R, sep = "||")
        pairs <- pairs[ref %in% restricted, ]
    }

    # generate the igraph object
    g <- igraph::graph_from_data_frame(pairs[, c("L", "R")], directed = TRUE)
    g.names <- igraph::vertex_attr(g, "name")
    g <- igraph::set_vertex_attr(g, name = "size", value = node.size)
    g <- igraph::set_vertex_attr(g, name = "label", value = g.names)
    g.types <- stats::setNames(rep("ligand", length(g.names)), g.names)
    g.types[g.names %in% pairs$R] <- "receptor"
    g <- igraph::set_vertex_attr(g, name = "node.type",
        value = g.types[g.names])
    g.colors <- rep("green", length(g.names))
    g.colors[g.names %in% pairs$R] <- "red"
    g <- igraph::set_vertex_attr(g, name = "color", value = g.colors)
    g <- igraph::set_edge_attr(g, name = "edge.type", value = "LR")
    el <- igraph::as_edgelist(g)
    pval <- NULL
    qval <- NULL
    corr <- NULL
    for (i in seq_len(nrow(el))) {
        j <- which(pairs$L == el[i, 1] & pairs$R == el[i, 2])
        pval <- c(pval, pairs$pval[j])
        qval <- c(qval, pairs$qval[j])
        corr <- c(corr, pairs$LR.corr[j])
    }
    g <- igraph::set_edge_attr(g, name = "corr", value = corr)
    g <- igraph::set_edge_attr(g, name = "pval", value = pval)
    g <- igraph::set_edge_attr(g, name = "log10.pval", value = -log10(pval))
    g <- igraph::set_edge_attr(g, name = "qval", value = qval)
    g <- igraph::set_edge_attr(g, name = "log10.qval", value = -log10(qval))
    
} # getLRNetwork


#' Internal function to generate a ligand-receptor-downstream signaling network
#'
#' @param pairs     A ligand-receptor table such as output by
#' \code{LRinter(BSRInference)}.
#' @param pw              A table defining the reference pathways.
#' @param tg.genes    Target gene list such as output by
#' \code{tgGenes(BSRInference)}.
#' @param tg.corr    Target gene correlation list (with the receptor) such as
#' output by \code{tgCorr(BSRInference)}.
#' @param id.col Column index or name in \code{pw} for the pathway IDs.
#' @param gene.col Column index or name in \code{pw} for the gene symbols.
#' @param min.cor Minimum correlation required for the target genes.
#' @param tg.pval Target gene P-value list such as returned by
#' \code{ tgPval(BSRInferenceComp)}.
#' @param max.pval     Maximum (regulation) P-value for target genes in case
#' a BSRInferenceComp object is used 
#' to generate the network.
#' @param min.logFC     Minimum logFC required for the target genes in case
#' a BSRInferenceComp object is used to generate the network.
#' @param pos.targets   A logical imposing that all the network targets must
#' display positive correlation or logFC a BSRInferenceComp object is used to
#' generate the network.
#' @param neg.targets   A logical imposing that all the network targets must
#' display negative correlation or logFC a BSRInferenceComp object is used to
#' generate the network. Correlations must be
#' <= -min.cor or logFC <= - min.logFC
#' with this option activated.
#'
#' @return An \code{igraph} object featuring the ligand-receptor-downstream
#' signaling network. Default colors and node sizes are assigned. In case
#' the \code{max.pval} parameter is set, it is assumed that \code{tg.pval}
#' is set as well and downstream signaling genes are selected by their
#' P-values in the comparison of clusters of samples.
#'
#' @importFrom foreach %do% %dopar%
#' @keywords internal
.edgesLRIntracell <- function(pairs, pw, tg.genes, tg.corr, id.col, gene.col,
    min.cor = 0.25, pos.targets = FALSE,
    neg.targets = FALSE, tg.pval = NULL,
    max.pval = NULL, tg.logFC = NULL, min.logFC = 0) {
    # local binding
    i <- NULL
    
    directed.int <- c(
        "controls-transport-of",
        "catalysis-precedes",
        "positively-controls-expression-of",
        "negatively-controls-expression-of",
        "regulates-transcription-of",
        "controls-dephospho-of",
        "controls-phospho-of",
        "regulates-dephospho-of",
        "regulates-phospho-of",
        "controls-state-change-of",
        "controls-state-of-by-metab"
    )

    arcs <- foreach::foreach(i = seq_len(nrow(pairs)), .combine = rbind) %do% {
        # arcs <- NULL
        # for (i in 1:nrow(pairs)) {
        r <- pairs$R[i]
        p <- pairs$pw.id[i]
        tg <- tg.genes[[i]]
        if (is.null(max.pval) && is.null(min.logFC)) {
            # selection on correlations
            corr <- tg.corr[[i]]
            if (pos.targets) {
                targets <- tg[corr >= min.cor]
            } else if (neg.targets) {
                targets <- tg[corr <= -min.cor]
            } else {
                targets <- tg[abs(corr) >= min.cor]
            }
        } else {
            # selection on P-values and logFC
            pval <- tg.pval[[i]]
            logFC <- tg.logFC[[i]]
            if (pos.targets) {
                targets <- tg[pval <= max.pval & logFC >= min.logFC]
            } else if (neg.targets) {
                targets <- tg[pval <= max.pval & logFC <= -min.logFC]
            } else {
                targets <- tg[pval <= max.pval & abs(logFC) >= min.logFC]
            }
        }

        # build a hybrid directed/undirected graph
        a.iter <- data.frame(
            from = pairs$L[i], to = r, edge.type = "LR",
            stringsAsFactors = FALSE
        )
        genes.in.pw <- pw[pw[[id.col]] == p, gene.col]
        int <- .SignalR$BulkSignalR_Network[
            .SignalR$BulkSignalR_Network$a.gn %in% genes.in.pw &
                .SignalR$BulkSignalR_Network$b.gn %in% genes.in.pw,
        ]
        directed <- int$type %in% directed.int
        ret <- int[!directed, c("b.gn", "a.gn")]
        names(ret) <- c("a.gn", "b.gn")
        d.int <- unique(rbind(int[, c("a.gn", "b.gn")], ret))
        d.int <- d.int[!is.na(d.int$a.gn) & !is.na(d.int$b.gn),]
        g <- igraph::graph_from_data_frame(d.int, directed = TRUE)
        
        # get the reachable targets (in the graph and at finite distance)
        targets <- intersect(targets, c(d.int$a.gn, d.int$b.gn))
        targets <- targets[!is.na(targets)]
        sp <- igraph::distances(g,
                                v = igraph::V(g)[igraph::V(g)$name == r],
                                to = igraph::V(g)[igraph::V(g)$name %in% targets],
                                mode = "out")
        targets <- colnames(sp)[!is.infinite(sp[r, ])]
        
        # keep shortest paths from the receptor to the targets only
        if ((r %in% d.int$a.gn || r %in% d.int$b.gn) && length(targets) > 0) {
            paths <- try(igraph::shortest_paths(g,
                from = igraph::V(g)[igraph::V(g)$name == r],
                to = igraph::V(g)[igraph::V(g)$name %in% targets],
                output = "vpath"
            ), silent = TRUE)
            if (inherits(paths, "try-error")) {
                paths <- NULL
            }
            if (is.null(paths)){
                stop("No directed graph found by igraph")
            }
            if (length(paths$vpath) > 0) {
                for (j in seq_len(length(paths$vpath))) {
                    vertices <- paths$vpath[[j]]
                    if (length(vertices) > 0) {
                        for (k in 2:length(vertices)) {
                            from <- igraph::V(g)$name[vertices[k - 1]]
                            to <- igraph::V(g)$name[vertices[k]]
                            edge.type <- .SignalR$BulkSignalR_Network[
                                .SignalR$BulkSignalR_Network$a.gn == from &
                                    .SignalR$BulkSignalR_Network$b.gn == to,
                                "type"
                            ][1]
                            a.iter <- rbind(a.iter, data.frame(
                                from = from, to = to,
                                edge.type = edge.type,
                                stringsAsFactors = FALSE
                            ))
                        }
                    }
                }
            }
        }
        # arcs <- rbind(arcs, unique(a.iter))
        unique(a.iter)
    }

    unique(arcs)
    
} # .edgesLRIntracell


#' Generate a ligand-receptor-downstream signaling network
#'
#' Generate a ligand-receptor network from a BSRInference object and add
#' the shortest paths from the receptors to correlated
#' target genes following Reactome and KEGG pathways.
#'
#' @param bsrinf        A BSRInference or BSRInferenceComp object.
#' @param pval.thres    P-value LR interaction threshold.
#' @param qval.thres    Q-value LR interaction threshold.
#' @param min.cor       Minimum correlation required for the target genes.
#' @param max.pval      Maximum P-value required for the target genes in case
#'   a BSRInferenceComp object is provided.
#' @param min.logFC     Minimum logFC required for the target genes in case
#'   a BSRInferenceComp object is provided.
#' @param pos.targets   A logical imposing that all the network targets must
#'   display positive correlation or logFC in case of a BSRInferenceComp object.
#' @param neg.targets   A logical imposing that all the network targets must
#'   display negative correlation or logFC in case of a BSRInferenceComp object.
#'   Correlations must be <= -min.cor or logFC <= - min.logFC
#'   with this option activated.
#' @param restrict.pw   A vector of pathway IDs to which receptor downstream
#' signaling is restricted.
#' @param node.size     Default node size in the network.
#' @return An \code{igraph} object featuring the ligand-receptor-downstream
#' signaling network. Default colors and node sizes are assigned,
#' which can be changed afterwards if necessary.
#'
#' The target genes to which the \code{min.cor} correlation is imposed are
#' those listed in \code{tgGenes(bsrinf)}, correlations are in
#' \code{tgCorr(bsrinf)}.
#' The construction of shortest paths from the receptors to those selected
#' targets adds other genes, which were either some targets with too low
#' correlation or genes along the shortest paths to reach the selected targets.
#' @import igraph
#' @importFrom methods is
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#' 
#' bsrinf.redBP <- reduceToBestPathway(bsrinf)
#'
#' pairs <- LRinter(bsrinf.redBP)
#' top <- unique(pairs[pairs$pval < 1e-20, c("pw.id", "pw.name")])
#'
#' gLRintra.res <- getLRIntracellNetwork(bsrinf.redBP,
#' qval.thres = 0.01,
#' restrict.pw = top[1,]$pw.id
#' )
#'
#' # write.graph(gLRintra, file="SDC-LR-intracellular-network.reduced.graphml",
#' # format="graphml")
getLRIntracellNetwork <- function(bsrinf, pval.thres = NULL, qval.thres = NULL,
    min.cor = 0.25, max.pval = NULL, min.logFC = NULL,
    pos.targets = FALSE, neg.targets = FALSE,
    restrict.pw = NULL, node.size = 5) {
    comp.obj <- is(bsrinf, "BSRInferenceComp")
    if (!is(bsrinf, "BSRInference") && !comp.obj) {
        stop("bsrinf must be a BSRInference or BSRInferenceComp object")
    }
    if (!is.null(max.pval) && !comp.obj) {
        stop("max.pval can only be specified for a BSRInferenceComp object")
    }
    if (!is.null(min.logFC) && !comp.obj) {
        stop("min.logFC can only be specified for a BSRInferenceComp object")
    }
    if (is.null(pval.thres) && is.null(qval.thres)) {
        stop("Either a P- or a Q-value threshold must be provided")
    }
    ipar <- inferenceParameters(bsrinf)
    if (ipar$ligand.reduced || ipar$receptor.reduced) {
        stop(
            "Does not work with inferences already reduced to the ",
            "ligands or the receptors"
        )
    }
    if (neg.targets && pos.targets) {
        stop("neg.targets and pos.targets cannot be TRUE simultaneously")
    }

    # get unique LR pairs with required statistical significance
    pairs <- LRinter(bsrinf)
    tg.genes <- tgGenes(bsrinf)
    tg.corr <- tgCorr(bsrinf)
    if (!is.null(pval.thres)) {
        good <- pairs$pval <= pval.thres
    } else {
        good <- pairs$qval <= qval.thres
    }
    pairs <- pairs[good, ]
    tg.genes <- tg.genes[good]
    tg.corr <- tg.corr[good]
    if (comp.obj) {
        tg.pval <- tgPval(bsrinf)[good]
        tg.logFC <- tgLogFC(bsrinf)[good]
    }

    pool <- unique(c(pairs$L, pairs$R))
    all.edges <- NULL

    # reactome pathways
    i.react <- grep("^R-", pairs$pw.id)
    pairs.react <- pairs[i.react, ]
    tg.genes.react <- tg.genes[i.react]
    tg.corr.react <- tg.corr[i.react]
    if (comp.obj) {
        tg.pval.react <- tg.pval[i.react]
        tg.logFC.react <- tg.logFC[i.react]
    }
    if (!is.null(restrict.pw)) {
        i.react <- which(pairs.react$pw.id %in% restrict.pw)
        pairs.react <- pairs.react[i.react, ]
        tg.genes.react <- tg.genes.react[i.react]
        tg.corr.react <- tg.corr.react[i.react]
        if (comp.obj) {
            tg.pval.react <- tg.pval.react[i.react]
            tg.logFC.react <- tg.logFC.react[i.react]
        }
    }
    if (nrow(pairs.react) > 0) {
        ids <- unique(.SignalR$BulkSignalR_Reactome[
            .SignalR$BulkSignalR_Reactome$`Gene name` %in% pool, "Reactome ID"])
        react <- .SignalR$BulkSignalR_Reactome[
            .SignalR$BulkSignalR_Reactome$`Reactome ID` %in% ids, ]
        if (!is.null(restrict.pw)) {
            react <- .SignalR$BulkSignalR_Reactome[
            .SignalR$BulkSignalR_Reactome$`Reactome ID` %in% restrict.pw, ]
        }
        all.edges <- .edgesLRIntracell(
            pairs.react, react, tg.genes.react,
            tg.corr.react, "Reactome ID", "Gene name", min.cor,
            pos.targets, neg.targets, tg.pval.react, max.pval,
            tg.logFC.react, min.logFC
        )
    }

    # GOBP
    i.go <- grep("^GO:", pairs$pw.id)
    pairs.go <- pairs[i.go, ]
    tg.genes.go <- tg.genes[i.go]
    tg.corr.go <- tg.corr[i.go]
    if (comp.obj) {
        tg.pval.go <- tg.pval[i.go]
        tg.logFC.go <- tg.logFC[i.go]
    }
    if (!is.null(restrict.pw)) {
        i.go <- which(pairs.go$pw.id %in% restrict.pw)
        pairs.go <- pairs.go[i.go, ]
        tg.genes.go <- tg.genes.go[i.go]
        tg.corr.go <- tg.corr.go[i.go]
        if (comp.obj) {
            tg.pval.go <- tg.pval.go[i.go]
            tg.logFC.go <- tg.logFC.go[i.go]
        }
    }
    if (nrow(pairs.go) > 0) {
        ids <- unique(.SignalR$BulkSignalR_Gobp[
            .SignalR$BulkSignalR_Gobp$`Gene name` %in% pool, "GO ID"])
        go <- .SignalR$BulkSignalR_Gobp[
        .SignalR$BulkSignalR_Gobp$`GO ID` %in% ids, ]
        if (!is.null(restrict.pw)) {
            go <- go[go$`GO ID` %in% restrict.pw, ]
        }
        all.edges <- unique(rbind(
            all.edges,
            .edgesLRIntracell(
                pairs.go, go, tg.genes.go,
                tg.corr.go, "GO ID", "Gene name", min.cor,
                pos.targets, neg.targets, tg.pval.go, max.pval,
                tg.logFC.go, min.logFC
            )
        ))
    }

    # generate igraph object -------------------

    # some LR interactions are also given as in-complex-with interactions,
    # get rid of them
    ref <- paste(all.edges[[1]], all.edges[[2]], sep = "||")
    dup <- ref[duplicated(ref)]
    all.edges <- all.edges[!(ref %in% dup) | 
    (ref %in% dup & all.edges$edge.type == "LR"), ]
    ref <- paste(all.edges[[1]], all.edges[[2]], sep = "||")
    duppl <- duplicated(ref)
    if (sum(duppl) > 0) {
        all.edges <- all.edges[!duppl, ]
    } # some duplicates remain, eliminate

    # build graph
    directed.int <- c(
        "controls-transport-of",
        "catalysis-precedes",
        "positively-controls-expression-of",
        "negatively-controls-expression-of",
        "regulates-transcription-of",
        "controls-dephospho-of",
        "controls-phospho-of",
        "regulates-dephospho-of",
        "regulates-phospho-of",
        "controls-state-change-of",
        "controls-state-of-by-metabo",
        "LR"
    )

    directed <- all.edges$edge.type %in% directed.int
    ret <- all.edges[!directed, ]
    from <- ret$from
    ret$from <- ret$to
    ret$to <- from
    
    d.int <- unique(rbind(all.edges, ret))
    d.int <- d.int[!is.na(d.int$from) & !is.na(d.int$to),]
    g <- igraph::graph_from_data_frame(d.int, directed = TRUE)

    g.names <- igraph::vertex_attr(g, "name")
    g <- igraph::set_vertex_attr(g, name = "size", value = node.size)
    g <- igraph::set_vertex_attr(g, name = "label", value = g.names)
    g.types <- stats::setNames(rep("downstream.gene", length(g.names)), g.names)
    g.types[g.names %in% pairs$R] <- "receptor"
    g.types[g.names %in% pairs$L] <- "ligand"
    g <- igraph::set_vertex_attr(g, name = "node.type",
        value = g.types[g.names])
    g.colors <- rep("gray80", length(g.names))
    g.colors[g.names %in% pairs$R] <- "red"
    g.colors[g.names %in% pairs$L] <- "green"
    g <- igraph::set_vertex_attr(g, name = "color", value = g.colors)
    
} # getLRIntracellNetwork

## ----setup, include = FALSE---------------------------------------------------
knitr::knit_hooks$set(optipng = knitr::hook_optipng)

## ----load-libs, message = FALSE,  warning = FALSE, results = FALSE------------
library(BulkSignalR)

## ----cash,eval=FALSE----------------------------------------------------------
# 
# # You can see hidden environment variable list as follows.
# ls(BulkSignalR:::.SignalR)
# 
# # Setup Pathways for reactome (can be done for GO-BP also)
# 
# subset <- c("REACTOME_BASIGIN_INTERACTIONS",
# "REACTOME_SYNDECAN_INTERACTIONS",
# "REACTOME_ECM_PROTEOGLYCANS",
# "REACTOME_CELL_JUNCTION_ORGANIZATION")
# 
# reactSubset <- BulkSignalR:::.SignalR$BulkSignalR_Reactome[
# BulkSignalR:::.SignalR$BulkSignalR_Reactome$`Reactome name` %in% subset,]
# 
# resetPathways(dataframe = reactSubset,
# resourceName = "Reactome")
# 
# cacheInfo()
# cacheVersion()
# 
# cacheClear()
# 
# # Ask to download again
# createResources(onRequest=TRUE)
# 
# 
# # Set up LRdb
# df <- data.frame(
#   ligand    = c("A2M", "AANAT", "AANAT", "ACE", "ACE"),
#   receptor  = c("LRP1", "MTNR1A", "MTNR1B", "AGTR2", "BDKRB2")
# )
# resetLRdb(df,switch=TRUE)
# 
# # Set up Network
# df <- data.frame(
#   a.gn   = c("A2M", "AANAT", "AANAT", "ACE", "ACE"),
#   type  = c("regulates-transcription-of",
#    "regulates-transcription-of",
#    "regulates-transcription-of",
#    "regulates-transcription-of",
#    "regulates-transcription-of"),
#   b.gn  = c("LRP1", "MTNR1A", "MTNR1B", "AGTR2", "BDKRB2")
# )
# 
# resetNetwork(df)
# 
# 

## ----session-info-------------------------------------------------------------
sessionInfo()

#' BulkSignalR ligand-receptor signature Object
#'
#' S4 class to represent gene signatures
#' of inferred ligand-receptor interactions, including
#' their reduced versions.
#'
#' @slot ligands   A list of ligands, one entry per LR interaction.
#' @slot receptors   A list of receptors, one entry per LR interaction.
#' @slot tg.genes  A list of target genes, one entry per LR interaction.
#' @slot pathways  An atomic vector of pathway names, one per interaction.
#' @slot tg.corr  A list of target genes correlation.
#'
#' @export
#' @examples
#' new("BSRSignature")
setClass("BSRSignature",
    slots = c(
        pathways = "character",
        ligands = "list",
        receptors = "list",
        tg.genes = "list",
        tg.corr = "list"
    ),
    prototype = list(
        pathways = "path 1",
        ligands = list("A"),
        receptors = list("B"),
        tg.genes = list(c("a", "b", "c")),
        tg.corr = list(c(0.1, 0.2, 0.3))
    )
)

setValidity(
    "BSRSignature",
    function(object) {
        if (!is.character(object@pathways)) {
            return("pathways is not character")
        }
        if (!is.list(object@ligands)) {
            return("ligands is not a list")
        }
        if (!is.list(object@receptors)) {
            return("receptors is not a list")
        }
        if (!is.list(object@tg.genes)) {
            return("tg.genes is not a list")
        }
        if (!is.list(object@tg.corr)) {
            return("tg.corr is not a list")
        }
        TRUE
    }
)
setMethod("show", "BSRSignature", function(object) {
    print(data.frame(
        L = vapply(
            object@ligands, function(x) paste(x, collapse = ";"),
            character(1)
        ),
        R = vapply(
            object@receptors, function(x) paste(x, collapse = ";"),
            character(1)
        ),
        pathways = object@pathways,
        tgGenes = vapply(
            object@tg.genes, function(x) paste(x, collapse = ";"),
            character(1)
        )
    )[seq_len(min(5, length(object@ligands))), ])
})

# Constructor ========================================================

#' Extract gene signatures of LR pair activity
#'
#' Obtain gene signatures reflecting ligand-receptor as well as
#' receptor downstream activity to
#' score ligand-receptor pairs across samples subsequently with
#' \code{"\link[=BSRDataModel-class]{scoreLRGeneSignatures}"}
#'
#' @name BSRSignature
#'
#' @param obj    BSRinference object.
#' @param pval.thres    P-value threshold.
#' @param qval.thres    Q-value threshold.
#' @param with.pw.id    A logical indicating whether the ID of a pathway
#' should be concatenated to its name.
#' @return A BSRSignature object containing a gene signature for each triple
#' ligand-receptor pair. A reduction to the best pathway
#' for each pair is automatically performed and the gene signature is
#' comprised of the ligand, the receptor,
#' and all the target genes with rank equal or superior to \code{pairs$rank}.
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#' 
#' bsrinf.redP <- reduceToPathway(bsrinf)
#' bsrsig.redP <- BSRSignature(bsrinf, qval.thres = 0.001)
#'
#' @importFrom foreach %do% %dopar%
#' @importFrom methods new
BSRSignature <- function(obj,
    pval.thres = NULL, qval.thres = NULL, with.pw.id = FALSE) {
    if (is.null(pval.thres) && is.null(qval.thres)) {
        stop("Either a P- or a Q-value threshold must be provided")
    }

    # reduce and select
    obj <- reduceToBestPathway(obj)
    pairs <- LRinter(obj)
    if (!is.null(pval.thres)) {
        selected <- pairs$pval <= pval.thres
    } else {
        selected <- pairs$qval <= qval.thres
    }

    # obtain the signature object
    pairs <- pairs[selected, ]
    ligands <- ligands(obj)[selected]
    receptors <- receptors(obj)[selected]
    if (with.pw.id) {
        pathways <- paste0(pairs$pw.id, ": ", pairs$pw.name)
    } else {
        pathways <- pairs$pw.name
    }
    tg.genes <- tgGenes(obj)[selected]
    tg.corrs <- tgCorr(obj)[selected]

    for (i in seq_len(nrow(pairs))) {
        tg <- tg.genes[[i]]
        tg.genes[[i]] <- tg[pairs$rank[i]:length(tg)]

        tc <- tg.corrs[[i]]
        tg.corrs[[i]] <- tc[pairs$rank[i]:length(tc)]
    }

    new("BSRSignature",
        ligands = ligands,
        receptors = receptors, 
        tg.genes = tg.genes, 
        tg.corr = tg.corrs,
        pathways =  pathways
    )
} # BSRSignature

# Accessors & setters ========================================================

setGeneric("pathways", signature="x",
    function(x) standardGeneric("pathways")
)
#' pathways accessor
#'
#' @name pathways
#' @aliases pathways,BSRSignature-method
#' @param x BSRSignature
#' @return pathways
#' @examples
#' bsr.sig <- new("BSRSignature")
#' pathways(bsr.sig)
#' @export
setMethod("pathways", "BSRSignature", function(x) x@pathways)

#' ligands accessor
#'
#' @name ligands
#' @aliases ligands,BSRSignature-method
#' @param x BSRSignature
#' @return ligands
#' @examples
#' bsr.sig <- new("BSRSignature")
#' ligands(bsr.sig)
#' @export
setMethod("ligands", "BSRSignature", function(x) x@ligands)

#' receptors accessor
#'
#' @name receptors
#' @aliases receptors,BSRSignature-method
#' @param x BSRSignature
#' @return receptors
#' @examples
#' bsr.sig <- new("BSRSignature")
#' ligands(bsr.sig)
#' @export
setMethod("receptors", "BSRSignature", function(x) x@receptors)

#' Target genes accessor
#'
#' @name tgGenes
#' @aliases tgGenes,BSRSignature-method
#' @param x BSRSignature
#' @return tgGenes
#' @examples
#' bsr.sig <- new("BSRSignature")
#' tgGenes(bsr.sig)
#' @export
setMethod("tgGenes", "BSRSignature", function(x) x@tg.genes)

#' Target gene correlations accessor
#'
#' @name tgCorr
#' @aliases tgCorr,BSRSignature-method
#' @param x BSRSignature
#' @return tgCorr
#' @examples
#' bsr.sig <- new("BSRSignature")
#' tgCorr(bsr.sig)
#' @export
setMethod("tgCorr", "BSRSignature", function(x) x@tg.corr)

#' BulkSignalR Data Model Object
#'
#' An S4 class to represent the expression data used for inferring
#' ligand-receptor interactions.
#'
#' @slot ncounts   Normalized read count matrix. Row names must be set to HUGO
#' official gene symbols.
#' @slot log.transformed  Logical indicating whether values in
#' \code{ncounts} were log2-transformed.
#' @slot normalization    Name of the normalization method.
#' @slot param            List containing the statistical model parameters.
#' @slot initial.organism         Organism for which the data were obtained.
#' @slot initial.orthologs        List of genes for which human
#' orthologs exist.
#' @export
#' @examples
#' BSRDataModel(
#'     counts = matrix(1.5,
#'                 nrow = 2, ncol = 2,
#'                 dimnames = list(c("A", "B"), c("C", "D"))
#'                    ),
#'     method = "TC",
#'     log.transformed = TRUE,
#'     normalize = FALSE,
#'     min.LR.found = 0
#' )
#'
setClass("BSRDataModel",
    slots = c(
        initial.organism = "character",
        initial.orthologs = "list",
        ncounts = "matrix",
        log.transformed = "logical",
        normalization = "character",
        param = "list"
    ),
    prototype = list(
        initial.organism = "hsapiens",
        initial.orthologs = list(), #list("A", "B", "C"),
        ncounts = matrix(1.0,
            nrow = 2, ncol = 1,
            dimnames = list(c("A", "B"), "C")
        ),
        log.transformed = FALSE,
        normalization = "UQ",
        param = list(spatial.smooth = FALSE)
    )
)

setValidity(
    "BSRDataModel",
    function(object) {
        if (!is.matrix(object@ncounts)) {
            return("specified normalized counts are not a matrix")
        }
        if (!is.numeric(object@ncounts)) {
            return("specified normalized counts are not numeric")
        }
        if (is.null(row.names(object@ncounts))) {
            return("specified normalized counts have no row names set")
        }
        if (nchar(object@normalization) == 0) {
            return("Normalization method must be set")
        }

        TRUE
    }
)

# NOTE: the constructor of BRSDataModel class is defined in the
# file dataPrepare.R for "historical" reasons

setMethod(
    "show", "BSRDataModel",
    function(object) {
        cat("Expression values are log2-transformed: ", object@log.transformed,
            "\n",
            sep = ""
        )
        cat("Normalization method: ", object@normalization, "\n", sep = "")
        cat("Organism: ", object@initial.organism, "\n", sep = "")
        cat("Statistical model parameters:\n")
        utils::str(object@param)
        cat("Expression data:\n")
        if (ncol(object@ncounts) > 8) {
            print(utils::head(object@ncounts[, seq_len(8)]))
        } else {
            print(utils::head(object@ncounts))
        }
    }
)


# Accessors & setters ========================================================

setGeneric("initialOrganism", signature="x",
    function(x) standardGeneric("initialOrganism")
)
#' organism accessor
#'
#' @name initialOrganism
#' @aliases initialOrganism,BSRDataModel-method
#' @param x Object BSRDataModel
#' @return initialOrganism
#' @examples
#' bsrdm <- BSRDataModel(
#'     counts = matrix(1.5,
#'                 nrow = 2, ncol = 2,
#'                 dimnames = list(c("A", "B"), c("C", "D"))
#'                    ),
#'     method = "TC",
#'     log.transformed = TRUE,
#'     normalize = FALSE,
#'     min.LR.found = 0
#' )
#' initialOrganism(bsrdm)
#' @export
setMethod("initialOrganism", "BSRDataModel", function(x) x@initial.organism)


setGeneric("initialOrthologs", signature="x",
    function(x) standardGeneric("initialOrthologs")
)
#' Model parameter accessor
#'
#' @name initialOrthologs
#' @aliases initialOrthologs,BSRDataModel-method
#' @param x Object BSRDataModel
#' @return initialOrthologs
#' @examples
#' bsrdm <- BSRDataModel(
#'     counts = matrix(1.5,
#'                 nrow = 2, ncol = 2,
#'                 dimnames = list(c("A", "B"), c("C", "D"))
#'                    ),
#'     method = "TC",
#'     log.transformed = TRUE,
#'     normalize = FALSE,
#'     min.LR.found = 0
#' )
#' initialOrthologs(bsrdm)
#' @export
setMethod("initialOrthologs", "BSRDataModel", function(x) x@initial.orthologs)


setGeneric("ncounts", signature="x",
    function(x) standardGeneric("ncounts")
)
#' Normalized count matrix accessor
#'
#' @name ncounts
#' @aliases ncounts,BSRDataModel-method
#' @param x object BSRDataModel
#' @return ncounts
#' @examples
#' bsrdm <- BSRDataModel(
#'     counts = matrix(1.5,
#'                 nrow = 2, ncol = 2,
#'                 dimnames = list(c("A", "B"), c("C", "D"))
#'                    ),
#'     method = "TC",
#'     log.transformed = TRUE,
#'     normalize = FALSE,
#'     min.LR.found = 0
#' )
#' ncounts(bsrdm)
#' @export
setMethod("ncounts", "BSRDataModel", function(x) x@ncounts)


setGeneric("ncounts<-", signature=c("x", "value"),
    function(x, value) standardGeneric("ncounts<-")
)
#' Normalized count matrix setter (internal use only)
#'
#' @param x object BSRDataModel
#' @param value value to be set for BSRDataModel
#' @return returns \code{NULL}
#' @keywords internal
setMethod("ncounts<-", "BSRDataModel", function(x, value) {
    x@ncounts <- value
    methods::validObject(x)
    x
})


setGeneric("parameters", signature="x",
    function(x) standardGeneric("parameters")
)
#' Model parameter accessor
#'
#' @name parameters
#' @aliases parameters,BSRDataModel-method
#' @param x BSRDataModel oject
#' @return param
#' @examples
#' bsrdm <- BSRDataModel(
#'     counts = matrix(1.5,
#'                 nrow = 2, ncol = 2,
#'                 dimnames = list(c("A", "B"), c("C", "D"))
#'                    ),
#'     method = "TC",
#'     log.transformed = TRUE,
#'     normalize = FALSE,
#'     min.LR.found = 0
#' )
#' parameters(bsrdm)
#' @export
setMethod("parameters", "BSRDataModel", function(x) x@param)


setGeneric("parameters<-", signature=c("x", "value"),
    function(x, value) standardGeneric("parameters<-")
)
#' Parameters dataModel setter (internal use only)
#'
#' @param x object BSRDataModel
#' @param value value to be set for BSRDataModel
#' @return returns \code{NULL}
#' @keywords internal
setMethod("parameters<-", "BSRDataModel", function(x, value) {
    x@param <- value
    methods::validObject(x)
    x
})


setGeneric("logTransformed", signature="x",
    function(x) standardGeneric("logTransformed")
)
#' log.transformed accessor
#'
#' @name logTransformed
#' @aliases logTransformed,BSRDataModel-method
#' @param x Object BRSDataModel
#' @return logTransformed
#' @examples
#' bsrdm <- BSRDataModel(
#'     counts = matrix(1.5,
#'                 nrow = 2, ncol = 2,
#'                 dimnames = list(c("A", "B"), c("C", "D"))
#'                    ),
#'     method = "TC",
#'     log.transformed = TRUE,
#'     normalize = FALSE,
#'     min.LR.found = 0
#' )
#' logTransformed(bsrdm)
#' @export
setMethod("logTransformed", "BSRDataModel", function(x) x@log.transformed)


setGeneric("normalization", signature="x",
    function(x) standardGeneric("normalization")
)
#' Normalization accessor
#'
#' @name normalization
#' @aliases normalization,BSRDataModel-method
#' @param x object BSRDatamModel
#' @return normalization
#' @examples
#' bsrdm <- BSRDataModel(
#'     counts = matrix(1.5,
#'                 nrow = 2, ncol = 2,
#'                 dimnames = list(c("A", "B"), c("C", "D"))
#'                    ),
#'     method = "TC",
#'     log.transformed = TRUE,
#'     normalize = FALSE,
#'     min.LR.found = 0
#' )
#' normalization(bsrdm)
#' @export
setMethod("normalization", "BSRDataModel", function(x) x@normalization)


setGeneric("learnParameters", signature="obj",
    function(obj, ...) standardGeneric("learnParameters")
)
#' Training of BulkSignalR model parameters
#'
#' Unique entry point for training the parameters behind
#' BulkSignalR statistical models.
#'
#' @name learnParameters
#' @aliases learnParameters,BSRDataModel-method
#'
#' @param obj   A BSRDatamodel without learned paramaters.
#' @param plot.folder   A folder name for generating control plots.
#' @param filename      Name of the output plot.
#' @param verbose       A logical activating progress messages for the user.
#' @param n.rand.LR     The number of random expression matrices to use for
#'   learning the ligand-receptor correlation distribution.
#' @param n.rand.RT     The number of random expression matrices to use for
#'   learning the receptor-target genes correlation distribution.
#' @param with.complex    A logical indicating whether receptor co-complex
#'   members should be included in the target genes.
#' @param max.pw.size     Maximum pathway size to consider from the pathway
#'   reference.
#' @param min.pw.size     Minimum pathway size to consider from the pathway
#'   reference.
#' @param min.positive    Minimum number of target genes to be found in a given
#'   pathway.
#' @param quick           A logical indicating whether approximate parameters
#'   for the receptor-target correlations should be used.
#' @param null.model      The null model to use for Spearman correlation
#'   null distributions.
#' @param min.corr.LR      The minimum ligand-receptor correlation required.
#'  
#' @details Estimates the model parameters that are stored in the
#'   slot \code{param}.
#'
#'   In a reference pathway, i.e., a Reactome pathway or the genes of a
#'   GOBP term, the target genes are the genes coding for proteins forming a
#'   complex with the receptor and the genes in the pathway downstream the
#'   receptor, which are given as regulated by the pathway. If
#'   \code{with.complex} is set to \code{FALSE}, then only the regulated genes
#'   are considered. Participation to a complex, being regulated, and
#'   pathway directed topologies are defined by Reactome and KEGG pathways
#'   as provided by PathwayCommons.
#'
#'   The \code{min.pw.size}, \code{max.pw.size}, and \code{min.positive}
#'   parameters should be identical to the
#'   values intended when searching for ligand-receptor pairs with
#'   \code{\link{.getCorrelatedLR}}) and  \code{\link{.checkReceptorSignaling}})
#'   Although the statistical distributions are rather robust, it is not
#'   advisable to use different parameters that could introduce unanticipated
#'   biases, but for saving compute time and exploring.
#'
#'   The maximum pathway size is used to limit the redundancy inherent to GOBP
#'   and Reactome. The minimum pathway size is used to avoid overspecific,
#'   noninformative results.
#'
#'   BulkSignalR approach relies on modeling (Spearman) correlations and
#'   different models of null distributions are available for this purpose
#'   (parameter \code{null.model}). By default, the "automatic" option is
#'   selected meaning that censored normal and mixed normal as well as
#'   an empirical model based on Gaussian kernels (R \code{density()} function)
#'   are compared to pick the one closest to the data. Preference is given
#'   to normal and then mixture of normal over the empirical version for
#'   comparable quality of fit. It is also to bypass the automatic selection.
#'   Fitting of an alpha-stable distribution is quite time consuming as the
#'   computation of its PDF is compute-intensive. Finally, in the automaic
#'   selection mode, the choice of the actual model will be done based on
#'   the L-R null assuming a similar shape for the R-T null (with
#'   different parameters though, unless \code{quick} was set to \code{TRUE}).
#'
#' Note that since the introduction of the use.full.network parameter
#' (April 29, 2024) in the BSRInference method parameters,
#' the pathway sizes are always computed before potential
#' intersection with the observed data (use.full.network set to FALSE) for
#' consistency. Accordingly, the minimum and maximum pathway default values
#' have been raised from 5 & 200 to 5 & 400 respectively. By default,
#' use.full.network is set to TRUE, meaning no intersection and hence larger
#' pathways.
#'
#' @return A BSRDataModel object with trained model parameters
#'
#' @export
#'
#' @examples
#' data(sdc, package = "BulkSignalR")
#' idx <- sample(nrow(sdc), 4000)
#' bsrdm <- BSRDataModel(sdc[idx, c("N22","SDC17")],min.LR.found = 20)
#' bsrdm <- learnParameters(bsrdm, n.rand.LR = 1L,
#' verbose=FALSE,quick=TRUE)
#' @importFrom methods new
setMethod(
    "learnParameters", "BSRDataModel",
    function(obj, plot.folder = NULL,
        verbose = FALSE, n.rand.LR = 5L, n.rand.RT = 2L, with.complex = TRUE,
        max.pw.size = 400, min.pw.size = 5, min.positive = 4, quick = FALSE,
        null.model = c(
            "automatic", "mixedNormal", "normal", "kernelEmpirical",
            "empirical", "stable"
            ), 
        filename = NULL, min.corr.LR = -1) {


        parameters(obj)$n.rand.LR <- as.integer(n.rand.LR)
        if (parameters(obj)$n.rand.LR < 1) {
            stop("Parameter n.rand.LR must be an integer > 0")
        }
        parameters(obj)$n.rand.RT <- as.integer(n.rand.RT)
        if (parameters(obj)$n.rand.RT < 1) {
            stop("Parameter n.rand.RT must be an integer > 0")
        }
        parameters(obj)$plot.folder <- plot.folder
        if (!is.null(plot.folder) && !file.exists(plot.folder)) {
            stop("The provided plot.folder does not exist")
        }
        parameters(obj)$file.name <- filename
        parameters(obj)$with.complex <- with.complex
        parameters(obj)$max.pw.size <- trunc(max.pw.size)
        if (parameters(obj)$max.pw.size < 1) {
            stop("Parameter max.pw.size must be an integer > 0")
        }
        parameters(obj)$min.pw.size <- trunc(min.pw.size)
        if (parameters(obj)$min.pw.size < 1 || 
            parameters(obj)$min.pw.size > parameters(obj)$max.pw.size) {
            stop(
                "Parameter min.pw.size must be",
                "an integer > 0 and <= than max.pw.size"
            )
        }
        parameters(obj)$min.positive <- trunc(min.positive)
        if (parameters(obj)$min.positive < 1) {
            stop("Parameter min.positive must be an integer > 0")
        }

        if (min.corr.LR < -1 || min.corr.LR > 1) {
            stop("min.corr.LR must lie in [-1;1]")
        }

        parameters(obj)$quick <- quick
        null.model <- match.arg(null.model)
        if (null.model == "normal") {
            trainModel <- .getGaussianParam
        } else if (null.model == "mixedNormal") {
            trainModel <- .getMixedGaussianParam
        } else if (null.model == "kernelEmpirical") {
            trainModel <- .getKernelEmpiricalParam
        } else if (null.model == "empirical") {
            trainModel <- .getEmpiricalParam
        } else if (null.model == "stable") {
            trainModel <- .getAlphaStableParam
        } else if (null.model == "automatic") {
            trainModel <- NULL
        } else {
            stop("No valid null model specified")
        }

        # LR correlation null ----------------

        if (verbose) {
            message("Learning ligand-receptor correlation null distribution...")
        }

        parameters(obj)$min.corr.LR <- min.corr.LR
        
        ds.LR.null <- .getEmpiricalNullCorrLR(obj)

        rc <- ds.LR.null[[1]]$corr
        if (length(ds.LR.null) > 1) {
            for (i in 2:length(ds.LR.null)) rc <- c(rc, ds.LR.null[[i]]$corr)
        }
        parameters(obj)$LR.0$n <- length(rc)

        # Null distribution model
        if (!is.null(plot.folder)) {
            file.name <- paste0(plot.folder, "/", filename, "_LR-null.pdf")
        } else {
            file.name <- NULL
        }

        if (is.null(trainModel)) {
            # automatic selection of the model
            np <- try(.getGaussianParam(rc, 
                "LR correlation (null)"), silent = TRUE)
            if (inherits(np, "try-error")) {
                np <- NULL
            }
            mp <- try(.getMixedGaussianParam(rc, "LR correlation (null)"),
                silent = TRUE
            )
            if (inherits(mp, "try-error")) {
                mp <- NULL
            }
            kp <- .getKernelEmpiricalParam(rc, "LR correlation (null)")
            if (verbose) {
              message("Automatic null model choice:")
              if (is.null(np)) {
                message("  Censored normal estimation did not converge")
              } else {
                message(
                    "  Censored normal D=", np$D,
                    ", Chi2=", np$Chi2
                )
              }
              if (is.null(mp)) {
                message("  Censored Mixture of normals",
                    " estimation did not converge")
              } else {
                message(
                    "  Censored mixture D=", mp$D,
                    ", Chi2=", mp$Chi2
                )
              }
              message(
                  "  Gaussian kernel empirical D=", kp$D,
                  ", Chi2=", kp$Chi2
              )
            }
            npchi <- ifelse(is.null(np), 100, sqrt(np$Chi2))
            mpchi <- ifelse(is.null(mp), 100, sqrt(mp$Chi2))
            kpchi <- sqrt(kp$Chi2)
            if ((npchi < 1.25 * mpchi) && (npchi < 2 * kpchi)) {
              trainModel <- .getGaussianParam
              if (verbose) {
                  message("  ==> select censored normal")
              }
            } else if (mpchi < 2 * kpchi) {
              trainModel <- .getMixedGaussianParam
              if (verbose) {
                  message("  ==> select censored mixture of 2 normals")
              }
            } else {
              trainModel <- .getKernelEmpiricalParam
              if (verbose) {
                  message("  ==> select Gaussian kernel-based empirical")
              }
            }
        }
        # actual training with the chosen model
        gp <- trainModel(rc, "LR correlation (null)",
                         verbose = verbose,
                         file.name = file.name
        )
        parameters(obj)$LR.0$model <- gp
        
        # RT correlation null ------------------------------------
        
        if (parameters(obj)$quick) {
          # RT correlations are assumed to be equal to LR correlations
          if (verbose) {
              message("Quick learning, receptor-target correlation null",
                  " distribution assumed to be equal to ligand-receptor...")
          }
          parameters(obj)$RT.0$n <- parameters(obj)$LR.0$n
          parameters(obj)$RT.0$model <- parameters(obj)$LR.0$model
        } else {
          # RT correlations are actually learnt
          if (verbose) {
              message("Learning receptor-target",
                  " correlation null distribution...")
          }
          ds.RT.null <- .getEmpiricalNull(obj)
          
          t <- ds.RT.null[[1]]
          if (length(ds.RT.null) > 1) {
            for (i in 2:length(ds.RT.null)) t <- rbind(t, ds.RT.null[[i]])
          }
          above <- unlist(strsplit(t$target.corr, split = "\\|"))
          r.corrs <- NULL
          for (i in seq_len(length(above))) {
            corr <- as.numeric(strsplit(above[i], split = ";")[[1]])
            r.corrs <- c(r.corrs, corr)
          }
          if (null.model == "stable") {
            # sub-sample randomized R-T correlations to limit compute time
            r.corrs <- sample(r.corrs, parameters(obj)$LR.0$n)
          }
          parameters(obj)$RT.0$n <- length(r.corrs)
          
          # fit null model
          if (!is.null(plot.folder)) {
            file.name <- paste0(plot.folder, "/", filename, "_RT-null.pdf")
          } else {
            file.name <- NULL
          }
          gp <- trainModel(r.corrs, "RT correlation (null)",
                           verbose = verbose, file.name = file.name
          )
          parameters(obj)$RT.0$model <- gp
        }
        
        if (verbose) {
            message("Learning of statistical model parameters completed")
        }
        obj
    }
    
) # learnParameters


# Scoring of gene signatures in a BSRSignature object ==========================

setGeneric("scoreLRGeneSignatures", signature="obj",
    function(obj, ...) standardGeneric("scoreLRGeneSignatures")
)
#' Score ligand-receptor gene signatures
#'
#' Compute ligand-receptor gene signature scores over a BSRDataModel.
#'
#' @name scoreLRGeneSignatures
#' @aliases scoreLRGeneSignatures,BSRDataModel-method
#'
#' @param obj           A BSRDataModel object.
#' @param sig           A BSRSignature object.
#' @param LR.weight    A number between 0 and 1 defining the relative weight
#' of the ligand and the receptor in the signature.
#' @param robust       A logical indicating that z-scores should be computed
#' with median and MAD instead of mean and standard deviation.
#' @param name.by.pathway     A logical indicating whether row names of the
#' resulting score matrix should be pathway names.
#' @param rownames.LRP A logical indicating, in case \code{name.by.pathway}
#' was set to TRUE, whether ligand and receptor names should be added on top.
#' No role if \code{name.by.pathway} was set to FALSE.
#' @param abs.z.score  A logical to use absolute z-scores (useful if the
#' activity of a paythway is reported by a mixture of up- and down-genes
#' whose z-score averages might hide actual activity).
#' @return A matrix containing the scores of each ligand-receptor gene
#' signature in each sample.
#'
#' @export
#' @examples
#' data(bsrdm, package = "BulkSignalR")
#' data(bsrinf, package = "BulkSignalR")
#' 
#' bsrinf.redBP <- reduceToBestPathway(bsrinf)
#' bsrsig.redBP <- BSRSignature(bsrinf.redBP, qval.thres = 0.001)
#' res <-scoreLRGeneSignatures(bsrdm, bsrsig.redBP,
#'     name.by.pathway = FALSE
#' )
#' @importFrom foreach %do% %dopar%
#' @importFrom methods is
#' @importFrom matrixStats rowMeans2 colSums2
setMethod("scoreLRGeneSignatures", "BSRDataModel", function(obj,
    sig, LR.weight = 0.5, robust = FALSE,
    name.by.pathway = FALSE, abs.z.score = FALSE,
    rownames.LRP = FALSE) {

    if (!is(sig, "BSRSignature")) {
        stop("sig must be a BSRSignature object")
    }
    if (LR.weight <= 0 || LR.weight >= 1) {
        stop("LRweight must reside in (0;1)")
    }


    if (initialOrganism(obj) != "hsapiens") {
        all.genes <- unlist(initialOrthologs(obj))
    } else {
        all.genes <- rownames(ncounts(obj))
    }

    # intersect signature gene names with RNA-seq data
    ncounts <- ncounts(obj)

    ligands <- vector("list", length(ligands(sig)))
    receptors <- vector("list", length(receptors(sig)))
    tg.genes <- vector("list", length(tgGenes(sig)))


    for (i in seq_along(ligands(sig))) {
        ligands[[i]] <- intersect(ligands(sig)[[i]], all.genes)
    }
    for (i in seq_along(receptors(sig))) {
        receptors[[i]] <- intersect(receptors(sig)[[i]], all.genes)
    }
    for (i in seq_along(tgGenes(sig))) {
        tg.genes[[i]] <- intersect(tgGenes(sig)[[i]], all.genes)
    }

    good <- lengths(ligands) > 0 &
        lengths(receptors) > 0 &
        lengths(tg.genes)  > 0

    ligands <- ligands[good]
    receptors <- receptors[good]
    tg.genes <- tg.genes[good]
    pathways <- pathways(sig)[good]

    # scale ncounts
    if (logTransformed(obj)) {
        ncounts <- 2**ncounts
    }
    if (robust) {
        z <- (ncounts - apply(ncounts, 1, stats::median)) 
        z <- z / apply(ncounts, 1, stats::mad)
    } else {
        z <- (ncounts - matrixStats::rowMeans2(ncounts)) / 
        apply(ncounts, 1, stats::sd)
    }
    if (abs.z.score) {
        z <- abs(z)
    }

    if (initialOrganism(obj) != "hsapiens") {
        rownames(z) <- all.genes
    }

    # compute the LR gene signatures
    i <- NULL
    pwn <- foreach::foreach(i = seq_len(length(pathways)), .combine = c) %do% {
        if (name.by.pathway) {
            if (rownames.LRP) {
                paste0(
                    "{", paste(ligands[[i]], collapse = ";"), "} / {",
                    paste(receptors[[i]], collapse = ";"), "} | ", pathways[[i]]
                )
            } else {
                pathways[[i]]
            }
        } else if (!name.by.pathway) {
            paste0(
                "{", paste(ligands[[i]], collapse = ";"), "} / {",
                paste(receptors[[i]], collapse = ";"), "}"
            )
        }
    }

    res <- matrix(0, nrow = length(pathways), ncol = ncol(ncounts), 
        dimnames = list(pwn, colnames(ncounts)))
    for (i in seq_len(length(pathways))) {
        # average ligand z-score
        zz <- z[ligands[[i]], ]
        if (is.matrix(zz)) {
            mL <- matrixStats::colSums2(zz) / length(ligands[[i]])
        } else {
            mL <- zz
        }

        # average receptor z-score
        zz <- z[receptors[[i]], ]
        if (is.matrix(zz)) {
            mR <- matrixStats::colSums2(zz) / length(receptors[[i]])
        } else {
            mR <- zz
        }

        # average target gene z-score
        zz <- z[tg.genes[[i]], ]
        if (is.matrix(zz)) {
            mT <- matrixStats::colSums2(zz) / length(tg.genes[[i]])
        } else {
            mT <- zz
        }

        res[i, ] <- LR.weight * 0.5 * (mL + mR) + (1 - LR.weight) * mT
    }

    res
}) # scoreLRGeneSignatures

# Pure display functions (for convenience) =====================================

#' Internal function to cut extreme values from a matrix
#'
#' @param m         A matrix.
#' @param p          Proportion of top and bottom values for thresholding.
#' @return A matrix with values beyond top and bottom thresholds repaced by
#' the latter thresholds.
#' @keywords internal
.cutExtremeValues <- function(m, p) {
    thres.lo <- stats::quantile(m, prob = p)
    thres.hi <- stats::quantile(m, prob = 1 - p)
    m[m > thres.hi] <- thres.hi
    m[m < thres.lo] <- thres.lo
    m
}

#' Bubble Plot to explore LR & Pathways
#'
#' Quick check to observe LR - Pathways association
#' with their respective correlation
#' and Q-values.
#'
#' @param bsrinf     BulkSignalR inference object.
#' @param pathways  Vector of pathway names to keep.
#' @param qval.thres     Maximum Q-value.
#' @param filter.L     Vector of ligands to keep.
#' @param filter.R     Vector of receptors to keep.
#' @param color     Main color used for the gradient.
#' @param pointsize Global point size.
#'
#' @return  A bubble plot displayed in the current viewport.
#'
#' This is a convenience function to propose a simple way
#' of representing LR - Pathways association
#' with their respective correlation
#' and Q-values.
#'
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#' pathways <- LRinter(bsrinf)[1,c("pw.name")]
#' bubblePlotPathwaysLR(bsrinf,
#' pathways = pathways,
#' qval.thres = 0.1,
#' color = "red",
#' pointsize = 8
#' )
#' @import ggplot2
bubblePlotPathwaysLR <- function(
    bsrinf,
    pathways,
    qval.thres = 1,
    filter.L = NULL,
    filter.R = NULL,
    color = "#16a647",
    pointsize = 6) {
    filtered.brinf <- LRinter(bsrinf)
    filtered.brinf <- filtered.brinf[filtered.brinf$qval < qval.thres, ]

    if (!is.null(filter.R) | !is.null(filter.L)) {
        filtered.brinf <- filtered.brinf[filtered.brinf$L %in% filter.L | 
        filtered.brinf$R %in% filter.R, ]
    }

    filtered.brinf$LR <- paste(filtered.brinf$L, filtered.brinf$R, sep = " / ")

    filtered.brinf <- filtered.brinf[, c("LR", "pw.name", "LR.corr", "qval")]
    filtered.brinf$log10.qval <- -log10(filtered.brinf$qval)
    filtered.brinf$log10.LR.corr <- -log10(abs(filtered.brinf$LR.corr)) 

    filtered.brinf <- filtered.brinf[filtered.brinf$pw.name %in% pathways, ]

    if (dim(filtered.brinf)[1] == 0) {
        stop("The pathways you have selected do no exist.")
    }

    limit.P <- 8
    if (length(pathways) >= limit.P) {
        message("We recommend less than ", limit.P, " pathways.")
        message("Too many pathways were given in input.")
    }
    limit.LR <- 50
    if (length(unique(filtered.brinf$LR)) > limit.LR) {
        message("Too many LR interactions detected (",
            length(unique(filtered.brinf$LR)), ").")
        message("We recommend less than ", limit.LR,
            " LR interactions to visualize.")
        message("Try to reduce (Qval-Threshold, number of pathways...).")
    }

    message(length(unique(filtered.brinf$LR)), " LR interactions detected.")

    plot(ggplot2::ggplot(
        filtered.brinf,
        ggplot2::aes_string(x = "LR", y = "pw.name")
    ) +
        ggplot2::geom_point(ggplot2::aes_string(
            size = "log10.LR.corr",
            fill = "log10.qval"
        ), alpha = 0.75, shape = 21) +
        labs(
            x = "", y = "", size = "-log10 (LR.corr)",
            fill = "-log10 (Qval)"
        ) +
        ggplot2::theme(
            legend.key = ggplot2::element_blank(),
            legend.key.size = unit(0.2, "cm"),
            legend.position = "right", legend.box = "horizontal",
            axis.text.x = ggplot2::element_text(
                colour = "black",
                size = pointsize, angle = 90, vjust = 0.3, hjust = 1
            ),
            axis.text.y = ggplot2::element_text(
                colour = "black",
                face = "bold", size = pointsize
            ),
            legend.text = ggplot2::element_text(
                size = pointsize - 1,
                face = "bold", colour = "black"
            ),
            legend.title = ggplot2::element_text(
                size = pointsize,
                face = "bold"
            ),
            panel.background = ggplot2::element_blank(),
            # panel.grid.minor = ggplot2::element_line(size = 0.25,
            # linetype = 'solid',colour = "grey"),
            panel.border = ggplot2::element_rect(
                colour = "black",
                fill = NA, size = 1.2
            ),
            panel.grid.major = ggplot2::element_line(colour = "grey95")
        ) +
        ggplot2::scale_fill_gradient(
            low = "white",
            high = color, space = "Lab", na.value = "grey50",
            guide = "colourbar", aesthetics = "fill"
        ) +
        ggplot2::scale_y_discrete(limits = rev(levels(filtered.brinf$pw.name))))

} # bubblePlotPathwaysLR


#' Heatmap function for gene signature expression
#'
#' Generate one heatmap used by
#' \code{signatureHeatmaps}.
#'
#' @param counts  Matrix of counts.
#' @param name    Name of the heatmap to generate.
#' @param h.height  Heatmap  height in cm.
#' @param fontsize  Font size for row (gene) names.
#' @param col.fontsize  Font size for column (sample) names.
#' @param legend.fontsize  Font size for the legends.
#' @param annot.fontsize   Font size for column annotation names.
#' @param scoring Vector of sample scores for a
#' chosen pathway. If NULL, then no column annotation is produced.
#' @param cols.scoring   Fixed colorRamp2 object.
#' @param hcl.palette   Palette from
#' HCL colormaps supported by ComplexHeatmap.
#'
#' @return A ComplexHeatmap object.
#'
#' This is a convenience function that relies
#' on the \code{ComplexHeatmap}
#' package.
#'
#' @import ComplexHeatmap
#' @importFrom circlize colorRamp2
#' @keywords internal
.customheatmap <- function(
    counts,
    name,
    fontsize = 6,
    col.fontsize = 6,
    legend.fontsize = 8,
    annot.fontsize = 8,
    h.height = 3,
    scoring = NULL,
    cols.scoring = NULL,
    hcl.palette = "Blues 3",
    show_column_names = FALSE) {
  
  counts.scaled <- t(scale(t(counts)))
  
  cols <- circlize::colorRamp2(
    breaks = c(-1, 0, 3),
    hcl_palette = hcl.palette, reverse = TRUE
  )
  
  if (!is.null(scoring)){
    top.annotation <- ComplexHeatmap::HeatmapAnnotation(
      LR.scoring = as.vector(scoring),
      border = c(LR.scoring = TRUE),
      show_legend = TRUE,
      simple_anno_size = grid::unit(2.5, "mm"),
      show_annotation_name = TRUE,
      col = list(LR.scoring = cols.scoring),
      annotation_name_gp= gpar(fontsize = annot.fontsize)
    )
  }
  
  di.gene <- stats::dist(counts.scaled)
  hc.gene <- stats::hclust(di.gene, method = "ward.D")
  dend.row <- stats::as.dendrogram(hc.gene)
  
  di.spl <- stats::dist(t(counts.scaled))
  hc.spl <- stats::hclust(di.spl, method = "ward.D")
  dend.spl <- stats::as.dendrogram(hc.spl)
  
  ComplexHeatmap::ht_opt(
    heatmap_border = TRUE,
    annotation_border = FALSE
  )
  
  if (!is.null(scoring)){
    ComplexHeatmap::Heatmap(counts.scaled,
                            name = name,
                            cluster_rows = dend.row, cluster_columns = dend.spl,
                            show_row_dend = FALSE, show_column_dend = TRUE,
                            col = cols, show_row_names = TRUE,
                            show_column_names = show_column_names,
                            use_raster = TRUE, raster_device = "png",
                            raster_quality = 8, raster_by_magick = FALSE,
                            rect_gp = grid::gpar(col = "white"),
                            row_names_gp = grid::gpar(fontsize = fontsize),
                            column_names_gp = grid::gpar(fontsize = col.fontsize),
                            top_annotation = top.annotation,
                            height = grid::unit(h.height, "cm"),
                            column_gap = grid::unit(0.5, "mm"),
                            heatmap_legend_param = list(
                              labels_gp = gpar(fontsize = legend.fontsize)
                            )
    )
  }
  else{
    ComplexHeatmap::Heatmap(counts.scaled,
                            name = name,
                            cluster_rows = dend.row, cluster_columns = dend.spl,
                            show_row_dend = FALSE, show_column_dend = TRUE,
                            col = cols, show_row_names = TRUE,
                            show_column_names = show_column_names,
                            use_raster = TRUE, raster_device = "png",
                            raster_quality = 8, raster_by_magick = FALSE,
                            rect_gp = grid::gpar(col = "white"),
                            row_names_gp = grid::gpar(fontsize = fontsize),
                            column_names_gp = grid::gpar(fontsize = col.fontsize),
                            height = grid::unit(h.height, "cm"),
                            column_gap = grid::unit(0.5, "mm"),
                            heatmap_legend_param = list(
                              labels_gp = gpar(fontsize = legend.fontsize)
                            )
    )
  }
  
} # .customheatmap


#' Heatmap function to dissect one pathway signature
#'
#' Plots a stack of three heatmaps to assess the expression of
#' the target genes or proteins in a chosen pathway, the receptor expressions,
#' and the ligand expressions.
#'
#' @param pathway        The chosen pathway name.
#' @param bsrdm     BulkSignalR data model object.
#' @param bsrsig     BulkSignalR signature object.
#' @param heights    A vector of 3 heights (in cm) for the 3 heatmaps.
#' @param fontsize    Font size for the gene names.
#' @param legend.fontsize  Font size for the legends.
#' @param title.fontsize   Font size for the pathway name as plot title.
#' @param col.fontsize  Font size for column (sample) names.
#' @param annot.fontsize   Font size for column annotation names.
#' @param ht.gap   Space between heatmaps (in mm).
#' @param show_column_names   Add column names in the heatmaps.

#' @return  A plot is created.
#'
#' @export
#' @examples
#' data(bsrdm, package = "BulkSignalR")
#' data(bsrinf, package = "BulkSignalR")
#' if(FALSE){
#' bsrinf.redP <- reduceToPathway(bsrinf)
#' bsrinf.redPBP <- reduceToBestPathway(bsrinf)
#' bsrsig.redPBP <- BSRSignature(bsrinf, qval.thres = 1)
#' pathway1 <- pathways(bsrsig.redPBP)[1]
#' signatureHeatmaps(
#' pathway = pathway1,
#' bsrdm = bsrdm,
#' bsrsig = bsrsig.redPBP,
#' show_column_names = TRUE)
#' }
#' @import ComplexHeatmap
#' @importFrom ComplexHeatmap %v%
#' @importFrom circlize colorRamp2
#' @import grid
signatureHeatmaps <- function(pathway,
                              bsrdm,
                              bsrsig,
                              heights = c(4,2,4),
                              fontsize = 6,
                              legend.fontsize = 8,
                              title.fontsize = 8,
                              col.fontsize = 6,
                              annot.fontsize = 8,
                              ht.gap = 3,
                              show_column_names = TRUE) {
  
  idx.path.sig <- which(pathways(bsrsig) == pathway)
  
  if (rlang::is_empty(idx.path.sig)) {
    stop("Pathway is not defined in signature.")
  }
  if (!is.numeric(heights)){
    stop("heights must be provided as a numerical vector.")
  }
  if (length(heights) != 3){
    stop("Three heights must be provided exactly.")
  }
  
  scoresPathway <- scoreLRGeneSignatures(bsrdm, bsrsig)
  
  counts <- ncounts(bsrdm)
  
  filter.L <- unique(unlist(ligands(bsrsig)[idx.path.sig]))
  
  counts.L <- counts[filter.L, ]
  
  palette.L <- "RdPu"
  cols.L <- circlize::colorRamp2(breaks = c(-1, 0, 1),
                                 hcl_palette = palette.L, reverse = TRUE)
  
  filter.R <- unique(unlist(receptors(bsrsig)[idx.path.sig]))
  filter.T <- unique(unlist(tgGenes(bsrsig)[idx.path.sig]))
  
  # Remove in receptors, genes that are potential targets.
  filter.R <- filter.R[!filter.R %in% filter.T]
  
  counts.R <- counts[filter.R, ]
  palette.R <- "YlGn"
  cols.R <- circlize::colorRamp2(breaks = c(-1, 0, 1),
                                 hcl_palette = palette.R, reverse = TRUE)
  
  counts.T <- counts[filter.T, ]
  palette.T <- "Blues 3"
  cols.T <- circlize::colorRamp2(breaks = c(-1, 0, 1),
                                 hcl_palette = palette.T, reverse = TRUE)
  
  cols.scoring <- circlize::colorRamp2(breaks = c(-1, 0, 1),
                                       colors = c("blue", "white", "red"))
  
  abundance.samples <- dim(counts.T)[2]
  abundance.genes <- c(dim(counts.T)[1], dim(counts.R)[1], dim(counts.L)[1])
  
  if (length(idx.path.sig) == 1){
    scoring <- as.vector(scoresPathway[idx.path.sig[1],])
  }
  else{
    scoring <- colMeans(scoresPathway[idx.path.sig,])
  }
  
  p.T <- .customheatmap(
    counts = counts.T,
    name = "Targets",
    h.height = heights[1],
    scoring = scoring,
    hcl.palette = palette.T,
    cols.scoring = cols.scoring,
    show_column_names = FALSE,
    fontsize = fontsize,
    legend.fontsize = legend.fontsize,
    annot.fontsize = annot.fontsize
  )
  
  p.R <- .customheatmap(
    counts = counts.R,
    name = "Receptors",
    h.height = heights[2],
    hcl.palette = palette.R,
    show_column_names = FALSE,
    fontsize = fontsize,
    legend.fontsize = legend.fontsize,
  )
  
  p.L <- .customheatmap(
    counts = counts.L,
    name = "Ligands",
    h.height = heights[3],
    hcl.palette = palette.L,
    show_column_names = show_column_names,
    fontsize = fontsize,
    legend.fontsize = legend.fontsize,
    col.fontsize = col.fontsize,
  )
  
  ComplexHeatmap::draw(p.T %v% p.R %v% p.L,
                       ht_gap = grid::unit(ht.gap, "mm"),
                       column_title = pathway,
                       column_title_gp = gpar(fontsize=title.fontsize))
  
} # signatureHeatmaps


#' Heatmap function for LR scores
#'
#' Generate a heatmap representing ligand-receptor gene
#' signature scores.
#'
#' @param mat.c         A matrix with the signature scores such as output by
#' \code{scoreLRGeneSignatures()}.
#' @param dend.row       A precomputed row dendrogram.
#' @param dend.spl       A precompute sample (column) dendrogram.
#' @param cols           A vector of colors to use for the heatmap.
#' @param pointsize      Heatmap font point size
#' @param bottom.annotation  \code{ComplexHeatmap} package bottom annotations.
#' @param n.col.clust    Number of column clusters.
#' @param n.row.clust    Number of row clusters.
#' @param gap.size       Gap size between clusters.
#' @param cut.p          Proportion of top and bottom values for thresholding.
#' @param row.names      A logical to turn on/off the display of row names.
#' @param column.names      A logical to turn on/off the display
#' of column (sample) names.
#' @param hcl.palette    support for HCL colormaps in ComplexHeatmap
#' using color mapping function with circlize::colorRamp2().
#' palettes are listed in grDevides::hcl.pals().
#' of row (gene) names.
#' @param reverse    A logical to reverse or not colors in hcl.palette.
#'
#' @return A heatmap. Since heatmap plotting tend to be slow on the screen,
#' it is advisable to plot in a file instead.
#'
#' If hcl.palette is set, the colors parameter won't be used.
#'
#' Extreme values (top and bottom) can be replaced by global quantiles at
#' \code{cut.p} and \code{1-cut.p}
#' to avoid color scales shrunk by a few outliers.
#'
#' This is a convenience function that relies on the \code{ComplexHeatmap}
#' package to propose a simple way
#' of representing signature scores. If more advanced features are needed or
#' more graphic parameters
#' should be controlled, users should implement their own function.
#' @export
#' @examples
#' data(bsrdm, package = "BulkSignalR")
#' data(bsrinf, package = "BulkSignalR")
#' 
#' bsrinf.redBP <- reduceToBestPathway(bsrinf)
#' bsrsig.redBP <- BSRSignature(bsrinf,
#'     qval.thres = 0.001
#' )
#'
#' scoresLR <- scoreLRGeneSignatures(bsrdm, bsrsig.redBP,
#'     name.by.pathway = FALSE
#' )
#' simpleHeatmap(scoresLR[1:3, ],
#'     column.names = TRUE,
#'     hcl.palette = "Cividis")
#' @import ComplexHeatmap
#' @importFrom circlize colorRamp2
simpleHeatmap <- function(mat.c,
    dend.row = NULL,
    dend.spl = NULL, cols = NULL, pointsize = 4,
    bottom.annotation = NULL, n.col.clust = 0,
    n.row.clust = 0, gap.size = 0.5, cut.p = 0.01, 
    row.names = TRUE,
    column.names = TRUE, hcl.palette = NULL,
    reverse = FALSE) {

    if (!requireNamespace("ComplexHeatmap", quietly = TRUE)) {
        stop(
            "Package \"ComplexHeatmap\" needed for this function ",
            "to work. Please install it."
        )
    }
    if (cut.p < 0 || cut.p > 0.1) {
        stop("cut.p must lie in [0;0.1]")
    }

    if (cut.p != 0) {
        mat.c.cut <- .cutExtremeValues(mat.c, cut.p)
    }

    if (is.null(cols)) {
        if (!requireNamespace("circlize", quietly = TRUE)) {
            stop(
                "Package \"circlize\" needed for this function to ",
                "work (generation of color scale). Please install it."
            )
        }

        cols <- circlize::colorRamp2(
            breaks = c(min(mat.c.cut), 0, max(mat.c.cut)),
            colors = c("royalblue3", "white", "orange")
        )

        if (!is.null(hcl.palette)) {
            cols <- circlize::colorRamp2(
                breaks = c(min(mat.c.cut), 0, max(mat.c.cut)), ,
                hcl_palette = hcl.palette, reverse = reverse
            )
        }
    }

    if (is.null(dend.spl)) {
        di.spl <- stats::dist(t(mat.c))
        hc.spl <- stats::hclust(di.spl, method = "ward.D")
        dend.spl <- stats::as.dendrogram(hc.spl)
    }
    if (is.null(dend.row)) {
        di.gene <- stats::dist(mat.c)
        hc.gene <- stats::hclust(di.gene, method = "ward.D")
        dend.row <- stats::as.dendrogram(hc.gene)
    }

    
    if (n.row.clust > 0) {
        if (n.col.clust > 0) {
            plot(ComplexHeatmap::Heatmap(mat.c,
                cluster_rows = dend.row,
                cluster_columns = dend.spl, col = cols,
                show_row_names = row.names,
                show_column_names = column.names, 
                use_raster = TRUE, raster_device = "png",
                raster_quality = 8, raster_by_magick = FALSE,
                row_names_gp = grid::gpar(fontsize = pointsize),
                show_row_dend = TRUE, bottom_annotation = bottom.annotation,
                split = n.row.clust, gap = grid::unit(gap.size, "mm"),
                column_split = n.col.clust,
                column_gap = grid::unit(gap.size, "mm")
            ))
        } else {
            plot(ComplexHeatmap::Heatmap(mat.c,
                cluster_rows = dend.row,
                cluster_columns = dend.spl, col = cols,
                show_row_names = row.names,
                show_column_names = column.names, 
                use_raster = TRUE, raster_device = "png",
                raster_quality = 8, raster_by_magick = FALSE,
                row_names_gp = grid::gpar(fontsize = pointsize),
                show_row_dend = TRUE, bottom_annotation = bottom.annotation,
                split = n.row.clust, gap = grid::unit(gap.size, "mm")
            ))
        }
    } else if (n.col.clust) {
        plot(ComplexHeatmap::Heatmap(mat.c,
            cluster_rows = dend.row,
            cluster_columns = dend.spl, col = cols, show_row_names = row.names,
            show_column_names = column.names, 
            use_raster = TRUE, raster_device = "png",
            raster_quality = 8, raster_by_magick = FALSE,
            row_names_gp = grid::gpar(fontsize = pointsize),
            column_names_gp = grid::gpar(fontsize = pointsize),
            show_row_dend = TRUE, bottom_annotation = bottom.annotation,
            column_split = n.col.clust, column_gap = grid::unit(gap.size, "mm")
        ))
    } else {
        plot(ComplexHeatmap::Heatmap(mat.c,
            cluster_rows = dend.row,
            cluster_columns = dend.spl, col = cols, 
            show_row_names = row.names,
            show_column_names = column.names,
            row_names_gp = grid::gpar(fontsize = pointsize),
            column_names_gp = grid::gpar(fontsize = pointsize),
            use_raster = TRUE, raster_device = "png",
            raster_quality = 8, raster_by_magick = FALSE,
            show_row_dend = TRUE, bottom_annotation = bottom.annotation
        ))
    }

    
} # simpleHeatmap


#' Generic gene signature scoring
#'
#' Scores generic gene signatures over the
#' samples of a BSRDataModel object.
#'
#' @param ds         A BSRDataModel object.
#' @param ref.signatures           Gene signatures.
#' @param robust       A logical indicating that z-scores
#' should be computed with median and MAD
#' instead of mean and standard deviation.
#' @details This function relies on
#' a simple average of gene z-scores over each signature.
#' It is no replacement for mode advanced methods
#' such as CIBERSORT or BisqueRNA.
#' It is provided for convenience.
#' @return A matrix containing the scores of
#' each gene signature in each sample.
#' Note that ligand-receptor gene
#' signature scores should be computed with
#' \code{"\link[=BSRDataModel-class]{scoreLRGeneSignatures}"}
#' instead.
#' @export
#' @examples
#' data(sdc, package = "BulkSignalR")
#' data(bsrdm, package = "BulkSignalR")
#' 
#' data(immune.signatures, package = "BulkSignalR")
#' imm.scores <- scoreSignatures(bsrdm, immune.signatures)
#' @importFrom methods is
#' @importFrom matrixStats rowMeans2
scoreSignatures <- function(ds, ref.signatures, robust = FALSE) {
    if (!is(ds, "BSRDataModel")) {
        stop("ds must be BSRDataModel object")
    }

    ref.signatures <- ref.signatures[ref.signatures$gene %in%
        rownames(ncounts(ds)), ]
    pool <- unique(ref.signatures$gene)

    # convert normalized counts into z-scores
    if (logTransformed(ds)) {
        ncounts <- 2**ncounts(ds)[pool, ]
    } else {
        ncounts <- ncounts(ds)
    }
    if (robust) {
        z <- (ncounts - apply(ncounts, 1, stats::median)) 
        z <- z / apply(ncounts, 1, stats::mad)
    } else {
        z <- (ncounts - matrixStats::rowMeans2(ncounts)) / 
        apply(ncounts, 1, stats::sd)
    }

    # compute the gene signature scores
    pop <- unique(ref.signatures$signature)
    sig <- matrix(0,
        nrow = length(pop), ncol = ncol(ncounts),
        dimnames = list(pop, colnames(ncounts))
    )
    for (p in pop) {
        n <- sum(ref.signatures$signature == p)
        if (n > 1) {
            sig[p, ] <- as.numeric(colSums(
                z[ref.signatures$gene[ref.signatures$signature == p], ]
            ) / n)
        } else {
            sig[p, ] <- as.numeric(
                z[ref.signatures$gene[ref.signatures$signature == p], ]
            )
        }
    }

    sig
    
} # scoreSignatures


#' Alluvial plot
#'
#' @description Representation of the links
#' between ligands, receptors, and pathways.
#'
#' @param bsrinf A BSRInference object.
#' @param keywords vector of keywoprds to filter pathways.
#' @param type filter on Ligand, Receptor or pathway id.
#' @param qval.thres threshold over Q-value.
#' @return NULL
#'
#' This is a convenience function that relies on the \code{ggalluvial}
#' package to propose a simple way
#' of representing ligands, receptors,
#  and downstream pathway associations.
#' @import ggplot2
#' @import ggalluvial
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#' alluvialPlot(bsrinf,
#'     keywords = c("LAMC1"),
#'     type = "L",
#'     qval.thres = 0.01)
alluvialPlot <- function(bsrinf, keywords, type = c("L", "R", "pw.id"),
                        qval.thres = 0.01) {
    interactions <- data.frame(
        L = unlist(ligands(bsrinf)),
        R = unlist(receptors(bsrinf)),
        pw.name = LRinter(bsrinf)$pw.name,
        pw.id = LRinter(bsrinf)$pw.id,
        qval = LRinter(bsrinf)$qval,
        targets = vapply(tgGenes(bsrinf),
            FUN = function(x) paste(x, collapse = "  "),
            character(1)
        )
    )

    type <- match.arg(type)

    if (type == "L") {
        subset.interactions <- interactions[interactions$L %in% keywords, ]
    }
    if (type == "R") {
        subset.interactions <- interactions[interactions$R %in% keywords, ]
    }
    if (type == "pw.id") {
        subset.interactions <- interactions[interactions$pw.id %in% keywords, ]
    }

    if (dim(subset.interactions)[1] == 0) {
        message(paste(keywords, collapse = " "),
            " for ", type, " not found.")
        stop("Try another value for filtering.")
    }
    subset.interactions <- subset.interactions[
    subset.interactions$qval <= qval.thres, ]
    subset.interactions$count <- 1
    subset.interactions <- subset.interactions[, 
    c("L", "R", "pw.name", "count")]

    stratum <- ggalluvial::StatStratum

    pl <- ggplot2::ggplot(
        subset.interactions,
        ggplot2::aes_string(y = "count", axis1 = "L", 
            axis2 = "R", axis3 = "pw.name")
    ) +
        ggalluvial::geom_alluvium(ggplot2::aes_string(fill = "R"),
            width = 1 / 12) +
        ggalluvial::geom_stratum(width = 1 / 12, 
            fill = "black", color = "grey") +
        ggplot2::geom_label(stat = stratum, 
            aes(label = ggplot2::after_stat(stratum))) +
        ggplot2::scale_x_discrete(limits = c("L", "R", "pw.name"),
            expand = c(0.5, 0.5)) +
        ggplot2::scale_fill_brewer(type = "qual", palette = "Set1") +
        ggplot2::ggtitle("Ligand-Receptor Interactions & Underlying Pathways")

    pl <- pl + ggplot2::theme_bw()
    pl <- pl + ggplot2::theme(legend.position = "none")
    pl <- pl + ggplot2::theme(
        axis.title = ggplot2::element_blank(),
        text = ggplot2::element_text(size = 12),
        plot.title = ggplot2::element_text(hjust = .5),
        axis.text.y = ggplot2::element_blank(),
        axis.text.x = ggplot2::element_blank(),
        axis.ticks = ggplot2::element_blank(),
        panel.grid = ggplot2::element_blank()
    )

    plot(pl)

} # alluvialPlot


#' Chord Diagram of LR interactions with correlations
#'
#' @description Chord diagram.
#'
#' @param bsrinf A BSRInference object
#' @param pw.id.filter One Pathway ID accepted only to
#  retrieve the respective LR interactions.
#' @param qval.thres Threshold over Q-values.
#' @param ligand Ligand
#' of the LR pair that you want to
#' highlight in the chord diagram.
#' @param receptor Receptor
#' of the LR pair that you want to highlight
#' in the chord diagram.
#' @param limit Number of interactions you can visualize.
#  Maximum set to 30.
#' @return Circos Plot on the screen or a file
#'
#' @import ComplexHeatmap
#' @importFrom circlize colorRamp2 circos.par chordDiagramFromDataFrame
#' @importFrom circlize circos.trackPlotRegion circos.text circos.axis
#' @importFrom circlize get.cell.meta.data circos.clear
#'
#' @export
#' @examples
#' data(bsrinf, package = "BulkSignalR")
#' chordDiagramLR(bsrinf,
#' pw.id.filter = "R-HSA-3000178",
#' limit = 20,
#' ligand="ADAM15", 
#' receptor="ITGAV"
#' )
chordDiagramLR <- function(
    bsrinf,
    pw.id.filter = NULL, qval.thres = 1,
    ligand = NULL, receptor = NULL,
    limit = 20) {

    if (limit > 40) {
        message("Number of selected interactions is too large", limit, ".")
        message("Number of visualised interactions sould be less than 40.")
    }

    if (!is.null(ligand) && !is.null(receptor)) {
        pair.to.highlight <- paste(ligand, receptor, sep = "-")
    } else {
        pair.to.highlight <- NULL
    }

    dataframe.bsrinf <- data.frame(
        ligands = unlist(ligands(bsrinf)),
        receptors = unlist(receptors(bsrinf)),
        corr = LRinter(bsrinf)$LR.corr,
        pw.id = LRinter(bsrinf)$pw.id,
        pathways = LRinter(bsrinf)$pw.name,
        pval = LRinter(bsrinf)$pval,
        qval = LRinter(bsrinf)$qval
    )

    dataframe.bsrinf$pair <- paste(dataframe.bsrinf$ligands,
        dataframe.bsrinf$receptors,
        sep = "-"
    )

    if (!is.null(pair.to.highlight) && 
        !(pair.to.highlight %in% dataframe.bsrinf$pair)) {
        stop(
            "Highlighted LR pair ", pair.to.highlight, " was not found for ",
            pw.id.filter, ".\n"
        )
    }

    # Filters
    if (!is.null(pw.id.filter)) {
        dataframe.bsrinf <- dataframe.bsrinf[
        dataframe.bsrinf$pw.id %in% pw.id.filter, ]
    }

    dataframe.bsrinf <- dataframe.bsrinf[
    dataframe.bsrinf$qval < qval.thres, ]


    if (dim(dataframe.bsrinf)[1] == 0) {
        stop("Pathway ID was not found.\n")
    }

    if (dim(dataframe.bsrinf)[1] < limit) {
        limit <- dim(dataframe.bsrinf)[1]
        message("Only ", limit, " interactions were found.\n")
    }

    dataframe.bsrinf <- dataframe.bsrinf[order(dataframe.bsrinf$qval), ]
    dataframe.bsrinf <- dataframe.bsrinf[seq_len(limit), ]
    dataframe.bsrinf <- unique(dataframe.bsrinf[,
        c("ligands", "receptors", "corr", "pair")])

    if (length(unique(dataframe.bsrinf$corr)) == 1){
        cr <- "#febd17" # for BSRInferenceComp class with all corr = 1
    }
    else{
        cr <- circlize::colorRamp2(c(
            min(dataframe.bsrinf$corr),
            max(dataframe.bsrinf$corr)
        ), c("white", "#febd17"))
    }

    myList.ligands <- rep("gray25", 
        times = length(dataframe.bsrinf$ligands))
    names(myList.ligands) <- as.list(dataframe.bsrinf$ligands)

    myList.receptors <- rep("#7fbb00", 
        times = length(dataframe.bsrinf$receptors))
    names(myList.receptors) <- as.list(dataframe.bsrinf$receptors)

    myList <- c(myList.receptors, myList.ligands)

    link.col <- rep("dodgerblue3", nrow(dataframe.bsrinf))

    link.lwd <- rep(1, nrow(dataframe.bsrinf))

    link.width <- rep(0.12, nrow(dataframe.bsrinf))

    if (!is.null(pair.to.highlight)) {
        index.filter <- which(pair.to.highlight == dataframe.bsrinf$pair)
        link.col[index.filter] <- "#d40000"
        link.lwd[index.filter] <- 3
        link.width[index.filter] <- 0.15
    }


    interactions <- data.frame(
        from = dataframe.bsrinf$ligands,
        to = dataframe.bsrinf$receptors,
        value = dataframe.bsrinf$corr
    )

    circlize::circos.par(points.overflow.warning = FALSE)

    circlize::chordDiagramFromDataFrame(interactions,
        # grid.col = grid.col,
        col = cr,
        annotationTrack = "grid",
        grid.col = myList,
        transparency = 0.7,
        preAllocateTracks = 1,
        directional = 1,
        direction.type = "arrows",
        link.arr.length = link.width,
        link.arr.width = link.width,
        link.arr.type = "triangle",
        link.arr.lty = "solid",
        link.arr.lwd = link.lwd,
        link.arr.col = link.col,
        big.gap = 2,
        small.gap = 1
    )

    circlize::circos.trackPlotRegion(
        track.index = 2,
        panel.fun = function(x, y) {
            xlim <- circlize::get.cell.meta.data("xlim")
            ylim <- circlize::get.cell.meta.data("ylim")
            sector.name <- circlize::get.cell.meta.data("sector.index")

            circlize::circos.text(mean(xlim), ylim[1] + 1.9, sector.name,
                facing = "clockwise", niceFacing = TRUE,
                adj = c(0, 0.5), cex = 0.7
            )

            circlize::circos.axis(
                h = "top", labels = FALSE, minor.ticks = FALSE,
                major.tick.length = 1,
                major.at = c(xlim),
                sector.index = sector.name,
                track.index = 2
            )
        }
    )

    # LEGEND

    lgd_points <- Legend(
        labels = c("Ligands", "Receptors"),
        type = "points", pch = 16,
        legend_gp = grid::gpar(col = c("gray25", "#7fbb00")),
        title_position = "topleft",
        labels_gp = grid::gpar(font = 6),
        title = "LR"
    )

    lgd_links <- Legend(
        at = c(round(min(interactions$value),
            digits = 2
        ), round(max(interactions$value),
            digits = 2
        )), col_fun = cr,
        title = "Correlation", direction = "horizontal",
        grid_width = unit(0.9, "mm"),
        grid_height = unit(1.3, "mm"),
        labels_gp = grid::gpar(font = 6),
        title_position = "topcenter",
    )

    lgd_list_vertical <- packLegend(lgd_points)

    draw(lgd_list_vertical,
        x = unit(2, "mm"),
        y = unit(2, "mm"),
        just = c("left", "bottom")
    )
    draw(lgd_links,
        x = unit(2.7, "inch"),
        y = unit(2, "mm"),
        just = c("left", "bottom")
    )

    circlize::circos.clear()


} # chordDiagramLR

####################################################
###     Create / Get Ressources                  ###
####################################################

#' Create all resources
#'
#' Create a cache for all resources (pathways, lrdb & network)
#' downloaded from the web when the library is first loaded.
#' This functionality is handled with BiocFileCache.
#'
#' @param onRequest logical TRUE if you want to force
#' downloading again. This will overwrite the
#' pre-existing local database. Default is TRUE.
#' @param verbose Default is FALSE
#' @return Returns `NULL`, invisibly. 
#' @importFrom curl has_internet
#' @importFrom cli cli_alert_danger cli_alert
#' @export
#' @examples
#' createResources(onRequest=FALSE)
createResources <- function(onRequest = TRUE, verbose = FALSE) {
    cacheDir <- .SignalR$BulkSignalR_CACHEDIR
    resourcesCacheDir <- paste(cacheDir, "resources", sep = "/")

    hasInternet <- tryCatch(expr={curl::has_internet()}, 
        error = FALSE)
    
    if (!hasInternet & 
    !dir.exists(resourcesCacheDir)) {
        cli::cli_alert_danger("Your internet connection is off:")
        stop(
        "- Remote resources cannot be downloaded.\n"
        )   
    }

    if (!hasInternet & 
        onRequest) {
        cli::cli_alert_danger("Your internet connection is off:")
        stop(
        "- Remote resources cannot be downloaded.\n"
        )   
    }

    # Do it once, onLoad
    if (!dir.exists(resourcesCacheDir) | onRequest) {
        .cacheAdd(fpath = .SignalR$BulkSignalR_GO_URL,
            cacheDir = resourcesCacheDir,
            resourceName = "GO-BP", 
            verbose = verbose, download = TRUE)
        .cacheAdd(fpath = .SignalR$BulkSignalR_Reactome_URL,
            cacheDir = resourcesCacheDir, 
            resourceName = "Reactome",
            verbose = verbose, download = TRUE)
        .cacheAdd(fpath = .SignalR$BulkSignalR_Network_URL,
            cacheDir = resourcesCacheDir, 
            resourceName = "Network",
            verbose = verbose, download = TRUE)
        .cacheAdd(fpath = .SignalR$BulkSignalR_LRdb_URL,
            cacheDir = resourcesCacheDir, 
            resourceName = "LRdb",
            verbose = verbose, download = TRUE)

    }

    cacheVersion(dir="resources")
    
    return(invisible(NULL))
}


#' Get resources from the cache
#'
#' Get resources (pathways, lrdb or network
#' stored in the cache.
#'
#' @param resourceName   Resource name.
#' @param cache   Logical. Default is FALSE
#' If TRUE, you will use environment variables.
#' @return Returns a data frame containing the requested
#' resource.
#' @importFrom cli cli_alert_danger
#' @export
#' @examples
#' reactome <- getResource(resourceName = "Reactome",cache=TRUE)
getResource <- function(resourceName = NULL, cache = FALSE) {
    if (!resourceName %in% c("GO-BP", "Reactome", "Network","LRdb")) {
        cli::cli_alert_danger(
        ".val {GO-BP, Reactome, LRdb & Network} are the only keywords alllowed.\n"
        )
        stop()
    }

    if (cache == TRUE) {
        cacheDir <- .SignalR$BulkSignalR_CACHEDIR
        resourcesCacheDir <- paste(cacheDir, "resources", sep = "/")

        # safeguard
        if (!dir.exists(resourcesCacheDir)) {
            cli::cli_alert_danger("Resources repository does not exist.\n")
            stop()
        }

        bfc <- BiocFileCache::BiocFileCache(resourcesCacheDir, ask = FALSE)

        if(resourceName=="LRdb") {
            cacheHits <- BiocFileCache::bfcquery(bfc, query = resourceName,
            field = "rname")
            dataframe <- utils::read.csv(bfc[[cacheHits$rid]],
            stringsAsFactors = FALSE, sep = "\t", check.names = FALSE) 
        } else {
            dataframe <- .readRDSFromCache(bfc = bfc,
            resourceName = resourceName)
        }
        
        if (resourceName == "LRdb") {
            if (!all(
                c("ligand","receptor") %in% 
                colnames(dataframe))) {
                cli::cli_alert_danger("Colnames are not well defined.\n")
                stop()
            }
        }
        if (resourceName == "Reactome") {
            if (!all(
                c("Reactome ID", "Gene name", "Reactome name") %in% 
                colnames(dataframe))) {
                cli::cli_alert_danger("Colnames are not well defined.\n")
                stop()
            }
            dataframe <- dataframe[, 
            c("Reactome ID", "Gene name", "Reactome name")]
        }

        if (resourceName == "GO-BP") {
            if (!all(
                c("GO ID", "Gene name", "GO name") %in% 
                colnames(dataframe))) {
                cli::cli_alert_danger("Colnames are not well defined.\n")
                stop()
            }
            dataframe <- dataframe[, c("GO ID", "Gene name", "GO name")]
        }

        if (resourceName == "Network") {
            if (!all(
                c("a.gn", "type", "b.gn") %in% colnames(dataframe))) {
                cli::cli_alert_danger("Colnames are not well defined.\n")
                stop()
            }
            dataframe <- dataframe[, c("a.gn", "type", "b.gn")]
        }
    } else if (cache == FALSE) {
        if (resourceName == "Reactome") {
            dataframe <- .SignalR$BulkSignalR_Reactome
        }

        if (resourceName == "GO-BP") {
            dataframe <- .SignalR$BulkSignalR_Gobp
        }

        if (resourceName == "Network") {
            dataframe <- .SignalR$BulkSignalR_Network
        }

        if (resourceName == "LRdb") {
            dataframe <- .SignalR$BulkSignalR_LRdb
        }
    }

    return(dataframe)
}


####################################################
###     / Parse / Format / Import Ressources     ###
####################################################

#' Import a refernce network of your own
#'
#' Network is a data frame that defines interactions between
#' genes. It's composed of 3 columns named as
#' follows:
#'
#' a.gn: Gene Symbol 1
#' type: controls-expression-of
#' b.gn: Gene Symbol 2
#'
#' When the user provides his own network,
#' 'type' should be set to 'controls-expression-of'.
#'
#' @param network   Network data frame made of 3 columns
#' a.gn, b.gn & type. 'a.gn' & 'b.gn' should be gene symbols
#' of gene interactions. 'type'  should be set as
#' 'controls-expression-of' when a user provides
#' his own file.
#'
#' @return Returns `NULL`, invisibly. 
#'
#' @importFrom cli cli_alert_info
#' @export
#' @examples
#'  BulkSignalR_Network <- getResource(resourceName = "Network",
#'  cache = FALSE)
#' resetNetwork(BulkSignalR_Network)
resetNetwork <- function(network) {
    if (!all(c("a.gn", "type", "b.gn") %in% colnames(network))) {
        stop("Column names of network should be defined as a.gn, type & b.gn.")
    }

    message("")
    cli::cli_alert_info("New resource defined for {.val Network}.\n")

    assign("BulkSignalR_Network", 
        network, envir = .SignalR)

    return(invisible(NULL))
} # resetNetwork


#' Import pathways from a file or data frame
#'
#' \code{resetPathways} is a function
#' we provide to users who want to refresh REACTOME
#' and GO-BP content included in BulkSignalR.
#' 
#' Pathways in `BulkSignalR` (as sets of genes/proteins) are defined
#' after Reactome and GOBP databases.
#' Those can be updated using
#' json files from
#' the Human Molecular Signatures Database (MSigDB)
#' at \url{https://www.gsea-msigdb.org/}
#' Gmt file format also can be imported.
#' A data frame can be used directly also.
#'
#' @param dataframe  Data frame formated as follows.
#' When \code{resourceName} is set to "Reactome",
#' dataframe colnames must be defined as
#' "Reactome ID", "Gene name", and "Reactome name"
#' When \code{resourceName} is set to "GO-BP",
#' #' dataframe colnames must be defined as
#' "GO ID", "Gene name", and "GO name"
#' @param file    Path to file.
#' @param fileType    Default is Json.
#' Other options are gmt or txt files.
#' @param resourceName    Two options "GO-BP" or "Reactome".
#'
#' @return Returns `NULL`, invisibly. 
#'
#' @importFrom cli cli_alert_info
#' @export
#' @examples
#' reactSubset <- getResource(resourceName = "Reactome",
#' cache = TRUE)
#' 
#' subset <- c("REACTOME_BASIGIN_INTERACTIONS",
#' "REACTOME_SYNDECAN_INTERACTIONS",
#' "REACTOME_ECM_PROTEOGLYCANS",
#' "REACTOME_CELL_JUNCTION_ORGANIZATION")
#' 
#' reactSubset <- reactSubset[
#' reactSubset$`Reactome name` %in% subset,]
#' 
#' resetPathways(dataframe = reactSubset,
#' resourceName = "Reactome")
resetPathways <- function(
    dataframe = NULL,
    file = NULL,
    fileType = c("json", "gmt", "txt"),
    resourceName = NULL) {

    if (!resourceName %in% c("GO-BP", "Reactome")) {
        stop("GO-BP and Reactome are the only keywords alllowed.")
    }

    if (!is.null(file)){

        fileType <- match.arg(fileType)

        if (!file.exists(file)) {
            stop("This file doesn't exist.")
        }

        if (fileType == "json") {
            dataframe <- .formatPathwaysFromJson(
                file = file,
                resourceName = resourceName
            )
        } else if (fileType == "gmt") {
            dataframe <- .formatPathwaysFromGmt(
                file = file,
                resourceName = resourceName
            )
        } else if (fileType == "txt") {
            dataframe <- .formatPathwaysFromTxt(
                file = file,
                resourceName = resourceName
            )
        } else {
            stop("File format accepted are `json` , `gmt` or `txt` only.")
        }

        message("")
        cli::cli_alert_info("New resource defined for {.val {resourceName}}.\n")

        if (resourceName == "Reactome") {
            assign("BulkSignalR_Reactome", dataframe,
                envir = .SignalR)
        }
        if (resourceName == "GO-BP") {
            assign("BulkSignalR_Gobp", dataframe,
                envir = .SignalR)
        }

    }
    else {

    if (resourceName == "Reactome") {
        if (!all(
            c("Reactome ID", "Gene name", "Reactome name") %in% 
            names(dataframe))) {
            mess <- "Colnames should be defined with specific names."
            cli::cli_alert_danger(mess)
            stop("Three columns must defined as",
                " 'Reactome ID','Gene name','Reactome name'.")
        }
    } else if (resourceName == "GO-BP") {
        if (!all(c("GO ID", "Gene name", "GO name") %in% names(dataframe))) {
            mess <- "Colnames should be defined with specific names."
            cli::cli_alert_danger(mess)
            stop("Three columns must defined as",
            " 'GO ID','Gene name','GO name'.")
        }
    } else {
        cli::cli_alert_danger("Resource name is not well defined.\n")
        stop("")
    }

    if (resourceName == "Reactome") {
        assign("BulkSignalR_Reactome", dataframe,
            envir = .SignalR)
    }
    if (resourceName == "GO-BP") {
        assign("BulkSignalR_Gobp", dataframe,
            envir = .SignalR)
    }
    }
    
    return(invisible(NULL))
} # resetPathways


#' Read dataframe from txt file
#'
#' @param file    Path to a tabular file.
#' @param resourceName    Two options "GO-BP"  "REACTOME".
#'
#' @return Dataframe with pathwayID, geneName and pathwayName
#'
.formatPathwaysFromTxt <- function(
    file,
    resourceName = NULL) {
    db <- utils::read.csv(file,
        stringsAsFactors = FALSE, sep = "\t", check.names = FALSE
    )

    if (resourceName == "Reactome") {
        if (!all(
            c("Reactome ID", "Gene name", "Reactome name") %in% 
            names(db))) {
            mess <- "Colnames should be defined with specific names."
            cli::cli_alert_danger(mess)
            stop("Three columns must defined as",
                " 'Reactome ID','Gene name','Reactome name'.")
        }
    } else if (resourceName == "GO-BP") {
        if (!all(c("GO ID", "Gene name", "GO name") %in% names(db))) {
            mess <- "Colnames should be defined with specific names."
            cli::cli_alert_danger(mess)
            stop("Three columns must defined as",
            " 'GO ID','Gene name','GO name'.")
        }
    } else {
        cli::cli_alert_danger("Resource name is not well defined.\n")
        stop("")
    }

    return(db)
} # .formatPathwaysFromTxt


#' Format a data frame according to json input
#'
#' @param file    Path to file.
#' @param resourceName    Two options "GO-BP" or "REACTOME".
#'
#' @return Data frame with pathwayID, geneName and pathwayName
#'
#' @importFrom foreach %do% %dopar%
#' @import doParallel
#' @import jsonlite
.formatPathwaysFromJson <- function(
    file,
    resourceName = NULL) {
    data <- jsonlite::read_json(file, simplifyVector = TRUE)
    indexPathway <- NULL

    if (!all(c("exactSource", "geneSymbols") %in% names(data[[1]]))) {
        cli::cli_alert_danger("Json format is invalid.\n")
        stop("Json must follow the standard",
        " from The Molecular Signatures Database (MSigDB).")
    }

    db <- foreach::foreach(
        indexPathway = seq_len(length(data)),
        .combine = "rbind"
    ) %dopar% {
        data.frame(
            pathwayID = rep(data[[indexPathway]]$exactSource,
                length(data[[indexPathway]]$exactSource)),
            geneName = unlist(data[[indexPathway]]$geneSymbols)[
            seq_len(length(unlist(data[[indexPathway]]$geneSymbols)))],
            pathwayName = rep(names(data)[[indexPathway]], 
                length(names(data)[[indexPathway]]))
        )
    }

    # Due to the fact React and Go are organized differently
    if (resourceName == "Reactome") {
        names(db) <- c("Reactome ID", "Gene name", "Reactome name")
    }

    if (resourceName == "GO-BP") {
        names(db) <- c("GO ID", "Gene name", "GO name")
    }

    return(db)
} # .formatPathwaysFromJson


#' Transform gmt file to data frame
#'
#' We note discrepancies between the formats available
#' from internet sources. Here, we consider a valid gmt file format defined
#' on each lines as follows:
#' First is Pathway name,
#' then comes the ID,
#' finally you will find genes symbols
#' part of the pathway defined on the line.
#'
#' You can find an example here.
#' - For Reactome. (Directly from their website)
#' \url{https://reactome.org/download/current/ReactomePathways.gmt.zip}
#' Note that you need to unzip the file to read the content.

#' The code is inspired from read.gmt function
#' from the gsa R package.
#'
#' @param file    Path to GMT file
#' @param resourceName   Two options "GO-BP" or "REACTOME"
#'
#' @return Dataframe with pathwayID, geneName and pathwayName
#'
#' @importFrom foreach %do% %dopar%
#' @import doParallel
.formatPathwaysFromGmt <- function(
    file,
    resourceName = NULL) {
    read1 <- scan(file,
        what = list("", ""), sep = "\t",
        quote = NULL, fill = TRUE, flush = TRUE, multi.line = FALSE
    )

    read1 <- stats::setNames(read1, c("Description", "ID"))
    read1 <- read1[c("ID", "Description")]

    geneset.ids <- read1[1][[1]]
    geneset.descriptions <- read1[2][[1]]

    read2 <- scan(file, what = "", sep = "\t", quote = NULL)
    read2 <- read2[read2 != ""]

    nn <- length(geneset.ids)
    n <- length(read2)
    ox <- rep(NA, nn)

    # Compute indice of pathway
    ii <- 1
    for (i in seq_len(nn)) {
        while ((read2[ii] != geneset.descriptions[i]) | 
            (read2[ii + 1] != geneset.ids[i])) {
            ii <- ii + 1
        }
        ox[i] <- ii
        ii <- ii + 1
    }

    genesets <- vector("list", nn)

    for (i in seq_len(nn - 1)) {
        i1 <- ox[i] + 2
        i2 <- ox[i + 1] - 1
        geneset.descriptions[i] <- read2[ox[i] + 1]
        geneset.ids[i] <- read2[ox[i]]
        genesets[[i]] <- read2[i1:i2]
    }

    geneset.ids[nn] <- read2[ox[nn]]
    geneset.descriptions[nn] <- read2[ox[nn] + 1]
    genesets[[nn]] <- read2[(ox[nn] + 2):n]

    data <- list(
        geneset.ids = geneset.ids,
        geneset.descriptions = geneset.descriptions,
        genesets = genesets
    )

    dataframeFromGmt <- foreach::foreach(
        i = seq_len(length(data$geneset.ids)),
        .combine = "rbind"
    ) %dopar% {
        data.frame(
            a = rep(data$geneset.ids[[i]], length(data$genesets[[i]])),
            b = data$genesets[[i]][seq_len(length(data$genesets[[i]]))],
            c = rep(data$geneset.descriptions[[i]], length(data$genesets[[i]]))
        )
    }

    # Due to the fact React and Go are organized differently
    if (resourceName == "Reactome") {
        names(dataframeFromGmt) <- 
        c("Reactome name", "Gene name", "Reactome ID")
        dataframeFromGmt <- dataframeFromGmt[
        c("Reactome ID", "Gene name", "Reactome name")]
    }

    if (resourceName == "GO-BP") {
        names(dataframeFromGmt) <- c("GO name", "Gene name", "GO ID")
        dataframeFromGmt <- dataframeFromGmt[c("GO ID", "Gene name", "GO name")]
    }

    return(dataframeFromGmt)
} # .formatPathwaysFromGmt

#' Modify LRdb database
#'
#' Users can provide a data frame with 2 columns named
#' ligand and receptor.
#' This can be used to extend or replace the existing
#' LRdb.
#'
#' @param db     A data frame with 2 columns named
#' ligand and receptor.
#' @param switch  A logical indicating whether LRdb should be extended only
#' (FALSE, default) or completely replaced (TRUE).
#'
#' @return Returns `NULL`, invisibly. 
#'
#' @importFrom cli cli_alert_info
#' @export
#' @examples
#' resetLRdb(db = data.frame(ligand = "A2M", receptor = "LRP1"), switch = FALSE)
resetLRdb <- function(db, switch = FALSE) {
    if (all(c("ligand","receptor") %in% names(db))) {
        if (switch) {
            assign("BulkSignalR_LRdb", unique(db[, c("ligand", "receptor")]),
                envir = .SignalR
            )
        } else {
            db <- rbind(
                .SignalR$BulkSignalR_LRdb[, c("ligand", "receptor")],
                db[, c("ligand", "receptor")]
            )
            assign("BulkSignalR_LRdb", unique(db), 
                envir = .SignalR)
        }
    } else {
        stop(
            "db should be a dataframe with",
            "2 columns named 'ligand' and 'receptor'."
        )
    }

    message("")
    cli::cli_alert_info(
        "New database defined for {.val LRdb}."
    )

    return(invisible(NULL))
    
} # resetLRdb

#' BulkSignalR Data Model Compare Object
#'
#' An S4 class to represent the expression data used for inferring
#' ligand-receptor interactions based on sample cluster comparisons.
#'
#' @slot comp   A named list of BSRClusterComp objects, one per
#' comparison.
#' @slot mu     A number representing the average value in the normalized and
#' lop1p-transformed gene expression matrix. This value is used to compute
#' the LR-score (cf. SingleCellSignalR paper, Cabello-Aguilar, et al.,
#' Nucleic Acids Res, 2020)
#' @export
#' @examples
#' new("BSRDataModelComp")
#'
setClass("BSRDataModelComp",
    contains = c("BSRDataModel"),
    slots = c(
        comp = "list",
        mu = "numeric"
    ),
    prototype = list(
        initial.organism = "hsapiens",
        initial.orthologs = list("A", "B", "C"),
        ncounts = matrix(1.0, nrow = 2, ncol = 1,
            dimnames = list(c("A", "B"), "C")),
        log.transformed = FALSE,
        normalization = "UQ",
        param = list(spatial.smooth = FALSE),
        comp = list(),
        mu = 1.0
    )
)

setValidity(
    "BSRDataModelComp",
    function(object) {
        if (length(object@comp) > 0) {
            if (is.null(names(object@comp))) {
                return("comp names must be set")
            }
            for (c in names(object@comp)) {
                if (!is(object@comp[[c]], "BSRClusterComp")) {
                    return("comp must contain objects of class BSRClusterComp")
                }
            }
        }
        if (!is.numeric(object@mu) || object@mu <= 0) {
            return("mu must be numeric and >0")
        }

        TRUE
    }
)

setMethod(
    "show", "BSRDataModelComp",
    function(object) {
        callNextMethod()
      cat("mu: ", object@mu, "\n", sep = "")
      cat("Defined comparisons:\n")
      utils::str(object@comp)
    }
)


# Accessors & setters ========================================================

setGeneric("comparison", signature="x",
    function(x) standardGeneric("comparison")
)
#' Comparisons list accessor
#'
#' @name comparison
#' @aliases comparison,BSRDataModelComp-method
#' @param x object BSRDataModelComp
#' @return comp
#' @examples
#' bsrdm.comp <- new("BSRDataModelComp")
#' comparison(bsrdm.comp)
#' @export
setMethod("comparison", "BSRDataModelComp", function(x) x@comp)


setGeneric("comparison<-", signature=c("x", "value"),
    function(x, value) standardGeneric("comparison<-")
)
#' Comparisons list setter (internal use only, use addComparison() otherwise)
#'
#' @param x object BSRDataModelComp
#' @param value value to be set for BSRDataModelComp
#' @return returns \code{NULL}
#' @keywords internal
setMethod("comparison<-", "BSRDataModelComp", function(x, value) {
    x@comp <- value
    methods::validObject(x)
    x
})


setGeneric("mu", signature="x",
    function(x) standardGeneric("mu")
)
#' Mu accessor
#'
#' @name mu
#' @aliases mu,BSRDataModelComp-method
#' @param x object BSRDataModelComp
#' @return mu
#' @examples
#' bsrdm.comp <- new("BSRDataModelComp")
#' mu(bsrdm.comp)
#' @export
setMethod("mu", "BSRDataModelComp", function(x) x@mu)


setGeneric("mu<-", signature=c("x", "value"),
    function(x, value) standardGeneric("mu<-")
)
#' Mu setter (internal use only)
#'
#' @param x object BSRDataModelComp
#' @param value value to be set for BSRDataModelComp
#' @return returns \code{NULL}
#' @keywords internal
setMethod("mu<-", "BSRDataModelComp", function(x, value) {
    x@mu <- value
    methods::validObject(x)
    x
})


#' Convert BSRDataModel to BSRDataModelComp
#' 
#' Enable the promotion of a BSRDataModel object into
#' a BSRDataModelComp object by
#' coercion.
#' 
#' @name coerce
#' @aliases coerce,BSRDataModel,BSRDataModelComp-method
#' @param from  BSRDataModel object  
#' @return A BSRDataModelComp object
#' @examples
#' bsrdm <- new("BSRDataModel")
#' bsrdm.comp <- as(bsrdm, "BSRDataModelComp")
#'
#' @importFrom methods is new
#' @exportMethod coerce
setAs("BSRDataModel", "BSRDataModelComp", function(from) {

    if (!is(from, "BSRDataModel")) {
        stop("bsrdm must be of class BSRDataModel")
    }
    m <- mean(ncounts(from))
    if (!logTransformed(from)) {
        m <- log1p(m) / log(2)
    } # approximate of mu on the log2-transformed matrix

    new("BSRDataModelComp", from, 
        comp = list(),
        mu = m) 
})


setGeneric("addClusterComp", signature="obj",
    function(obj, ...) standardGeneric("addClusterComp")
)
#' Add a comparison between two clusters of samples
#'
#' Add a comparison to a BSRDataModelComp object.
#'
#' @name addClusterComp
#' @aliases addClusterComp,BSRDataModelComp-method
#'
#' @param obj    A BSRDataModelComp object output by
#'   \code{\link{setAs}}.
#' @param cmp   A BSRClusterComp object to add.
#' @param cmp.name  The name of the comparison to add.
#'
#' @details Add \code{cmp} to the list of comparisons contained in
#' \code{obj}.
#'
#' @return A BSRDataModelComp object.
#'
#' @export
#'
#' @examples
#' # prepare data
#' data(sdc, package = "BulkSignalR")
#' normal <- grep("^N", names(sdc))
#' bsrdm <- BSRDataModel(sdc[, -normal])
#'
#' # define the comparison
#' bsrdm.comp <- as(bsrdm, "BSRDataModelComp")
#' colA <- as.integer(1:3)
#' colB <- as.integer(12:15)
#' n <- nrow(ncounts(bsrdm.comp))
#' stats <- data.frame(
#'     pval = runif(n), logFC = rnorm(n, 0, 2),
#'     expr = runif(n, 0, 10)
#' )
#' rownames(stats) <- rownames(ncounts(bsrdm.comp))
#' bsrcc <- BSRClusterComp(bsrdm.comp, colA, colB, stats)
#'
#' bsrdm.comp <- addClusterComp(bsrdm.comp, bsrcc, "random.example")
#' @importFrom methods new
setMethod("addClusterComp", "BSRDataModelComp", function(obj, cmp,
    cmp.name) {
    if (!is(cmp, "BSRClusterComp")) {
        stop("cmp must be of class BSRClusterComp")
    }
    if (!is.character(cmp.name)) {
        stop("cmp.name must be of type character")
    }
    if (length(cmp.name) == 0) {
        stop("cmp.name must have length > 0")
    }
    if (cmp.name %in% names(comparison(obj))) {
        stop("cmp.name is already in the list of comparisons")
    }

    tmp <- c(comparison(obj), list(cmp))
    names(tmp)[length(tmp)] <- cmp.name
    comparison(obj) <- tmp
    obj
    
}) # addClusterComp


setGeneric("removeClusterComp", signature="obj",
    function(obj, ...) standardGeneric("removeClusterComp")
)
#' Remove a comparison from a BSRDataModelComp object.
#'
#' @name removeClusterComp
#' @aliases removeClusterComp,BSRDataModelComp-method
#'
#' @param obj    A BSRDataModelComp object output by
#'   \code{\link{setAs}}.
#' @param cmp.name  The name of the comparison to remove.
#'
#' @details Remove the comparison with \code{cmp.name} from the list of
#' comparisons contained in \code{obj}.
#'
#' @return A BSRDataModelComp object.
#'
#' @export
#'
#' @examples
#' # prepare data
#' data(sdc, package = "BulkSignalR")
#' normal <- grep("^N", names(sdc))
#' bsrdm <- BSRDataModel(sdc[, -normal])
#'
#' # define the comparison
#' bsrdm.comp <- as(bsrdm, "BSRDataModelComp")
#' colA <- as.integer(1:3)
#' colB <- as.integer(12:15)
#' n <- nrow(ncounts(bsrdm.comp))
#' stats <- data.frame(
#'     pval = runif(n), logFC = rnorm(n, 0, 2),
#'     expr = runif(n, 0, 10)
#' )
#' rownames(stats) <- rownames(ncounts(bsrdm.comp))
#' bsrcc <- BSRClusterComp(bsrdm.comp, colA, colB, stats)
#'
#' bsrdm.comp <- addClusterComp(bsrdm.comp, bsrcc, "random.example")
#' bsrdm.comp <- removeClusterComp(bsrdm.comp, "random.example")
#'
setMethod("removeClusterComp", "BSRDataModelComp", function(obj, cmp.name) {
    if (!is.character(cmp.name)) {
        stop("cmp.name must be of type character")
    }
    if (length(cmp.name) == 0) {
        stop("cmp.name must have length > 0")
    }
    if (!(cmp.name %in% names(comparison(obj)))) {
        stop("cmp.name must be in the list of comparisons")
    }

    if (length(comparison(obj)) == 1) {
        comparison(obj) <- list()
    } else {
        comparison(obj) <- comparison(obj)[names(comparison(obj)) != cmp.name]
    }

    obj
    
}) # removeClusterComp


# Constructor ==================================================================

#' Constructor of the BSRDataModelComp class
#'
#' Take a matrix or data frame containing RNA sequencing,
#' microarray, or expression proteomics data as input parameter
#' and return a BSRDataModelComp
#' object ready for subsequent use.
#' 
#' @param counts     A table or matrix of read counts.
#' @param species    Data were obtained for this organism.
#' @param normalize  A logical indicating whether \code{counts} should be
#'   normalized according to \code{method} or if it was normalized beforehand.
#' @param symbol.col The index of the column containing the gene symbols in case
#'   those are not the row names of \code{counts} already.
#' @param min.count  The minimum read count of a gene to be considered expressed
#'   in a sample.
#' @param prop       The minimum proportion of samples where a gene must be
#'   expressed higher than \code{min.count} to keep that gene.
#' @param method     The normalization method ('UQ' for upper quartile or 'TC'
#'   for total count). If \code{normalize==FALSE}, then method must be
#'   used to document the name of the normalization method applied by the user.
#' @param UQ.pc      Percentile for upper-quartile normalization, number
#' between 0 and 1 (in case the default 0.75 - hence the name - is not
#' appropriate).
#' @param log.transformed  A logical indicating whether expression data were
#'   already log2-transformed, e.g., some microarray data.
#' @param min.LR.found  The minimum number of ligands or receptors found in
#'   \code{count} row names after eliminating the rows containing too many
#'   zeros according to \code{min.count} and \code{prop}.
#' @param conversion.dict  Correspondence table of HUGO gene symbols
#' human/nonhuman. Not used unless the organism is different from human.
#' @param x.col In a SpatialExperiment object, the index of the column
#' containing the x coordinates in the dafaframe returned by rowData(), usually 
#' named array_row.
#' @param y.col  In a SpatialExperiment object, the index of the column
#' containing the y coordinates in the dafaframe returned by rowData(), usually 
#' named array_col.
#' @param barcodeID.col   In a SpatialExperiment object, the index of the column
#' containing the barcodeID in the dafaframe returned by colData(), usually
#' named barcode_id.
#'
#' @return A BSRModelDataComp object with empty model parameters.
#'
#' @details The \code{counts} matrix or table should be provided with expression
#'   levels of protein coding genes in each samples (column) and
#'   \code{rownames(counts)} set to HUGO official gene symbols.
#'   For commodity, it is also possible 
#'   to provide \code{counts} with the
#'   gene symbols stored in one of its columns. This column must be specified
#'   with \code{symbol.col}. In such a case, \code{BSRDataModel} will extract
#'   this column and use it to set the row names. Because row names must be
#'   unique, \code{BSRDataModel} will eliminate rows with duplicated gene
#'   symbols by keeping the rows with maximum average expression. Gene symbol
#'   duplication may occur in protein coding genes after genome alignment
#'   due to errors in genome feature annotation files (GTF/GFF), where a handful
#'   of deprecated gene annotations might remain, or
#'   some genes are not given their fully specific symbols. If your read count
#'   extraction pipeline does not take care of this phenomenon, the maximum mean
#'   expression selection strategy implemented here should solve this difficulty
#'   for the sake of inferring ligand-receptor interactions.
#'
#'   If \code{normalize} is \code{TRUE} then normalization is performed
#'   according to \code{method}. If those two simple methods are not satisfying,
#'   then it is possible to provide a pre-normalized matrix setting
#'   \code{normalize} to \code{FALSE}. In such a case, the parameter
#'   \code{method} must be used to document the name of the normalization
#'   algorithm used.
#'
#'   In case proteomic or microarray data are provided, \code{min.count} must be
#'   understood as its equivalent with respect to those data types.
#'
#' @importFrom matrixStats rowMeans2 rowSums2 colSums2
#' @export
#' @examples
#' data(sdc, package = "BulkSignalR")
#' idx <- sample(nrow(sdc), 4000)
#' bsrdm.comp <- BSRDataModelComp(sdc[idx, c("N22","SDC17")],
#'                                normalize=FALSE, method="UQ")
BSRDataModelComp <- function(
    counts, normalize = TRUE, symbol.col = NULL, min.count = 10,
    prop = 0.1, method = c("UQ", "TC"), 
    log.transformed = FALSE, min.LR.found = 80,
    species = "hsapiens", conversion.dict = NULL,
    UQ.pc = 0.75,x.col = NULL, y.col = NULL,
    barcodeID.col = NULL) {
  
  bsrdm <- BSRDataModel(counts = counts,
                        normalize = normalize,
                        symbol.col = symbol.col,
                        min.count = min.count,
                        prop = prop,
                        method = method,
                        log.transformed = log.transformed,
                        min.LR.found = min.LR.found,
                        species = species,
                        conversion.dict = conversion.dict,
                        UQ.pc = UQ.pc,
                        x.col = x.col,
                        y.col = y.col,
                        barcodeID.col = barcodeID.col)
  as(bsrdm, "BSRDataModelComp")
  
} # BSRDataModelComp


# Scoring of gene signatures in a BSRSignature object ==========================

#' Score ligand-receptor gene signatures
#'
#' Compute ligand-receptor gene signature scores over a BSRDataModelComp
#' specific comparison.
#'
#' @name scoreLRGeneSignatures
#' @aliases scoreLRGeneSignatures,BSRDataModelComp-method
#'
#' @param obj           A BSRDataModelComp object.
#' @param sig           A BSRSignatureComp object.
#' @param LR.weight    A number between 0 and 1 defining the relative weight
#' of the ligand and the receptor in the signature.
#' @param robust       A logical indicating that z-scores should be computed
#' with median and MAD instead of mean and standard deviation.
#' @param name.by.pathway     A logical indicating whether row names of the
#' resulting score matrix should be pathway names.
#' @param rownames.LRP A logical indicating, in case \code{name.by.pathway}
#' was set to TRUE, whether ligand and receptor names should be added on top.
#' No role if \code{name.by.pathway} was set to FALSE.
#' @param abs.z.score  A logical to use absolute z-scores (useful if the
#' activity of a paythway is reported by a mixture of up- and down-genes
#' whose z-score averages might hide actual activity).
#' @return A matrix containing the scores of each ligand-receptor gene
#' signature in each sample.
#'
#' @export
#' @examples
#' # prepare data
#' data(bsrdm.comp, package = "BulkSignalR")
#' data(bsrinf.comp, package = "BulkSignalR")
#'
#' # reduction
#' bsrinf.red <- reduceToBestPathway(bsrinf.comp)
#' # signature extraction and scoring
#' bsrsig.red <- BSRSignatureComp(bsrinf.red, qval.thres = 1e-6)
#' scores.red <- scoreLRGeneSignatures(bsrdm.comp, bsrsig.red,
#'     name.by.pathway = TRUE, rownames.LRP = TRUE
#' )
#' @importFrom foreach %do% %dopar%
#' @importFrom methods is
#' @importFrom matrixStats rowMeans2 colSums2
setMethod("scoreLRGeneSignatures", "BSRDataModelComp", function(obj,
    sig, LR.weight = 0.5, robust = FALSE,
    name.by.pathway = FALSE, abs.z.score = FALSE,
    rownames.LRP = FALSE) {
    
    if (!is(sig, "BSRSignatureComp")) {
        stop("sig must be a BSRSignature object")
    }
    if (LR.weight <= 0 || LR.weight >= 1) {
        stop("LRweight must reside in (0;1)")
    }

    # retrieve the BSRClusterComp object
    cmp.name <- comparisonName(sig)
    if (!(cmp.name %in% names(comparison(obj)))) {
        stop("The comparison name in sig is not present in obj")
    }
    cc <- comparison(obj)[[cmp.name]]

    # species management
    if (initialOrganism(obj) != "hsapiens") {
        all.genes <- unlist(initialOrthologs(obj))
    } else {
        all.genes <- rownames(ncounts(obj))
    }

    # get the ncount matrix with proper columns
    ncounts <- ncounts(obj)[, c(colClusterA(cc), colClusterB(cc))]

    # intersect signature gene names with RNA-seq data
    ligands <- list()
    receptors <- list()
    tg.genes <- list()

    for (i in seq_along(ligands(sig))) {
        ligands[[i]] <- intersect(ligands(sig)[[i]], all.genes)
    }
    for (i in seq_along(receptors(sig))) {
        receptors[[i]] <- intersect(receptors(sig)[[i]], all.genes)
    }
    for (i in seq_along(tgGenes(sig))) {
        tg.genes[[i]] <- intersect(tgGenes(sig)[[i]], all.genes)
    }

    good <- vapply(ligands, length, integer(1)) > 0 &
        vapply(receptors, length, integer(1)) > 0 &
        vapply(tg.genes, length, integer(1)) > 0

    ligands <- ligands[good]
    receptors <- receptors[good]
    tg.genes <- tg.genes[good]
    pathways <- pathways(sig)[good]

    # scale ncounts
    if (logTransformed(obj)) {
        ncounts <- 2**ncounts
    }
    if (robust) {
        z <- (ncounts - apply(ncounts, 1, stats::median)) 
        z <- z / apply(ncounts, 1, stats::mad)
    } else {
        z <- (ncounts - matrixStats::rowMeans2(ncounts)) / 
        apply(ncounts, 1, stats::sd)
    }
    if (abs.z.score) {
        z <- abs(z)
    }

    if (initialOrganism(obj) != "hsapiens") {
        rownames(z) <- all.genes
    }

    # compute the LR gene signatures
    i <- NULL
    pwn <- foreach::foreach(i = seq_len(length(pathways)), .combine = c) %do% {
        if (name.by.pathway) {
            if (rownames.LRP) {
                paste0(
                    "{", paste(ligands[[i]], collapse = ";"), "} / {",
                    paste(receptors[[i]], collapse = ";"), "} | ", pathways[[i]]
                )
            } else {
                pathways[[i]]
            }
        } else if (!name.by.pathway) {
            paste0(
                "{", paste(ligands[[i]], collapse = ";"), "} / {",
                paste(receptors[[i]], collapse = ";"), "}"
            )
        }
    }

    res <- matrix(0, nrow = length(pathways),
        ncol = ncol(ncounts), dimnames = list(pwn, colnames(ncounts)))
    for (i in seq_len(length(pathways))) {
        # average ligand z-score
        zz <- z[ligands[[i]], ]
        if (is.matrix(zz)) {
            mL <- matrixStats::colSums2(zz) / length(ligands[[i]])
        } else {
            mL <- zz
        }

        # average receptor z-score
        zz <- z[receptors[[i]], ]
        if (is.matrix(zz)) {
            mR <- matrixStats::colSums2(zz) / length(receptors[[i]])
        } else {
            mR <- zz
        }

        # average target gene z-score
        zz <- z[tg.genes[[i]], ]
        if (is.matrix(zz)) {
            mT <- matrixStats::colSums2(zz) / length(tg.genes[[i]])
        } else {
            mT <- zz
        }

        res[i, ] <- LR.weight * 0.5 * (mL + mR) + (1 - LR.weight) * mT
    }

    res
    
}) # scoreLRGeneSignatures

#' Salivary duct carcinoma transcriptoms
#'
#' A data set containing the read counts of salivary duct carcinomas (SDCs)
#'   and adjacent normal tissues.
#'
#' @format A data frame with 19764 rows and 26 variables.
#' @source \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE138581}
#' @usage data(sdc)
"sdc"

#' A skinny BSRDataModel object related to sdc.
#'
#' Output from the `learnParameters` function to get
#' BulkSignalR statistical model parameters.
#' 
#' @format An example of an object created by `BSRDataModel`
#' applied to an sdc subset (Patients N20,N22,SDC17,SDC25) and
#' 10 000 genes sampled (seed set to 123)
#' `learnParameters` was also called to get statistical 
#' model parameters.
#' @usage data(bsrdm)
"bsrdm"

#' A skinny BSRInference object related to sdc.
#'
#' From the previous object `bsrdm`, 
#' you can generate inferences by calling its
#' constructor `BSRInference`. 
#' The resulting BSRInference object is `bsrinf`,
#' It contains all the inferred L-R interactions 
#' with their associated pathways and corrected p-values.
#' 
#' @format An example of an object created by inference function 
#' @usage data(bsrinf)
"bsrinf"

#' A skinny BSRDataModelComp object related to sdc.
#'
#' See Vignette BulkSignalR-Differential.
#' 
#' @format An example of an BSRDataModelComp object
#' @usage data(bsrdm.comp)
"bsrdm.comp"

#' A skinny BSRInferenceComp object related to sdc.
#'
#' See Vignette BulkSignalR-Differential.
#' 
#' @format An example of an BSRInferenceComp object
#' @usage data(bsrinf.comp)
"bsrinf.comp"

#' A skinny BSRDataModel object related to a spatial data set
#' 
#' Obtained from STexampleData::Visium_humanDLPFC. 
#' A single sample (sample 151673)
#' of human brain dorsolateral prefrontal cortex (DLPFC)
#' in the human brain, measured using the 10x Genomics 
#' Visium platform. This is a subset of the full dataset 
#' published by Maynard and Collado-Torres et al. (2021).
#' The subset is reproduced in the vignette.
#' name.idx <- c("10x32","3x47","4x50",
#' "17x111","5x59","0x20","8x100",
#' "8x108","14x30","11x39") 
#' 
#' Output from the `learnParameters` function to get
#' BulkSignalR statistical model parameters for a subset
#' of a spatial data set.
#' 
#' @format An example of an object created by `BSRDataModel`
#' applied to a subset of a spatial data set.
#' `learnParameters` was also called to get statistical 
#' model parameters.
#' @source \url{http://spatial.libd.org/spatialLIBD/}
#' @usage data(bsrdm.spa)
"bsrdm.spa"

#' A skinny BSR-inference object related to spatial data set
#'
#' Output from the `learnParameters` function to get
#' BulkSignalR statistical model parameters.
#' 
#' @format From the previous object `bsrdm.spa`, 
#' you can generate inferences by calling its
#' method `BSRInference`. 
#' The resulting BSRInference object is `bsrinf.spa`,
#' It contains all the inferred L-R interactions 
#' with their associated pathways and corrected p-values.
#' `learnParameters` was also called to train the statistical 
#' model parameters.
#' @source \url{http://spatial.libd.org/spatialLIBD/}
#' @usage data(bsrinf.spa)
"bsrinf.spa"

#' A skinny data frame used in the spatial workflow
#'
#' Data frame subset describing the spatial spots
#' 
#' @format Data frame that contains the following columns:
#' barcode_id,sample_id, in_tissue,array_row
#' array_col, ground_truth, reference, cell_count, idSpatial
#' 
#' barcode_id is the id of the spot
#' idSpatial is the spatial id of the spot(array_rowXarray_col)
#' ground_truth is the label (Layer1/2 were only kept)
#' 
#' They are the mandatory data necessary to
#' generate plots for the spatial workflow.
#' 
#' @source \url{http://spatial.libd.org/spatialLIBD/}
#' @usage data(annotation.spa)
"annotation.spa"

#' Mouse transcriptoms across tissues
#'
#' A data set containing RPKM values of brain and liver.
#'
#'
#' @format A data frame with 24543 rows and 8 variables.
#' @source Bin Li & al., Scientific Reports, 2017;
#' @usage data(bodyMap.mouse)
"bodyMap.mouse"

#' A skinny BSRInference object related to bodyMap.mouse
#' 
#' see related workflow for non human organism
#' in the vignette
#'
#' @format An example of an object created by inference function
#' @usage data(bsrinf.mouse)
"bsrinf.mouse"

#' A skinny data frame used in the mouse workflow
#'
#' Synthetic object used during the call to the
#' function `resetToInitialOrganism``
#'
#' @format An example of a data frame created by findOrthoGenes
#' @usage data(ortholog.dict)
"ortholog.dict"

#' Tumor microenvironment gene signatures
#'
#' A data set containing gene signatures for some immune and stromal
#'   cell populations that are present in the microenvironment of a tumor.
#'
#' @format A data frame with 209 rows and 2 variables: \describe{
#'   \item{gene}{HUGO gene symbol} \item{signature}{cell population name} }
#' @source Becht & al., Genome Biol, 2016; Angelova et al., Genome Biol, 2015.
#' @usage data(tme.signatures)
"tme.signatures"


#' Partial EMT gene signature
#'
#' A data set containing a partial EMT gene signature.
#'
#' @format A data frame with 100 rows and 1 variables:
#' \describe{
#'   \item{gene}{HUGO gene symbol}
#' }
#' @source Puram, SV & al., Cell, 2017.
#' @usage data(p.EMT)
"p.EMT"


#' Immune cell gene signatures
#'
#' A data set containing gene signatures for general immune cell populations.
#'
#' @format A data frame with 1541 rows and 2 variables:
#' \describe{
#'   \item{gene}{HUGO gene symbol}
#'   \item{signature}{cell population name}
#' }
#' @source PanglaoDB (Franzén et al., Database, 2019).
#' @usage data(immune.signatures)
"immune.signatures"

#' Smooth spatial expression data
#'
#' @param bsrdm  A BSRDataModel object containing the expression data to smooth.
#' @param areas  A data.frame containing at least the x and y
#' coordinates of the locations.
#' @param nnn    Number of nearest-neighbor locations to use for smoothing
#' each location. In case \code{radius} is set, then it is the maximum number
#' of nearest neighbors within the radius.
#' @param radius  A maximal distance to include neighbors in the smoothing.
#' @param weight.ratio  The weight given to the central location.
#' @param x.col  Column name in \code{areas} containing x coordinates.
#' @param y.col  Column name in \code{areas} containing y coordinates.
#'
#' @details The expression data contained in a BSRDataModel object are
#' smoothed using a weighted average of nearby locations.
#'
#' Two strategies are available to identify the neighbors. It is possible to
#' simply set the number of nearest-neighbors (parameter \code{nnn}). An
#' alternative consists in providing a distance radius (\code{radius}) along
#' with a a maximum number of nearest-neighbors within the radius
#' (\code{nnn.radius}). To properly define the radius, the user must know the
#' location coordinates. The strategy with the radius enables having corner
#' locations with two neighbors only and border locations with three
#' neighbors only, whereas to simply set a maximum of four neighbors for
#' instance would retrieve the four closest neighbors in every case.
#'
#' For each location, its nearest-neighbors are found and a weighted average
#' computed with \code{weight.ratio} given to the central location itself
#' and a total weight of 1-\code{weight.ratio} shared within the neighbors
#' based on the inverse of their distances. In case \code{radius} is set,
#' some locations may have less than \code{nnn} neighbors (see above).
#' At such locations, the weight given to
#' the central location is augmented according to
#' 1-(1-\code{weight.ratio})*(number of neighbors)/\code{nnn}.
#'
#' @export
#'
#' @return A BSRDataModel object containing the smoothed ncounts.
#'
#' @importFrom RANN nn2
#'
#' @examples  
#' data(bsrdm.spa, package = "BulkSignalR")
#' data(annotation.spa, package = "BulkSignalR")
#' sm.bsrdm <- smoothSpatialCounts(bsrdm.spa, annotation.spa,
#' radius = 1.2, nnn = 4)
#' 
smoothSpatialCounts <- function(bsrdm, areas, nnn = 4,
                                radius = NULL, weight.ratio = 0.5,
                                x.col = "array_col", y.col = "array_row") {
    if (!is.null(parameters(bsrdm)$spatial.smooth) && 
        (parameters(bsrdm)$spatial.smooth)) {
        warning("Spatial smoothing or ligand max ",
            "has been already applied to these data")
    }
    if (!all(c(x.col, y.col) %in% names(areas))) {
        stop("One of x.col or y.col is not in names(areas)")
    }
    if ((weight.ratio >= 1) || (weight.ratio <= 0)) {
        stop("weight.ratio must lie in ]0 ; 1[")
    }

    # gets neighbor spots and their distances
    spots <- as.data.frame(areas[, c(x.col, y.col)])
    if (is.null(radius)) {
        # RANN standard search
        neighb <- RANN::nn2(spots, k = nnn + 1)
    } else {
        # RANN radius search
        neighb <- RANN::nn2(spots, k = nnn + 1, 
            radius = radius, searchtype = "radius")

    }
    # determine weights
    c <- neighb$nn.idx
    c[, 1] <- 0
    c[c > 0] <- 1
    ratios <- 1 - (1 - weight.ratio) * rowSums(c) / nnn
    d <- 1 / (neighb$nn.dists * c)[, -1]
    d[is.infinite(d)] <- 0
    tot <- rowSums(d) / (1 - ratios)
    weights <- sweep(d, 1, tot, "/")
    weights[is.nan(weights)] <- 0
    weights <- cbind(ratios, weights)

    # smooth the transcriptomes
    orig <- ncounts(bsrdm)
    smooth <- orig
    for (i in seq_len(ncol(orig))) {
        used <- neighb$nn.idx[i, ] > 0
        if (sum(used) > 1) {
            smooth[, i] <- orig[, neighb$nn.idx[i, used]] %*% weights[i, used]
        } else {
            smooth[, i] <- orig[, neighb$nn.idx[i, used]] * weights[i, used]
        }
    }

    ncounts(bsrdm) <- smooth
    bsrdm@param$spatial.smooth <- TRUE
    bsrdm
} # smoothSpatialCounts


#' Get maximal ligand expression at nearby locations
#'
#' @param bsrdm  A BSRDataModel object containing the expression data to smooth.
#' @param areas  A data.frame containing at least the x and y
#' coordinates of the locations.
#' @param nnn    Number of nearest-neighbor locations to use for smoothing
#' each location. In case \code{radius} is set, then it is the maximum number
#' of nearest neighbors within the radius.
#' @param radius  A maximal distance to include neighbors in the smoothing.
#' @param x.col  Column name in \code{areas} containing x coordinates.
#' @param y.col  Column name in \code{areas} containing y coordinates.
#'
#' @details Ligand expression data contained in a BSRDataModel object are
#' modified to consider the possibility that the ligand of a L-R interaction
#' might be expressed at nearby locations. This is achieved replacing each
#' ligand expression by its maximum over the central location and its
#' neighbors. Since ligands and receptors are never used as gene targets
#' in computing the receptor downstream signal correlations, this
#' substitution is compatible with our statistical model. Moreover,
#' the reciprocal configuration where the ligand is expressed at the
#' central location and hits a receptors at a neighbor location is
#' covered when the same ligand maximization scheme is applied to
#' the neighbor. L-R localization and gene signature scoring is defined
#' by the location at which the receptor is expressed after applying
#' this function.
#'
#' Two strategies are available to identify the neighbors. It is possible to
#' simply set the number of nearest-neighbors (parameter \code{nnn}). An
#' alternative consists in providing a distance radius (\code{radius}) along
#' with a a maximum number of nearest-neighbors within the radius
#' (\code{nnn.radius}). To properly define the radius, the user must know the
#' location coordinates. The strategy with the radius enables having corner
#' locations with two neighbors only and border locations with three
#' neighbors only, whereas to simply set a maximum of four neighbors for
#' instance would retrieve the four closest neighbors in every case.
#'
#' @export
#'
#' @return A BSRDataModel object containing the maximized ligand expressions.
#'
#' @importFrom RANN nn2
#'
#' @examples
#' data(bsrdm.spa, package = "BulkSignalR")
#' data(annotation.spa, package = "BulkSignalR")
#' 
#' max.bsrdm <- maxLigandSpatialCounts(bsrdm.spa, annotation.spa,
#' radius = 1.2, nnn = 4)
#'
maxLigandSpatialCounts <- function(bsrdm, areas, nnn = 4, radius = NULL,
    x.col = "array_col", y.col = "array_row") {
    if (!is.null(parameters(bsrdm)$spatial.smooth) && 
        (parameters(bsrdm)$spatial.smooth)) {
        warning("Spatial smoothing or ligand max has",
        " been already applied to these data")
    }
    if (!all(c(x.col, y.col) %in% names(areas))) {
        stop("One of x.col or y.col is not in names(areas)")
    }

    # gets neighbor spots and their distances
    spots <- as.data.frame(areas[, c(x.col, y.col)])
    if (is.null(radius)) {
        # RANN standard search
        neighb <- RANN::nn2(spots, k = nnn + 1)
    } else {
        # RANN radius search
        neighb <- RANN::nn2(spots, k = nnn + 1, 
            radius = radius, searchtype = "radius")
    }

    # maximize ligands in the transcriptomes
    orig <- ncounts(bsrdm)
    maxi <- orig
    ligands <- intersect(.SignalR$BulkSignalR_LRdb$ligand, rownames(orig))
    for (i in seq_len(ncol(orig))) {
        used <- neighb$nn.idx[i, ] > 0
        if (sum(used) > 1) {
            maxi[ligands, i] <- apply(orig[ligands, neighb$nn.idx[i, used]],
                1, max)
        } else {
            maxi[ligands, i] <- orig[ligands, i]
        }
    }
    bsrdm@ncounts <- maxi
    bsrdm@param$spatial.smooth <- TRUE
    bsrdm
} # maxLigandSpatialCounts



#' L-R interaction score spatial display
#'
#' Generate a plot with scores at the spatial coordinates of the corresponding
#' sample locations. Not limited to BulkSignalR gene signature scores.
#'
#' @param v  A named vector containing the scores, names must be
#' the IDs of each location.
#' @param areas  A data.frame containing at least the x and y
#' coordinates of the locations as well as the unique IDs of spatial locations.
#' In case \code{ref.plot} is set to TRUE,
#' a label column is required additionally.
#' @param inter.name  Interaction name to display as plot title.
#' @param rev.y  A Boolean indicating whether low y coordinates should be
#' at the top of the plot.
#' @param ref.plot  A Boolean indicating whether a reference map of the tissue
#' with area labels should be plot aside.
#' @param ref.plot.only  A Boolean indicating that only the reference plot
#' should be output.
#' @param image.raster  Raster object image to plot raw tissue image as
#' reference.
#' @param x.col  Column name in \code{areas} containing x coordinates.
#' @param y.col  Column name in \code{areas} containing y coordinates.
#' @param label.col  Column name in \code{areas} containing area labels.
#' @param idSpatial.col  Column name in \code{areas} containing the unique
#' IDs of spatial locations.
#' @param cut.p  Proportion of top and bottom values for thresholding.
#' @param low.color  Color for low score values.
#' @param mid.color  Color for score = 0.
#' @param high.color  Color for high score values.
#' @param title.fs Title font size.
#' @param legend.fs Legend items font size.
#' @param axis.fs Axis ticks font size.
#' @param label.fs Legend titles and axis names font size.
#' @param dot.size Dot size.
#' @param legend.dot.factor A factor applied to obtain the legend dot size.
#' @param ref.colors A vector of colors to bypass those automatically chosen
#' by ggplot2 for the tissue areas in the reference plot.
#' @details A single (scores) or side-by-side (reference tissue & scores) plot
#' is generated.
#' @return A spatial plot
#' @export
#' @examples
#' data(bsrinf.spa, package = "BulkSignalR")
#' data(bsrdm.spa, package = "BulkSignalR")
#' data(annotation.spa, package = "BulkSignalR")
#' 
#' thres <- 0.01
#' bsrinf.red <- reduceToBestPathway(bsrinf.spa)
#' s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
#' scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)
#' 
#' inter <- "{SLIT2} / {GPC1}"
#' 
#' spatialPlot(scores.red[inter, ], annotation.spa, inter,
#'    ref.plot = TRUE, ref.plot.only = FALSE,
#'    image.raster = NULL, dot.size = 1,
#'    label.col = "ground_truth")
#' 
#' @import ggplot2
#' @import grid
#' @importFrom gridExtra grid.arrange
#' @importFrom scales rescale
spatialPlot <- function(v, areas, inter.name, rev.y = TRUE, ref.plot = FALSE,
                        ref.plot.only = FALSE, image.raster = NULL,
                        x.col = "array_col", y.col = "array_row",
                        label.col = "label", idSpatial.col = "idSpatial",
                        cut.p = 0.01, low.color = "royalblue3",
                        mid.color = "white", high.color = "orange",
                        title.fs = 12, legend.fs = 10, axis.fs = 10,
                        label.fs = 12, dot.size = 0.5, legend.dot.factor = 10,
                        ref.colors = NULL) {
    x <- y <- label <- score <- NULL

    if (cut.p < 0 || cut.p > 0.1) {
        stop("cut.p must lie in [0;0.1]")
    }

    if (!all(c(x.col, y.col, label.col, idSpatial.col) %in% names(areas))) {
        stop(
            "One of x.col, y.col, idSpatial.col, or label.col is not in ",
            "names(areas)"
        )
    }

    # put the scores in the right order for display at (x,y) coordinates
    v <- as.numeric(v[areas[[idSpatial.col]]])
    w <- .cutExtremeValues(v, cut.p)
    if (ref.plot || ref.plot.only) {
        # include the labels
        tissue <- data.frame(
            x = areas[[x.col]], y = areas[[y.col]],
            label = factor(areas[[label.col]]),
            score = w
        )
    } else {
        # no label needed
        tissue <- data.frame(
            x = areas[[x.col]], y = areas[[y.col]],
            score = w
        )
    }

    if (rev.y) {
        # revert y-axis
        ymin <- min(tissue$y)
        ymax <- max(tissue$y)
        tissue$y <- ymax - tissue$y + ymin
    }

    # reference tissue areas are required on the side
    if (ref.plot || ref.plot.only) {
        if (is.null(image.raster)) {
            ref <- ggplot2::ggplot(data = tissue, ggplot2::aes(x = x, y = y)) +
                ggplot2::ggtitle("Reference tissue") +
                ggplot2::geom_point(ggplot2::aes(color = label),
                    size = dot.size) +
                ggplot2::theme_set(ggplot2::theme_bw(base_size = 10))+
                ggplot2::theme(axis.text=ggplot2::element_text(
                    size = axis.fs))+
                ggplot2::theme(axis.title=ggplot2::element_text(
                    size = label.fs))+
                ggplot2::theme(legend.title=ggplot2::element_text(
                    size = label.fs))+
                ggplot2::theme(legend.text=ggplot2::element_text(
                    size = legend.fs))+
                ggplot2::theme(plot.title=ggplot2::element_text(
                    size = title.fs))+
                ggplot2::guides(colour=ggplot2::guide_legend(
                    override.aes = list(size = dot.size * legend.dot.factor)
                ))
            if (!is.null(ref.colors)) {
                if (length(unique(tissue$label)) != length(ref.colors)) {
                    stop("The number of reference colors",
                    " do not match the number of labels")
                }
                ref <- ref + ggplot2::scale_color_manual(values = ref.colors)
            }
        } else {
            ref <- grid::rasterGrob(image.raster)
        }
    }

    if (ref.plot.only) {
        # do not plot L-R scores
        lr <- ref
        if (!is.null(image.raster)) {
            lr <- image.raster
        }
        ref.plot <- FALSE
    } else {
        # plot L-R scores
        if (min(w) >= 0 || max(w) <= 0) {
            # single gradient
            lr <- ggplot2::ggplot(data = tissue, ggplot2::aes(x = x, y = y)) +
                ggplot2::ggtitle(inter.name) +
                ggplot2::geom_point(ggplot2::aes(color = score),
                    size = dot.size) +
                ggplot2::scale_color_gradient(low = low.color, 
                    high = high.color) +
                ggplot2::theme_set(ggplot2::theme_bw(base_size = 10)) +
                ggplot2::theme(axis.text = ggplot2::element_text(
                    size = axis.fs)) +
                ggplot2::theme(axis.title = ggplot2::element_text(
                    size = label.fs)) +
                ggplot2::theme(legend.title = ggplot2::element_text(
                    size = label.fs)) +
                ggplot2::theme(legend.text = ggplot2::element_text(
                    size = legend.fs)) +
                ggplot2::theme(plot.title = ggplot2::element_text(
                    size = title.fs))
        } else {
            # dual gradient
            lr <- ggplot2::ggplot(data = tissue, ggplot2::aes(x = x, y = y)) +
                ggplot2::ggtitle(inter.name) +
                ggplot2::geom_point(ggplot2::aes(color = score),
                    size = dot.size) +
                ggplot2::scale_color_gradientn(
                    colors = c(low.color, mid.color, high.color),
                    values = scales::rescale(c(min(w) - 1e-6, 0, 
                        max(w) + 1e-6), c(0, 1))
                ) +
                ggplot2::theme_set(ggplot2::theme_bw(base_size = 10)) +
                ggplot2::theme(axis.text = ggplot2::element_text(
                    size = axis.fs)) +
                ggplot2::theme(axis.title = ggplot2::element_text(
                    size = label.fs)) +
                ggplot2::theme(legend.title = ggplot2::element_text(
                    size = label.fs)) +
                ggplot2::theme(legend.text = ggplot2::element_text(
                    size = legend.fs)) +
                ggplot2::theme(plot.title = ggplot2::element_text(
                    size = title.fs))
        }
    }

    if (ref.plot) {
        gridExtra::grid.arrange(ref, lr, ncol = 2)
    } else {
        if (!is.null(image.raster)) {
            plot(lr)
        }
        if (is.null(image.raster)) {
            lr
        }
    }
} # spatialPlot


#' Generate L-R interaction score spatial plots in a folder
#'
#' Generate a series of individual spatial score plots in a folder.
#' Not limited to BulkSignalR gene signature scores.
#'
#' @param scores  A matrix of scores, one L-R interaction per row and
#' spatial locations in the columns. This matrix is typically obtained
#' from BulkSignalR functions \code{scoreLRGeneSignatures} or \code{scScoring}.
#' @param areas  A data.frame containing at least the x and y
#' coordinates of the locations as well as the unique IDs of spatial locations.
#' In case \code{ref.plot} is set to TRUE,
#' a label column is required additionally.
#' @param plot.folder  The folder name in which the plot files will be written.
#' @param width  The width of each individual plot.
#' @param height  The height of each individual plot.
#' @param pointsize  PDF font point size.
#' @param rev.y  A Boolean indicating whether low y coordinates should be
#' at the top of the plot.
#' @param ref.plot  A Boolean indicating whether a reference map of the tissue
#' with area labels should be plot aside.
#' @param image.raster  Raster object image to plot raw tissue image as
#' reference.
#' @param x.col  Column name in \code{areas} containing x coordinates.
#' @param y.col  Column name in \code{areas} containing y coordinates.
#' @param label.col  Column name in \code{areas} containing area labels.
#' @param idSpatial.col  Column name in \code{areas} containing the unique
#' IDs of spatial locations.
#' @param cut.p  Proportion of top and bottom values for thresholding.
#' @param low.color  Color for low score values.
#' @param mid.color  Color for score = 0.
#' @param high.color  Color for high score values.
#' @param title.fs Title font size.
#' @param legend.fs Legend items font size.
#' @param axis.fs Axis ticks font size.
#' @param label.fs Legend titles and axis names font size.
#' @param dot.size Dot size.
#' @param ref.colors A vector of colors to bypass those automatically chosen
#' by ggplot2 for the tissue areas in the reference plot.
#' @details A set of PDF files are created in the provided folder.
#' @return Create PDF file and returns `NULL`, invisibly. 
#' @export
#' @examples
#' data(bsrdm.spa, package = "BulkSignalR")
#' data(bsrinf.spa, package = "BulkSignalR")
#' data(annotation.spa, package = "BulkSignalR")
#' 
#' thres <- 0.01
#' bsrinf.red <- reduceToBestPathway(bsrinf.spa)
#' s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
#' scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)
#' 
#' generateSpatialPlots(scores.red[1:2,],
#' annotation.spa, ".", label.col = "ground_truth")
#' 
generateSpatialPlots <- function(scores, areas, plot.folder, 
    width = 5, height = 3,
    pointsize = 8, rev.y = TRUE, ref.plot = TRUE, image.raster = NULL, 
    x.col = "array_col", y.col = "array_row",
    label.col = "label", idSpatial.col = "idSpatial",
    cut.p = 0.01, low.color = "royalblue3",
    mid.color = "white", high.color = "orange",
    title.fs = 12, legend.fs = 10, axis.fs = 10,
    label.fs = 12, dot.size = 0.5, ref.colors = NULL) {
    for (i in seq_len(nrow(scores))) {
        inter <- gsub("\\}", "", gsub("\\{", "", rownames(scores)[i]))
        fn <- gsub(" +/ +", "-", inter, perl = TRUE)

        grDevices::pdf(
            paste0(plot.folder, "/interaction-plot-", fn),
            width = width,
            height = height, useDingbats = FALSE, pointsize = pointsize
        )
        figure <- spatialPlot(scores[i, areas[[idSpatial.col]]], areas, inter,
            rev.y = rev.y, ref.plot = ref.plot,
            image.raster = image.raster,
            x.col = x.col, y.col = y.col,
            label.col = label.col, idSpatial.col = idSpatial.col,
            cut.p = cut.p, low.color = low.color, mid.color = mid.color,
            high.color = high.color, title.fs = title.fs,
            legend.fs = legend.fs, axis.fs = axis.fs, label.fs = label.fs,
            dot.size = dot.size, ref.colors = ref.colors
        )
        if (ref.plot) {
            figure
        } else {
            plot(figure)
        }

        grDevices::dev.off()
    }

    return(invisible(NULL))

} # generateSpatialPlots



#' Generate a visual index of spatial score distributions
#'
#' Generate an index made of series of small individual spatial score plots
#' in a PDF. Not limited to BulkSignalR gene signature scores.
#'
#' @param scores  A matrix of scores, one L-R interaction per row and
#' spatial locations in the columns. This matrix is typically obtained
#' from BulkSignalR functions \code{scoreLRGeneSignatures} or \code{scScoring}.
#' @param areas  A data.frame containing at least the x and y
#' coordinates of the locations, the unique IDs of spatial locations, and
#' a tissue label column.
#' @param out.file File name for the output PDF.
#' @param image.raster  Raster object image to plot raw tissue image as
#' reference.
#' @param ref.plot  A Boolean indicating whether a reference map of the tissue
#' with area labels should be plot first.
#' @param cut.p  Proportion of top and bottom values for thresholding.
#' @param low.color  Color for low score values.
#' @param mid.color  Color for score = 0.
#' @param high.color  Color for high score values.
#' @param title.fs Title font size.
#' @param legend.fs Legend items font size.
#' @param axis.fs Axis ticks font size.
#' @param label.fs Legend titles and axis names font size.
#' @param dot.size Dot size.
#' @param ref.colors A vector of colors to bypass those automatically chosen
#' by ggplot2 for the tissue areas in the reference plot.
#' @param x.col  Column name in \code{areas} containing x coordinates.
#' @param y.col  Column name in \code{areas} containing y coordinates.
#' @param label.col  Column name in \code{areas} containing area labels.
#' @param idSpatial.col  Column name in \code{areas} containing the unique
#' IDs of spatial locations.
#' @param base.h  Width of each plot.
#' @param base.v  Height of each plot.
#' @param ratio the vertical/horizontal ratio.
#' 
#' @details A PDF file is created that contains the index.
#' @return Create PDF file and returns `NULL`, invisibly. 
#' 
#' @export
#' @examples
#' data(bsrdm.spa, package = "BulkSignalR")
#' data(bsrinf.spa, package = "BulkSignalR")
#' data(annotation.spa, package = "BulkSignalR")
#' 
#' thres <- 0.01
#' bsrinf.red <- reduceToBestPathway(bsrinf.spa)
#' s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
#' scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)
#' 
#' # generate visual index on disk in pdf file
#' spatialIndexPlot(scores.red[1:2,], annotation.spa,  
#' label.col = "ground_truth",
#' out.file = "spatialIndexPlot")
#' 
#' @import grid
#' @importFrom gridExtra grid.arrange
spatialIndexPlot <- function(scores, areas, out.file, ref.plot = TRUE,
                            image.raster = NULL,
                            x.col = "array_col", y.col = "array_row",
                            label.col = "label", idSpatial.col = "idSpatial",
                            cut.p = 0.01, low.color = "royalblue3",
                            mid.color = "white", high.color = "orange",
                            title.fs = 12, legend.fs = 10, axis.fs = 10,
                            label.fs = 12, dot.size = 0.25, ratio = 1.25,
                            base.v = 2.5, base.h = 3, ref.colors = NULL) {
    # one reference plot at the beginning
    if (ref.plot) {
        if (is.null(image.raster)) {
            plots <- list(spatialPlot(scores[1, ], areas, "",
                x.col = x.col, y.col = y.col,
                label.col = label.col, idSpatial.col = idSpatial.col,
                ref.plot.only = TRUE, cut.p = cut.p,
                low.color = low.color,
                mid.color = mid.color, high.color = high.color,
                title.fs = title.fs, legend.fs = legend.fs,
                axis.fs = axis.fs, label.fs = label.fs,
                dot.size = dot.size, ref.colors = ref.colors
            ))
        } else {
            plots <- list(grid::rasterGrob(image.raster))
        }
    } else {
        plots <- list()
    }

    # actual plots
    for (i in seq_len(nrow(scores))) {
        inter <- gsub("\\}", "", gsub("\\{", "", rownames(scores)[i]))
        plots <- c(plots, list(
            spatialPlot(scores[i, ], areas, inter,
                x.col = x.col, y.col = y.col,
                label.col = label.col, idSpatial.col = idSpatial.col,
                ref.plot = FALSE, cut.p = cut.p,
                low.color = low.color,
                mid.color = mid.color, high.color = high.color,
                title.fs = title.fs, legend.fs = legend.fs,
                axis.fs = axis.fs,
                label.fs = label.fs,
                dot.size = dot.size
            )
        ))
    }

    # file output at matrix dimension computation
    m <- length(plots)
    n <- round(sqrt(m / ratio * base.v / base.h))
    l <- m %/% n
    if (l * n < m) {
        l <- l + 1
    }
    grDevices::pdf(out.file, width = n * base.h, 
        height = l * base.v, useDingbats = FALSE, pointsize = 6)
    gridExtra::grid.arrange(grobs = plots, ncol = n, nrow = l)
    grDevices::dev.off()

    return(invisible(NULL))

} # spatialIndexPlot


#' Generate separated plots for a L-R interaction
#'
#' Generate a detailed view related to a chosen interaction made of series of
#' small individual spatial plots: tissue organization (optional), gene
#' signature score, ligand and receptor expression.
#'
#' @param v  A named vector containing the gene signature scores for the
#' L-R interaction including the contribution of the pathway, names must be
#' the IDs of each location. Alternatively, v can be a gene signature score
#' matrix such as those returned by \code{scoreLRGeneSignatures} and the
#' row named "{\code{L}} / {\code{R}}" will be used.
#' @param L  The name of the ligand.
#' @param R  The name of the receptor.
#' @param ncounts  The (normalized) expression matrix with column names equal
#' to the IDs of each location.
#' @param areas  A data.frame containing at leastcluster_columns the x and y
#' coordinates of the locations as well as the unique IDs of spatial locations.
#' In case \code{ref.plot} is set to TRUE,
#' a label column is required additionally.
#' @param inter.name  Interaction name to display as plot title,
#' equal to "\code{L} / \code{R}" unless specified.
#' @param rev.y  A Boolean indicating whether low y coordinates should be
#' at the top of the plot.
#' @param ref.plot  A Boolean indicating whether a reference map of the tissue
#' with area labels should be plot aside.
#' @param image.raster  Raster object image to plot raw tissue image as
#' reference.
#' @param x.col  Column name in \code{areas} containing x coordinates.
#' @param y.col  Column name in \code{areas} containing y coordinates.
#' @param label.col  Column name in \code{areas} containing area labels.
#' @param idSpatial.col  Column name in \code{areas} containing the unique
#' IDs of spatial locations.
#' @param cut.p  Proportion of top and bottom values for thresholding.
#' @param low.color  Color for low score values.
#' @param mid.color  Color for score = 0.
#' @param high.color  Color for high score values.
#' @param title.fs Title font size.
#' @param legend.fs Legend items font size.
#' @param axis.fs Axis ticks font size.
#' @param label.fs Legend titles and axis names font size.
#' @param dot.size Dot size.
#' @param ref.colors A vector of colors to bypass those automatically chosen
#' by ggplot2 for the tissue areas in the reference plot.
#' @param legend.dot.factor A factor applied to obtain the legend dot size.
#' @details A set of spatial plots are generated including an optional
#' reference tissue plot (image or areas represented), the gene signature
#' scores, the ligand expression values, and the receptor expression values.
#' @return A set of spatial plots. 
#' @export
#' @examples
#' data(bsrdm.spa, package = "BulkSignalR")
#' data(bsrinf.spa, package = "BulkSignalR")
#' data(annotation.spa, package = "BulkSignalR")
#' 
#' thres <- 0.01
#' bsrinf.red <- reduceToBestPathway(bsrinf.spa)
#' s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
#' scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)
#' 
#' separatedLRPlot(scores.red, "SLIT2", "GPC1", 
#' ncounts(bsrdm.spa), 
#' annotation.spa,
#' label.col = "ground_truth")
#' 
#' @import grid
#' @importFrom gridExtra grid.arrange
separatedLRPlot <- function(v, L, R, ncounts, areas, 
    inter.name = NULL, rev.y = TRUE,
    ref.plot = TRUE, image.raster = NULL,
    x.col = "array_col", y.col = "array_row",
    label.col = "label", idSpatial.col = "idSpatial",
    cut.p = 0.01, low.color = "royalblue3",
    mid.color = "white", high.color = "orange",
    title.fs = 12, legend.fs = 10, axis.fs = 10,
    label.fs = 12, dot.size = 0.5, legend.dot.factor = 10,
    ref.colors = NULL) {
    if (is.matrix(v)) {
        v <- v[paste0("{", L, "} / {", R, "}"), ]
    }
    if (is.null(inter.name)) {
        inter.name <- paste(L, "/", R)
    }

    # one reference plot at the beginning
    if (ref.plot) {
        if (is.null(image.raster)) {
            plots <- list(spatialPlot(v, areas, "",
                ref.plot.only = TRUE,
                x.col = x.col, y.col = y.col,
                label.col = label.col, idSpatial.col = idSpatial.col,
                dot.size = dot.size,
                legend.dot.factor = legend.dot.factor,
                ref.colors = ref.colors
            ))
        } else {
            plots <- list(grid::rasterGrob(image.raster))
        }
    } else {
        plots <- list()
    }

    # gene signature scores
    plots <- c(plots, list(
        spatialPlot(v, areas, inter.name,
            rev.y = rev.y, ref.plot = FALSE,
            x.col = x.col, y.col = y.col,
            label.col = label.col, idSpatial.col = idSpatial.col,
            cut.p = cut.p, low.color = low.color,
            mid.color = mid.color, high.color = high.color,
            title.fs = title.fs, legend.fs = legend.fs, axis.fs = axis.fs,
            label.fs = label.fs, dot.size = dot.size
        )
    ))

    # ligand and receptor plots
    plots <- c(plots, list(
        spatialPlot(ncounts[L, ], areas,
            inter.name = L, rev.y = rev.y, ref.plot = FALSE,
            x.col = x.col, y.col = y.col,
            label.col = label.col, idSpatial.col = idSpatial.col,
            cut.p = cut.p, low.color = low.color,
            mid.color = mid.color, high.color = high.color,
            title.fs = title.fs, legend.fs = legend.fs, axis.fs = axis.fs,
            label.fs = label.fs, dot.size = dot.size
        ),
        spatialPlot(ncounts[R, ], areas,
            inter.name = R, rev.y = rev.y, ref.plot = FALSE,
            x.col = x.col, y.col = y.col,
            label.col = label.col, idSpatial.col = idSpatial.col,
            cut.p = cut.p, low.color = low.color,
            mid.color = mid.color, high.color = high.color,
            title.fs = title.fs, legend.fs = legend.fs, axis.fs = axis.fs,
            label.fs = label.fs, dot.size = dot.size
        )
    ))

    # display
    gridExtra::grid.arrange(grobs = plots, ncol = 2, nrow = 2)
} # separatedLRPlot


#' Statistical association of scores with area labels
#'
#' Compute the statistical association of L-R interaction score spatial
#' distributions with tissue area labels.
#' Not limited to BulkSignalR gene signature scores.
#'
#' @param scores  A matrix of scores, one L-R interaction per row and
#' spatial locations in the columns. This matrix is typically obtained
#' from BulkSignalR functions \code{scoreLRGeneSignatures} or \code{scScoring}.
#' @param areas  A data.frame containing at least the x and y
#' coordinates of the locations, the unique IDs of spatial locations, and
#' a label column.
#' @param test  The chosen statistical test or statistics
#' (see details below).
#' @param label.col  Column name in \code{areas} containing area labels.
#' @param idSpatial.col  Column name in \code{areas} containing the unique
#' IDs of spatial locations.
#' @param fdr.proc  Multiple hypothesis correction procedure, see
#' \code{multtest}.
#' @return A data.frame with the names of the interactions, the value of the
#' chosen statistics, and the corresponding Q-value.
#' @details In case the
#' nonparametric Kruskal-Wallis test is chosen, additional columns are provided
#' testing each label for significantly larger scores (Kruskal-Wallis is global
#' and only says whether one or several labels show a bias). Individual
#' labels are tested with Wilcoxon and two columns are added *per* label,
#' one for the statistics and one for a Bonferroni-corrected P-value over
#' all the labels.
#'
#' In case an actual statistical test is chosen, a parametric test (ANOVA) and
#' a non-parametric test (Kruskal-Wallis) are available for the global analysis.
#' Individual labels are tested with T-tests or Wilcoxon (Bonferroni-corrected)
#' accordingly.
#'
#' In case a statistics is preferred, Spearman correlation or explained variance
#' (r2 or coefficient of determination, through linear models) 
#' are available. They mesure the relationship
#' between each individual area and \code{scores}. For the explained variance,
#' a global value (R2) is also computed from a multi-linear model (the same as
#' what is used for the ANOVA).
#' @export
#' @examples
#' data(bsrdm.spa, package = "BulkSignalR")
#' data(bsrinf.spa, package = "BulkSignalR")
#' data(annotation.spa, package = "BulkSignalR")
#' thres <- 0.01
#' #bsrinf.red <- reduceToBestPathway(bsrinf.spa)
#' #s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
#' #scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)
#' 
#' # Run in other examples no need to be run again
#' # spatialAssociation(scores.red[c(1:2),], areas = annotation.spa,
#' # label.col = "ground_truth")
#' @import multtest
#' @importFrom foreach %do%
spatialAssociation <- function(scores, areas, 
    test = c("Kruskal-Wallis", "ANOVA", "Spearman", "r2"),
    label.col = "label", idSpatial.col = "idSpatial",
    fdr.proc = c(
    "BH", "Bonferroni",
    "Holm", "Hochberg", "SidakSS", "SidakSD", 
    "BY", "ABH", "TSBH")) {

    test <- match.arg(test)
    fdr.proc <- match.arg(fdr.proc)
    if (!(label.col %in% names(areas))) {
        stop("label.col is not in names(areas)")
    }
    if (!(idSpatial.col %in% names(areas))) {
        stop("idSpatial.col is not in names(areas)")
    }

    labels <- factor(areas[[label.col]])
    ul <- unique(labels)
    n <- length(ul)
    i <- lab <- NULL
    res <- foreach::foreach(i = seq_len(nrow(scores)), .combine = rbind) %do% {
        inter <- gsub("\\}", "", gsub("\\{", "", rownames(scores)[i]))
        v <- scores[i, areas[[idSpatial.col]]]
        if (test == "Kruskal-Wallis") {
            kw <- stats::kruskal.test(v, labels)

            # specific associations with each label
            pvals <- foreach::foreach(lab = ul, .combine = c) %do% {
                wt <- stats::wilcox.test(
                    x = v[labels == lab], y = v[labels != lab],
                    alternative = "greater"
                )
                list(min(c(wt$p.value * n, 1))) # Bonferroni correction
            }
            names(pvals) <- ul

            cbind(
                data.frame(
                    interaction = inter, pval = kw$p.value, H = kw$statistic,
                    stringsAsFactors = FALSE
                ),
                pvals
            )
        } else if (test == "ANOVA") {
            # ANOVA
            df <- data.frame(score = v, label = labels)
            my.lm <- stats::lm(score ~ label, data = df)
            ano <- stats::anova(my.lm)

            # specific associations with each label
            pvals <- foreach::foreach(lab = ul, .combine = c) %do% {
                tt <- stats::t.test(
                    x = v[labels == lab], y = v[labels != lab],
                    alternative = "greater"
                )
                list(min(c(tt$p.value * n, 1))) # Bonferroni correction
            }
            names(pvals) <- ul

            cbind(
                data.frame(
                    interaction = inter, pval = ano$`Pr(>F)`[1],
                    F = ano$`F value`[1], stringsAsFactors = FALSE
                ),
                pvals
            )
        } else if (test == "Spearman") {
            # Spearman correlation

            # specific correlations with each label
            corrs <- foreach::foreach(lab = ul, .combine = c) %do% {
                local <- rep(0, length(labels))
                local[labels == lab] <- 1
                co <- stats::cor(v, local, method = "spearman")
                list(co)
            }
            names(corrs) <- ul

            cbind(data.frame(interaction = inter, 
                stringsAsFactors = FALSE), corrs)
        } else {
            # r2 from linear regressions

            df <- data.frame(score = v, label = labels)
            my.lm <- stats::lm(score ~ label, data = df)
            ano <- stats::anova(my.lm)
            R2 <- ano$`Sum Sq`[1] / sum(ano$`Sum Sq`)

            # specific r2 for each label
            r2s <- foreach::foreach(lab = ul, .combine = c) %do% {
                local <- rep(0, length(labels))
                local[labels == lab] <- 1
                dfl <- data.frame(score = v, local = local)
                my.llm <- stats::lm(score ~ local, data = dfl)
                lano <- stats::anova(my.llm)
                r2 <- lano$`Sum Sq`[1] / sum(lano$`Sum Sq`)
                list(r2)
            }
            names(r2s) <- ul

            cbind(
                data.frame(interaction = inter, 
                    global.R2 = R2, stringsAsFactors = FALSE),
                r2s
            )
        }
    }

    # multiple hypothesis correction on the global association P-values
    if (test %in% c("Kruskal-Wallis", "ANOVA")) {
        rawp <- res$pval
        adj <- multtest::mt.rawp2adjp(rawp, fdr.proc)
        res$qval <- adj$adjp[order(adj$index), fdr.proc]
        label.index.stop <- ncol(res) - 1
        res <- res[, c(seq_len(3), 
        ncol(res), 4:label.index.stop)] # put Q-values in column 4
    }

    rownames(res) <- res$interaction
    res
} # spatialAssociation


#' Heatmap plot of association of scores with area labels
#'
#' Plot a heatmap featuring Q-values 
#' or values of statistical association between
#' L-R interaction score spatial distributions and tissue area labels.
#'
#' @param associations  A statistical association data.frame generated
#' by the function \code{spatialAssociation}.
#' @param qval.thres  The maximum Q-value to consider in the plot (a
#' L-R interaction must associate with one label at least with a Q-value
#' smaller or equal to this threshold).
#' @param absval.thres  The minimum value to consider in the plot (a
#' L-R interaction must associate with one label at least with an absolute
#' value larger or equal to this threshold).
#' @param  colors  A function returning a color for a given value such as
#' generated by \code{circlize::colorRamp2}.
#' @details Display a heatmap linking L-R interactions to labels.
#' @export
#' @return   ComplexHeatmap::Heatmap object
#' @examples
#' data(bsrdm.spa, package = "BulkSignalR")
#' data(bsrinf.spa, package = "BulkSignalR")
#' data(annotation.spa, package = "BulkSignalR")
#' 
#' thres <- 0.01
#' bsrinf.red <- reduceToBestPathway(bsrinf.spa)
#' s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
#' scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)
#' 
#' # statistical association with tissue areas based on correlations
#' 
#' assoc.bsr.corr <- spatialAssociation(scores.red[c(1:10), ],
#' areas = annotation.spa, label.col = "ground_truth",test = "Spearman")
#'
#' spatialAssociationPlot(assoc.bsr.corr)
#' @import ComplexHeatmap
#' @importFrom circlize colorRamp2
spatialAssociationPlot <- function(associations, qval.thres = 0.01, 
    absval.thres = 0,colors = NULL) {
    # transform and filter data
    if (sum(c("pval", "qval") %in% names(associations)) == 2) {
        # log-scale on Q-values
        mat <- data.matrix(associations[, -(seq_len(4))])
        mat[mat == 0] <- min(mat[mat > 0])
        mat <- -log10(mat)
        thres <- -log10(abs(qval.thres))
        good <- apply(mat, 1, max) >= thres
        mat <- mat[good, ]
    } else {
        # linear scale
        if ("global.R2" %in% names(associations)) {
            mat <- data.matrix(associations[, -(seq_len(2))])
        } else {
            mat <- data.matrix(associations[, -1])
        }
        thres <- abs(absval.thres)
        good <- apply(abs(mat), 1, max) >= thres
        mat <- mat[good, ]
    }

    if (is.null(colors)) {
        # create a color scale
        if (min(mat) >= 0) {
            colscale <- circlize::colorRamp2(
                breaks = c(
                    0, thres - 1e-10,
                    seq(thres, max(mat), length.out = 10)
                ),
                colors = c(
                    "lightgray", "lightgray",
                    grDevices::hcl.colors(10, "Viridis")
                )
            )
        } else if (thres == 0) {
            colscale <- circlize::colorRamp2(
                breaks = c(min(mat), 0, max(mat)),
                colors = c("royalblue", "white", "orangered")
            )
        } else {
            colscale <- circlize::colorRamp2(
                breaks = c(min(mat), -thres, thres, max(mat)),
                colors = c("royalblue", "white", "white", "orangered")
            )
        }
    } else {
        colscale <- colors
    }

    # plot heatmap
    di.lab <- stats::dist(t(mat))
    hc.lab <- stats::hclust(di.lab, method = "ward.D")
    dend.lab <- stats::as.dendrogram(hc.lab)
    di.int <- stats::dist(mat)
    hc.int <- stats::hclust(di.int, method = "ward.D")
    dend.int <- stats::as.dendrogram(hc.int)
    ComplexHeatmap::Heatmap(mat,
        col = colscale, cluster_rows = dend.int,
        cluster_columns = dend.lab, show_row_dend = FALSE,
        show_column_dend = FALSE
    )
} # spatialAssociationPlot


#' 2D-projection of spatial score distributions
#'
#' Use PCA or t-SNE to obtain a 2D-projection of a set of spatial scores
#' or associations.
#' This plot summarizes the diversity of patterns occuring in a spatial
#' dataset. Use the function \code{spatialIndexPlot} to create a large
#' visual index of many spatial distributions.
#' Not limited to BulkSignalR gene signature scores.
#'
#' @param scores  A matrix of scores, one L-R interaction per row and
#' spatial locations in the columns. This matrix is typically obtained
#' from BulkSignalR functions \code{scoreLRGeneSignatures} or \code{scScoring}.
#' @param associations  A statistical association data.frame generated
#' by the function \code{spatialAssociation}.
#' @param proj  Projection method : 'PCA' or 'tSNE' are available arguements.
#' @param score.based  A logical indicating whether the plot should be
#' based on scores or the associations directly.
#' @param qval.thres  The maximum Q-value to consider in the plot (a
#' L-R interaction must associate with one label at least with a Q-value
#' smaller or equal to this threshold). Relevant for Kruskal-Wallis and
#' ANOVA tests in \code{spatialAssociation}.
#' @param val.thres  The minimum value to consider in the plot (a
#' L-R interaction must associate with one label at least with a value
#' larger or equal to this threshold). Relevant for Spearman and r2
#' associations in \code{spatialAssociation}.
#' @param with.names  A logical indicating whether L-R names should be plotted.
#' @param text.fs Point label font size in case \code{with.names} is TRUE.
#' @param legend.fs Legend items font size.
#' @param axis.fs Axis ticks font size.
#' @param label.fs Legend titles and axis names font size.
#' @param dot.size Dot size.
#' @param perplexity  Perplexity parameter for t-SNE.
#' @details Display a 2D-projection of the score spatial distributions.
#' @return Display a 2D-projection of the score spatial distributions.
#' 
#' @export
#' @examples
#' data(bsrdm.spa, package = "BulkSignalR")
#' data(bsrinf.spa, package = "BulkSignalR")
#' data(annotation.spa, package = "BulkSignalR")
#' 
#' thres <- 0.01
#' bsrinf.red <- reduceToBestPathway(bsrinf.spa)
#' s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
#' scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)
#' 
#' # statistical association with tissue areas based on correlations
#' # For display purpose, we only use a subset here
#'
#' 
#' assoc.bsr.corr <- spatialAssociation(scores.red[c(1:3), ],
#' annotation.spa, label.col = "ground_truth",test = "Spearman")
#'
#' spatialDiversityPlot(scores.red[c(1:3),],assoc.bsr.corr)
#' 
#' @importFrom foreach %do%
#' @importFrom Rtsne Rtsne
#' @importFrom ggrepel geom_text_repel
#' @import ggplot2
spatialDiversityPlot <- function(scores, associations, proj = c("PCA", "tSNE"),
                                score.based = FALSE,
                                qval.thres = 0.01, val.thres = 0, 
                                with.names = FALSE, text.fs = 2.5,
                                legend.fs = 10, axis.fs = 10,
                                label.fs = 12, dot.size = 1,
                                perplexity = 10) {
    i <- PC1 <- PC2 <- name <- label <- NULL
    tSNE1 <- tSNE2 <- log.scale <- remove.col <- NULL

    proj <- match.arg(proj)

    # adapt to the type of associations
    if (sum(c("pval", "qval") %in% names(associations)) == 2) {
        remove.col <- seq_len(4)
        log.scale <- TRUE
    } else {
        # linear scale
        if ("global.R2" %in% names(associations)) {
            remove.col <- seq_len(2)
        } else {
            remove.col <- 1
        }
        log.scale <- FALSE
    }
    labels <- names(associations)[-remove.col]

    cols <- stats::setNames(
        c(grDevices::rainbow(length(labels), s = 0.5), "lightgray"),
        c(labels, "non_signif")
    )

    # find the strongest association for each L-R interaction
    best.label <- foreach::foreach(
        i = seq_len(nrow(associations)),
        .combine = c
    ) %do% {
        if (log.scale) {
            qvals <- as.numeric(associations[i, -remove.col])
            if (min(qvals) > qval.thres) {
                "non_signif"
            } else {
                labels[which.min(qvals)]
            }
        } else {
            values <- as.numeric(associations[i, -remove.col])
            if (max(values) < val.thres) {
                "non_signif"
            } else {
                labels[which.max(values)]
            }
        }
    }

    # the plot itself
    if (proj == "PCA") {
        if (score.based) {
            pca <- stats::prcomp(scores, scale. = TRUE)
        } else if (log.scale) {
            pca <- stats::prcomp(-log10(
                data.matrix(associations[, -remove.col])),
                scale. = TRUE
            )
        } else {
            pca <- stats::prcomp(
                data.matrix(associations[, -remove.col]), scale. = TRUE)
        }
        dat <- data.frame(PC1 = pca$x[, 1],
            PC2 = pca$x[, 2], label = best.label, name = rownames(scores))
        if (with.names) {
            p <- ggplot2::ggplot(data = dat, 
                ggplot2::aes(x = PC1, y = PC2, label = name)) +
                ggrepel::geom_text_repel(size = text.fs, max.overlaps = Inf) +
                ggplot2::geom_point(ggplot2::aes(color = label),
                    size = dot.size)
        } else {
            p <- ggplot2::ggplot(data = dat, ggplot2::aes(x = PC1, y = PC2)) +
                ggplot2::geom_point(ggplot2::aes(color = label),
                    size = dot.size)
        }

        p <- p + ggplot2::theme_set(ggplot2::theme_bw(base_size = 10)) +
            ggplot2::theme(axis.text = ggplot2::element_text(
                size = axis.fs)) +
            ggplot2::theme(axis.title = ggplot2::element_text(
                size = label.fs)) +
            ggplot2::theme(legend.title = ggplot2::element_text(
                size = label.fs)) +
            ggplot2::theme(legend.text = ggplot2::element_text(
                size = legend.fs))
        p
    } else {
        if (score.based) {
            tsne <- Rtsne::Rtsne(scores, perplexity = perplexity)
        } else if (log.scale) {
            tsne <- Rtsne::Rtsne(-log10(
                data.matrix(associations[, -remove.col])),
                perplexity = perplexity, check_duplicates = FALSE
            )
        } else {
            tsne <- Rtsne::Rtsne(data.matrix(associations[, -remove.col]),
                perplexity = perplexity, check_duplicates = FALSE
            )
        }
        dat <- data.frame(tSNE1 = tsne$Y[, 1], 
            tSNE2 = tsne$Y[, 2], label = best.label, name = rownames(scores))
        if (with.names) {
            p <- ggplot2::ggplot(data = dat,
                ggplot2::aes(x = tSNE1, y = tSNE2, label = name)) +
                ggrepel::geom_text_repel(size = text.fs) +
                ggplot2::geom_point(ggplot2::aes(color = label),
                    size = dot.size)
        } else {
            p <- ggplot2::ggplot(data = dat, 
                ggplot2::aes(x = tSNE1, y = tSNE2)) +
                ggplot2::geom_point(ggplot2::aes(color = label),
                    size = dot.size)
        }

        p <- p + ggplot2::theme_set(ggplot2::theme_bw(base_size = 10)) +
            ggplot2::theme(axis.text = ggplot2::element_text(
                size = axis.fs)) +
            ggplot2::theme(axis.title = ggplot2::element_text(
                size = label.fs)) +
            ggplot2::theme(legend.title = ggplot2::element_text(
                size = label.fs)) +
            ggplot2::theme(legend.text = ggplot2::element_text(
                size = legend.fs))
        p
    }
} # spatialDiversityPlot

#' BulkSignalR Cluster Comparison Object
#'
#' An S4 class to represent the comparison of two clusters of samples to
#' infer LR interactions based on the resulting P-values,
#' log-fold-changes (logFC), and expression values.
#'
#' @slot col.clusterA   Column indices for the samples in cluster A.
#' @slot col.clusterB   Column indices for the samples in cluster B.
#' @slot differential.stats  Comparison statistics A versus B 
#' as a data.frame and
#' containing at least 3 columns named 'pval', 'logFC', and 'expr'.
#'
#' @export
#' @examples
#' new("BSRClusterComp")
setClass("BSRClusterComp",
    slots = c(
        col.clusterA = "integer",
        col.clusterB = "integer",
        differential.stats = "data.frame"
    ),
    prototype = list(
        col.clusterA = as.integer(c(1, 2)),
        col.clusterB = as.integer(c(3, 4)),
        differential.stats = data.frame(
            pval = c(0.01, 0.01),
            logFC = c(1, -1), expr = c(1, 2)
        )
    )
)

setValidity(
    "BSRClusterComp",
    function(object) {
        if (!is.integer(object@col.clusterA)) {
            return("col.clusterA indices are not all integers")
        }
        if (length(object@col.clusterA) < 1) {
            return("col.clusterA empty")
        }
        if (!is.integer(object@col.clusterB)) {
            return("col.clusterB indices are not all integers")
        }
        if (length(object@col.clusterB) < 1) {
            return("col.clusterB empty")
        }
        if (length(intersect(object@col.clusterA, 
        object@col.clusterB)) > 0) {
            return("col.cluster1 and col.clusterB are not disjoint")
        }
        if (!is.data.frame(object@differential.stats)) {
            return("specified stats are not a data.frame")
        }

        TRUE
    }
)

setMethod(
    "show", "BSRClusterComp",
    function(object) {
        if (length(object@col.clusterA) > 5) {
            cat("Cluster A columns:", 
            object@col.clusterA[seq_len(5)], "...\n")
        } else {
            cat(
                "Cluster A columns:",
                object@col.clusterA[
                    seq_len(length(object@col.clusterA))], "\n"
            )
        }
        if (length(object@col.clusterB) > 5) {
            cat("Cluster B columns:", object@col.clusterB[seq_len(5)], "...\n")
        } else {
            cat(
                "Cluster B columns:",
                object@col.clusterB[
                    seq_len(length(object@col.clusterB))], "\n"
            )
        }
        print(utils::head(object@differential.stats))
    }
)

# Constructor ====================================

#' Definition of the comparison between two clusters of samples
#'
#' Define the columns of the expression matrix that belong to each cluster,
#' and store the result of the cluster differences statistical analysis
#' obtained by an external tool such as edgeR or DESeq2 in a dedicated
#' data frame.
#'
#' @name BSRClusterComp
#'
#' @param obj    A BSRDataModelComp object.
#' @param col.clusterA   Cluster A column indices.
#' @param col.clusterB   Cluster B column indices.
#' @param differential.stats  A data.frame containing statistics about
#' the differential
#' analysis cluster A versus B. \code{differentialStats} must contain 
#' at least the
#' columns 'pval' (for P-values), 'logFC' for log-fold-changes A/B, and
#' 'expr' for the expression of the genes in cluster A.
#'
#' @details Create a BSRClusterComp object describing a comparison
#' of two clusters of columns taken from the expression matrix
#' in the BSRDataModelComp object \code{obj}. Such a cluster comparison
#' description is the basis for inferring LRIs from differential
#' expression P-values instead of correlation analysis.
#'
#' The rows of \code{differentialStats} must be in the same order 
#' as those of the count
#' matrix in \code{obj}. Alternatively, \code{differentialStats}
#' rows can be named and a 1-1 correspondence must exist between these names
#' and those of the count matrix.
#'
#' @return A BSRClusterComp object.
#'
#' @export
#'
#' @examples
#' # prepare data
#' data(sdc, package = "BulkSignalR")
#' normal <- grep("^N", names(sdc))
#' bsrdm <- BSRDataModel(sdc[, -normal])
#'
#' # define the comparison
#' bsrdm.comp <- as(bsrdm, "BSRDataModelComp")
#' colA <- as.integer(1:3)
#' colB <- as.integer(12:15)
#' n <- nrow(ncounts(bsrdm.comp))
#' stats <- data.frame(
#'     pval = runif(n), logFC = rnorm(n, 0, 2),
#'     expr = runif(n, 0, 10)
#' )
#' rownames(stats) <- rownames(ncounts(bsrdm.comp))
#' bsrcc <- BSRClusterComp(bsrdm.comp, colA, colB, stats)
#'
#' @importFrom methods new
BSRClusterComp <- function(obj,
    col.clusterA, 
    col.clusterB, 
    differential.stats) {
    if (!is.integer(col.clusterA)) {
        stop("col.clusterA must contain integer indices")
    }
    if (!is.integer(col.clusterB)) {
        stop("col.clusterB must contain integer indices")
    }
    if (length(intersect(col.clusterA, col.clusterB)) > 0) {
        stop("col.clusterA and col.clusterB must be disjoint")
    }
    if (any(col.clusterA < 1 | col.clusterA > ncol(ncounts(obj)))) {
        stop("col.clusterA indices must fall in [1; ncol(ncounts)]")
    }
    if (any(col.clusterB < 1 | col.clusterB > ncol(ncounts(obj)))) {
        stop("col.clusterB indices must fall in [1; ncol(ncounts)]")
    }
    if (!is.data.frame(differential.stats)) {
        stop("differential.stats must be a data.frame")
    }
    if (!all(c("pval", "logFC", "expr") %in% names(differential.stats))) {
        stop("differential.stats data.frame must contain",
            " columns named 'pval', 'logFC', and 'expr'")
    }
    if (nrow(differential.stats) != nrow(ncounts(obj))) {
        stop("differential.stats and ncounts(obj) number of rows differ")
    }
    if (!is.null(rownames(differential.stats)) &&
        (sum(rownames(differential.stats) %in% 
        rownames(ncounts(obj))) != nrow(differential.stats))) {
        stop("differential.stats rownames defined",
            " but do not all match ncounts(obj)")
    }
    if (is.null(rownames(differential.stats))) {
        rownames(differential.stats) <- rownames(ncounts(obj))
    }

    new("BSRClusterComp", col.clusterA = col.clusterA, 
    col.clusterB = col.clusterB, 
    differential.stats = differential.stats)
} # BSRClusterComp


# Accessors & setters ====================================

setGeneric("colClusterA", signature="x",
    function(x) standardGeneric("colClusterA")
)
#' Cluster A columns accessor
#'
#' @name colClusterA
#' @aliases colClusterA,BSRClusterComp-method
#' @param x object BSRClusterComp
#' @return col.clusterA
#' @examples
#' bsrcc <- new("BSRClusterComp")
#' colClusterA(bsrcc)
#' @export
setMethod("colClusterA", "BSRClusterComp", function(x) x@col.clusterA)

setGeneric("colClusterA<-", signature=c("x", "value"),
    function(x, value) standardGeneric("colClusterA<-")
)
#' Cluster A columns setter (internal use only)
#'
#' @param x object BSRClusterComp
#' @param value value to be set for BSRClusterComp
#' @return returns \code{NULL}
#' @keywords internal
setMethod("colClusterA<-", "BSRClusterComp", function(x, value) {
    x@col.clusterA <- value
    methods::validObject(x)
    x
})

setGeneric("colClusterB", signature="x",
    function(x) standardGeneric("colClusterB")
)
#' Cluster B columns accessor
#'
#' @name colClusterB
#' @aliases colClusterB,BSRClusterComp-method
#' @param x object BSRClusterComp
#' @return col.clusterB
#' @examples
#' bsrcc <- new("BSRClusterComp")
#' colClusterB(bsrcc)
#' @export
setMethod("colClusterB", "BSRClusterComp", function(x) x@col.clusterB)

setGeneric("colClusterB<-", signature=c("x", "value"),
    function(x, value) standardGeneric("colClusterB<-")
)
#' Cluster B columns setter (internal use only)
#'
#' @param x object BSRClusterComp
#' @param value value to be set for BSRClusterComp
#' @return returns \code{NULL}
#' @keywords internal
setMethod("colClusterB<-", "BSRClusterComp", function(x, value) {
    x@col.clusterB <- value
    methods::validObject(x)
    x
})


setGeneric("differentialStats", signature="x",
    function(x) standardGeneric("differentialStats")
)
#' Cluster comparison statistics accessor
#'
#' @name differentialStats
#' @aliases differentialStats,BSRClusterComp-method
#' @param x BSRClusterComp object
#' @return diffferential.stats
#' @examples
#' bsrcc <- new("BSRClusterComp")
#' differentialStats(bsrcc)
#' @export
setMethod("differentialStats", "BSRClusterComp", 
function(x) x@differential.stats)

setGeneric("differentialStats<-", signature=c("x", "value"),
    function(x, value) standardGeneric("differentialStats<-")
)
#' Cluster comparison statistics setter (internal use only)
#'
#' @param x object BSRClusterComp
#' @param value value to be set for BSRClusterComp
#' @return returns \code{NULL}
#' @keywords internal
setMethod("differentialStats<-", "BSRClusterComp", function(x, value) {
    x@differential.stats <- value
    methods::validObject(x)
    x
})

---
title: "BulkSignalR : </br> Inference of ligand-receptor interactions 
from bulk data or spatial transcriptomics"
author:
  - name: Jean-Philippe Villemin
    affiliation:
    - Institut de Recherche en Cancérologie de Montpellier,
     Inserm, Montpellier, France 
    email: jean-philippe.villemin@inserm.fr
  - name: Jacques Colinge
    affiliation:
    - Institut de Recherche en Cancérologie de Montpellier,
     Inserm, Montpellier, France 
    email: jacques.colinge@inserm.fr
date: "`r format(Sys.Date(), '%m/%d/%Y')`"
output: 
  rmarkdown::html_vignette:
      self_contained: true
      toc: true
      toc_depth: 4
      highlight: pygments
      fig_height: 3
      fig_width: 3
      fig_caption: no
      code_folding: show
package: BulkSignalR
vignette: >
  %\VignetteIndexEntry{BulkSignalR-Differential}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::knit_hooks$set(optipng = knitr::hook_optipng)
options(rmarkdown.html_vignette.check_title = FALSE)
```
```{r load-libs, message = FALSE,  warning = FALSE, results = FALSE}
library(BulkSignalR)
```

## Cluster-based differential analysis

By default, `BulkSignalR` inference of L-R interactions relies on correlation
analysis across all the samples. That is, a putative L-R interaction is assessed
for coherent expression of the ligand, the receptor, and target genes in a
pathway below receptor across all the samples.

Alternatively, it is possible to define sample clusters based on an independent
analysis (outside of `BulkSignalR`) to group similar samples. Based on cluster
definitions, it is for instance possible to assess whether L-R interactions are
significantly stronger in a cluster compared to all the samples, to all the
samples but the chosen cluster, or in comparison with another cluster.
In practice, the user must simply provide a table of differential gene or
protein expression analysis that is relevant for the selected comparison.

In this mode, L-R interactions are inferred based on gene or protein
regulation-associated P-values instead of P-values resulting from the
correlation analysis in the default mode. As already stated, differential
P-value estimations are left to the user to compute. With bulk data, commonly
used libraries such as `DESeq2`, `EdgeR`, or `limma` constitute obvious options.

In the next chunk of code, we illustrate the differential mode using 
Salivary Duct Carcinoma (SDC) samples where we compare two clusters 
of patients.  

We first create a **BSRDataModelComp** object as follows:

```{r diffmode1,eval=TRUE}
data(sdc, package = "BulkSignalR")
normal <- grep("^N", names(sdc))
bsrdm.comp <- BSRDataModelComp(sdc[, -normal])
```

To make a very small example we generate random values,
but user should provide his own logFC and
associated pvalues from DGE ouputs.  

```{r diffmode2,eval=TRUE}
colA <- as.integer(1:3)
colB <- as.integer(12:15)

n <- nrow(ncounts(bsrdm.comp))
stats <- data.frame(pval = runif(n),
logFC = rnorm(n, 0, 2),
expr = runif(n, 0, 10))
rownames(stats) <- rownames(ncounts(bsrdm.comp))
```    

We define the cluster comparison and add it.  

```{r diffmode3,eval=TRUE}

bsrcc <- BSRClusterComp(bsrdm.comp, colA, colB, stats)
bsrdm.comp <- addClusterComp(bsrdm.comp, bsrcc, "random.example")

```

Finally we infer ligand-receptor interactions from the comparison. 
We use a subset of the reference to speed up inference 
in the context of the vignette.

```{r diffmode4,eval=TRUE}

subset <- c("REACTOME_BASIGIN_INTERACTIONS",
"REACTOME_SYNDECAN_INTERACTIONS",
"REACTOME_ECM_PROTEOGLYCANS",
"REACTOME_CELL_JUNCTION_ORGANIZATION")

reactSubset <- BulkSignalR:::.SignalR$BulkSignalR_Reactome[
BulkSignalR:::.SignalR$BulkSignalR_Reactome$`Reactome name` %in% subset,]

resetPathways(dataframe = reactSubset,
resourceName = "Reactome")

bsrinf.comp <- BSRInferenceComp(bsrdm.comp,
reference="REACTOME",
max.pval = 1, 
"random.example")

head(LRinter(bsrinf.comp))

```  

\

##  Technical notes

The three basic `BulkSignalR` S4 classes 
BSRDataModel, BSRInference, and BSRSignature 
were derived into new classes to add the functionalities required by the
differential mode:  

* **BSRDataModelComp**, denoted here as `bsrdm.comp` is a daughter class of 
**BSRDataModel**, previously denoted `bsrdm`  
* **BSRInferenceComp**, denoted here as `bsrinf.comp` is a daughter class of
**BSRInference**, previously denoted `bsrinf` 
* **BSRSignatureComp**, denoted here as `bsrsig.comp` is a daughter class of 
**BSRSignature**, previously denoted `bsrsig` 

The new S4 class **BSRClusterComp** is meant to represent the comparison
between two clusters of samples. It primarily contains the results of the
user-provided differential analysis in a table containing gene/protein names,
P-values, and log-fold-changes (logFC).

Usually, we recommend instantiating a `BSRDataModel` object to contain the
expression matrix underlying the differential analysis. Then, it is
promoted into a `BSRDataModelComp` object such that cluster definitions
and comparisons can be added to it. This can be for instance done with
``as(bsrdm, "BSRDataModelComp")``. The `addClusterComp` method
of a `BSRDataModelComp` object allows adding one or several comparisons between
clusters, each added comparison being defined in a `BSRClusterComp` object.

\
    
Thank you for reading this guide and for using `BulkSignalR`.  

\

## Session Information

```{r session-info}
sessionInfo()
```

---
title: "BulkSignalR : </br> Inference of ligand-receptor interactions 
from bulk data or spatial transcriptomics"
author:
  - name: Jean-Philippe Villemin
    affiliation:
    - Institut de Recherche en Cancérologie de Montpellier,
     Inserm, Montpellier, France 
    email: jean-philippe.villemin@inserm.fr
  - name: Jacques Colinge
    affiliation:
    - Institut de Recherche en Cancérologie de Montpellier,
     Inserm, Montpellier, France 
    email: jacques.colinge@inserm.fr
date: "`r format(Sys.Date(), '%m/%d/%Y')`"
abstract: > 
  <p> BulkSignalR is used to infer ligand-receptor (L-R) interactions from bulk
   expression (transcriptomics/proteomics) data, or spatial
   transcriptomics. Potential L-R interactions are taken from the
   LR*db* database, which is  included in our other package SingleCellSignalR,
   available from
   Bioconductor. Inferences rely on a statistical model linking potential
   L-R interactions with biological pathways from Reactome or biological
   processes from GO.
   
   A number of visualization and data summary functions are proposed to
   help navigating the predicted interactions.<br/><br/>
  BulkSignalR package version: `r packageVersion("BulkSignalR")`
output: 
  rmarkdown::html_vignette:
      self_contained: true
      toc: true
      toc_depth: 4
      highlight: pygments
      fig_height: 3
      fig_width: 3
      fig_caption: no
      code_folding: show
package: BulkSignalR
vignette: >
  %\VignetteIndexEntry{BulkSignalR-Main}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::knit_hooks$set(optipng = knitr::hook_optipng)
```

```{r load-libs, message = FALSE,  warning = FALSE, results = FALSE}
library(BulkSignalR)
library(igraph)
library(STexampleData)
```


# Introduction

    

## What is it for? 

`BulkSignalR` is a Bioconductor library that enables the inference of L-R
interactions from bulk expression data sets, *i.e.*, from transcriptomics
(RNA-seq or microarrays) or expression proteomics.

`BulkSignalR` also applies to spatial transcriptomics such as
10x Genomics VISIUM:TM:, and a set of
functions dedicated to spatial data has been added to better support
this particular use of the library.

## Starting point 

There is a variety of potential data sources that can be used with `BulkSignalR`,
*e.g.*, expression proteomics or RNA sequencing. Such data are typically
represented as a matrix of numbers representing the abundance of molecules
(gene transcripts or proteins) across samples. This matrix may have been
normalized already or not prior the use of `BulkSignalR`.
In both cases, the latter matrix is the input data for `BulkSignalR` analysis.

It is mandatory that genes/proteins are represented as rows of the
expression matrix and the samples as columns. HUGO gene symbols
(even for proteins) must be used to ensure matching LR*db*,
Reactome, and GOBP contents.  

You can also work with an object from the `SummarizedExperiment` or
`SpatialExperiment` Bioconductor classes directly.

## How does it work? 

As represented in the figure below, only a few steps are required in order to 
perform a `BulkSignalR` analysis. Three S4 objects will be sequentially constructed:

* **BSRDataModel**, denoted here as `bsrdm`  
* **BSRInference**, denoted here as `bsrinf`  
* **BSRSignature**, denoted here as `bsrsig`

\

**BSRDataModel** comprises the expression data matrix and the 
parameters of the statistical model learned from this matrix.

**BSRInference** provides various lists that contain the ligands, receptors,
downstream pathways as well as the target genes, their correlations and statistical significance for all the L-R interactions.

**BSRSignature** contains gene signatures associated with the triples
(ligand, receptor, downstream pathway)
stored in a `BSRInference` object. Those signatures are comprised
of the ligand and the receptor obviously, but also the pathway
target genes that were used in the statistical model. Gene signatures
are meant to report the L-R interaction as a global phenomenon integrating
its downstream effect. Indeed, signatures can be subsequently scored with a
dedicated function allowing the user to obtain a numerical value representing
the activity of the L-R interactions (with downstream consequences)
across the samples. Signature scores are returned as a matrix (one row *per*
L-R interaction and one column *per* sample).

Because of the occurrence of certain receptors in multiple pathways,
and also because some ligands may bind several receptors, or *vice versa*,
BSRInference objects may contain redundant data depending on how
the user wants to look at them. We therefore provide a range of
reduction operators meant to obtain reduced BSRInference objects
(see below).  

\

Furthermore, we provide several handy functions to explore the data 
through different plots (heatmaps, alluvial plots, chord diagrams or networks).

`BulkSignalR` library functions have many parameters that can be
changed by the user to fit specific needs (see Reference Manual for details). 

    
    
## Parallel mode settings

Users can reduce compute time
by using several processors in parallel.

```{r parallel, message=FALSE, warning=FALSE}
library(doParallel)
n.proc <- 1
cl <- makeCluster(n.proc)
registerDoParallel(cl)

# To add at the end of your script:
# stopCluster(cl)
```

**Notes:**
For operating systems that can fork such as the UNIX-like systems,
it might be preferable to use the library `doMC` that is faster (less
overhead). This is transparent to `BulkSignalR`.

In case you need to reproduce results exactly and since statistical
model parameter learning involves the generation of randomized expression
matrices, you might want to use `set.seed()`. In a parallel mode,
`iseed` that is a parameter of `clusterSetRNGStream` must be used.

\
    
# First Example  
    
## Loading the data
    
Here, we load salivary duct carcinoma (SDC) bulk transcriptomes integrated as 
`sdc` in `BulkSignalR`.

```{r loading, eval=TRUE}
data(sdc)
head(sdc)
```  

## Building a BSRDataModel object  

The constructor `BSRDataModel` creates the first object from SDC data above.

```{r BSRDataModel, eval=TRUE,cache=FALSE}
bsrdm <- BSRDataModel(counts = sdc)
bsrdm
```  
`learnParameters` updates a BSRDataModel object with the parameters
necessary for `BulkSignalR` statistical modeling. 


```{r learnParameters,eval=TRUE,warning=FALSE,fig.dim = c(7,3)}
bsrdm <- learnParameters(bsrdm, quick=TRUE)
bsrdm
``` 

## Building a BSRInference object

From the previous object `bsrdm`, you can generate inferences by calling its
method
`BSRInference`. The returned BSRInference object,
contains all the inferred L-R interactions 
with their associated pathways and corrected p-values.  

From there, you can already access L-R interactions using `LRinter(bsrinf)`,
which returns a summary table.


```{r BSRInference,eval=TRUE}

# We use a subset of the reference to speed up
# inference in the context of the vignette.
subset <- c("REACTOME_BASIGIN_INTERACTIONS",
"REACTOME_SYNDECAN_INTERACTIONS",
"REACTOME_ECM_PROTEOGLYCANS",
"REACTOME_CELL_JUNCTION_ORGANIZATION")

reactSubset <- BulkSignalR:::.SignalR$BulkSignalR_Reactome[
BulkSignalR:::.SignalR$BulkSignalR_Reactome$`Reactome name` %in% subset,]

resetPathways(dataframe = reactSubset,
              resourceName = "Reactome")

bsrinf <- BSRInference(bsrdm,
    min.cor = 0.3,
    reference="REACTOME")

LRinter.dataframe <- LRinter(bsrinf)

head(LRinter.dataframe[
    order(LRinter.dataframe$qval),
    c("L", "R", "LR.corr", "pw.name", "qval")])

```

You can finally filter out non-significant L-R interactions and order them 
by Q-values before saving them into a file for instance.

```{r BSRInferenceTfile, eval=FALSE}
write.table(LRinter.dataframe[order(LRinter.dataframe$qval), ],
    "./sdc_LR.tsv",
    row.names = FALSE,
    sep = "\t",
    quote = FALSE
)
```
    

## Reduction strategies

The output of `BSRInference` is exhaustive and can thus contain
redundancy due to the redundancy present in the reference databases (Reactome,
KEGG, GOBP) and multilateral interactions in LR*db*.
To alleviate this issue, we propose several strategies.

### Reducing a BSRInference object to pathways

With `reduceToPathway`, all the L-R interactions with their receptors included
in a certain pathway are aggregated to only report the downstream pathway
once. For a given pathway, the reported P-values and target genes are those 
of best (smallest P-value) L-R interaction that was part of the aggregation.
Nothing is recomputed, we simply merge data.

```{r ReduceToPathway, eval=TRUE}
bsrinf.redP <- reduceToPathway(bsrinf)
```

### Reducing a BSRInference object to the best pathways

With ` reduceToBestPathway`, a BSRInference object is reduced to only report
one pathway per L-R interaction. The pathway with the smallest P-value 
is selected.
A same pathways might occur multiple times with due different L-R interactions
that all have their receptor in this pathway.

```{r ReduceToBestPathway, eval=TRUE}
bsrinf.redBP <- reduceToBestPathway(bsrinf)
```

### Reducing to ligands or receptors

As already mentioned, several ligands might bind to single receptor (or
several shared receptors) and the converse might happen also. Two
reduction operators enable users to either aggregate all the ligands
of a same receptor or all the receptors bound by a same ligand:

```{r ReduceToLigand,eval=TRUE}
bsrinf.L <- reduceToLigand(bsrinf)
bsrinf.R <- reduceToReceptor(bsrinf)
```

### Combined reductions

Combinations are possible.

For instance, users can apply the `reduceToPathway` and `reduceToBestPathway` 
functions sequentially to maximize the reduction effect. In case the exact
same sets of aggregated ligands and receptors obtained with `reduceToPathway`
was associated with several pathways, the pathway with the best P-value
would be kept by `reduceToBestPathway`.

```{r doubleReduction, eval=TRUE}
bsrinf.redP <- reduceToPathway(bsrinf)
bsrinf.redPBP <- reduceToBestPathway(bsrinf.redP)
```


## Building a BSRSignature object

Gene signatures for a given, potentially reduced BSRInference object
are generated by the `BSRSignature` constructor, which returns a BSRSignature
object.

To follow the activity of L-R interactions across the samples of
a data set, `scoreLRGeneSignatures` computes a score for each gene signature
in each sample.
Then, heatmaps can be generated to represent differences, *e.g.*, using
the built-in utility function `simpleHeatmap`.

Hereafter, we show different workflows of reductions combined with gene
signature scoring and display.


###  Scoring by ligand-receptor

```{r scoringLR,eval=TRUE,fig.dim = c(5,4)}
bsrsig.redBP <- BSRSignature(bsrinf.redBP, qval.thres=0.001)

scoresLR <- scoreLRGeneSignatures(bsrdm, bsrsig.redBP,
    name.by.pathway=FALSE
)

simpleHeatmap(scoresLR[1:20, ],
    hcl.palette="Cividis",
    pointsize=8)

```

###  Scoring by pathway

```{r scoringPathway,eval=TRUE,fig.dim = c(7,3)}    

bsrsig.redPBP <- BSRSignature(bsrinf.redPBP, qval.thres=0.01)

scoresPathway <- scoreLRGeneSignatures(bsrdm, bsrsig.redPBP,
    name.by.pathway=TRUE
)

simpleHeatmap(scoresPathway,
    hcl.palette="Blue-Red 2",
    pointsize=8)
```

## Other visualization utilities


### Heatmap of ligand-receptor-target genes expression

After computing gene signatures score, we may wish to look at the
expression of the genes involved in that signature. For instance,
we can display three heatmaps corresponding to the scaled (z-scores)
expression of ligands (pink), receptors (green), and target genes (blue).

On the top of each individual heatmap, the whole signature score from
`scoreLRGeneSignatures` is reported for reference.

```{r heatmapMulti,eval=TRUE,fig.dim = c(7,10)}   
pathway1 <- pathways(bsrsig.redPBP)[1]
signatureHeatmaps(
    pathway = pathway1,
    bsrdm = bsrdm,
    heights = c(3,2,15),
    bsrsig = bsrsig.redPBP
)
```        

### AlluvialPlot

`alluvial.plot` is a function that enable users to represent the different
interactions between ligands, receptors, and pathways stored in a
BSRInference object.

Obviously, it is possible to filter by ligand, receptor, or pathway to limit
output complexity. This
is achieved by specifying a key word in the chosen category. A threshold on
L-R interaction Q-values can be applied in addition.

```{r AlluvialPlot,eval=TRUE,fig.dim = c(8,3)}   
alluvialPlot(bsrinf,
    keywords = c("LAMC1"),
    type = "L",
    qval.thres = 0.01
)
```


### BubblePlot

`bubblePlotPathwaysLR` is a handy way to visualize the strengths of several
L-R interactions in relation with their receptor downstream pathways.

A vector of pathways of interest can be provided to limit the complexity
of the plot.

```{r BubblePlot,eval=TRUE,fig.dim = c(8,4)} 

pathways <- LRinter(bsrinf)[1,c("pw.name")]
bubblePlotPathwaysLR(bsrinf,
    pathways = pathways,
    qval.thres = 0.001,
    color = "red",
    pointsize = 8
)
```


### Chordiagram

`chord.diagram.LR` is a function that enables users to feature the different
L-R interactions involved in a specific pathway.

L-R correlations strengths are drawn using a yellow color-scale.  
Ligands are in grey, whereas receptors are in green.  
You can also highlight in red one specific interaction by passing values 
of a L-R pair as follows  `ligand="FYN", receptor="SPN"`.  

```{r Chordiagram,eval=TRUE,fig.dim = c(6,4.5)} 
chordDiagramLR(bsrinf,
    pw.id.filter = "R-HSA-210991",
    limit = 20,
    ligand="FYN", 
    receptor="SPN"
)
```

\

# Network Analysis

Since `BulkSignalR` relies on intracellular networks to estimate the statistical
significance of (ligand, receptor, pathway) triples, links from receptors to
target genes are naturally accessible. Different functions enable users to
exploit this graphical data for plotting or further data analysis.

Furthermore, networks can be exported in text files and graphML objects 
to be further explored with Cytoscape (www.cytoscape.org), 
yEd (www.yworks.com), or similar software tools.

```{r network1, eval=TRUE} 
# Generate a ligand-receptor network and export it in .graphML
# for Cytoscape or similar tools
gLR <- getLRNetwork(bsrinf.redBP, qval.thres = 1e-3)

# save to file
# write.graph(gLR,file="SDC-LR-network.graphml",format="graphml")

# As an alternative to Cytoscape, you can play with igraph package functions.
plot(gLR,
    edge.arrow.size = 0.1,
    vertex.label.color = "black",
    vertex.label.family = "Helvetica",
    vertex.label.cex = 0.1
)
```

```{r network2,  eval=TRUE} 
# You can apply other functions.


# Community detection
u.gLR <- as_undirected(gLR) # most algorithms work for undirected graphs only
comm <- cluster_edge_betweenness(u.gLR)
# plot(comm,u.gLR,
#     vertex.label.color="black",
#     vertex.label.family="Helvetica",
#     vertex.label.cex=0.1)

# Cohesive blocks
cb <- cohesive_blocks(u.gLR)
plot(cb, u.gLR,
    vertex.label.color = "black",
    vertex.label.family = "Helvetica",
    vertex.label.cex = 0.1,
    edge.color = "black"
)
```


```{r network3, eval=FALSE,warning=FALSE} 
# For the next steps, we just share the code below, but graph generation
# functions are commented to lighten the vignette.

# Generate a ligand-receptor network complemented with intra-cellular,
# receptor downstream pathways [computations are a bit longer here]
#
# You can save to a file for cystoscape or plot with igraph.

gLRintra <- getLRIntracellNetwork(bsrinf.redBP, qval.thres = 1e-3)

lay <- layout_with_kk(gLRintra)
# plot(gLRintra,
#     layout=lay,
#     edge.arrow.size=0.1,
#     vertex.label.color="black",
#     vertex.label.family="Helvetica",
#     vertex.label.cex=0.1)

# Reduce complexity by focusing on strongly targeted pathways
pairs <- LRinter(bsrinf.redBP)
top <- unique(pairs[pairs$pval <  1e-3, c("pw.id", "pw.name")])
top
gLRintra.res <- getLRIntracellNetwork(bsrinf.redBP,
    qval.thres = 0.01,
    restrict.pw = top$pw.id
)
lay <- layout_with_fr(gLRintra.res)

# plot(gLRintra.res,
#     layout=lay,
#     edge.arrow.size=0.1,
#     vertex.label.color="black",
#     vertex.label.family="Helvetica",
#     vertex.label.cex=0.4)
```

\

# Non-human data

In order to process data from non-human organisms,
users only need to specify a few additional parameters and all the
other steps of the analysis remain unchanged.

By default, `BulksignalR` works with *Homo sapiens*.
We implemented a strategy using ortholog genes (mapped by the 
`orthogene` BioConductor package) in `BulkSignalR` directly.

The function `findOrthoGenes` creates a correspondence table between
human and another organism.
`convertToHuman`  then converts an initial expression matrix to
a *Homo sapiens* equivalent.

When calling `BSRDataModel`, the user only needs to pass this transformed
matrix, the actual non-human organism name, and the correspondence table.
Then, L-R interaction inference is performed as for human data.
Finally, users can switch back to gene names relative to the original organism
via `resetToInitialOrganism`.
The rest of the workflow is executed as usual for
computing gene signatures and visualizing.

\

```{r mouse,eval=TRUE,warning=FALSE}
data(bodyMap.mouse)

ortholog.dict <- findOrthoGenes(
    from_organism = "mmusculus",
    from_values = rownames(bodyMap.mouse)
)

matrix.expression.human <- convertToHuman(
    counts = bodyMap.mouse,
    dictionary = ortholog.dict
)

bsrdm <- BSRDataModel(
    counts = matrix.expression.human,
    species = "mmusculus",
    conversion.dict = ortholog.dict
)

bsrdm <- learnParameters(bsrdm,quick=TRUE)

bsrinf <- BSRInference(bsrdm,reference="REACTOME")

bsrinf <- resetToInitialOrganism(bsrinf, conversion.dict = ortholog.dict)

# For example, if you want to explore L-R interactions
# you can proceed as shown above for a human dataset.

bsrinf.redBP <- reduceToBestPathway(bsrinf)
bsrsig.redBP <- BSRSignature(bsrinf.redBP, qval.thres=0.001)
scoresLR <- scoreLRGeneSignatures(bsrdm, bsrsig.redBP, name.by.pathway=FALSE)
head(LRinter(bsrinf.redBP))
#simpleHeatmap(scoresLR, column.names=TRUE,
#              pointsize=8)
```

\

# Spatial Transcriptomics

`BulkSignalR` analysis can be applied to spatial transcriptomics (ST) at medium
resolution such as with 10x Genomics Visium:TM: or Nanostring GeoMx:TM:.
L-R interactions that display significant occurrence in a tissue are
retrieved. Additional functions
have been introduced to facilitate the visualization and analysis of the
results in a spatial context.

To account for the rather limited dynamics of ST data and dropouts, it is
usually recommended to release specific parameters controlling the
training of the statistical model. Namely, minimum correlation can be set
at -1 during training and the minimum number of target genes in a
pathway should be reduced to 2 instead of the default at 4.
Also, the thresholds on L-R interaction Q-values should
be raised at 1% or 5% instead of 0.1%.

A key spatial plot function is `spatialPlot` that enables visualizing
L-R interaction gene signature scores at their spatial coordinates with a
potential reference plot (raw tissue image or user-defined areas) on the side.

As we published `BulkSignalR` paper, we provided example scripts to apply
the library to spatial data represented in tabular text files
[BulkSignalR github companion](
https://github.com/jcolinge/BulkSignalR_companion).
It is also possible to work with an object of the `SpatialExperiment` 
Bioconductor class. In addition, the `VisiumIO` package allows users
to readily import Visium data from the 10X Space Ranger pipeline and retrieve 
a `SpatialExperiment object`.

\

```{r spatial1,eval=TRUE,message=FALSE}
# load data =================================================

# We re-initialize the environment variable of Reactome
# to have all the pathways (we reduced it to a subset above to fasten
# example computations)
reactSubset <- getResource(resourceName = "Reactome",
cache = TRUE)

resetPathways(dataframe = reactSubset,
resourceName = "Reactome")

# Few steps of pre-process to subset a spatialExperiment object
# from STexampleData package ==================================

spe <- Visium_humanDLPFC()
set.seed(123)

speSubset <- spe[, colData(spe)$ground_truth%in%c("Layer1","Layer2")]

idx <- sample(ncol(speSubset), 10)
speSubset <- speSubset[, idx]

my.image.as.raster <- SpatialExperiment::imgRaster(speSubset, 
    sample_id = imgData(spe)$sample_id[1], image_id = "lowres")

colData(speSubset)$idSpatial <- paste(colData(speSubset)[[4]],
                colData(speSubset)[[5]],sep = "x")


annotation <- colData(speSubset)

```

```{r spatial2,eval=TRUE,warning=FALSE}
# prepare data =================================================

bsrdm <- BSRDataModel(speSubset,
    min.count = 1,
    prop = 0.01,
    method = "TC",
    symbol.col = 2,
    x.col = 4,
    y.col = 5, 
    barcodeID.col = 1)

bsrdm <- learnParameters(bsrdm,
    quick = TRUE,
    min.positive = 2,
    verbose = TRUE)

bsrinf <- BSRInference(bsrdm, min.cor = -1,reference="REACTOME")

# spatial analysis ============================================

bsrinf.red <- reduceToBestPathway(bsrinf)
pairs.red <- LRinter(bsrinf.red)

thres <- 0.01
min.corr <- 0.01
pairs.red <- pairs.red[pairs.red$qval < thres & pairs.red$LR.corr > min.corr,]

head(pairs.red[
    order(pairs.red$qval),
    c("L", "R", "LR.corr", "pw.name", "qval")])

s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
scores.red <- scoreLRGeneSignatures(bsrdm,s.red)

head(scores.red)

```

From here, one can start exploring the ST data 
through different plots.

**Note:** As we work on a much reduced data set to fasten 
the vignette generation, only a few point are
displayed in each plot. Actual plots are a lot more informative.


```{r spatialPlot3,eval=TRUE,fig.dim = c(6,4.5)}
# Visualization ============================================

# plot one specific interaction

# we have to follow the syntax with {} 
# to be compatible with the reduction functions
inter <- "{SLIT2} / {GPC1}"

# with raw tissue reference
spatialPlot(scores.red[inter, ], annotation, inter,
    ref.plot = TRUE, ref.plot.only = FALSE,
    image.raster = NULL, dot.size = 1,
    label.col = "ground_truth"
)

# or with synthetic image reference
spatialPlot(scores.red[inter, ], annotation, inter,
    ref.plot = TRUE, ref.plot.only = FALSE,
    image.raster = my.image.as.raster, dot.size = 1,
    label.col = "ground_truth"

)
```  

You can dissect one interaction to separately visualize the expression of the
ligand and the receptor involved in a specific L-R interaction.

```{r spatialPlot4,eval=TRUE,fig.dim = c(6,4.5)}

separatedLRPlot(scores.red, "SLIT2", "GPC1", 
    ncounts(bsrdm), 
    annotation,
    label.col = "ground_truth")

```

```{r spatialPlot5,eval=TRUE}

# generate a visual index in a pdf file directly
spatialIndexPlot(scores.red, annotation,  
    label.col = "ground_truth",
    out.file="spatialIndexPlot")

```

\

Finally, we provide a function to assess the statistical associations of
L-R interaction signature scores with user-defined areas of the sample.
Based on these associations, a further visualization function can represent the
latter in the form of a heatmap.  

\

```{r spatialPlot6,eval=TRUE,fig.dim = c(6,4.5)}

# statistical association with tissue areas based on correlations
# For display purpose, we only use a subset here
assoc.bsr.corr <- spatialAssociation(scores.red[c(1:17), ],
annotation, label.col = "ground_truth",test = "Spearman")

head(assoc.bsr.corr)

spatialAssociationPlot(assoc.bsr.corr)

```

We also provide 2D-projections (see the `spatialDiversityPlot` function) to
assess the diversity among L-R interaction spatial distributions over
an entire data set. Other functions such as `generateSpatialPlots` can
output multiple individual spatial plots in a graphic file directly.

\


Note that we describe additional use cases in the
[BulkSignalR github companion](
https://github.com/jcolinge/BulkSignalR_companion). 

\


See the reference manual for all the details.  


```{r inferCells,eval=FALSE,include=FALSE}

## Additional functions

# This is not part of the main workflow for analyzing L-R interactions but
# we offer convenience functions for inferring cell types. However 
# we recommend using other software packages specifically dedicated to this
# purpose.

# Inferring cell types


data(sdc, package = "BulkSignalR")
bsrdm <- BSRDataModel(counts = sdc)
bsrdm <- learnParameters(bsrdm)
bsrinf <- BSRInference(bsrdm)

# Common TME cell type signatures
data(immune.signatures, package = "BulkSignalR")
unique(immune.signatures$signature)
immune.signatures <- immune.signatures[immune.signatures$signature %in% c(
    "B cells", "Dentritic cells", "Macrophages",
    "NK cells", "T cells", "T regulatory cells"
), ]
data("tme.signatures", package = "BulkSignalR")
signatures <- rbind(immune.signatures,
tme.signatures[tme.signatures$signature %in% 
c("Endothelial cells", "Fibroblasts"), ])
tme.scores <- scoreSignatures(bsrdm, signatures)

# assign cell types to interactions
lr2ct <- assignCellTypesToInteractions(bsrdm, bsrinf, tme.scores)
head(lr2ct)

# cellular network computation and plot
g.table <- cellularNetworkTable(lr2ct)

gCN <- cellularNetwork(g.table)

plot(gCN, edge.width = 5 * E(gCN)$score)

gSummary <- summarizedCellularNetwork(g.table)
plot(gSummary, edge.width = 1 + 30 * E(gSummary)$score)

# relationship with partial EMT---
# Should be tested HNSCC data instead of SDC!!

# find the ligands
data(p.EMT, package = "BulkSignalR")
gs <- p.EMT$gene
triggers <- relateToGeneSet(bsrinf, gs)
triggers <- triggers[triggers$n.genes > 1, ] # at least 2 target genes in the gs
ligands.in.gs <- intersect(triggers$L, gs)
triggers <- triggers[!(triggers$L %in% ligands.in.gs), ]
ligands <- unique(triggers$L)

# link to cell types
cf <- cellTypeFrequency(triggers, lr2ct, min.n.genes = 2)
missing <- setdiff(rownames(tme.scores), names(cf$s))
cf$s[missing] <- 0
cf$t[missing] <- 0

op <- par(mar = c(2, 10, 2, 2))
barplot(cf$s, col = "lightgray", horiz = T, las = 2)
par(op)

# random selections based on random gene sets
qval.thres <- 1e-3
inter <- LRinter(bsrinf)
tg <- tgGenes(bsrinf)
tcor <- tgCorr(bsrinf)
good <- inter$qval <= qval.thres
inter <- inter[good, ]
tg <- tg[good]
tcor <- tcor[good]
all.targets <- unique(unlist(tg))
r.cf <- list()
for (k in 1:100) { # should 1000 or more
    r.gs <- sample(all.targets, length(intersect(gs, all.targets)))
    r.triggers <- relateToGeneSet(bsrinf, r.gs, qval.thres = qval.thres)
    r.triggers <- r.triggers[r.triggers$n.genes > 1, ]
    r.ligands.in.gs <- intersect(r.triggers$L, r.gs)
    r.triggers <- r.triggers[!(r.triggers$L %in% r.ligands.in.gs), ]
    r <- cellTypeFrequency(r.triggers, lr2ct, min.n.genes = 2)
    missing <- setdiff(rownames(tme.scores), names(r$s))
    r$s[missing] <- 0
    r$t[missing] <- 0
    o <- order(names(r$t))
    r$s <- r$s[o]
    r$t <- r$t[o]
    r.cf <- c(r.cf, list(r))
}
r.m.s <- foreach(i = seq_len(length(r.cf)), .combine = rbind) %do% {
    r.cf[[i]]$s
}

# plot results
op <- par(mar = c(2, 10, 2, 2))
boxplot(r.m.s, col = "lightgray", horizontal = T, las = 2)
pts <- data.frame(x = as.numeric(cf$s[colnames(r.m.s)]), cty = colnames(r.m.s))
stripchart(x ~ cty, data = pts, add = TRUE, pch = 19, col = "red")
par(op)
for (cty in rownames(tme.scores)) {
    cat(cty, ": P=", sum(r.m.s[, cty] >= cf$s[cty]) / nrow(r.m.s),
    "\n", sep = "")
}
```

\

# Acknowledgements

We thank Guillaume Tosato for his help with the figures and
Gauthier Gadouas for testing the software on different platforms.

\

Thank you for reading this guide and for using `BulkSignalR`.

\

# Session Information

```{r session-info}
sessionInfo()
```

---
title: "BulkSignalR : </br> Inference of ligand-receptor interactions 
from bulk data or spatial transcriptomics"
author:
  - name: Jean-Philippe Villemin
    affiliation:
    - Institut de Recherche en Cancérologie de Montpellier,
     Inserm, Montpellier, France 
    email: jean-philippe.villemin@inserm.fr
  - name: Jacques Colinge
    affiliation:
    - Institut de Recherche en Cancérologie de Montpellier,
     Inserm, Montpellier, France 
    email: jacques.colinge@inserm.fr
date: "`r format(Sys.Date(), '%m/%d/%Y')`"
output: 
  rmarkdown::html_vignette:
      self_contained: true
      toc: true
      toc_depth: 4
      highlight: pygments
      fig_height: 3
      fig_width: 3
      fig_caption: no
      code_folding: show
package: BulkSignalR
vignette: >
  %\VignetteIndexEntry{BulkSignalR-Configure}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::knit_hooks$set(optipng = knitr::hook_optipng)
```

```{r load-libs, message = FALSE,  warning = FALSE, results = FALSE}
library(BulkSignalR)
```

## Configure local cache and hidden environment variables

`BulkSignalR` donwloads resource files just once after the first 
installation.  

A mecanism handled by `BioCFileCache` package informs the user 
if the local files are up to date. 
Several functions as `cacheVersion` and `cacheInfo` give
informations about the content of the local cache and its synchronization 
state  with its remote source.  

Cache directory can be cleaned by `cacheClear` function and and forced to be 
created again using `createResources(onRequest=TRUE)`. Otherwise it will be
automatically recreated when the package is next loaded.   

The knowledge database (pathways - GO-BP / Reactome, Ligand &
recepors - LRdb, and gene Network are handled inside a hidden environment 
variable  called `.SignalR`)

The sources can be rewritten using `resetPathways`, `resetLRdb` 
and `resetNetwork`.  

Several data versions can be access remotely from :
[here](https://partage-dev.montp.inserm.fr:9192/CBSB/SignalR/resources/)
These datasets can be used to reset the sources locally.  

```{r cash,eval=FALSE}

# You can see hidden environment variable list as follows.
ls(BulkSignalR:::.SignalR)

# Setup Pathways for reactome (can be done for GO-BP also)

subset <- c("REACTOME_BASIGIN_INTERACTIONS",
"REACTOME_SYNDECAN_INTERACTIONS",
"REACTOME_ECM_PROTEOGLYCANS",
"REACTOME_CELL_JUNCTION_ORGANIZATION")

reactSubset <- BulkSignalR:::.SignalR$BulkSignalR_Reactome[
BulkSignalR:::.SignalR$BulkSignalR_Reactome$`Reactome name` %in% subset,]

resetPathways(dataframe = reactSubset,
resourceName = "Reactome")

cacheInfo()
cacheVersion()

cacheClear()

# Ask to download again
createResources(onRequest=TRUE)


# Set up LRdb
df <- data.frame(
  ligand    = c("A2M", "AANAT", "AANAT", "ACE", "ACE"),
  receptor  = c("LRP1", "MTNR1A", "MTNR1B", "AGTR2", "BDKRB2")
)
resetLRdb(df,switch=TRUE)

# Set up Network
df <- data.frame(
  a.gn   = c("A2M", "AANAT", "AANAT", "ACE", "ACE"),
  type  = c("regulates-transcription-of",
   "regulates-transcription-of",
   "regulates-transcription-of", 
   "regulates-transcription-of", 
   "regulates-transcription-of"),
  b.gn  = c("LRP1", "MTNR1A", "MTNR1B", "AGTR2", "BDKRB2")
)

resetNetwork(df)


```  

\
    
Thank you for reading this guide and for using `BulkSignalR`.  

\

## Session Information

```{r session-info}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{annotation.spa}
\alias{annotation.spa}
\title{A skinny data frame used in the spatial workflow}
\format{
Data frame that contains the following columns:
barcode_id,sample_id, in_tissue,array_row
array_col, ground_truth, reference, cell_count, idSpatial

barcode_id is the id of the spot
idSpatial is the spatial id of the spot(array_rowXarray_col)
ground_truth is the label (Layer1/2 were only kept)

They are the mandatory data necessary to
generate plots for the spatial workflow.
}
\source{
\url{http://spatial.libd.org/spatialLIBD/}
}
\usage{
data(annotation.spa)
}
\description{
Data frame subset describing the spatial spots
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelTraining.R
\name{.getMixedGaussianParam}
\alias{.getMixedGaussianParam}
\title{Internal function to fit a censored mixed-Gaussian distribution}
\usage{
.getMixedGaussianParam(d, title, verbose = FALSE, file.name = NULL)
}
\arguments{
\item{d}{A vector of values to fit.}

\item{title}{A plot title.}

\item{verbose}{Provide details on computations.}

\item{file.name}{The file name of a PDF file.}
}
\value{
A list with the mean (\code{mu}) and standard deviation
  (\code{sigma}) estimates of each distribution along with the
  weight alpha applied to the first distribution.

  If \code{file.name} is provided, a control plot is generated in a PDF with
  a data histogram and the fitted Gaussian. \code{title} is used to give this
  plot a main title.
}
\description{
Maximum-likelihood estimators are used.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{alluvialPlot}
\alias{alluvialPlot}
\title{Alluvial plot}
\usage{
alluvialPlot(bsrinf, keywords, type = c("L", "R", "pw.id"), qval.thres = 0.01)
}
\arguments{
\item{bsrinf}{A BSRInference object.}

\item{keywords}{vector of keywoprds to filter pathways.}

\item{type}{filter on Ligand, Receptor or pathway id.}

\item{qval.thres}{threshold over Q-value.}
}
\value{
NULL

This is a convenience function that relies on the \code{ggalluvial}
package to propose a simple way
of representing ligands, receptors,
}
\description{
Representation of the links
between ligands, receptors, and pathways.
}
\examples{
data(bsrinf, package = "BulkSignalR")
alluvialPlot(bsrinf,
    keywords = c("LAMC1"),
    type = "L",
    qval.thres = 0.01)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRClusterComp-class.R
\docType{class}
\name{BSRClusterComp-class}
\alias{BSRClusterComp-class}
\title{BulkSignalR Cluster Comparison Object}
\description{
An S4 class to represent the comparison of two clusters of samples to
infer LR interactions based on the resulting P-values,
log-fold-changes (logFC), and expression values.
}
\section{Slots}{

\describe{
\item{\code{col.clusterA}}{Column indices for the samples in cluster A.}

\item{\code{col.clusterB}}{Column indices for the samples in cluster B.}

\item{\code{differential.stats}}{Comparison statistics A versus B 
as a data.frame and
containing at least 3 columns named 'pval', 'logFC', and 'expr'.}
}}

\examples{
new("BSRClusterComp")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInferenceComp-class.R
\name{tgExpr<-,BSRInferenceComp-method}
\alias{tgExpr<-,BSRInferenceComp-method}
\title{Target gene expression setter (internal use only)}
\usage{
\S4method{tgExpr}{BSRInferenceComp}(x) <- value
}
\arguments{
\item{x}{BSRInferenceComp object}

\item{value}{value to be set for bsrinf}
}
\value{
returns \code{NULL}
}
\description{
Target gene expression setter (internal use only)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resource.R
\name{.formatPathwaysFromGmt}
\alias{.formatPathwaysFromGmt}
\title{Transform gmt file to data frame}
\usage{
.formatPathwaysFromGmt(file, resourceName = NULL)
}
\arguments{
\item{file}{Path to GMT file}

\item{resourceName}{Two options "GO-BP" or "REACTOME"}
}
\value{
Dataframe with pathwayID, geneName and pathwayName
}
\description{
We note discrepancies between the formats available
from internet sources. Here, we consider a valid gmt file format defined
on each lines as follows:
First is Pathway name,
then comes the ID,
finally you will find genes symbols
part of the pathway defined on the line.
}
\details{
You can find an example here.
- For Reactome. (Directly from their website)
\url{https://reactome.org/download/current/ReactomePathways.gmt.zip}
Note that you need to unzip the file to read the content.
The code is inspired from read.gmt function
from the gsa R package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAssign.R
\name{assignCellTypesToInteractions}
\alias{assignCellTypesToInteractions}
\title{Assign cell types to L-R interactions}
\usage{
assignCellTypesToInteractions(
  bsrdm,
  bsrinf,
  ct.scores,
  normalize.scores = TRUE,
  min.weight = 0.1,
  min.r2 = 0.25,
  min.r2.after = 0.35,
  lasso = TRUE,
  qval.thres = 0.001
)
}
\arguments{
\item{bsrdm}{A BSRDataModel object.}

\item{bsrinf}{A BSRInference object.}

\item{ct.scores}{A matrix of cell type signature scores.}

\item{normalize.scores}{A logical indicating whether scores should be
normalized before assigning cell types.}

\item{min.weight}{Minimum weight to keep in the linear model (cell types
with lower weights will be discarded) if \code{lasso==TRUE}. Otherwise,
minimum correlation coefficient of each individual cell type.}

\item{min.r2}{Minimum r2 between a candidate cell type and a L-R gene
signature score.}

\item{min.r2.after}{Minimum r2 between the proposed linear model and
a L-R gene signature score to retain the model.}

\item{lasso}{Logical indicating that the LASSO (or linear regression if
only one cell type satisfies the \code{min.r2} criterion) should be
used. Otherwise, Spearman linear correlation is used.}

\item{qval.thres}{Maximum Q-value of the L-R pairs to be considered.}
}
\value{
A data.frame containing the cell type assignments for each
L-R interaction. Unique interactions are considered only (thanks to
\code{"\link[=BSRInference-class]{reduceToBestPathway}"} that is applied
internally). An interaction can be associated with several cell types
or none. In case it is associated with a single cell type, it is labelled
autocrine (indicative only).

Cell type signature scores must be provided. They can be computed
with BulkSignalR utility function \code{"\link{scoreSignatures}"}, but
also any other external tool such as CIBERSORT or BisqueRNA. In case
such a tool would score cell types in a nonlinear fashion, we
recommend to transform the score matrix to restore a linear relationship
cell type abundance/score. By default, cell type (and L-R gene
signature) scores are normalized between 0 and 1 to make the weights of each
cel type in the linear models as comparable as possible.
}
\description{
Generate a data.frame linking interactions to cell types.
}
\examples{
data(bsrdm, package = "BulkSignalR")
data(bsrinf, package = "BulkSignalR")
data(immune.signatures, package = "BulkSignalR")
data(tme.signatures, package = "BulkSignalR")

immune.signatures <- immune.signatures[immune.signatures$signature \%in\%
    c("T cells"), ]

signatures <- rbind(immune.signatures, tme.signatures[
    tme.signatures$signature \%in\% c("Fibroblasts"),
])

tme.scores <- scoreSignatures(bsrdm, signatures)

# assignment
lr2ct <- assignCellTypesToInteractions(bsrdm, bsrinf, tme.scores)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAssign.R
\name{cellularNetwork}
\alias{cellularNetwork}
\title{Build a cellular network}
\usage{
cellularNetwork(tab)
}
\arguments{
\item{tab}{The data.frame output by \code{"\link{cellularNetworkTable}"}.}
}
\value{
A igraph object containing all the links in the cellular network.
}
\description{
Generate a igraph object including all the links between cell types.
}
\examples{
data(bsrdm, package = "BulkSignalR")
data(bsrinf, package = "BulkSignalR")
data("tme.signatures", package = "BulkSignalR")
data(immune.signatures, package = "BulkSignalR")

immune.signatures <- immune.signatures[immune.signatures$signature \%in\%
    c("T cells"), ]

signatures <- rbind(immune.signatures, tme.signatures[
    tme.signatures$signature \%in\% c("Fibroblasts"),
])

tme.scores <- scoreSignatures(bsrdm, signatures)

# assignment
lr2ct <- assignCellTypesToInteractions(bsrdm, bsrinf, tme.scores)

# cellular network
g.table <- cellularNetworkTable(lr2ct)
gCN <- cellularNetwork(g.table)

#plot(gCN, edge.width=5*E(gCN)$score)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInferenceComp-class.R,
%   R/BSRSignatureComp-class.R
\name{comparisonName}
\alias{comparisonName}
\alias{comparisonName,BSRInferenceComp-method}
\alias{comparisonName,BSRSignatureComp-method}
\title{Comparison name accessor}
\usage{
\S4method{comparisonName}{BSRInferenceComp}(x)

\S4method{comparisonName}{BSRSignatureComp}(x)
}
\arguments{
\item{x}{BSRSignatureComp object}
}
\value{
cmp.name

cmp.name
}
\description{
Comparison name accessor

Comparison name accessor
}
\examples{
bsrinf <- new("BSRInferenceComp")
comparisonName(bsrinf)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInferenceComp-class.R
\name{LRinterScore}
\alias{LRinterScore}
\alias{LRinterScore,BSRInferenceComp-method}
\title{Simplified LRinter accessor with focus on the LR-score}
\usage{
\S4method{LRinterScore}{BSRInferenceComp}(x)
}
\arguments{
\item{x}{BSRInferenceComp object}
}
\value{
LRinterScore
}
\description{
Simplified LRinter accessor with focus on the LR-score
}
\examples{
data(bsrinf.comp, package = "BulkSignalR")
LRinterScore(bsrinf.comp)[5,]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/networks.R
\name{getLRNetwork}
\alias{getLRNetwork}
\title{Generate a ligand-receptor network}
\usage{
getLRNetwork(
  bsrinf,
  pval.thres = NULL,
  qval.thres = NULL,
  node.size = 5,
  red.pairs = NULL
)
}
\arguments{
\item{bsrinf}{A BSRInference object.}

\item{pval.thres}{P-value threshold.}

\item{qval.thres}{Q-value threshold.}

\item{node.size}{Default node size in the network.}

\item{red.pairs}{A data frame with columns L (ligands) and R
(receptors) that restrict LR pairs to those listed.}
}
\value{
An \code{igraph} object featuring the ligand-receptor network.
Default colors and node sizes are assigned,
which can be changed afterwards if necessary.
}
\description{
Generate a ligand-receptor network from a ligand-receptor table.
}
\examples{
data(bsrinf, package = "BulkSignalR")

gLR <- getLRNetwork(bsrinf, qval.thres = 1e-4)
# plot(gLR)
# write.graph(gLR, file="SDC-LR-network.graphml", format="graphml")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelTraining.R
\name{.getEmpiricalParam}
\alias{.getEmpiricalParam}
\title{Internal function to fit an empirical distribution}
\usage{
.getEmpiricalParam(d, title, verbose = FALSE, file.name = NULL)
}
\arguments{
\item{d}{A vector of values to fit.}

\item{title}{A plot title.}

\item{verbose}{Provide details on computations.}

\item{file.name}{The file name of a PDF file.}
}
\value{
A list with the step function implementing the CDF of
  the empirical distribution (\code{empirCDF}).

  If \code{file.name} is provided, a control plot is generated in a PDF with
  a data histogram and the fitted Gaussian. \code{title} is used to give this
  plot a main title.
}
\description{
Based on stats::ecdf.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R, R/BSRSignature-class.R
\name{tgGenes}
\alias{tgGenes}
\alias{tgGenes,BSRInference-method}
\alias{tgGenes,BSRSignature-method}
\title{Target genes accessor}
\usage{
\S4method{tgGenes}{BSRInference}(x)

\S4method{tgGenes}{BSRSignature}(x)
}
\arguments{
\item{x}{BSRSignature}
}
\value{
tgGenes

tgGenes
}
\description{
Target genes accessor

Target genes accessor
}
\examples{
bsr.sig <- new("BSRSignature")
tgGenes(bsr.sig)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inference.R
\name{.checkReceptorSignaling}
\alias{.checkReceptorSignaling}
\title{Internal function to check receptor signaling downstream}
\usage{
.checkReceptorSignaling(
  ds,
  lr,
  reference = c("REACTOME-GOBP", "REACTOME", "GOBP"),
  max.pw.size = 400,
  min.pw.size = 5,
  min.positive = 4,
  use.full.network = FALSE,
  restrict.pw = NULL,
  with.complex = TRUE
)
}
\arguments{
\item{ds}{A BSRDataModel object.}

\item{lr}{A table as returned by \code{.getCorrelatedLR()}.}

\item{reference}{Which pathway reference should be used ("REACTOME"
for Reactome, "GOBP" for GO Biological Process,
or "REACTOME-GOBP" for both).}

\item{max.pw.size}{Maximum pathway size to consider from the pathway
reference.}

\item{min.pw.size}{Minimum pathway size to consider from the pathway
reference.}

\item{min.positive}{Minimum number of target genes to be found in a given
pathway.}

\item{use.full.network}{A logical to avoid limiting the reference network
to the detected genes and use the whole reference network.}

\item{restrict.pw}{A list of pathway IDs to restrict the application of
the function.}

\item{with.complex}{A logical indicating whether receptor co-complex
members should be included in the target genes.}
}
\value{
A data frame extending \code{lr} content with the pathways found to
contain the receptors and data about target gene correlations with those
receptors. Strings in semi-colon-separated format are used to report
target genes and their Spearman correlations with the receptor in the
data frame. The target genes are sorted according to the correlation
coefficient.

In a pathway of the reference, i.e., a Reactome pathway or the genes of a
GOBP term, the target genes are the
genes coding for proteins forming a complex with the receptor and the
genes in the pathway downstream the receptor,
which are given as regulated by the pathway. If \code{with.complex} is
set to \code{FALSE}, then only the
regulated genes are considered. Participation to a complex and being
regulated as well as the pathway directed topologies
are defined by Reactome and KEGG pathways as provided by PathwayCommons.

The maximum pathway size is used to limit the redundancy inherent to GOBP
and Reactome. The minimum pathway size is
used to avoid overspecific, noninformative results.
}
\description{
Assess the existence of correlations between a receptor,
part of a ligand-receptor pair, and
genes coding for proteins forming a complex with the receptor or genes
regulated by the receptor downstream signaling.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRClusterComp-class.R
\name{colClusterA}
\alias{colClusterA}
\alias{colClusterA,BSRClusterComp-method}
\title{Cluster A columns accessor}
\usage{
\S4method{colClusterA}{BSRClusterComp}(x)
}
\arguments{
\item{x}{object BSRClusterComp}
}
\value{
col.clusterA
}
\description{
Cluster A columns accessor
}
\examples{
bsrcc <- new("BSRClusterComp")
colClusterA(bsrcc)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{.checkRDSFromCache}
\alias{.checkRDSFromCache}
\title{Check for valid RDS cache file.}
\usage{
.checkRDSFromCache(bfc, resourceName)
}
\arguments{
\item{bfc}{Object of class BiocFileCache, created by a call to
BiocFileCache::BiocFileCache()}

\item{resourceName}{keyword associated to a specific resource name.}
}
\value{
logical TRUE/FALSE
}
\description{
This function checks whether a cache entry is a valid RDS file.
Returns TRUE if the cache entry is valid, FALSE otherwise.
In the case of an invalid file, the cache entry and file are
deleted.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bsrdm}
\alias{bsrdm}
\title{A skinny BSRDataModel object related to sdc.}
\format{
An example of an object created by `BSRDataModel`
applied to an sdc subset (Patients N20,N22,SDC17,SDC25) and
10 000 genes sampled (seed set to 123)
`learnParameters` was also called to get statistical 
model parameters.
}
\usage{
data(bsrdm)
}
\description{
Output from the `learnParameters` function to get
BulkSignalR statistical model parameters.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModelComp-class.R
\name{addClusterComp}
\alias{addClusterComp}
\alias{addClusterComp,BSRDataModelComp-method}
\title{Add a comparison between two clusters of samples}
\usage{
\S4method{addClusterComp}{BSRDataModelComp}(obj, cmp, cmp.name)
}
\arguments{
\item{obj}{A BSRDataModelComp object output by
\code{\link{setAs}}.}

\item{cmp}{A BSRClusterComp object to add.}

\item{cmp.name}{The name of the comparison to add.}
}
\value{
A BSRDataModelComp object.
}
\description{
Add a comparison to a BSRDataModelComp object.
}
\details{
Add \code{cmp} to the list of comparisons contained in
\code{obj}.
}
\examples{
# prepare data
data(sdc, package = "BulkSignalR")
normal <- grep("^N", names(sdc))
bsrdm <- BSRDataModel(sdc[, -normal])

# define the comparison
bsrdm.comp <- as(bsrdm, "BSRDataModelComp")
colA <- as.integer(1:3)
colB <- as.integer(12:15)
n <- nrow(ncounts(bsrdm.comp))
stats <- data.frame(
    pval = runif(n), logFC = rnorm(n, 0, 2),
    expr = runif(n, 0, 10)
)
rownames(stats) <- rownames(ncounts(bsrdm.comp))
bsrcc <- BSRClusterComp(bsrdm.comp, colA, colB, stats)

bsrdm.comp <- addClusterComp(bsrdm.comp, bsrcc, "random.example")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial.R
\name{maxLigandSpatialCounts}
\alias{maxLigandSpatialCounts}
\title{Get maximal ligand expression at nearby locations}
\usage{
maxLigandSpatialCounts(
  bsrdm,
  areas,
  nnn = 4,
  radius = NULL,
  x.col = "array_col",
  y.col = "array_row"
)
}
\arguments{
\item{bsrdm}{A BSRDataModel object containing the expression data to smooth.}

\item{areas}{A data.frame containing at least the x and y
coordinates of the locations.}

\item{nnn}{Number of nearest-neighbor locations to use for smoothing
each location. In case \code{radius} is set, then it is the maximum number
of nearest neighbors within the radius.}

\item{radius}{A maximal distance to include neighbors in the smoothing.}

\item{x.col}{Column name in \code{areas} containing x coordinates.}

\item{y.col}{Column name in \code{areas} containing y coordinates.}
}
\value{
A BSRDataModel object containing the maximized ligand expressions.
}
\description{
Get maximal ligand expression at nearby locations
}
\details{
Ligand expression data contained in a BSRDataModel object are
modified to consider the possibility that the ligand of a L-R interaction
might be expressed at nearby locations. This is achieved replacing each
ligand expression by its maximum over the central location and its
neighbors. Since ligands and receptors are never used as gene targets
in computing the receptor downstream signal correlations, this
substitution is compatible with our statistical model. Moreover,
the reciprocal configuration where the ligand is expressed at the
central location and hits a receptors at a neighbor location is
covered when the same ligand maximization scheme is applied to
the neighbor. L-R localization and gene signature scoring is defined
by the location at which the receptor is expressed after applying
this function.

Two strategies are available to identify the neighbors. It is possible to
simply set the number of nearest-neighbors (parameter \code{nnn}). An
alternative consists in providing a distance radius (\code{radius}) along
with a a maximum number of nearest-neighbors within the radius
(\code{nnn.radius}). To properly define the radius, the user must know the
location coordinates. The strategy with the radius enables having corner
locations with two neighbors only and border locations with three
neighbors only, whereas to simply set a maximum of four neighbors for
instance would retrieve the four closest neighbors in every case.
}
\examples{
data(bsrdm.spa, package = "BulkSignalR")
data(annotation.spa, package = "BulkSignalR")

max.bsrdm <- maxLigandSpatialCounts(bsrdm.spa, annotation.spa,
radius = 1.2, nnn = 4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModel-class.R
\name{logTransformed}
\alias{logTransformed}
\alias{logTransformed,BSRDataModel-method}
\title{log.transformed accessor}
\usage{
\S4method{logTransformed}{BSRDataModel}(x)
}
\arguments{
\item{x}{Object BRSDataModel}
}
\value{
logTransformed
}
\description{
log.transformed accessor
}
\examples{
bsrdm <- BSRDataModel(
    counts = matrix(1.5,
                nrow = 2, ncol = 2,
                dimnames = list(c("A", "B"), c("C", "D"))
                   ),
    method = "TC",
    log.transformed = TRUE,
    normalize = FALSE,
    min.LR.found = 0
)
logTransformed(bsrdm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{scoreSignatures}
\alias{scoreSignatures}
\title{Generic gene signature scoring}
\usage{
scoreSignatures(ds, ref.signatures, robust = FALSE)
}
\arguments{
\item{ds}{A BSRDataModel object.}

\item{ref.signatures}{Gene signatures.}

\item{robust}{A logical indicating that z-scores
should be computed with median and MAD
instead of mean and standard deviation.}
}
\value{
A matrix containing the scores of
each gene signature in each sample.
Note that ligand-receptor gene
signature scores should be computed with
\code{"\link[=BSRDataModel-class]{scoreLRGeneSignatures}"}
instead.
}
\description{
Scores generic gene signatures over the
samples of a BSRDataModel object.
}
\details{
This function relies on
a simple average of gene z-scores over each signature.
It is no replacement for mode advanced methods
such as CIBERSORT or BisqueRNA.
It is provided for convenience.
}
\examples{
data(sdc, package = "BulkSignalR")
data(bsrdm, package = "BulkSignalR")

data(immune.signatures, package = "BulkSignalR")
imm.scores <- scoreSignatures(bsrdm, immune.signatures)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelTraining.R
\name{.buildPermutationIndices}
\alias{.buildPermutationIndices}
\title{Internal function to generate expression matrix permutation indices}
\usage{
.buildPermutationIndices(ncounts, n.bins = 20)
}
\arguments{
\item{ncounts}{A matrix of normalized read counts.}

\item{n.bins}{Number of bins.}
}
\value{
A list containing a vectors of row indices. The vector at index
  \code{i} in the list contains the row indices of rows with mean normalized
  read count in bin \code{i}.
}
\description{
Internal function to generate expression matrix permutation indices
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{.cacheCheckIn}
\alias{.cacheCheckIn}
\title{Check existence of a record in the cache.}
\usage{
.cacheCheckIn(bfc, resourceName)
}
\arguments{
\item{bfc}{Object of class BiocFileCache, created by a call to
BiocFileCache::BiocFileCache()}

\item{resourceName}{keyword associated to a specific resource name.}
}
\value{
logical This function returns TRUE if a record with
the requested keyword already exists in the file cache,
otherwise it returns FALSE.
}
\description{
Check whether the cache record exists or not by passing
to the function an associated keyword
related to the resource we are looking for.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bsrinf.comp}
\alias{bsrinf.comp}
\title{A skinny BSRInferenceComp object related to sdc.}
\format{
An example of an BSRInferenceComp object
}
\usage{
data(bsrinf.comp)
}
\description{
See Vignette BulkSignalR-Differential.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRSignature-class.R
\name{pathways}
\alias{pathways}
\alias{pathways,BSRSignature-method}
\title{pathways accessor}
\usage{
\S4method{pathways}{BSRSignature}(x)
}
\arguments{
\item{x}{BSRSignature}
}
\value{
pathways
}
\description{
pathways accessor
}
\examples{
bsr.sig <- new("BSRSignature")
pathways(bsr.sig)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModelComp-class.R
\name{mu}
\alias{mu}
\alias{mu,BSRDataModelComp-method}
\title{Mu accessor}
\usage{
\S4method{mu}{BSRDataModelComp}(x)
}
\arguments{
\item{x}{object BSRDataModelComp}
}
\value{
mu
}
\description{
Mu accessor
}
\examples{
bsrdm.comp <- new("BSRDataModelComp")
mu(bsrdm.comp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelTraining.R
\name{.getEmpiricalNull}
\alias{.getEmpiricalNull}
\title{Sampling of correlations downstream the receptors null distribution}
\usage{
.getEmpiricalNull(obj)
}
\arguments{
\item{obj}{A BSRDatamodel without learned paramaters.}
}
\value{
A list of \code{n.rand} tables such as output by
  \code{.checkReceptorSignaling}. Each table is computed from a randomized
  expression matrix (randomized \code{ncounts}).
}
\description{
Perform receptor downstream analysis with
\code{.checkReceptorSignaling} based on randomized expression data and
ligand-receptor pairs selected from the same randomized data.
}
\details{
A large number of correlations 
  (ligand-receptor and receptor-downstream target genes) 
  is reported in each randomized matrix. Therefore,
  \code{n.rand} should be
  given a modest value to avoid unnecessarily long computations.

  See \code{\link{.checkReceptorSignaling}} for more details about the
  parameters.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R
\docType{class}
\name{BSRInference-class}
\alias{BSRInference-class}
\title{BulkSignalR Inference Object}
\description{
An S4 class to represent inferred ligand-receptor interactions.
}
\details{
This class contains inferred LR interactions along with
their statistical significance. Data representation supports subsequent
reductions to pathways, etc. See reduction functions
\code{"\link[=BSRInference-class]{reduceToBestPathway}"},
\code{"\link[=BSRInference-class]{reduceToLigand}"},
\code{"\link[=BSRInference-class]{reduceToReceptor}"} and
\code{"\link[=BSRInference-class]{reduceToPathway}"}.
}
\section{Slots}{

\describe{
\item{\code{LRinter}}{A data frame describing the (ligand,receptor,pathway) triples
with P- and Q-values.}

\item{\code{ligands}}{A list of ligands, one entry per LR interaction.}

\item{\code{receptors}}{A list of receptors, one entry per LR interaction.}

\item{\code{tg.genes}}{A list of target genes, one entry per LR interaction.}

\item{\code{tg.corr}}{A list of target gene correlations to the receptor, one
entry per interaction}

\item{\code{inf.param}}{The parameters used for the inference.}
}}

\examples{
new("BSRInference")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R,
%   R/BSRInferenceComp-class.R
\name{reduceToPathway}
\alias{reduceToPathway}
\alias{reduceToPathway,BSRInference-method}
\alias{reduceToPathway,BSRInferenceComp-method}
\title{Aggregate ligands and receptors at the pathway level}
\usage{
\S4method{reduceToPathway}{BSRInference}(obj)

\S4method{reduceToPathway}{BSRInferenceComp}(obj)
}
\arguments{
\item{obj}{BSRInferenceComp object}
}
\value{
A BSRInference object reduced to only report one row per pathway.
The information of which ligand interacted with which receptor is lost as
all the ligands and all the receptors forming pairs related to a certain
pathway are combined.
For a given pathway, the reported P-values and target genes are those of
the best ligand-receptor pair that
was in this pathway.
Receptors and ligands are combined in two semi-colon-separated
lists surrounded by curly brackets in the tabular slot \code{LRinter},
while the list representation slots (\code{ligands} and
\code{receptors}) are update accordingly.

A BSRInferenceComp object reduced to only report one row per pathway.
The information of which ligand interacted with which receptor is lost as
all the ligands and all the receptors forming pairs related to a certain
pathway are combined.
For a given pathway, the reported P-values and target genes are those of
the best ligand-receptor pair that was in this pathway.
The same logic applies to the LR-score, and the ligand and receptor
expression.
Receptors and ligands are combined in two semi-colon-separated
lists surrounded by curly brackets in the tabular slot \code{LRinter},
while the list representation slots (\code{ligands} and
\code{receptors}) are update accordingly.
}
\description{
Simplifies a ligand-receptor inference object to focus on
the pathways.

Simplifies a ligand-receptor inference object to focus on
the pathways.
}
\examples{
data(bsrinf, package = "BulkSignalR")

bsrinf.redP <- reduceToPathway(bsrinf)
data(bsrinf.comp, package = "BulkSignalR")

bsrinf.redP <- reduceToPathway(bsrinf.comp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inference.R
\name{.getCorrelatedLR}
\alias{.getCorrelatedLR}
\title{Get correlated ligand-receptor pairs}
\usage{
.getCorrelatedLR(ds, min.cor = 0.25, restrict.genes = NULL)
}
\arguments{
\item{ds}{A BSRDataModel object.}

\item{min.cor}{The minimum correlation required.}

\item{restrict.genes}{A list of gene symbols that restricts ligands and
receptors.}
}
\value{
A data frame containing putative ligand-receptor pairs along
  with their correlations above \code{min.cor}. This table is the first step
  of a ligand-receptor analysis.
}
\description{
Internal function to compute the Spearman correlations
of all the ligand-receptor
pairs in LRdb and return those above a minimum value.
}
\details{
The \code{restrict.genes} parameter is used for special cases where
  LRdb must be further restricted to a subset.
  The putative ligand-receptor pairs has 3 columns : R, L and corr.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R
\name{LRinter}
\alias{LRinter}
\alias{LRinter,BSRInference-method}
\title{LRinter accessor}
\usage{
\S4method{LRinter}{BSRInference}(x)
}
\arguments{
\item{x}{BSRInference object}
}
\value{
LRinter
}
\description{
LRinter accessor
}
\examples{
bsrinf <- new ("BSRInference")
LRinter(bsrinf)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModel-class.R
\docType{class}
\name{BSRDataModel-class}
\alias{BSRDataModel-class}
\title{BulkSignalR Data Model Object}
\description{
An S4 class to represent the expression data used for inferring
ligand-receptor interactions.
}
\section{Slots}{

\describe{
\item{\code{ncounts}}{Normalized read count matrix. Row names must be set to HUGO
official gene symbols.}

\item{\code{log.transformed}}{Logical indicating whether values in
\code{ncounts} were log2-transformed.}

\item{\code{normalization}}{Name of the normalization method.}

\item{\code{param}}{List containing the statistical model parameters.}

\item{\code{initial.organism}}{Organism for which the data were obtained.}

\item{\code{initial.orthologs}}{List of genes for which human
orthologs exist.}
}}

\examples{
BSRDataModel(
    counts = matrix(1.5,
                nrow = 2, ncol = 2,
                dimnames = list(c("A", "B"), c("C", "D"))
                   ),
    method = "TC",
    log.transformed = TRUE,
    normalize = FALSE,
    min.LR.found = 0
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelTraining.R
\name{.cdfAlphaStable}
\alias{.cdfAlphaStable}
\title{Internal function to compute a censored alpha-) stable CDF}
\usage{
.cdfAlphaStable(x, par)
}
\arguments{
\item{x}{A vector of observed values.}

\item{par}{A list containing the censored stable model parameters.}
}
\value{
A vector of probabilities P(X<x|par).
}
\description{
Internal function to compute a censored alpha-) stable CDF
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataPrepare.R, R/resource.R
\name{resetLRdb}
\alias{resetLRdb}
\title{Modify LRdb database}
\usage{
resetLRdb(db, switch = FALSE)

resetLRdb(db, switch = FALSE)
}
\arguments{
\item{db}{A data frame with 2 columns named
ligand and receptor.}

\item{switch}{A logical indicating whether LRdb should be extended only
(FALSE, default) or completely replaced (TRUE).}
}
\value{
Returns `NULL`, invisibly.

Returns `NULL`, invisibly.
}
\description{
Users can provide a data frame with 2 columns named
ligand and receptor.
This can be used to extend or replace the existing
LRdb.

Users can provide a data frame with 2 columns named
ligand and receptor.
This can be used to extend or replace the existing
LRdb.
}
\examples{
resetLRdb(db = data.frame(ligand = "A2M", receptor = "LRP1"), switch = FALSE)
resetLRdb(db = data.frame(ligand = "A2M", receptor = "LRP1"), switch = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R,
%   R/BSRInferenceComp-class.R
\name{reduceToLigand}
\alias{reduceToLigand}
\alias{reduceToLigand,BSRInference-method}
\alias{reduceToLigand,BSRInferenceComp-method}
\title{Aggregate the receptors of a same ligand}
\usage{
\S4method{reduceToLigand}{BSRInference}(obj)

\S4method{reduceToLigand}{BSRInferenceComp}(obj)
}
\arguments{
\item{obj}{BSRInferenceComp object}
}
\value{
A BSRInference object but reduced to one row per ligand.
All the receptors are combined in a
semi-colon-separated list surrounded by curly brackets in the tabular
slot \code{LRinter}, and in vectors in the \code{ligands} (list) slot.

The reported P-value and target genes are those from the pathway with
the smallest P-value.

A BSRInferenceComp object but reduced to one row per ligand.
All the receptors are combined in a
semi-colon-separated list surrounded by curly brackets in the tabular
slot \code{LRinter}, and in vectors in the \code{ligands} (list) slot.

The reported P-value and target genes are those from the pathway with
the smallest P-value.
The same logic applies to the LR-score, and the receptor
expression.
}
\description{
Simplifies a ligand-receptor table to focus on the ligands.

Simplifies a ligand-receptor table to focus on the ligands.
}
\examples{
data(bsrinf, package = "BulkSignalR")

bsrinf.redL <- reduceToLigand(bsrinf)

data(bsrinf.comp, package = "BulkSignalR")

bsrinf.redL <- reduceToLigand(bsrinf.comp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resource.R
\name{resetPathways}
\alias{resetPathways}
\title{Import pathways from a file or data frame}
\usage{
resetPathways(
  dataframe = NULL,
  file = NULL,
  fileType = c("json", "gmt", "txt"),
  resourceName = NULL
)
}
\arguments{
\item{dataframe}{Data frame formated as follows.
When \code{resourceName} is set to "Reactome",
dataframe colnames must be defined as
"Reactome ID", "Gene name", and "Reactome name"
When \code{resourceName} is set to "GO-BP",
#' dataframe colnames must be defined as
"GO ID", "Gene name", and "GO name"}

\item{file}{Path to file.}

\item{fileType}{Default is Json.
Other options are gmt or txt files.}

\item{resourceName}{Two options "GO-BP" or "Reactome".}
}
\value{
Returns `NULL`, invisibly.
}
\description{
\code{resetPathways} is a function
we provide to users who want to refresh REACTOME
and GO-BP content included in BulkSignalR.
}
\details{
Pathways in `BulkSignalR` (as sets of genes/proteins) are defined
after Reactome and GOBP databases.
Those can be updated using
json files from
the Human Molecular Signatures Database (MSigDB)
at \url{https://www.gsea-msigdb.org/}
Gmt file format also can be imported.
A data frame can be used directly also.
}
\examples{
reactSubset <- getResource(resourceName = "Reactome",
cache = TRUE)

subset <- c("REACTOME_BASIGIN_INTERACTIONS",
"REACTOME_SYNDECAN_INTERACTIONS",
"REACTOME_ECM_PROTEOGLYCANS",
"REACTOME_CELL_JUNCTION_ORGANIZATION")

reactSubset <- reactSubset[
reactSubset$`Reactome name` \%in\% subset,]

resetPathways(dataframe = reactSubset,
resourceName = "Reactome")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R
\name{inferenceParameters}
\alias{inferenceParameters}
\alias{inferenceParameters,BSRInference-method}
\title{Inference parameters accessor}
\usage{
\S4method{inferenceParameters}{BSRInference}(x)
}
\arguments{
\item{x}{BRSInference object.}
}
\value{
inf.param
}
\description{
Inference parameters accessor
}
\examples{
bsrinf <- new ("BSRInference")
inferenceParameters(bsrinf)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R
\name{BSRInference}
\alias{BSRInference}
\title{Inference of ligand-receptor interactions}
\usage{
BSRInference(
  obj,
  rank.p = 0.55,
  min.cor = 0.25,
  restrict.genes = NULL,
  reference = c("REACTOME-GOBP", "REACTOME", "GOBP"),
  max.pw.size = NULL,
  min.pw.size = NULL,
  min.positive = NULL,
  use.full.network = FALSE,
  restrict.pw = NULL,
  with.complex = NULL,
  fdr.proc = c("BH", "Bonferroni", "Holm", "Hochberg", "SidakSS", "SidakSD", "BY", "ABH",
    "TSBH")
)
}
\arguments{
\item{obj}{A BSRDataModel output by \code{\link{BSRDataModel}} with
statistical model parameters trained by the
\code{"\link[=BSRDataModel-class]{learnParameters}"} method.}

\item{rank.p}{A number between 0 and 1 defining the rank of the last
considered target genes.}

\item{min.cor}{The minimum Spearman correlation required between
the ligand and the receptor.}

\item{restrict.genes}{A list of gene symbols that restricts ligands and
receptors.}

\item{reference}{Which pathway reference should be used ("REACTOME"
for Reactome, "GOBP" for GO Biological Process,
or "REACTOME-GOBP" for both).}

\item{max.pw.size}{Maximum pathway size to consider from the pathway
reference.}

\item{min.pw.size}{Minimum pathway size to consider from the pathway
reference.}

\item{min.positive}{Minimum number of target genes to be found in a given
pathway.}

\item{use.full.network}{A logical to avoid limiting the reference network
to the detected genes and use the whole reference network.}

\item{restrict.pw}{A list of pathway IDs to restrict the application of
the function.}

\item{with.complex}{A logical indicating whether receptor co-complex
members should be included in the target genes.}

\item{fdr.proc}{The procedure for adjusting P-values according to
\code{\link[multtest]{mt.rawp2adjp}}.}
}
\value{
A BSRInference object with initial inferences set.
}
\description{
Computes putative LR interactions along with their statistical confidence.
In this initial inference, all the relevant pathways are reported,
see reduction functions to reduce this list.
}
\details{
Perform the initial ligand-receptor inference. Initial means that
no reduction is applied. All the (ligand, receptor, downstream pathway)
triples are reported, i.e., a given LR pair may appear multiple times
with different pathways downstream the receptor. Specific reduction
functions are available from the package to operate subsequent
simplifications based on the BSRInference object created by the initial
inference.

Parameters defining minimum/maximum pathway sizes, etc. are set to NULL
by default, meaning that their values will be taken from what was set
during the training of the statistical model with
\code{"\link[=BSRDataModel-class]{learnParameters}"}

To use different
values at the time of inference sounds like a bad idea, although this
could be used to explore without retraining the underlying model.
Retraining of the model with adjusted parameters is advised following
such an exploration.
}
\examples{
data(bsrdm, package = "BulkSignalR")
data(immune.signatures, package = "BulkSignalR")

# We use a subset of the reference to speed up
# inference in the context of the example.

reactSubset <- getResource(resourceName = "Reactome",
cache = FALSE)

subset <- c("REACTOME_BASIGIN_INTERACTIONS",
"REACTOME_SYNDECAN_INTERACTIONS",
"REACTOME_ECM_PROTEOGLYCANS",
"REACTOME_CELL_JUNCTION_ORGANIZATION")

reactSubset <- reactSubset[
reactSubset$`Reactome name` \%in\% subset,]

resetPathways(dataframe = reactSubset,
resourceName = "Reactome")

bsrinf <- BSRInference(bsrdm,
    min.cor = 0.2,restrict.genes=immune.signatures$gene,
    reference="REACTOME")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bsrinf.spa}
\alias{bsrinf.spa}
\title{A skinny BSR-inference object related to spatial data set}
\format{
From the previous object `bsrdm.spa`, 
you can generate inferences by calling its
method `BSRInference`. 
The resulting BSRInference object is `bsrinf.spa`,
It contains all the inferred L-R interactions 
with their associated pathways and corrected p-values.
`learnParameters` was also called to train the statistical 
model parameters.
}
\source{
\url{http://spatial.libd.org/spatialLIBD/}
}
\usage{
data(bsrinf.spa)
}
\description{
Output from the `learnParameters` function to get
BulkSignalR statistical model parameters.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resource.R
\name{getResource}
\alias{getResource}
\title{Get resources from the cache}
\usage{
getResource(resourceName = NULL, cache = FALSE)
}
\arguments{
\item{resourceName}{Resource name.}

\item{cache}{Logical. Default is FALSE
If TRUE, you will use environment variables.}
}
\value{
Returns a data frame containing the requested
resource.
}
\description{
Get resources (pathways, lrdb or network
stored in the cache.
}
\examples{
reactome <- getResource(resourceName = "Reactome",cache=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bodyMap.mouse}
\alias{bodyMap.mouse}
\title{Mouse transcriptoms across tissues}
\format{
A data frame with 24543 rows and 8 variables.
}
\source{
Bin Li & al., Scientific Reports, 2017;
}
\usage{
data(bodyMap.mouse)
}
\description{
A data set containing RPKM values of brain and liver.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataPrepare.R
\name{.checkInteroperabilityForCounts}
\alias{.checkInteroperabilityForCounts}
\title{Internal function to check and extract a
count matrix if a more complex object than a simple matrix or data frame
is given as parameter. Main usage is to link with Bioconductor objects.}
\usage{
.checkInteroperabilityForCounts(
  counts,
  symbol.col,
  x.col,
  y.col,
  barcodeID.col
)
}
\arguments{
\item{counts}{A table or matrix of read counts (or protein abundance).
It can also be a SummarizedExperiment or SpatialExperiment
object from which the count matrix should be extracted.
See \code{\link{BSRDataModel}}.}

\item{symbol.col}{The index of the column containing the gene symbols in case
those are not the row names of \code{counts} already. In a
SpatialExperiment object, the index in the data frame returned by rowData().}

\item{x.col}{In a SpatialExperiment object, the index of the column
containing the x coordinates in the dafaframe returned by rowData(), usually 
named array_row.}

\item{y.col}{In a SpatialExperiment object, the index of the column
containing the y coordinates in the dafaframe returned by rowData(), usually 
named array_col.}

\item{barcodeID.col}{In a SpatialExperiment object, the index of the column
containing the barcodeID in the dafaframe returned by colData(), usually
named barcode_id.}
}
\value{
A matrix of count (or abundance) values
}
\description{
Internal function to check and extract a
count matrix if a more complex object than a simple matrix or data frame
is given as parameter. Main usage is to link with Bioconductor objects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModelComp-class.R
\name{comparison}
\alias{comparison}
\alias{comparison,BSRDataModelComp-method}
\title{Comparisons list accessor}
\usage{
\S4method{comparison}{BSRDataModelComp}(x)
}
\arguments{
\item{x}{object BSRDataModelComp}
}
\value{
comp
}
\description{
Comparisons list accessor
}
\examples{
bsrdm.comp <- new("BSRDataModelComp")
comparison(bsrdm.comp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{.cutExtremeValues}
\alias{.cutExtremeValues}
\title{Internal function to cut extreme values from a matrix}
\usage{
.cutExtremeValues(m, p)
}
\arguments{
\item{m}{A matrix.}

\item{p}{Proportion of top and bottom values for thresholding.}
}
\value{
A matrix with values beyond top and bottom thresholds repaced by
the latter thresholds.
}
\description{
Internal function to cut extreme values from a matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInferenceComp-class.R
\name{sourceComparisonName}
\alias{sourceComparisonName}
\alias{sourceComparisonName,BSRInferenceComp-method}
\title{Source comparison name accessor}
\usage{
\S4method{sourceComparisonName}{BSRInferenceComp}(x)
}
\arguments{
\item{x}{BSRInferenceComp object}
}
\value{
src.comp.name
}
\description{
Source comparison name accessor
}
\examples{
bsrinf <- new("BSRInferenceComp")
sourceComparisonName(bsrinf)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial.R
\name{spatialDiversityPlot}
\alias{spatialDiversityPlot}
\title{2D-projection of spatial score distributions}
\usage{
spatialDiversityPlot(
  scores,
  associations,
  proj = c("PCA", "tSNE"),
  score.based = FALSE,
  qval.thres = 0.01,
  val.thres = 0,
  with.names = FALSE,
  text.fs = 2.5,
  legend.fs = 10,
  axis.fs = 10,
  label.fs = 12,
  dot.size = 1,
  perplexity = 10
)
}
\arguments{
\item{scores}{A matrix of scores, one L-R interaction per row and
spatial locations in the columns. This matrix is typically obtained
from BulkSignalR functions \code{scoreLRGeneSignatures} or \code{scScoring}.}

\item{associations}{A statistical association data.frame generated
by the function \code{spatialAssociation}.}

\item{proj}{Projection method : 'PCA' or 'tSNE' are available arguements.}

\item{score.based}{A logical indicating whether the plot should be
based on scores or the associations directly.}

\item{qval.thres}{The maximum Q-value to consider in the plot (a
L-R interaction must associate with one label at least with a Q-value
smaller or equal to this threshold). Relevant for Kruskal-Wallis and
ANOVA tests in \code{spatialAssociation}.}

\item{val.thres}{The minimum value to consider in the plot (a
L-R interaction must associate with one label at least with a value
larger or equal to this threshold). Relevant for Spearman and r2
associations in \code{spatialAssociation}.}

\item{with.names}{A logical indicating whether L-R names should be plotted.}

\item{text.fs}{Point label font size in case \code{with.names} is TRUE.}

\item{legend.fs}{Legend items font size.}

\item{axis.fs}{Axis ticks font size.}

\item{label.fs}{Legend titles and axis names font size.}

\item{dot.size}{Dot size.}

\item{perplexity}{Perplexity parameter for t-SNE.}
}
\value{
Display a 2D-projection of the score spatial distributions.
}
\description{
Use PCA or t-SNE to obtain a 2D-projection of a set of spatial scores
or associations.
This plot summarizes the diversity of patterns occuring in a spatial
dataset. Use the function \code{spatialIndexPlot} to create a large
visual index of many spatial distributions.
Not limited to BulkSignalR gene signature scores.
}
\details{
Display a 2D-projection of the score spatial distributions.
}
\examples{
data(bsrdm.spa, package = "BulkSignalR")
data(bsrinf.spa, package = "BulkSignalR")
data(annotation.spa, package = "BulkSignalR")

thres <- 0.01
bsrinf.red <- reduceToBestPathway(bsrinf.spa)
s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)

# statistical association with tissue areas based on correlations
# For display purpose, we only use a subset here


assoc.bsr.corr <- spatialAssociation(scores.red[c(1:3), ],
annotation.spa, label.col = "ground_truth",test = "Spearman")

spatialDiversityPlot(scores.red[c(1:3),],assoc.bsr.corr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resource.R
\name{.formatPathwaysFromJson}
\alias{.formatPathwaysFromJson}
\title{Format a data frame according to json input}
\usage{
.formatPathwaysFromJson(file, resourceName = NULL)
}
\arguments{
\item{file}{Path to file.}

\item{resourceName}{Two options "GO-BP" or "REACTOME".}
}
\value{
Data frame with pathwayID, geneName and pathwayName
}
\description{
Format a data frame according to json input
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModelComp-class.R
\name{removeClusterComp}
\alias{removeClusterComp}
\alias{removeClusterComp,BSRDataModelComp-method}
\title{Remove a comparison from a BSRDataModelComp object.}
\usage{
\S4method{removeClusterComp}{BSRDataModelComp}(obj, cmp.name)
}
\arguments{
\item{obj}{A BSRDataModelComp object output by
\code{\link{setAs}}.}

\item{cmp.name}{The name of the comparison to remove.}
}
\value{
A BSRDataModelComp object.
}
\description{
Remove a comparison from a BSRDataModelComp object.
}
\details{
Remove the comparison with \code{cmp.name} from the list of
comparisons contained in \code{obj}.
}
\examples{
# prepare data
data(sdc, package = "BulkSignalR")
normal <- grep("^N", names(sdc))
bsrdm <- BSRDataModel(sdc[, -normal])

# define the comparison
bsrdm.comp <- as(bsrdm, "BSRDataModelComp")
colA <- as.integer(1:3)
colB <- as.integer(12:15)
n <- nrow(ncounts(bsrdm.comp))
stats <- data.frame(
    pval = runif(n), logFC = rnorm(n, 0, 2),
    expr = runif(n, 0, 10)
)
rownames(stats) <- rownames(ncounts(bsrdm.comp))
bsrcc <- BSRClusterComp(bsrdm.comp, colA, colB, stats)

bsrdm.comp <- addClusterComp(bsrdm.comp, bsrcc, "random.example")
bsrdm.comp <- removeClusterComp(bsrdm.comp, "random.example")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial.R
\name{spatialPlot}
\alias{spatialPlot}
\title{L-R interaction score spatial display}
\usage{
spatialPlot(
  v,
  areas,
  inter.name,
  rev.y = TRUE,
  ref.plot = FALSE,
  ref.plot.only = FALSE,
  image.raster = NULL,
  x.col = "array_col",
  y.col = "array_row",
  label.col = "label",
  idSpatial.col = "idSpatial",
  cut.p = 0.01,
  low.color = "royalblue3",
  mid.color = "white",
  high.color = "orange",
  title.fs = 12,
  legend.fs = 10,
  axis.fs = 10,
  label.fs = 12,
  dot.size = 0.5,
  legend.dot.factor = 10,
  ref.colors = NULL
)
}
\arguments{
\item{v}{A named vector containing the scores, names must be
the IDs of each location.}

\item{areas}{A data.frame containing at least the x and y
coordinates of the locations as well as the unique IDs of spatial locations.
In case \code{ref.plot} is set to TRUE,
a label column is required additionally.}

\item{inter.name}{Interaction name to display as plot title.}

\item{rev.y}{A Boolean indicating whether low y coordinates should be
at the top of the plot.}

\item{ref.plot}{A Boolean indicating whether a reference map of the tissue
with area labels should be plot aside.}

\item{ref.plot.only}{A Boolean indicating that only the reference plot
should be output.}

\item{image.raster}{Raster object image to plot raw tissue image as
reference.}

\item{x.col}{Column name in \code{areas} containing x coordinates.}

\item{y.col}{Column name in \code{areas} containing y coordinates.}

\item{label.col}{Column name in \code{areas} containing area labels.}

\item{idSpatial.col}{Column name in \code{areas} containing the unique
IDs of spatial locations.}

\item{cut.p}{Proportion of top and bottom values for thresholding.}

\item{low.color}{Color for low score values.}

\item{mid.color}{Color for score = 0.}

\item{high.color}{Color for high score values.}

\item{title.fs}{Title font size.}

\item{legend.fs}{Legend items font size.}

\item{axis.fs}{Axis ticks font size.}

\item{label.fs}{Legend titles and axis names font size.}

\item{dot.size}{Dot size.}

\item{legend.dot.factor}{A factor applied to obtain the legend dot size.}

\item{ref.colors}{A vector of colors to bypass those automatically chosen
by ggplot2 for the tissue areas in the reference plot.}
}
\value{
A spatial plot
}
\description{
Generate a plot with scores at the spatial coordinates of the corresponding
sample locations. Not limited to BulkSignalR gene signature scores.
}
\details{
A single (scores) or side-by-side (reference tissue & scores) plot
is generated.
}
\examples{
data(bsrinf.spa, package = "BulkSignalR")
data(bsrdm.spa, package = "BulkSignalR")
data(annotation.spa, package = "BulkSignalR")

thres <- 0.01
bsrinf.red <- reduceToBestPathway(bsrinf.spa)
s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)

inter <- "{SLIT2} / {GPC1}"

spatialPlot(scores.red[inter, ], annotation.spa, inter,
   ref.plot = TRUE, ref.plot.only = FALSE,
   image.raster = NULL, dot.size = 1,
   label.col = "ground_truth")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{.cacheAdd}
\alias{.cacheAdd}
\title{Add cache for resources.}
\usage{
.cacheAdd(fpath, cacheDir, resourceName, verbose = FALSE, download = TRUE)
}
\arguments{
\item{fpath}{Path to file on the web or local system.}

\item{cacheDir}{Absolute path to cache directory.}

\item{resourceName}{Ressource name.}

\item{verbose}{Default FALSE}

\item{download}{Logical(TRUE/FALSE) Default TRUE for download.}
}
\value{
Returns `NULL`, invisibly.
}
\description{
Add cache for resources (pathways, lrdb, or network)
downloaded from the web or local database.
This part is handled with BiocFileCache.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModel-class.R
\name{learnParameters}
\alias{learnParameters}
\alias{learnParameters,BSRDataModel-method}
\title{Training of BulkSignalR model parameters}
\usage{
\S4method{learnParameters}{BSRDataModel}(
  obj,
  plot.folder = NULL,
  verbose = FALSE,
  n.rand.LR = 5L,
  n.rand.RT = 2L,
  with.complex = TRUE,
  max.pw.size = 400,
  min.pw.size = 5,
  min.positive = 4,
  quick = FALSE,
  null.model = c("automatic", "mixedNormal", "normal", "kernelEmpirical", "empirical",
    "stable"),
  filename = NULL,
  min.corr.LR = -1
)
}
\arguments{
\item{obj}{A BSRDatamodel without learned paramaters.}

\item{plot.folder}{A folder name for generating control plots.}

\item{verbose}{A logical activating progress messages for the user.}

\item{n.rand.LR}{The number of random expression matrices to use for
learning the ligand-receptor correlation distribution.}

\item{n.rand.RT}{The number of random expression matrices to use for
learning the receptor-target genes correlation distribution.}

\item{with.complex}{A logical indicating whether receptor co-complex
members should be included in the target genes.}

\item{max.pw.size}{Maximum pathway size to consider from the pathway
reference.}

\item{min.pw.size}{Minimum pathway size to consider from the pathway
reference.}

\item{min.positive}{Minimum number of target genes to be found in a given
pathway.}

\item{quick}{A logical indicating whether approximate parameters
for the receptor-target correlations should be used.}

\item{null.model}{The null model to use for Spearman correlation
null distributions.}

\item{filename}{Name of the output plot.}

\item{min.corr.LR}{The minimum ligand-receptor correlation required.}
}
\value{
A BSRDataModel object with trained model parameters
}
\description{
Unique entry point for training the parameters behind
BulkSignalR statistical models.
}
\details{
Estimates the model parameters that are stored in the
  slot \code{param}.

  In a reference pathway, i.e., a Reactome pathway or the genes of a
  GOBP term, the target genes are the genes coding for proteins forming a
  complex with the receptor and the genes in the pathway downstream the
  receptor, which are given as regulated by the pathway. If
  \code{with.complex} is set to \code{FALSE}, then only the regulated genes
  are considered. Participation to a complex, being regulated, and
  pathway directed topologies are defined by Reactome and KEGG pathways
  as provided by PathwayCommons.

  The \code{min.pw.size}, \code{max.pw.size}, and \code{min.positive}
  parameters should be identical to the
  values intended when searching for ligand-receptor pairs with
  \code{\link{.getCorrelatedLR}}) and  \code{\link{.checkReceptorSignaling}})
  Although the statistical distributions are rather robust, it is not
  advisable to use different parameters that could introduce unanticipated
  biases, but for saving compute time and exploring.

  The maximum pathway size is used to limit the redundancy inherent to GOBP
  and Reactome. The minimum pathway size is used to avoid overspecific,
  noninformative results.

  BulkSignalR approach relies on modeling (Spearman) correlations and
  different models of null distributions are available for this purpose
  (parameter \code{null.model}). By default, the "automatic" option is
  selected meaning that censored normal and mixed normal as well as
  an empirical model based on Gaussian kernels (R \code{density()} function)
  are compared to pick the one closest to the data. Preference is given
  to normal and then mixture of normal over the empirical version for
  comparable quality of fit. It is also to bypass the automatic selection.
  Fitting of an alpha-stable distribution is quite time consuming as the
  computation of its PDF is compute-intensive. Finally, in the automaic
  selection mode, the choice of the actual model will be done based on
  the L-R null assuming a similar shape for the R-T null (with
  different parameters though, unless \code{quick} was set to \code{TRUE}).

Note that since the introduction of the use.full.network parameter
(April 29, 2024) in the BSRInference method parameters,
the pathway sizes are always computed before potential
intersection with the observed data (use.full.network set to FALSE) for
consistency. Accordingly, the minimum and maximum pathway default values
have been raised from 5 & 200 to 5 & 400 respectively. By default,
use.full.network is set to TRUE, meaning no intersection and hence larger
pathways.
}
\examples{
data(sdc, package = "BulkSignalR")
idx <- sample(nrow(sdc), 4000)
bsrdm <- BSRDataModel(sdc[idx, c("N22","SDC17")],min.LR.found = 20)
bsrdm <- learnParameters(bsrdm, n.rand.LR = 1L,
verbose=FALSE,quick=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModelComp-class.R
\name{coerce}
\alias{coerce}
\alias{coerce,BSRDataModel,BSRDataModelComp-method}
\title{Convert BSRDataModel to BSRDataModelComp}
\arguments{
\item{from}{BSRDataModel object}
}
\value{
A BSRDataModelComp object
}
\description{
Enable the promotion of a BSRDataModel object into
a BSRDataModelComp object by
coercion.
}
\examples{
bsrdm <- new("BSRDataModel")
bsrdm.comp <- as(bsrdm, "BSRDataModelComp")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bsrdm.spa}
\alias{bsrdm.spa}
\title{A skinny BSRDataModel object related to a spatial data set}
\format{
An example of an object created by `BSRDataModel`
applied to a subset of a spatial data set.
`learnParameters` was also called to get statistical 
model parameters.
}
\source{
\url{http://spatial.libd.org/spatialLIBD/}
}
\usage{
data(bsrdm.spa)
}
\description{
Obtained from STexampleData::Visium_humanDLPFC. 
A single sample (sample 151673)
of human brain dorsolateral prefrontal cortex (DLPFC)
in the human brain, measured using the 10x Genomics 
Visium platform. This is a subset of the full dataset 
published by Maynard and Collado-Torres et al. (2021).
The subset is reproduced in the vignette.
name.idx <- c("10x32","3x47","4x50",
"17x111","5x59","0x20","8x100",
"8x108","14x30","11x39")
}
\details{
Output from the `learnParameters` function to get
BulkSignalR statistical model parameters for a subset
of a spatial data set.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAssign.R
\name{cellularNetworkTable}
\alias{cellularNetworkTable}
\title{Build a table describing a cellular network}
\usage{
cellularNetworkTable(lr, autocrine = FALSE)
}
\arguments{
\item{lr}{The data.frame output by
\code{"\link{assignCellTypesToInteractions}"}.}

\item{autocrine}{A logical indicating whether autocrine interactions should
be included.}
}
\value{
A data.frame containing all the links in the cellular network. A
link is created between two cell types as soon as there was a L-R interaction
that was associated with both cell types. The link is given a score
equal to the geometric mean of each cell type assignment r2.
}
\description{
Generate a data.frame including all the links between cell types
mediated by L-R interactions with their respective weights.
}
\examples{
data(bsrdm, package = "BulkSignalR")
data(bsrinf, package = "BulkSignalR")
data(immune.signatures, package = "BulkSignalR")
data(tme.signatures, package = "BulkSignalR")

immune.signatures <- immune.signatures[immune.signatures$signature \%in\%
    c("T cells"), ]

signatures <- rbind(immune.signatures, tme.signatures[
    tme.signatures$signature \%in\% c("Fibroblasts"),
])

tme.scores <- scoreSignatures(bsrdm, signatures)

# assignment
lr2ct <- assignCellTypesToInteractions(bsrdm, bsrinf, tme.scores)

# cellular network
g.table <- cellularNetworkTable(lr2ct)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{cacheInfo}
\alias{cacheInfo}
\title{Get cache content information.}
\usage{
cacheInfo(dir = c("resources"))
}
\arguments{
\item{dir}{Directory to remove in order to clean the cache.
Can be only 'resources'}
}
\value{
Returns `NULL`, invisibly.
}
\description{
Get cache content information for a specific cache directory.
}
\examples{
cacheInfo()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial.R
\name{spatialAssociation}
\alias{spatialAssociation}
\title{Statistical association of scores with area labels}
\usage{
spatialAssociation(
  scores,
  areas,
  test = c("Kruskal-Wallis", "ANOVA", "Spearman", "r2"),
  label.col = "label",
  idSpatial.col = "idSpatial",
  fdr.proc = c("BH", "Bonferroni", "Holm", "Hochberg", "SidakSS", "SidakSD", "BY", "ABH",
    "TSBH")
)
}
\arguments{
\item{scores}{A matrix of scores, one L-R interaction per row and
spatial locations in the columns. This matrix is typically obtained
from BulkSignalR functions \code{scoreLRGeneSignatures} or \code{scScoring}.}

\item{areas}{A data.frame containing at least the x and y
coordinates of the locations, the unique IDs of spatial locations, and
a label column.}

\item{test}{The chosen statistical test or statistics
(see details below).}

\item{label.col}{Column name in \code{areas} containing area labels.}

\item{idSpatial.col}{Column name in \code{areas} containing the unique
IDs of spatial locations.}

\item{fdr.proc}{Multiple hypothesis correction procedure, see
\code{multtest}.}
}
\value{
A data.frame with the names of the interactions, the value of the
chosen statistics, and the corresponding Q-value.
}
\description{
Compute the statistical association of L-R interaction score spatial
distributions with tissue area labels.
Not limited to BulkSignalR gene signature scores.
}
\details{
In case the
nonparametric Kruskal-Wallis test is chosen, additional columns are provided
testing each label for significantly larger scores (Kruskal-Wallis is global
and only says whether one or several labels show a bias). Individual
labels are tested with Wilcoxon and two columns are added *per* label,
one for the statistics and one for a Bonferroni-corrected P-value over
all the labels.

In case an actual statistical test is chosen, a parametric test (ANOVA) and
a non-parametric test (Kruskal-Wallis) are available for the global analysis.
Individual labels are tested with T-tests or Wilcoxon (Bonferroni-corrected)
accordingly.

In case a statistics is preferred, Spearman correlation or explained variance
(r2 or coefficient of determination, through linear models) 
are available. They mesure the relationship
between each individual area and \code{scores}. For the explained variance,
a global value (R2) is also computed from a multi-linear model (the same as
what is used for the ANOVA).
}
\examples{
data(bsrdm.spa, package = "BulkSignalR")
data(bsrinf.spa, package = "BulkSignalR")
data(annotation.spa, package = "BulkSignalR")
thres <- 0.01
#bsrinf.red <- reduceToBestPathway(bsrinf.spa)
#s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
#scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)

# Run in other examples no need to be run again
# spatialAssociation(scores.red[c(1:2),], areas = annotation.spa,
# label.col = "ground_truth")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial.R
\name{spatialAssociationPlot}
\alias{spatialAssociationPlot}
\title{Heatmap plot of association of scores with area labels}
\usage{
spatialAssociationPlot(
  associations,
  qval.thres = 0.01,
  absval.thres = 0,
  colors = NULL
)
}
\arguments{
\item{associations}{A statistical association data.frame generated
by the function \code{spatialAssociation}.}

\item{qval.thres}{The maximum Q-value to consider in the plot (a
L-R interaction must associate with one label at least with a Q-value
smaller or equal to this threshold).}

\item{absval.thres}{The minimum value to consider in the plot (a
L-R interaction must associate with one label at least with an absolute
value larger or equal to this threshold).}

\item{colors}{A function returning a color for a given value such as
generated by \code{circlize::colorRamp2}.}
}
\value{
ComplexHeatmap::Heatmap object
}
\description{
Plot a heatmap featuring Q-values 
or values of statistical association between
L-R interaction score spatial distributions and tissue area labels.
}
\details{
Display a heatmap linking L-R interactions to labels.
}
\examples{
data(bsrdm.spa, package = "BulkSignalR")
data(bsrinf.spa, package = "BulkSignalR")
data(annotation.spa, package = "BulkSignalR")

thres <- 0.01
bsrinf.red <- reduceToBestPathway(bsrinf.spa)
s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)

# statistical association with tissue areas based on correlations

assoc.bsr.corr <- spatialAssociation(scores.red[c(1:10), ],
areas = annotation.spa, label.col = "ground_truth",test = "Spearman")

spatialAssociationPlot(assoc.bsr.corr)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R
\name{.geneNameConversion}
\alias{.geneNameConversion}
\title{Convert gene symbols to another organism}
\usage{
.geneNameConversion(genes, conversion.dict)
}
\arguments{
\item{genes}{The genes you want to convert.}

\item{conversion.dict}{A data frame containing
gene names for the source species and Homo sapiens.}
}
\value{
Depend on the type of input genes
LRinter return a vector of genes
tgGenes receptors ligands: return list of list of genes
}
\description{
Convert gene symbols to another organism
based on a dictionary with human and orthologs in the other species.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataPrepare.R
\name{.testCacheFiles}
\alias{.testCacheFiles}
\title{Check there is a well formatted cache}
\usage{
.testCacheFiles()
}
\value{
Returns `NULL`, invisibly.
}
\description{
Check there is a well formatted cache
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelTraining.R
\name{.getEmpiricalNullCorrLR}
\alias{.getEmpiricalNullCorrLR}
\title{Sampling of ligand-receptor correlation null distribution}
\usage{
.getEmpiricalNullCorrLR(obj)
}
\arguments{
\item{obj}{A BSRDatamodel without learned paramaters.}
}
\value{
A list of \code{n.rand.LR} tables such as output by
  \code{\link{.getCorrelatedLR}}. Each table is computed from a randomized
  expression matrix (randomized \code{ncounts}).
}
\description{
Perform a ligand-receptor Spearman correlation analysis based
on randomized expression data.
}
\details{
A large number of correlations is reported in each randomized
  matrix. Therefore, \code{n.rand.LR} 
  should be given a modest value to avoid unnecessarily long
  computations.

  See \code{\link{.getCorrelatedLR}} for more details about the parameters.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/networks.R
\name{getLRIntracellNetwork}
\alias{getLRIntracellNetwork}
\title{Generate a ligand-receptor-downstream signaling network}
\usage{
getLRIntracellNetwork(
  bsrinf,
  pval.thres = NULL,
  qval.thres = NULL,
  min.cor = 0.25,
  max.pval = NULL,
  min.logFC = NULL,
  pos.targets = FALSE,
  neg.targets = FALSE,
  restrict.pw = NULL,
  node.size = 5
)
}
\arguments{
\item{bsrinf}{A BSRInference or BSRInferenceComp object.}

\item{pval.thres}{P-value LR interaction threshold.}

\item{qval.thres}{Q-value LR interaction threshold.}

\item{min.cor}{Minimum correlation required for the target genes.}

\item{max.pval}{Maximum P-value required for the target genes in case
a BSRInferenceComp object is provided.}

\item{min.logFC}{Minimum logFC required for the target genes in case
a BSRInferenceComp object is provided.}

\item{pos.targets}{A logical imposing that all the network targets must
display positive correlation or logFC in case of a BSRInferenceComp object.}

\item{neg.targets}{A logical imposing that all the network targets must
display negative correlation or logFC in case of a BSRInferenceComp object.
Correlations must be <= -min.cor or logFC <= - min.logFC
with this option activated.}

\item{restrict.pw}{A vector of pathway IDs to which receptor downstream
signaling is restricted.}

\item{node.size}{Default node size in the network.}
}
\value{
An \code{igraph} object featuring the ligand-receptor-downstream
signaling network. Default colors and node sizes are assigned,
which can be changed afterwards if necessary.

The target genes to which the \code{min.cor} correlation is imposed are
those listed in \code{tgGenes(bsrinf)}, correlations are in
\code{tgCorr(bsrinf)}.
The construction of shortest paths from the receptors to those selected
targets adds other genes, which were either some targets with too low
correlation or genes along the shortest paths to reach the selected targets.
}
\description{
Generate a ligand-receptor network from a BSRInference object and add
the shortest paths from the receptors to correlated
target genes following Reactome and KEGG pathways.
}
\examples{
data(bsrinf, package = "BulkSignalR")

bsrinf.redBP <- reduceToBestPathway(bsrinf)

pairs <- LRinter(bsrinf.redBP)
top <- unique(pairs[pairs$pval < 1e-20, c("pw.id", "pw.name")])

gLRintra.res <- getLRIntracellNetwork(bsrinf.redBP,
qval.thres = 0.01,
restrict.pw = top[1,]$pw.id
)

# write.graph(gLRintra, file="SDC-LR-intracellular-network.reduced.graphml",
# format="graphml")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{immune.signatures}
\alias{immune.signatures}
\title{Immune cell gene signatures}
\format{
A data frame with 1541 rows and 2 variables:
\describe{
  \item{gene}{HUGO gene symbol}
  \item{signature}{cell population name}
}
}
\source{
PanglaoDB (Franzén et al., Database, 2019).
}
\usage{
data(immune.signatures)
}
\description{
A data set containing gene signatures for general immune cell populations.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRClusterComp-class.R
\name{differentialStats<-,BSRClusterComp-method}
\alias{differentialStats<-,BSRClusterComp-method}
\title{Cluster comparison statistics setter (internal use only)}
\usage{
\S4method{differentialStats}{BSRClusterComp}(x) <- value
}
\arguments{
\item{x}{object BSRClusterComp}

\item{value}{value to be set for BSRClusterComp}
}
\value{
returns \code{NULL}
}
\description{
Cluster comparison statistics setter (internal use only)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInferenceComp-class.R
\docType{class}
\name{BSRInferenceComp-class}
\alias{BSRInferenceComp-class}
\title{BulkSignalR cluster comparison-based inference object}
\description{
An S4 class to represent ligand-receptor interactions inferred from
a comparison between two clusters of samples. This class inherits from
BSRInference.
}
\details{
This class is contains inferred LR interactions along with
their statistical significance. Data representation supports subsequent
reductions to pathways, etc. See reduction functions
\code{"\link[=BSRInferenceComp-class]{reduceToBestPathway}"},
\code{"\link[=BSRInferenceComp-class]{reduceToLigand}"},
\code{"\link[=BSRInferenceComp-class]{reduceToReceptor}"} and
\code{"\link[=BSRInferenceComp-class]{reduceToPathway}"}.
}
\section{Slots}{

\describe{
\item{\code{cmp.name}}{The name of the BSRClusterComp object in a BSRDataModelComp
object comp list.}

\item{\code{src.cmp.name}}{The name of an optional BSRClusterComp object in a
BSRDataModelComp object comp list in case paracrine inferences were
performed.}

\item{\code{tg.pval}}{A list of target gene P-values, one
entry per interaction}

\item{\code{tg.logFC}}{A list of target gene logFC, one
entry per interaction}

\item{\code{tg.expr}}{A list of target gene expression, one
entry per interaction}
}}

\examples{
new("BSRInferenceComp")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModel-class.R
\name{normalization}
\alias{normalization}
\alias{normalization,BSRDataModel-method}
\title{Normalization accessor}
\usage{
\S4method{normalization}{BSRDataModel}(x)
}
\arguments{
\item{x}{object BSRDatamModel}
}
\value{
normalization
}
\description{
Normalization accessor
}
\examples{
bsrdm <- BSRDataModel(
    counts = matrix(1.5,
                nrow = 2, ncol = 2,
                dimnames = list(c("A", "B"), c("C", "D"))
                   ),
    method = "TC",
    log.transformed = TRUE,
    normalize = FALSE,
    min.LR.found = 0
)
normalization(bsrdm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resource.R
\name{.formatPathwaysFromTxt}
\alias{.formatPathwaysFromTxt}
\title{Read dataframe from txt file}
\usage{
.formatPathwaysFromTxt(file, resourceName = NULL)
}
\arguments{
\item{file}{Path to a tabular file.}

\item{resourceName}{Two options "GO-BP"  "REACTOME".}
}
\value{
Dataframe with pathwayID, geneName and pathwayName
}
\description{
Read dataframe from txt file
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataPrepare.R
\name{findOrthoGenes}
\alias{findOrthoGenes}
\title{Orthologs Gene Names}
\usage{
findOrthoGenes(
  from_organism,
  from_values,
  method = c("gprofiler", "homologene", "babelgene")
)
}
\arguments{
\item{from_organism}{An organism defined as in Ensembl:
drerio, mmusculus, celegans, dmelanogaster, etc. This is the source organism
from which you want to convert the gene names to Homo sapiens.}

\item{from_values}{A vector of gene names from the current species studied.}

\item{method}{Ortholog mapping method.}
}
\value{
Return a data frame with 2 columns containing the gene names
for the two species.
First column is the gene name from the source organism
and the second column corresponds to the  homologous gene name
in  Homo sapiens.
}
\description{
By default, BulkSignalR is designed to work with Homo sapiens.
In order to work with other organisms, gene names need to be first converted
to human following an orthology mapping process.
}
\examples{
data(bodyMap.mouse)

idx <- sample(nrow(bodyMap.mouse), 20)
bodyMap.mouse <- bodyMap.mouse[idx,]

ortholog.dict <- findOrthoGenes(
    from_organism = "mmusculus",
    from_values = rownames(bodyMap.mouse)
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resource.R
\name{resetNetwork}
\alias{resetNetwork}
\title{Import a refernce network of your own}
\usage{
resetNetwork(network)
}
\arguments{
\item{network}{Network data frame made of 3 columns
a.gn, b.gn & type. 'a.gn' & 'b.gn' should be gene symbols
of gene interactions. 'type'  should be set as
'controls-expression-of' when a user provides
his own file.}
}
\value{
Returns `NULL`, invisibly.
}
\description{
Network is a data frame that defines interactions between
genes. It's composed of 3 columns named as
follows:
}
\details{
a.gn: Gene Symbol 1
type: controls-expression-of
b.gn: Gene Symbol 2

When the user provides his own network,
'type' should be set to 'controls-expression-of'.
}
\examples{
 BulkSignalR_Network <- getResource(resourceName = "Network",
 cache = FALSE)
resetNetwork(BulkSignalR_Network)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bsrinf}
\alias{bsrinf}
\title{A skinny BSRInference object related to sdc.}
\format{
An example of an object created by inference function
}
\usage{
data(bsrinf)
}
\description{
From the previous object `bsrdm`, 
you can generate inferences by calling its
constructor `BSRInference`. 
The resulting BSRInference object is `bsrinf`,
It contains all the inferred L-R interactions 
with their associated pathways and corrected p-values.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRClusterComp-class.R
\name{differentialStats}
\alias{differentialStats}
\alias{differentialStats,BSRClusterComp-method}
\title{Cluster comparison statistics accessor}
\usage{
\S4method{differentialStats}{BSRClusterComp}(x)
}
\arguments{
\item{x}{BSRClusterComp object}
}
\value{
diffferential.stats
}
\description{
Cluster comparison statistics accessor
}
\examples{
bsrcc <- new("BSRClusterComp")
differentialStats(bsrcc)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRSignature-class.R
\docType{class}
\name{BSRSignature-class}
\alias{BSRSignature-class}
\title{BulkSignalR ligand-receptor signature Object}
\description{
S4 class to represent gene signatures
of inferred ligand-receptor interactions, including
their reduced versions.
}
\section{Slots}{

\describe{
\item{\code{ligands}}{A list of ligands, one entry per LR interaction.}

\item{\code{receptors}}{A list of receptors, one entry per LR interaction.}

\item{\code{tg.genes}}{A list of target genes, one entry per LR interaction.}

\item{\code{pathways}}{An atomic vector of pathway names, one per interaction.}

\item{\code{tg.corr}}{A list of target genes correlation.}
}}

\examples{
new("BSRSignature")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BulkSignalR.R
\docType{package}
\name{BulkSignalR-package}
\alias{BulkSignalR}
\alias{BulkSignalR-package}
\title{BulkSignalR: A package to infer ligand-receptor
interactions from bulk transcriptomics or proteomics}
\description{
Inference of ligand-receptor interactions from bulk
(transcriptomic or proteomic) data. BulkSignalR bases its inferences
on the LRdb database. It relies on a statistical model that
is specific to bulk data sets. Different visualization and data
summary functions are proposed to help navigating results.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/jcolinge/BulkSignalR}
  \item Report bugs at \url{https://github.com/jcolinge/BulkSignalR/issues}
}

}
\author{
\strong{Maintainer}: Jean-Philippe Villemin \email{jpvillemin@gmail.com} (\href{https://orcid.org/0000-0002-1838-5880}{ORCID})

Authors:
\itemize{
  \item Jacques Colinge \email{jacques.colinge@inserm.fr} (\href{https://orcid.org/0000-0003-2466-4824}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ortholog.dict}
\alias{ortholog.dict}
\title{A skinny data frame used in the mouse workflow}
\format{
An example of a data frame created by findOrthoGenes
}
\usage{
data(ortholog.dict)
}
\description{
Synthetic object used during the call to the
function `resetToInitialOrganism``
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{.customheatmap}
\alias{.customheatmap}
\title{Heatmap function for gene signature expression}
\usage{
.customheatmap(
  counts,
  name,
  fontsize = 6,
  col.fontsize = 6,
  legend.fontsize = 8,
  annot.fontsize = 8,
  h.height = 3,
  scoring = NULL,
  cols.scoring = NULL,
  hcl.palette = "Blues 3",
  show_column_names = FALSE
)
}
\arguments{
\item{counts}{Matrix of counts.}

\item{name}{Name of the heatmap to generate.}

\item{fontsize}{Font size for row (gene) names.}

\item{col.fontsize}{Font size for column (sample) names.}

\item{legend.fontsize}{Font size for the legends.}

\item{annot.fontsize}{Font size for column annotation names.}

\item{h.height}{Heatmap  height in cm.}

\item{scoring}{Vector of sample scores for a
chosen pathway. If NULL, then no column annotation is produced.}

\item{cols.scoring}{Fixed colorRamp2 object.}

\item{hcl.palette}{Palette from
HCL colormaps supported by ComplexHeatmap.}
}
\value{
A ComplexHeatmap object.

This is a convenience function that relies
on the \code{ComplexHeatmap}
package.
}
\description{
Generate one heatmap used by
\code{signatureHeatmaps}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAssign.R
\name{cellTypeFrequency}
\alias{cellTypeFrequency}
\title{Cell type frequencies in relations to gene sets}
\usage{
cellTypeFrequency(rel, lr, min.n.genes = 1)
}
\arguments{
\item{rel}{The data.frame output by
\code{"\link[=BSRInference-class]{relateToGeneSet}"}.}

\item{lr}{The data.frame output by
\code{"\link{assignCellTypesToInteractions}"}.}

\item{min.n.genes}{Minimum number of genes in the gene set for
one (L,R,pathway) triple.}
}
\value{
A list of two slots: t for counting how many times each cell type
is involved; s for summing the weights of each involved cell type.
}
\description{
Count how many times and with which weights cell types were involved
in the (L,R,pathway) triples that targeted genes in a gene set.
}
\examples{
data(bsrdm, package = "BulkSignalR")
data(bsrinf, package = "BulkSignalR")
data(immune.signatures, package = "BulkSignalR")
data(tme.signatures, package = "BulkSignalR")
data(p.EMT, package = "BulkSignalR")

immune.signatures <- immune.signatures[immune.signatures$signature \%in\%
    c("T cells"), ]

signatures <- rbind(immune.signatures, tme.signatures[
    tme.signatures$signature \%in\% c("Fibroblasts"),
])

tme.scores <- scoreSignatures(bsrdm, signatures)

# assignment
lr2ct <- assignCellTypesToInteractions(bsrdm, bsrinf, tme.scores)

# relate to p-EMT (should be done in HNSCC normally, not in SDC)
p.EMT <- p.EMT$gene
triggers <- relateToGeneSet(bsrinf, p.EMT)

# counts
cf <- cellTypeFrequency(triggers, lr2ct)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInferenceComp-class.R,
%   R/BSRSignatureComp-class.R
\name{tgLogFC}
\alias{tgLogFC}
\alias{tgLogFC,BSRInferenceComp-method}
\alias{tgLogFC,BSRSignatureComp-method}
\title{Target gene logFC accessor}
\usage{
\S4method{tgLogFC}{BSRInferenceComp}(x)

\S4method{tgLogFC}{BSRSignatureComp}(x)
}
\arguments{
\item{x}{BSRSignatureComp object}
}
\value{
tgLogFC

tg.logFC
}
\description{
Target gene logFC accessor

Target gene logFC accessor
}
\examples{
bsrinf <- new("BSRInferenceComp")
tgLogFC(bsrinf)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R
\name{getPathwayStats}
\alias{getPathwayStats}
\alias{getPathwayStats,BSRInference-method}
\title{Basic statistics about hit pathways}
\usage{
\S4method{getPathwayStats}{BSRInference}(obj, pval.thres = NULL, qval.thres = NULL)
}
\arguments{
\item{obj}{BSRinf object.}

\item{pval.thres}{P-value threshold.}

\item{qval.thres}{Q-value threshold.}
}
\value{
A table with the pathways selected after the chosen threshold was
  applied to rows in \code{LRinter(obj)}.
Each pathway is reported along with various statistics:
the number of selected receptors in this pathway, the total number of
receptors described this pathway,
the number of selected ligand-receptor pairs hitting this pathway,
and the total number of
ligand-receptor pairs described that could hit this pathway.

Obviously, one could imagine computing enrichment in receptors or
ligand-receptor pairs
based on such statistics, but the actual meaning of such an analysis
would be ambiguous since
the pathways were already selected as significantly regulated by the
receptor. We thus did not implement
this (hypergeometric test) computation.
}
\description{
Basic statistics about hit pathways
}
\examples{
data(bsrinf, package = "BulkSignalR")

pw.stat <- getPathwayStats(bsrinf)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tme.signatures}
\alias{tme.signatures}
\title{Tumor microenvironment gene signatures}
\format{
A data frame with 209 rows and 2 variables: \describe{
  \item{gene}{HUGO gene symbol} \item{signature}{cell population name} }
}
\source{
Becht & al., Genome Biol, 2016; Angelova et al., Genome Biol, 2015.
}
\usage{
data(tme.signatures)
}
\description{
A data set containing gene signatures for some immune and stromal
  cell populations that are present in the microenvironment of a tumor.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R,
%   R/BSRInferenceComp-class.R
\name{rescoreInference}
\alias{rescoreInference}
\alias{rescoreInference,BSRInference-method}
\alias{rescoreInference,BSRInferenceComp-method}
\title{Inference re-scoring}
\usage{
\S4method{rescoreInference}{BSRInference}(
  obj,
  param,
  rank.p = 0.55,
  fdr.proc = c("BH", "Bonferroni", "Holm", "Hochberg", "SidakSS", "SidakSD", "BY", "ABH",
    "TSBH")
)

\S4method{rescoreInference}{BSRInferenceComp}(
  obj,
  param = NULL,
  rank.p = 0.55,
  fdr.proc = c("BH", "Bonferroni", "Holm", "Hochberg", "SidakSS", "SidakSD", "BY", "ABH",
    "TSBH")
)
}
\arguments{
\item{obj}{BSRInferecenceComp object.}

\item{param}{NULL by default}

\item{rank.p}{A number between 0 and 1 defining the rank of the last
considered target genes.}

\item{fdr.proc}{The procedure for adjusting P-values according to
\code{\link[multtest]{mt.rawp2adjp}}.}
}
\value{
A BSRInference object.

A BSRInferenceComp object.
}
\description{
A method to re-score an existing BSRInference object
(P- and Q-value estimations).

A method to re-score an existing BSRInferenceComp object
(P- and Q-value estimations).
}
\details{
A BSRInference object should be created by calling
\code{"\link[=BSRDataModel-class]{BSRInference}"}

Parameters controlling the estimation
of the statistical significance of the ligand/receptor/pathway triples
(\code{param}) are provided at the time of calling the latter method.

Nonetheless, it
might be useful to change the initially-provided parameters, in
which case this method should not be called.

A BSRInferenceComp object should be created by calling
\code{"\link[=BSRInferenceComp-class]{BSRInferenceComp}"}
}
\examples{
data(bsrinf, package = "BulkSignalR")
data(bsrdm, package = "BulkSignalR")

bsrinf.new <- rescoreInference(bsrinf,
                      param = parameters(bsrdm)
              )
data(bsrinf.comp, package = "BulkSignalR")

bsrinf.less <- rescoreInference(bsrinf.comp, 
rank.p = 0.75)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R,
%   R/BSRInferenceComp-class.R
\name{LRinterShort}
\alias{LRinterShort}
\alias{LRinterShort,BSRInference-method}
\alias{LRinterShort,BSRInferenceComp-method}
\title{Simplified LRinter accessor reporting the essential columns}
\usage{
\S4method{LRinterShort}{BSRInference}(x)

\S4method{LRinterShort}{BSRInferenceComp}(x)
}
\arguments{
\item{x}{BSRInferenceComp object}
}
\value{
LRinterShort

LRinterShort
}
\description{
Simplified LRinter accessor reporting the essential columns

Simplified LRinter accessor reporting the essential columns
}
\examples{
data(bsrinf.comp, package = "BulkSignalR")
LRinterShort(bsrinf.comp)[5,]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModelComp-class.R
\docType{class}
\name{BSRDataModelComp-class}
\alias{BSRDataModelComp-class}
\title{BulkSignalR Data Model Compare Object}
\description{
An S4 class to represent the expression data used for inferring
ligand-receptor interactions based on sample cluster comparisons.
}
\section{Slots}{

\describe{
\item{\code{comp}}{A named list of BSRClusterComp objects, one per
comparison.}

\item{\code{mu}}{A number representing the average value in the normalized and
lop1p-transformed gene expression matrix. This value is used to compute
the LR-score (cf. SingleCellSignalR paper, Cabello-Aguilar, et al.,
Nucleic Acids Res, 2020)}
}}

\examples{
new("BSRDataModelComp")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R
\name{inferenceParameters<-,BSRInference-method}
\alias{inferenceParameters<-,BSRInference-method}
\title{Inference parameters setter (internal use only)}
\usage{
\S4method{inferenceParameters}{BSRInference}(x) <- value
}
\arguments{
\item{x}{BRSInference object.}

\item{value}{value to be set.}
}
\value{
returns \code{NULL}
}
\description{
Inference parameters setter (internal use only)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInferenceComp-class.R,
%   R/BSRSignatureComp-class.R
\name{tgExpr}
\alias{tgExpr}
\alias{tgExpr,BSRInferenceComp-method}
\alias{tgExpr,BSRSignatureComp-method}
\title{Target gene expression accessor}
\usage{
\S4method{tgExpr}{BSRInferenceComp}(x)

\S4method{tgExpr}{BSRSignatureComp}(x)
}
\arguments{
\item{x}{BSRSignatureComp object}
}
\value{
tgExpr

tg.expr
}
\description{
Target gene expression accessor

Target gene expression accessor
}
\examples{
bsrinf <- new("BSRInferenceComp")
tgExpr(bsrinf)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/networks.R
\name{.edgesLRIntracell}
\alias{.edgesLRIntracell}
\title{Internal function to generate a ligand-receptor-downstream signaling network}
\usage{
.edgesLRIntracell(
  pairs,
  pw,
  tg.genes,
  tg.corr,
  id.col,
  gene.col,
  min.cor = 0.25,
  pos.targets = FALSE,
  neg.targets = FALSE,
  tg.pval = NULL,
  max.pval = NULL,
  tg.logFC = NULL,
  min.logFC = 0
)
}
\arguments{
\item{pairs}{A ligand-receptor table such as output by
\code{LRinter(BSRInference)}.}

\item{pw}{A table defining the reference pathways.}

\item{tg.genes}{Target gene list such as output by
\code{tgGenes(BSRInference)}.}

\item{tg.corr}{Target gene correlation list (with the receptor) such as
output by \code{tgCorr(BSRInference)}.}

\item{id.col}{Column index or name in \code{pw} for the pathway IDs.}

\item{gene.col}{Column index or name in \code{pw} for the gene symbols.}

\item{min.cor}{Minimum correlation required for the target genes.}

\item{pos.targets}{A logical imposing that all the network targets must
display positive correlation or logFC a BSRInferenceComp object is used to
generate the network.}

\item{neg.targets}{A logical imposing that all the network targets must
display negative correlation or logFC a BSRInferenceComp object is used to
generate the network. Correlations must be
<= -min.cor or logFC <= - min.logFC
with this option activated.}

\item{tg.pval}{Target gene P-value list such as returned by
\code{ tgPval(BSRInferenceComp)}.}

\item{max.pval}{Maximum (regulation) P-value for target genes in case
a BSRInferenceComp object is used 
to generate the network.}

\item{min.logFC}{Minimum logFC required for the target genes in case
a BSRInferenceComp object is used to generate the network.}
}
\value{
An \code{igraph} object featuring the ligand-receptor-downstream
signaling network. Default colors and node sizes are assigned. In case
the \code{max.pval} parameter is set, it is assumed that \code{tg.pval}
is set as well and downstream signaling genes are selected by their
P-values in the comparison of clusters of samples.
}
\description{
Internal function to generate a ligand-receptor-downstream signaling network
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{simpleHeatmap}
\alias{simpleHeatmap}
\title{Heatmap function for LR scores}
\usage{
simpleHeatmap(
  mat.c,
  dend.row = NULL,
  dend.spl = NULL,
  cols = NULL,
  pointsize = 4,
  bottom.annotation = NULL,
  n.col.clust = 0,
  n.row.clust = 0,
  gap.size = 0.5,
  cut.p = 0.01,
  row.names = TRUE,
  column.names = TRUE,
  hcl.palette = NULL,
  reverse = FALSE
)
}
\arguments{
\item{mat.c}{A matrix with the signature scores such as output by
\code{scoreLRGeneSignatures()}.}

\item{dend.row}{A precomputed row dendrogram.}

\item{dend.spl}{A precompute sample (column) dendrogram.}

\item{cols}{A vector of colors to use for the heatmap.}

\item{pointsize}{Heatmap font point size}

\item{bottom.annotation}{\code{ComplexHeatmap} package bottom annotations.}

\item{n.col.clust}{Number of column clusters.}

\item{n.row.clust}{Number of row clusters.}

\item{gap.size}{Gap size between clusters.}

\item{cut.p}{Proportion of top and bottom values for thresholding.}

\item{row.names}{A logical to turn on/off the display of row names.}

\item{column.names}{A logical to turn on/off the display
of column (sample) names.}

\item{hcl.palette}{support for HCL colormaps in ComplexHeatmap
using color mapping function with circlize::colorRamp2().
palettes are listed in grDevides::hcl.pals().
of row (gene) names.}

\item{reverse}{A logical to reverse or not colors in hcl.palette.}
}
\value{
A heatmap. Since heatmap plotting tend to be slow on the screen,
it is advisable to plot in a file instead.

If hcl.palette is set, the colors parameter won't be used.

Extreme values (top and bottom) can be replaced by global quantiles at
\code{cut.p} and \code{1-cut.p}
to avoid color scales shrunk by a few outliers.

This is a convenience function that relies on the \code{ComplexHeatmap}
package to propose a simple way
of representing signature scores. If more advanced features are needed or
more graphic parameters
should be controlled, users should implement their own function.
}
\description{
Generate a heatmap representing ligand-receptor gene
signature scores.
}
\examples{
data(bsrdm, package = "BulkSignalR")
data(bsrinf, package = "BulkSignalR")

bsrinf.redBP <- reduceToBestPathway(bsrinf)
bsrsig.redBP <- BSRSignature(bsrinf,
    qval.thres = 0.001
)

scoresLR <- scoreLRGeneSignatures(bsrdm, bsrsig.redBP,
    name.by.pathway = FALSE
)
simpleHeatmap(scoresLR[1:3, ],
    column.names = TRUE,
    hcl.palette = "Cividis")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{p.EMT}
\alias{p.EMT}
\title{Partial EMT gene signature}
\format{
A data frame with 100 rows and 1 variables:
\describe{
  \item{gene}{HUGO gene symbol}
}
}
\source{
Puram, SV & al., Cell, 2017.
}
\usage{
data(p.EMT)
}
\description{
A data set containing a partial EMT gene signature.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial.R
\name{smoothSpatialCounts}
\alias{smoothSpatialCounts}
\title{Smooth spatial expression data}
\usage{
smoothSpatialCounts(
  bsrdm,
  areas,
  nnn = 4,
  radius = NULL,
  weight.ratio = 0.5,
  x.col = "array_col",
  y.col = "array_row"
)
}
\arguments{
\item{bsrdm}{A BSRDataModel object containing the expression data to smooth.}

\item{areas}{A data.frame containing at least the x and y
coordinates of the locations.}

\item{nnn}{Number of nearest-neighbor locations to use for smoothing
each location. In case \code{radius} is set, then it is the maximum number
of nearest neighbors within the radius.}

\item{radius}{A maximal distance to include neighbors in the smoothing.}

\item{weight.ratio}{The weight given to the central location.}

\item{x.col}{Column name in \code{areas} containing x coordinates.}

\item{y.col}{Column name in \code{areas} containing y coordinates.}
}
\value{
A BSRDataModel object containing the smoothed ncounts.
}
\description{
Smooth spatial expression data
}
\details{
The expression data contained in a BSRDataModel object are
smoothed using a weighted average of nearby locations.

Two strategies are available to identify the neighbors. It is possible to
simply set the number of nearest-neighbors (parameter \code{nnn}). An
alternative consists in providing a distance radius (\code{radius}) along
with a a maximum number of nearest-neighbors within the radius
(\code{nnn.radius}). To properly define the radius, the user must know the
location coordinates. The strategy with the radius enables having corner
locations with two neighbors only and border locations with three
neighbors only, whereas to simply set a maximum of four neighbors for
instance would retrieve the four closest neighbors in every case.

For each location, its nearest-neighbors are found and a weighted average
computed with \code{weight.ratio} given to the central location itself
and a total weight of 1-\code{weight.ratio} shared within the neighbors
based on the inverse of their distances. In case \code{radius} is set,
some locations may have less than \code{nnn} neighbors (see above).
At such locations, the weight given to
the central location is augmented according to
1-(1-\code{weight.ratio})*(number of neighbors)/\code{nnn}.
}
\examples{
 
data(bsrdm.spa, package = "BulkSignalR")
data(annotation.spa, package = "BulkSignalR")
sm.bsrdm <- smoothSpatialCounts(bsrdm.spa, annotation.spa,
radius = 1.2, nnn = 4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRSignature-class.R
\name{BSRSignature}
\alias{BSRSignature}
\title{Extract gene signatures of LR pair activity}
\usage{
BSRSignature(obj, pval.thres = NULL, qval.thres = NULL, with.pw.id = FALSE)
}
\arguments{
\item{obj}{BSRinference object.}

\item{pval.thres}{P-value threshold.}

\item{qval.thres}{Q-value threshold.}

\item{with.pw.id}{A logical indicating whether the ID of a pathway
should be concatenated to its name.}
}
\value{
A BSRSignature object containing a gene signature for each triple
ligand-receptor pair. A reduction to the best pathway
for each pair is automatically performed and the gene signature is
comprised of the ligand, the receptor,
and all the target genes with rank equal or superior to \code{pairs$rank}.
}
\description{
Obtain gene signatures reflecting ligand-receptor as well as
receptor downstream activity to
score ligand-receptor pairs across samples subsequently with
\code{"\link[=BSRDataModel-class]{scoreLRGeneSignatures}"}
}
\examples{
data(bsrinf, package = "BulkSignalR")

bsrinf.redP <- reduceToPathway(bsrinf)
bsrsig.redP <- BSRSignature(bsrinf, qval.thres = 0.001)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModelComp-class.R
\name{comparison<-,BSRDataModelComp-method}
\alias{comparison<-,BSRDataModelComp-method}
\title{Comparisons list setter (internal use only, use addComparison() otherwise)}
\usage{
\S4method{comparison}{BSRDataModelComp}(x) <- value
}
\arguments{
\item{x}{object BSRDataModelComp}

\item{value}{value to be set for BSRDataModelComp}
}
\value{
returns \code{NULL}
}
\description{
Comparisons list setter (internal use only, use addComparison() otherwise)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAssign.R
\name{relateToGeneSet}
\alias{relateToGeneSet}
\title{Relate ligands to a gene set}
\usage{
relateToGeneSet(bsrinf, gs, min.cor = 0.25, qval.thres = 0.001)
}
\arguments{
\item{bsrinf}{BSRInference object.}

\item{gs}{The gene set.}

\item{min.cor}{Minimum Spearman correlation between the receptor of a triple
(L,R,pw) and a gene of the gene set.}

\item{qval.thres}{Maximum Q-value imposed to the (L,R,pw) triples to be
considered.}
}
\value{
A data.frame listing all the (L,R,pathway) triples that lead
to at least one gene in the gene set. The number of genes found
by each triple is indicated in the column \code{n.genes}.
}
\description{
Finds ligands related to a gene set by following receptor, and
receptor downstream pathway targets.
}
\examples{
data(bsrdm, package = "BulkSignalR")
data(bsrinf, package = "BulkSignalR")

data(p.EMT, package = "BulkSignalR")
p.EMT <- p.EMT$gene
triggers <- relateToGeneSet(bsrinf, p.EMT) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resource.R
\name{createResources}
\alias{createResources}
\title{Create all resources}
\usage{
createResources(onRequest = TRUE, verbose = FALSE)
}
\arguments{
\item{onRequest}{logical TRUE if you want to force
downloading again. This will overwrite the
pre-existing local database. Default is TRUE.}

\item{verbose}{Default is FALSE}
}
\value{
Returns `NULL`, invisibly.
}
\description{
Create a cache for all resources (pathways, lrdb & network)
downloaded from the web when the library is first loaded.
This functionality is handled with BiocFileCache.
}
\examples{
createResources(onRequest=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{cacheVersion}
\alias{cacheVersion}
\title{Check whether remote resource files have been changed.}
\usage{
cacheVersion(dir = c("resources"))
}
\arguments{
\item{dir}{Directory for which you want to check Version.
Can be only 'resources'.}
}
\value{
Returns `NULL`, invisibly.
}
\description{
Check to see whether some resource
has been updated.
}
\examples{
cacheVersion()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelTraining.R
\name{.buildPermutatedCountMatrix}
\alias{.buildPermutatedCountMatrix}
\title{Internal function to generate a randomized expression matrix}
\usage{
.buildPermutatedCountMatrix(ncounts, pind)
}
\arguments{
\item{ncounts}{A matrix of normalized read counts.}

\item{pind}{Permutation indices such as returned by
\code{\link{.buildPermutationIndices}}.}
}
\value{
\code{ncount} with shuffled row names (gene symbols). Shuffling is
  performed within rows of comparable average expression.
}
\description{
Internal function to generate a randomized expression matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R, R/BSRSignature-class.R
\name{tgCorr}
\alias{tgCorr}
\alias{tgCorr,BSRInference-method}
\alias{tgCorr,BSRSignature-method}
\title{Target gene correlations accessor}
\usage{
\S4method{tgCorr}{BSRInference}(x)

\S4method{tgCorr}{BSRSignature}(x)
}
\arguments{
\item{x}{BSRSignature}
}
\value{
tgCorr
}
\description{
Target gene correlations accessor

Target gene correlations accessor
}
\examples{
bsr.sig <- new("BSRSignature")
tgCorr(bsr.sig)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModelComp-class.R
\name{mu<-,BSRDataModelComp-method}
\alias{mu<-,BSRDataModelComp-method}
\title{Mu setter (internal use only)}
\usage{
\S4method{mu}{BSRDataModelComp}(x) <- value
}
\arguments{
\item{x}{object BSRDataModelComp}

\item{value}{value to be set for BSRDataModelComp}
}
\value{
returns \code{NULL}
}
\description{
Mu setter (internal use only)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regulatedInference.R
\name{.pValuesRegulatedLR}
\alias{.pValuesRegulatedLR}
\title{Internal function to assign P-values to LR interactions}
\usage{
.pValuesRegulatedLR(
  pairs,
  param,
  rank.p = 0.75,
  fdr.proc = c("BH", "Bonferroni", "Holm", "Hochberg", "SidakSS", "SidakSD", "BY", "ABH",
    "TSBH")
)
}
\arguments{
\item{pairs}{A data frame output by
\code{checkRegulatedReceptorSignaling}.}

\item{param}{A list containing the statistical model parameters.}

\item{rank.p}{A number between 0 and 1 defining the rank of the last
considered target genes.}

\item{fdr.proc}{The procedure for adjusting P-values according to
\code{\link[multtest]{mt.rawp2adjp}}.}
}
\value{
A data.frame with the data in \code{pairs} complemented with
P-values and adjusted P-values.
}
\description{
Estimate the P-value of each ligand-receptor pair based
on the data frame output by \code{\link{.checkRegulatedReceptorSignaling}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataPrepare.R
\name{convertToHuman}
\alias{convertToHuman}
\title{Transpose to Human Gene Names}
\usage{
convertToHuman(counts, dictionary)
}
\arguments{
\item{counts}{A table or matrix of read counts.}

\item{dictionary}{A data frame where the first column 
is made of gene symbols for the actual organism 
and row names are the ortholog human gene symbols.}
}
\value{
Return a counts matrix transposed for Human.
}
\description{
By default, BulkSignalR is designed to work with Homo sapiens.
In order to work with other organisms, gene names need to be first converted
to human by orthology.
}
\examples{
data(bodyMap.mouse)

idx <- sample(nrow(bodyMap.mouse), 500)
bodyMap.mouse <- bodyMap.mouse[idx,]

ortholog.dict <- findOrthoGenes(
    from_organism = "mmusculus",
    from_values = rownames(bodyMap.mouse)
)

matrix.expression.human <- convertToHuman(
    counts = bodyMap.mouse,
    dictionary = ortholog.dict
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R,
%   R/BSRInferenceComp-class.R
\name{reduceToBestPathway}
\alias{reduceToBestPathway}
\alias{reduceToBestPathway,BSRInference-method}
\alias{reduceToBestPathway,BSRInferenceComp-method}
\title{Keep one pathway per ligand-receptor pair}
\usage{
\S4method{reduceToBestPathway}{BSRInference}(obj)

\S4method{reduceToBestPathway}{BSRInferenceComp}(obj)
}
\arguments{
\item{obj}{BSRInferenceComp object}
}
\value{
A BSRInference object reduced to only report one pathway per
ligand-receptor pair. The pathway with the
smallest P-value is selected.

A BSRInferenceComp object reduced to only report one pathway per
ligand-receptor pair. The pathway with the
smallest P-value is selected.
}
\description{
Keep one pathway per ligand-receptor pair

Keep one pathway per ligand-receptor pair
}
\details{
Ligand-receptor pairs
are evaluated in relation with pathways that allow checking receptor
downstream correlations. It is thus possible
that several pathways are reported for a same LR pair.

Ligand-receptor pairs
are evaluated in relation with pathways that allow checking receptor
downstream correlations. It is thus possible
that several pathways are reported for a same LR pair.
}
\examples{
data(bsrinf, package = "BulkSignalR")
bsrinf.redBP <- reduceToBestPathway(bsrinf)

data(bsrinf.comp, package = "BulkSignalR")


reduceToBestPathway(bsrinf.comp)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{signatureHeatmaps}
\alias{signatureHeatmaps}
\title{Heatmap function to dissect one pathway signature}
\usage{
signatureHeatmaps(
  pathway,
  bsrdm,
  bsrsig,
  heights = c(4, 2, 4),
  fontsize = 6,
  legend.fontsize = 8,
  title.fontsize = 8,
  col.fontsize = 6,
  annot.fontsize = 8,
  ht.gap = 3,
  show_column_names = TRUE
)
}
\arguments{
\item{pathway}{The chosen pathway name.}

\item{bsrdm}{BulkSignalR data model object.}

\item{bsrsig}{BulkSignalR signature object.}

\item{heights}{A vector of 3 heights (in cm) for the 3 heatmaps.}

\item{fontsize}{Font size for the gene names.}

\item{legend.fontsize}{Font size for the legends.}

\item{title.fontsize}{Font size for the pathway name as plot title.}

\item{col.fontsize}{Font size for column (sample) names.}

\item{annot.fontsize}{Font size for column annotation names.}

\item{ht.gap}{Space between heatmaps (in mm).}

\item{show_column_names}{Add column names in the heatmaps.}
}
\value{
A plot is created.
}
\description{
Plots a stack of three heatmaps to assess the expression of
the target genes or proteins in a chosen pathway, the receptor expressions,
and the ligand expressions.
}
\examples{
data(bsrdm, package = "BulkSignalR")
data(bsrinf, package = "BulkSignalR")
if(FALSE){
bsrinf.redP <- reduceToPathway(bsrinf)
bsrinf.redPBP <- reduceToBestPathway(bsrinf)
bsrsig.redPBP <- BSRSignature(bsrinf, qval.thres = 1)
pathway1 <- pathways(bsrsig.redPBP)[1]
signatureHeatmaps(
pathway = pathway1,
bsrdm = bsrdm,
bsrsig = bsrsig.redPBP,
show_column_names = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelTraining.R
\name{.shufflePermutationIndices}
\alias{.shufflePermutationIndices}
\title{Internal function to shuffle permutation indices}
\usage{
.shufflePermutationIndices(pind)
}
\arguments{
\item{pind}{Permutation indices such as returned by
\code{\link{.buildPermutationIndices}}.}
}
\value{
A list with same structure as \code{pind} with shuffled indices
  within each bin.
}
\description{
Internal function to shuffle permutation indices
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bsrinf.mouse}
\alias{bsrinf.mouse}
\title{A skinny BSRInference object related to bodyMap.mouse}
\format{
An example of an object created by inference function
}
\usage{
data(bsrinf.mouse)
}
\description{
see related workflow for non human organism
in the vignette
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{bubblePlotPathwaysLR}
\alias{bubblePlotPathwaysLR}
\title{Bubble Plot to explore LR & Pathways}
\usage{
bubblePlotPathwaysLR(
  bsrinf,
  pathways,
  qval.thres = 1,
  filter.L = NULL,
  filter.R = NULL,
  color = "#16a647",
  pointsize = 6
)
}
\arguments{
\item{bsrinf}{BulkSignalR inference object.}

\item{pathways}{Vector of pathway names to keep.}

\item{qval.thres}{Maximum Q-value.}

\item{filter.L}{Vector of ligands to keep.}

\item{filter.R}{Vector of receptors to keep.}

\item{color}{Main color used for the gradient.}

\item{pointsize}{Global point size.}
}
\value{
A bubble plot displayed in the current viewport.

This is a convenience function to propose a simple way
of representing LR - Pathways association
with their respective correlation
and Q-values.
}
\description{
Quick check to observe LR - Pathways association
with their respective correlation
and Q-values.
}
\examples{
data(bsrinf, package = "BulkSignalR")
pathways <- LRinter(bsrinf)[1,c("pw.name")]
bubblePlotPathwaysLR(bsrinf,
pathways = pathways,
qval.thres = 0.1,
color = "red",
pointsize = 8
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regulatedInference.R
\name{.getRegulatedLR}
\alias{.getRegulatedLR}
\title{Get regulated ligand-receptor pairs.}
\usage{
.getRegulatedLR(
  ds,
  cc,
  scc = NULL,
  max.pval = 0.01,
  min.logFC = 1,
  neg.receptors = FALSE,
  restrict.genes = NULL
)
}
\arguments{
\item{ds}{A BSRDataModel object}

\item{cc}{A BSRClusterComp object.}

\item{scc}{An optional BSRClusterComp object in case ligand
regulation was assessed in a separate cluster comparison.}

\item{max.pval}{The maximum P-value imposed to both the ligand
and the receptor.}

\item{min.logFC}{The maximum log2 fold-change allowed for
both the receptor and the ligand.}

\item{neg.receptors}{A logical indicating whether receptors are only
allowed to be upregulated (FALSE), or up- and downregulated (TRUE).}

\item{restrict.genes}{A list of gene symbols that restricts ligands and
receptors.}
}
\value{
A data frame containing putative ligand-receptor pairs along
  with the product of their respective P-values. This table is the first step
  of a ligand-receptor analysis.
}
\description{
Internal function to return all the pairs of ligands and receptors
having both a P-value below a given threshold.
}
\details{
The \code{restrict.genes} parameter is used for special cases where
  LRdb must be further restricted to a subset.
  The putative ligand-receptor pairs has 6 columns : R, L, LR.pval, corr,
  L.logFC, and R.logFC. 
  Note that correlations are currently set to 1 to avoid
  lengthy computations with scRNA-seq data and multiple cell
  populations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModelComp-class.R
\name{BSRDataModelComp}
\alias{BSRDataModelComp}
\title{Constructor of the BSRDataModelComp class}
\usage{
BSRDataModelComp(
  counts,
  normalize = TRUE,
  symbol.col = NULL,
  min.count = 10,
  prop = 0.1,
  method = c("UQ", "TC"),
  log.transformed = FALSE,
  min.LR.found = 80,
  species = "hsapiens",
  conversion.dict = NULL,
  UQ.pc = 0.75,
  x.col = NULL,
  y.col = NULL,
  barcodeID.col = NULL
)
}
\arguments{
\item{counts}{A table or matrix of read counts.}

\item{normalize}{A logical indicating whether \code{counts} should be
normalized according to \code{method} or if it was normalized beforehand.}

\item{symbol.col}{The index of the column containing the gene symbols in case
those are not the row names of \code{counts} already.}

\item{min.count}{The minimum read count of a gene to be considered expressed
in a sample.}

\item{prop}{The minimum proportion of samples where a gene must be
expressed higher than \code{min.count} to keep that gene.}

\item{method}{The normalization method ('UQ' for upper quartile or 'TC'
for total count). If \code{normalize==FALSE}, then method must be
used to document the name of the normalization method applied by the user.}

\item{log.transformed}{A logical indicating whether expression data were
already log2-transformed, e.g., some microarray data.}

\item{min.LR.found}{The minimum number of ligands or receptors found in
\code{count} row names after eliminating the rows containing too many
zeros according to \code{min.count} and \code{prop}.}

\item{species}{Data were obtained for this organism.}

\item{conversion.dict}{Correspondence table of HUGO gene symbols
human/nonhuman. Not used unless the organism is different from human.}

\item{UQ.pc}{Percentile for upper-quartile normalization, number
between 0 and 1 (in case the default 0.75 - hence the name - is not
appropriate).}

\item{x.col}{In a SpatialExperiment object, the index of the column
containing the x coordinates in the dafaframe returned by rowData(), usually 
named array_row.}

\item{y.col}{In a SpatialExperiment object, the index of the column
containing the y coordinates in the dafaframe returned by rowData(), usually 
named array_col.}

\item{barcodeID.col}{In a SpatialExperiment object, the index of the column
containing the barcodeID in the dafaframe returned by colData(), usually
named barcode_id.}
}
\value{
A BSRModelDataComp object with empty model parameters.
}
\description{
Take a matrix or data frame containing RNA sequencing,
microarray, or expression proteomics data as input parameter
and return a BSRDataModelComp
object ready for subsequent use.
}
\details{
The \code{counts} matrix or table should be provided with expression
  levels of protein coding genes in each samples (column) and
  \code{rownames(counts)} set to HUGO official gene symbols.
  For commodity, it is also possible 
  to provide \code{counts} with the
  gene symbols stored in one of its columns. This column must be specified
  with \code{symbol.col}. In such a case, \code{BSRDataModel} will extract
  this column and use it to set the row names. Because row names must be
  unique, \code{BSRDataModel} will eliminate rows with duplicated gene
  symbols by keeping the rows with maximum average expression. Gene symbol
  duplication may occur in protein coding genes after genome alignment
  due to errors in genome feature annotation files (GTF/GFF), where a handful
  of deprecated gene annotations might remain, or
  some genes are not given their fully specific symbols. If your read count
  extraction pipeline does not take care of this phenomenon, the maximum mean
  expression selection strategy implemented here should solve this difficulty
  for the sake of inferring ligand-receptor interactions.

  If \code{normalize} is \code{TRUE} then normalization is performed
  according to \code{method}. If those two simple methods are not satisfying,
  then it is possible to provide a pre-normalized matrix setting
  \code{normalize} to \code{FALSE}. In such a case, the parameter
  \code{method} must be used to document the name of the normalization
  algorithm used.

  In case proteomic or microarray data are provided, \code{min.count} must be
  understood as its equivalent with respect to those data types.
}
\examples{
data(sdc, package = "BulkSignalR")
idx <- sample(nrow(sdc), 4000)
bsrdm.comp <- BSRDataModelComp(sdc[idx, c("N22","SDC17")],
                               normalize=FALSE, method="UQ")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInferenceComp-class.R,
%   R/BSRSignatureComp-class.R
\name{tgPval}
\alias{tgPval}
\alias{tgPval,BSRInferenceComp-method}
\alias{tgPval,BSRSignatureComp-method}
\title{Target gene P-values accessor}
\usage{
\S4method{tgPval}{BSRInferenceComp}(x)

\S4method{tgPval}{BSRSignatureComp}(x)
}
\arguments{
\item{x}{BSRSignatureComp object}
}
\value{
tgPval

tg.pval
}
\description{
Target gene P-values accessor

Target gene P-values accessor
}
\examples{
bsrinf <- new("BSRInferenceComp")
tgPval(bsrinf)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRSignatureComp-class.R
\docType{class}
\name{BSRSignatureComp-class}
\alias{BSRSignatureComp-class}
\title{BulkSignalR ligand-receptor signature object for cluster comparisons}
\description{
S4 class to represent gene signatures associated with ligand-receptor
interactions that were inferred from the comparison of two clusters
of samples. This class inherits from BSRSignature.
}
\section{Slots}{

\describe{
\item{\code{cmp.name}}{The name of the comparison.}

\item{\code{tg.pval}}{A list of target genes P-values.}

\item{\code{tg.logFC}}{A list of target genes logFC.}

\item{\code{tg.expr}}{A list of target genes expression}
}}

\examples{
new("BSRSignatureComp")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial.R
\name{generateSpatialPlots}
\alias{generateSpatialPlots}
\title{Generate L-R interaction score spatial plots in a folder}
\usage{
generateSpatialPlots(
  scores,
  areas,
  plot.folder,
  width = 5,
  height = 3,
  pointsize = 8,
  rev.y = TRUE,
  ref.plot = TRUE,
  image.raster = NULL,
  x.col = "array_col",
  y.col = "array_row",
  label.col = "label",
  idSpatial.col = "idSpatial",
  cut.p = 0.01,
  low.color = "royalblue3",
  mid.color = "white",
  high.color = "orange",
  title.fs = 12,
  legend.fs = 10,
  axis.fs = 10,
  label.fs = 12,
  dot.size = 0.5,
  ref.colors = NULL
)
}
\arguments{
\item{scores}{A matrix of scores, one L-R interaction per row and
spatial locations in the columns. This matrix is typically obtained
from BulkSignalR functions \code{scoreLRGeneSignatures} or \code{scScoring}.}

\item{areas}{A data.frame containing at least the x and y
coordinates of the locations as well as the unique IDs of spatial locations.
In case \code{ref.plot} is set to TRUE,
a label column is required additionally.}

\item{plot.folder}{The folder name in which the plot files will be written.}

\item{width}{The width of each individual plot.}

\item{height}{The height of each individual plot.}

\item{pointsize}{PDF font point size.}

\item{rev.y}{A Boolean indicating whether low y coordinates should be
at the top of the plot.}

\item{ref.plot}{A Boolean indicating whether a reference map of the tissue
with area labels should be plot aside.}

\item{image.raster}{Raster object image to plot raw tissue image as
reference.}

\item{x.col}{Column name in \code{areas} containing x coordinates.}

\item{y.col}{Column name in \code{areas} containing y coordinates.}

\item{label.col}{Column name in \code{areas} containing area labels.}

\item{idSpatial.col}{Column name in \code{areas} containing the unique
IDs of spatial locations.}

\item{cut.p}{Proportion of top and bottom values for thresholding.}

\item{low.color}{Color for low score values.}

\item{mid.color}{Color for score = 0.}

\item{high.color}{Color for high score values.}

\item{title.fs}{Title font size.}

\item{legend.fs}{Legend items font size.}

\item{axis.fs}{Axis ticks font size.}

\item{label.fs}{Legend titles and axis names font size.}

\item{dot.size}{Dot size.}

\item{ref.colors}{A vector of colors to bypass those automatically chosen
by ggplot2 for the tissue areas in the reference plot.}
}
\value{
Create PDF file and returns `NULL`, invisibly.
}
\description{
Generate a series of individual spatial score plots in a folder.
Not limited to BulkSignalR gene signature scores.
}
\details{
A set of PDF files are created in the provided folder.
}
\examples{
data(bsrdm.spa, package = "BulkSignalR")
data(bsrinf.spa, package = "BulkSignalR")
data(annotation.spa, package = "BulkSignalR")

thres <- 0.01
bsrinf.red <- reduceToBestPathway(bsrinf.spa)
s.red  <- BSRSignature(bsrinf.red, qval.thres=thres)
scores.red <- scoreLRGeneSignatures(bsrdm.spa,s.red)

generateSpatialPlots(scores.red[1:2,],
annotation.spa, ".", label.col = "ground_truth")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{chordDiagramLR}
\alias{chordDiagramLR}
\title{Chord Diagram of LR interactions with correlations}
\usage{
chordDiagramLR(
  bsrinf,
  pw.id.filter = NULL,
  qval.thres = 1,
  ligand = NULL,
  receptor = NULL,
  limit = 20
)
}
\arguments{
\item{bsrinf}{A BSRInference object}

\item{pw.id.filter}{One Pathway ID accepted only to}

\item{qval.thres}{Threshold over Q-values.}

\item{ligand}{Ligand
of the LR pair that you want to
highlight in the chord diagram.}

\item{receptor}{Receptor
of the LR pair that you want to highlight
in the chord diagram.}

\item{limit}{Number of interactions you can visualize.}
}
\value{
Circos Plot on the screen or a file
}
\description{
Chord diagram.
}
\examples{
data(bsrinf, package = "BulkSignalR")
chordDiagramLR(bsrinf,
pw.id.filter = "R-HSA-3000178",
limit = 20,
ligand="ADAM15", 
receptor="ITGAV"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inference.R
\name{.downstreamSignaling}
\alias{.downstreamSignaling}
\title{Internal function to check receptor signaling downstream}
\usage{
.downstreamSignaling(
  lr,
  pw,
  pw.size,
  rncounts,
  id.col,
  gene.col,
  pw.col,
  min.positive,
  with.complex = TRUE
)
}
\arguments{
\item{lr}{A data frame as returned by
\code{.getCorrelatedLR()}.}

\item{pw}{A table defining the reference pathways.}

\item{pw.size}{A named vector with pathway sizes (names are pathway
IDs).}

\item{rncounts}{A matrix of normalized read counts with at
least all the ligands, receptors, and genes in the reference pathways.}

\item{id.col}{Column index or name in \code{pw} for the pathway IDs.}

\item{gene.col}{Column index or name in \code{pw} for the gene
symbols.}

\item{pw.col}{Column index or name in \code{pw} for the pathway
names.}

\item{min.positive}{Minimum number of target genes to be found in a given
pathway.}

\item{with.complex}{A logical indicating whether receptor co-complex
members should be included in the target genes.}
}
\value{
A table reporting all the ligand-receptor pairs provided in \code{lr}
  along with the pathways found and data about target gene correlations with
  the receptor.
}
\description{
Internal function to check receptor signaling downstream
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRClusterComp-class.R
\name{BSRClusterComp}
\alias{BSRClusterComp}
\title{Definition of the comparison between two clusters of samples}
\usage{
BSRClusterComp(obj, col.clusterA, col.clusterB, differential.stats)
}
\arguments{
\item{obj}{A BSRDataModelComp object.}

\item{col.clusterA}{Cluster A column indices.}

\item{col.clusterB}{Cluster B column indices.}

\item{differential.stats}{A data.frame containing statistics about
the differential
analysis cluster A versus B. \code{differentialStats} must contain 
at least the
columns 'pval' (for P-values), 'logFC' for log-fold-changes A/B, and
'expr' for the expression of the genes in cluster A.}
}
\value{
A BSRClusterComp object.
}
\description{
Define the columns of the expression matrix that belong to each cluster,
and store the result of the cluster differences statistical analysis
obtained by an external tool such as edgeR or DESeq2 in a dedicated
data frame.
}
\details{
Create a BSRClusterComp object describing a comparison
of two clusters of columns taken from the expression matrix
in the BSRDataModelComp object \code{obj}. Such a cluster comparison
description is the basis for inferring LRIs from differential
expression P-values instead of correlation analysis.

The rows of \code{differentialStats} must be in the same order 
as those of the count
matrix in \code{obj}. Alternatively, \code{differentialStats}
rows can be named and a 1-1 correspondence must exist between these names
and those of the count matrix.
}
\examples{
# prepare data
data(sdc, package = "BulkSignalR")
normal <- grep("^N", names(sdc))
bsrdm <- BSRDataModel(sdc[, -normal])

# define the comparison
bsrdm.comp <- as(bsrdm, "BSRDataModelComp")
colA <- as.integer(1:3)
colB <- as.integer(12:15)
n <- nrow(ncounts(bsrdm.comp))
stats <- data.frame(
    pval = runif(n), logFC = rnorm(n, 0, 2),
    expr = runif(n, 0, 10)
)
rownames(stats) <- rownames(ncounts(bsrdm.comp))
bsrcc <- BSRClusterComp(bsrdm.comp, colA, colB, stats)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataPrepare.R
\name{.testRemoteServer}
\alias{.testRemoteServer}
\title{Check whether the reference DB server is up}
\usage{
.testRemoteServer()
}
\value{
Returns `NULL`, invisibly.
}
\description{
Check whether the reference DB server is up
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInferenceComp-class.R
\name{BSRInferenceComp}
\alias{BSRInferenceComp}
\title{Inference of ligand-receptor interactions based on regulation}
\usage{
BSRInferenceComp(
  obj,
  cmp.name,
  src.cmp.name = NULL,
  rank.p = 0.55,
  max.pval = 0.01,
  min.logFC = 1,
  neg.receptors = FALSE,
  pos.targets = FALSE,
  neg.targets = FALSE,
  min.t.logFC = 0.5,
  restrict.genes = NULL,
  use.full.network = FALSE,
  reference = c("REACTOME-GOBP", "REACTOME", "GOBP"),
  max.pw.size = 400,
  min.pw.size = 5,
  min.positive = 2,
  restrict.pw = NULL,
  with.complex = TRUE,
  fdr.proc = c("BH", "Bonferroni", "Holm", "Hochberg", "SidakSS", "SidakSD", "BY", "ABH",
    "TSBH")
)
}
\arguments{
\item{obj}{A BSRDataModelComp object.}

\item{cmp.name}{The name of the cluster comparison that should be used
for the inference. Autocrine interactions if only this comparison name is
provided, paracrine if a source comparison name is provided as well.}

\item{src.cmp.name}{The name of the source cluster comparison that should
be used for paracrine interaction inferences.}

\item{rank.p}{A number between 0 and 1 defining the rank of the last
considered target genes.}

\item{max.pval}{The maximum P-value imposed to both the ligand
and the receptor.}

\item{min.logFC}{The minimum log2 fold-change allowed for
both the receptor and the ligand.}

\item{neg.receptors}{A logical indicating whether receptors are only
allowed to be upregulated (FALSE), or up- and downregulated (TRUE).}

\item{pos.targets}{A logical imposing that all the network targets must
display positive logFC, i.e. logFC >= min.t.logFC.}

\item{neg.targets}{A logical imposing that all the network targets must
display negative logFC, i.e. logFC <= - min.t.logFC.}

\item{min.t.logFC}{The minimum log2 fold-change allowed for
targets in case pos.targets or neg.targets are used.}

\item{restrict.genes}{A list of gene symbols that restricts ligands and
receptors.}

\item{use.full.network}{A logical to avoid limiting the reference network
to the detected genes and use the whole reference network.}

\item{reference}{Which pathway reference should be used ("REACTOME"
for Reactome, "GOBP" for GO Biological Process,
or "REACTOME-GOBP" for both).}

\item{max.pw.size}{Maximum pathway size to consider from the pathway
reference.}

\item{min.pw.size}{Minimum pathway size to consider from the pathway
reference.}

\item{min.positive}{Minimum number of target genes to be found in a given
pathway.}

\item{restrict.pw}{A list of pathway IDs to restrict the application of
the function.}

\item{with.complex}{A logical indicating whether receptor co-complex
members should be included in the target genes.}

\item{fdr.proc}{The procedure for adjusting P-values according to
\code{\link[multtest]{mt.rawp2adjp}}.}
}
\value{
A BSRInferenceComp object with initial inferences set.
}
\description{
This method supports two configurations that we refer to
as paracrine and autocrine.
}
\details{
In the autocrine case, a single cluster comparison name is provided.
In the corresponding cluster comparison, a group of samples A was
compared to a group of samples B to determine fold-changes and associated
P-values. The inferred ligand-receptor interactions take place in the
samples of group A. A typical single-cell example would be a population of
macrophages (group A) compared to all the other populations (group B) to
represent specific increased or decreased expression in macrophages. The
resulting ligand-receptor interactions will be autocrine interactions
that are exacerbated (or reduced depending on the chosen parameters) in
macrophages.

In the paracrine case, two cluster comparison names must be provided.
For instance, a first comparison could involve macrophages versus all
the other cell populations as above. The second comparison could be
B-cells against all the other populations. Now, calling
\code{BSRInferenceComp()}
with comparison macrophages *versus* the rest and, as source comparison,
B-cells *versus* the rest, will result in inferring interactions between
B-cells (ligands) and macrophages (receptors and downstream pathways). To
obtain macrophages to B-cells paracrine interactions, it is necessary to call
the method a second time with permuted cluster comparison names. Another
example in spatial transcriptomics could be two thin bands at the boundary of
two tissue regions, one emitting the ligand and the other one expressing the
receptor.

In this initial inference, all the receptor-containing pathways are reported,
see reduction functions to reduce this list.

Perform the initial ligand-receptor inference. Initial means that
no reduction is applied. All the (ligand, receptor, downstream pathway)
triples are reported, i.e., a given LR pair may appear multiple times
with different pathways downstream the receptor. Specific reduction
functions are available from the package to operate subsequent
simplifications based on the BSRInferenceComp object created by this method.

Here, ligand-receptor interactions are inferred based on gene or protein
regulation-associated P-values when comparing two clusters of samples. Since
a BSRDataModelComp object can contain several such comparisons, the name
of the comparison to use must be specified (parameter \code{cmp.name}).

Note that since the introduction of the use.full.network parameter
(April 29, 2024), the pathway sizes are always computed before potential
intersection with the observed data (use.full.network set to FALSE) for
consistency. Accordingly, the minimum and maximum pathway default values
have been raised from 5 & 200 to 5 & 400 respectively. By default,
use.full.network is set to FALSE.

In addition to statistical significance estimated according to BulkSignalR
statistical model, we compute SingleCellSignalR original LR-score,
based on L and R cluster average expression. 
In the paracrine case, L average expression
is taken from the source cluster.
}
\examples{
data(bsrdm.comp, package = "BulkSignalR")
data(immune.signatures, package = "BulkSignalR")

# infer ligand-receptor interactions from the comparison
bsrinf.comp <- BSRInferenceComp(bsrdm.comp, max.pval = 1, 
reference="REACTOME",
"random.example")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInferenceComp-class.R
\name{updateInference}
\alias{updateInference}
\alias{updateInference,BSRInferenceComp-method}
\title{Inference updating}
\usage{
\S4method{updateInference}{BSRInferenceComp}(
  obj,
  bsrcc,
  ncounts,
  src.bsrcc = NULL,
  rank.p = 0.55,
  max.pval = 0.01,
  min.logFC = 1,
  min.LR.score = 0,
  neg.receptors = FALSE,
  pos.targets = FALSE,
  neg.targets = FALSE,
  min.t.logFC = 0.5,
  min.positive = 2,
  fdr.proc = c("BH", "Bonferroni", "Holm", "Hochberg", "SidakSS", "SidakSD", "BY", "ABH",
    "TSBH")
)
}
\arguments{
\item{obj}{BSRInferenceComp object.}

\item{bsrcc}{BSRClusterComp object relative to target cells.}

\item{ncounts}{Matrix counts normalized.}

\item{src.bsrcc}{BSRClusterComp object relative to source cells.}

\item{rank.p}{A number between 0 and 1 defining the rank of the last
considered target genes.}

\item{max.pval}{The maximum P-value imposed to both the ligand
and the receptor.}

\item{min.logFC}{The minimum log2 fold-change allowed for
both the receptor and the ligand.}

\item{min.LR.score}{The minimum LR-score allowed for the interaction.}

\item{neg.receptors}{A logical indicating whether receptors are only
allowed to be upregulated (FALSE), or up- and downregulated (TRUE).}

\item{pos.targets}{A logical imposing that all the network targets must
display positive logFC, i.e. logFC >= min.t.logFC.}

\item{neg.targets}{A logical imposing that all the network targets must
display negative logFC, i.e. logFC <= - min.t.logFC.}

\item{min.t.logFC}{The minimum log2 fold-change allowed for
targets in case pos.targets or neg.targets are used.}

\item{min.positive}{Minimum number of target genes to be found in a given
pathway.}

\item{fdr.proc}{The procedure for adjusting P-values according to
\code{\link[multtest]{mt.rawp2adjp}}.}
}
\value{
A BSRInferenceComp object. The main application of this
method is to take a "universal" inference obtained by assigning
each gene to good logFC, P-values and expression levels whose role
is to find all the reachable targets per receptor/pathway, and
to update it by using actual logFC, P-values, and expression data.
The benefit is to save time when multiple sample comparisons are
performed, only one network exploration is necessary. Note that
if a restrictive logic such as \code{positive.targets=TRUE} is used,
the result will be correct provided all the targets were in the
initial BSRInferenceComp object. If a restriction on the targets
was applied, then the update is likely to miss some targets, i.e.,
the statistical analysis will be wrong.

In case no L-R interaction is above the required thresholds, the
value `NULL` is returned.

Note that correlations are set to 1 to avoid
lengthy computations with scRNA-seq data and multiple cell
populations.

The main role of this method is to support our SingleCellSignalR Version 2
package.
}
\description{
A method to update the data underlying statistical significance estimations
prior to rescoring for an existing BSRInferenceComp object
(P- and Q-value estimations as well as LR-score).
}
\details{
A BSRInferenceComp object should be created by calling
\code{"\link[=BSRInferenceComp-class]{BSRInferenceComp}"}
}
\examples{
data(bsrdm.comp, package = "BulkSignalR")
data(bsrinf.comp, package = "BulkSignalR")
colA <- as.integer(1:2)
colB <- as.integer(3:4)

#bsrdm.comp <- as(bsrdm, "BSRDataModelComp")

n <- nrow(ncounts(bsrdm.comp))
stats <- data.frame(pval = runif(n),
logFC = rnorm(n, 0, 2),
expr = runif(n, 0, 10))
rownames(stats) <- rownames(ncounts(bsrdm.comp))


# update
stats$pval <- stats$pval / 100
stats$logFC <- stats$logFC + 0.5

bsrcc.2 <- BSRClusterComp(bsrdm.comp, colA, colB, stats)
bsrinf.updated <- updateInference(bsrinf.comp, bsrcc.2,
max.pval = 1, min.logFC = 0.1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{sdc}
\alias{sdc}
\title{Salivary duct carcinoma transcriptoms}
\format{
A data frame with 19764 rows and 26 variables.
}
\source{
\url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE138581}
}
\usage{
data(sdc)
}
\description{
A data set containing the read counts of salivary duct carcinomas (SDCs)
  and adjacent normal tissues.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{.readRDSFromCache}
\alias{.readRDSFromCache}
\title{Read RDS from the cache.}
\usage{
.readRDSFromCache(bfc, resourceName)
}
\arguments{
\item{bfc}{Object of class BiocFileCache, created by a call to
BiocFileCache::BiocFileCache()}

\item{resourceName}{keyword associated to a specific resourceName}
}
\value{
Returns BiocFileCache::bfcquery object or FALSE
}
\description{
Access  resources (pathways or network
stored in the cache.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModel-class.R
\name{ncounts<-,BSRDataModel-method}
\alias{ncounts<-,BSRDataModel-method}
\title{Normalized count matrix setter (internal use only)}
\usage{
\S4method{ncounts}{BSRDataModel}(x) <- value
}
\arguments{
\item{x}{object BSRDataModel}

\item{value}{value to be set for BSRDataModel}
}
\value{
returns \code{NULL}
}
\description{
Normalized count matrix setter (internal use only)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R, R/BSRSignature-class.R
\name{ligands}
\alias{ligands}
\alias{ligands,BSRInference-method}
\alias{ligands,BSRSignature-method}
\title{ligands accessor}
\usage{
\S4method{ligands}{BSRInference}(x)

\S4method{ligands}{BSRSignature}(x)
}
\arguments{
\item{x}{BSRSignature}
}
\value{
ligands

ligands
}
\description{
ligands accessor

ligands accessor
}
\examples{
bsr.sig <- new("BSRSignature")
ligands(bsr.sig)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{cacheClear}
\alias{cacheClear}
\title{Delete cache content.}
\usage{
cacheClear(dir = c("resources"))
}
\arguments{
\item{dir}{Directory to remove. Can be only 'resources'.}
}
\value{
Returns `NULL`, invisibly.
}
\description{
Delete the content of cache directory.
}
\examples{
cacheClear(dir="resources")
# need to recreate database in order to run examples well
createResources(verbose=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRInference-class.R
\name{resetToInitialOrganism}
\alias{resetToInitialOrganism}
\alias{resetToInitialOrganism,BSRInference-method}
\title{Reset gene names to initial organism provided in the first instance}
\usage{
\S4method{resetToInitialOrganism}{BSRInference}(obj, conversion.dict)
}
\arguments{
\item{obj}{BSRInference object}

\item{conversion.dict}{A dictionnary}
}
\value{
An BSRInference object updated for gene names.
The gene names are replaced by the ones from
the organism provided in the first instance.
}
\description{
Reset gene names to initial organism provided in the first instance
}
\examples{
data(bodyMap.mouse, package = "BulkSignalR")
data(bsrinf.mouse, package = "BulkSignalR")
data(ortholog.dict, package = "BulkSignalR")

#idx <- sample(nrow(bodyMap.mouse), 7500)

#bodyMap.mouse <- bodyMap.mouse[idx,1:3]

#ortholog.dict <- findOrthoGenes(
#    from_organism = "mmusculus",
#    from_values = rownames(bodyMap.mouse)
#)

#matrix.expression.human <- convertToHuman(
#    counts = bodyMap.mouse,
#    dictionary = ortholog.dict
#)

#bsrdm <- BSRDataModel(
#    counts = matrix.expression.human,
#    species = "mmusculus",
#    conversion.dict = ortholog.dict
#)

#bsrdm <- learnParameters(bsrdm,
#    quick = TRUE  
#)

#reactSubset <- getResource(resourceName = "Reactome",
#cache = TRUE)

#subset <- c("REACTOME_BASIGIN_INTERACTIONS",
#"REACTOME_SYNDECAN_INTERACTIONS",
#"REACTOME_ECM_PROTEOGLYCANS",
#"REACTOME_CELL_JUNCTION_ORGANIZATION")

#reactSubset <- reactSubset[
#reactSubset$`Reactome name` \%in\% subset,]

#bsrinf.mouse <- BSRInference(bsrdm,reference="REACTOME")

bsrinf <- resetToInitialOrganism(bsrinf.mouse, 
conversion.dict = ortholog.dict)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModel-class.R,
%   R/BSRDataModelComp-class.R
\name{scoreLRGeneSignatures}
\alias{scoreLRGeneSignatures}
\alias{scoreLRGeneSignatures,BSRDataModel-method}
\alias{scoreLRGeneSignatures,BSRDataModelComp-method}
\title{Score ligand-receptor gene signatures}
\usage{
\S4method{scoreLRGeneSignatures}{BSRDataModel}(
  obj,
  sig,
  LR.weight = 0.5,
  robust = FALSE,
  name.by.pathway = FALSE,
  abs.z.score = FALSE,
  rownames.LRP = FALSE
)

\S4method{scoreLRGeneSignatures}{BSRDataModelComp}(
  obj,
  sig,
  LR.weight = 0.5,
  robust = FALSE,
  name.by.pathway = FALSE,
  abs.z.score = FALSE,
  rownames.LRP = FALSE
)
}
\arguments{
\item{obj}{A BSRDataModelComp object.}

\item{sig}{A BSRSignatureComp object.}

\item{LR.weight}{A number between 0 and 1 defining the relative weight
of the ligand and the receptor in the signature.}

\item{robust}{A logical indicating that z-scores should be computed
with median and MAD instead of mean and standard deviation.}

\item{name.by.pathway}{A logical indicating whether row names of the
resulting score matrix should be pathway names.}

\item{abs.z.score}{A logical to use absolute z-scores (useful if the
activity of a paythway is reported by a mixture of up- and down-genes
whose z-score averages might hide actual activity).}

\item{rownames.LRP}{A logical indicating, in case \code{name.by.pathway}
was set to TRUE, whether ligand and receptor names should be added on top.
No role if \code{name.by.pathway} was set to FALSE.}
}
\value{
A matrix containing the scores of each ligand-receptor gene
signature in each sample.

A matrix containing the scores of each ligand-receptor gene
signature in each sample.
}
\description{
Compute ligand-receptor gene signature scores over a BSRDataModel.

Compute ligand-receptor gene signature scores over a BSRDataModelComp
specific comparison.
}
\examples{
data(bsrdm, package = "BulkSignalR")
data(bsrinf, package = "BulkSignalR")

bsrinf.redBP <- reduceToBestPathway(bsrinf)
bsrsig.redBP <- BSRSignature(bsrinf.redBP, qval.thres = 0.001)
res <-scoreLRGeneSignatures(bsrdm, bsrsig.redBP,
    name.by.pathway = FALSE
)
# prepare data
data(bsrdm.comp, package = "BulkSignalR")
data(bsrinf.comp, package = "BulkSignalR")

# reduction
bsrinf.red <- reduceToBestPathway(bsrinf.comp)
# signature extraction and scoring
bsrsig.red <- BSRSignatureComp(bsrinf.red, qval.thres = 1e-6)
scores.red <- scoreLRGeneSignatures(bsrdm.comp, bsrsig.red,
    name.by.pathway = TRUE, rownames.LRP = TRUE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRClusterComp-class.R
\name{colClusterB}
\alias{colClusterB}
\alias{colClusterB,BSRClusterComp-method}
\title{Cluster B columns accessor}
\usage{
\S4method{colClusterB}{BSRClusterComp}(x)
}
\arguments{
\item{x}{object BSRClusterComp}
}
\value{
col.clusterB
}
\description{
Cluster B columns accessor
}
\examples{
bsrcc <- new("BSRClusterComp")
colClusterB(bsrcc)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regulatedInference.R
\name{.downstreamRegulatedSignaling}
\alias{.downstreamRegulatedSignaling}
\title{Internal function to check receptor signaling downstream}
\usage{
.downstreamRegulatedSignaling(
  lr,
  pw,
  pw.size,
  rncounts,
  stats,
  id.col,
  gene.col,
  pw.col,
  min.positive,
  pos.targets = FALSE,
  neg.targets = FALSE,
  min.t.logFC = 0.5,
  with.complex = TRUE
)
}
\arguments{
\item{lr}{A data frame as returned by
\code{.getRegulatedLR()}.}

\item{pw}{A table defining the reference pathways.}

\item{pw.size}{A named vector with pathway sizes (names are pathway
IDs).}

\item{rncounts}{A matrix of normalized read counts with at
least all the ligands, receptors, and genes in the reference pathways.}

\item{stats}{A data.frame with a column 'pval' and
\code{rownames(stats)} assigned to gene symbols. Rows must at least
  include all the ligands, receptors, and genes in the reference pathways.}

\item{id.col}{Column index or name in \code{pw} for the pathway IDs.}

\item{gene.col}{Column index or name in \code{pw} for the gene
symbols.}

\item{pw.col}{Column index or name in \code{pw} for the pathway
names.}

\item{min.positive}{Minimum number of target genes to be found in a given
pathway.}

\item{pos.targets}{A logical imposing that all the network targets must
display positive logFC, i.e. logFC >= min.t.logFC.}

\item{neg.targets}{A logical imposing that all the network targets must
display negative logFC, i.e. logFC <= - min.t.logFC.}

\item{min.t.logFC}{The minimum log2 fold-change allowed for
targets in case pos.targets or neg.targets are used.}

\item{with.complex}{A logical indicating whether receptor co-complex
members should be included in the target genes.}
}
\value{
A table reporting all the ligand-receptor pairs provided in \code{lr}
  along with the pathways found and data about target gene regulation
  P-values. Note that correlations are currently set to 1 to avoid
  lengthy computations with scRNA-seq data and multiple cell
  populations.
}
\description{
Internal function to check receptor signaling downstream
}
\keyword{internal}
